dnl #########################################################################
dnl
dnl Project : Linux Explorer & Mindseye
dnl Copyright 1996,1997. All Rights Reserved.
dnl
dnl $Id: aclocal.m4,v 1.3 2001/07/07 12:50:53 ainvar Exp $
dnl
dnl Revision 1.3  1998/06/24 18:59:00  source
dnl plugin + bug fixes
dnl
dnl Revision 1.2  1998/05/14 20:40:40  source
dnl release
dnl
dnl Revision 1.1.1.1  1997/11/22 15:59:09  source
dnl import to CVS rep
dnl
dnl Revision 1.6  1997/06/24 13:03:57  ruben
dnl Fixed cannot-find-mesa-in-my-total-weird-path bug in the AC_PATH_GL
dnl macro. problem may persist in AC_PATH_QT, please report if so...
dnl
dnl Revision 1.7 1997/06/24 maku
dnl added -lGLU / -lMesaGLU to GL_LIBS
dnl
dnl #########################################################################


dnl #########################################################################
dnl Synopsis:
dnl     AC_PATH_USR_LOCALSHARE
dnl Purpose:
dnl     To include local includes and libraries in the searchpath
dnl     Currently: only /usr/share/ include and lib and
dnl     /usr/local include and lib are added to the searchpath
dnl
dnl Access:
dnl     None
dnl
dnl Defines: (to DEFS or config.h)
dnl     None
dnl Substitutions: (to Makefile from Makefile.in)
dnl     CFLAGS  - Will have the gcc/g++ specific flags
dnl Requirements:
dnl     AC_PROG_CC must be called first
dnl Returns:
dnl     gcc flags in CFLAGS or configure will warn that you are not using gcc
dnl #########################################################################

AC_DEFUN(AC_PATH_USR_LOCALSHARE,
[
AC_PROVIDE([$0])

AC_CHECKING(["for extra includes"])

for ac_dir in               \
  /usr/local/include        \
  /usr/share/include        \
  ; \
do
  if test -d "$ac_dir"; then
    CPPFLAGS="${CPPFLAGS} -I$ac_dir"
    AC_MSG_RESULT(["+ Adding -I$ac_dir"])
  fi
done


AC_CHECKING(["for extra library dirs"])
for ac_dir in               \
  /usr/local/lib            \
  /usr/share/lib            \
  ; \
do
  if test -d "$ac_dir"; then
    LIBS="${LIBS} -L$ac_dir"
    AC_MSG_RESULT(["+ Adding -L$ac_dir"])
  fi
done

])

dnl 
dnl #########################################################################
dnl Synopsis:
dnl        AC_C_GCCFLAGS
dnl    Purpose:
dnl        To set g++ specific flags
dnl Access:
dnl
dnl --enable-gcc-warn-all   when using gcc, warn on everything    
dnl --enable-gcc-error-all  when using gcc, turn all warnings in errors
dnl 
dnl
dnl Defines: (to DEFS or config.h)
dnl        None
dnl    Substitutions: (to Makefile from Makefile.in)
dnl        CFLAGS  - Will have the gcc/g++ specific flags
dnl Requirements:
dnl        AC_PROG_CC must be called first
dnl    Returns:
dnl        gcc flags in CFLAGS or configure will warn that you are not using gcc
dnl #########################################################################

AC_DEFUN(AC_C_GCCFLAGS,
[
AC_PROVIDE([$0])
AC_REQUIRE([AC_PROG_CC])

ac_cv_with_gcc_warn_all=
ac_cv_with_gcc_error_all=


AC_ARG_ENABLE(gcc-warn-all,
    [  --enable-gcc-warn-all   when using gcc, warn on everything],
    [  ac_cv_enable_gcc_warn_all="$enableval" ])

AC_ARG_ENABLE(gcc-error-all,
    [  --enable-gcc-error-all  when using gcc, turn all warnings in errors],
    [  ac_cv_enable_gcc_error_all="$enableval" ])

if test "$GCC" = yes ; then
    AC_MSG_CHECKING(["for extra gcc flags"])
    if test "${ac_cv_with_gcc_error_all}" = yes; then
        CFLAGS="${CFLAGS} -Werror"
    fi
    if test "${ac_cv_with_gcc_warn_all}" = yes; then
        CFLAGS="${CFLAGS} -Wall"
    fi
    AC_MSG_RESULT(["$CFLAGS"])
else
    AC_MSG_WARN(["Using gcc specific flags disabled because you don\'t have gcc"])
fi
])

dnl #########################################################################
dnl Synopsis:
dnl        AC_C_GXXFLAGS
dnl    Purpose:
dnl        To set g++ specific flags
dnl Access:
dnl
dnl --enable-g++-warn-all   when using g++, warn on everything    
dnl --enable-g++-error-all  when using g++, turn all warnings in errors
dnl 
dnl
dnl Defines: (to DEFS or config.h)
dnl        None
dnl    Substitutions: (to Makefile from Makefile.in)
dnl        CXXFLAGS  - Will have the g++/g++ specific flags
dnl Requirements:
dnl        AC_PROG_CC must be called first
dnl    Returns:
dnl        g++ flags in CXXFLAGS or configure will warn that you are not using g++
dnl #########################################################################

AC_DEFUN(AC_C_GXXFLAGS,
[
AC_PROVIDE([$0])
AC_REQUIRE([AC_PROG_CXX])

ac_cv_enable_gxx_warn_all=
ac_cv_enable_gxx_error_all=


AC_ARG_ENABLE(gxx-warn-all,
    [  --enable-gxx-warn-all   when using g++, warn on everything],
    [  ac_cv_enable_gxx_warn_all="$enableval" ],
    [  ac_cv_enable_gxx_warn_all="no" ])

AC_ARG_ENABLE(gxx-error-all,
    [  --enable-gxx-error-all  when using g++, turn all warnings in errors],
    [  ac_cv_enable_gxx_error_all="$enableval" ],
    [  ac_cv_enable_gxx_error_all="no" ])
    

if test "$GXX" = yes ; then
    AC_MSG_CHECKING(["for extra g++ flags"])
    if test "${ac_cv_enable_gxx_error_all}" = yes; then
        CXXFLAGS="${CXXFLAGS} -Werror"
    fi
    if test "${ac_cv_enable_gxx_warn_all}" = yes; then
        CXXFLAGS="${CXXFLAGS} -Wall"
    fi
    AC_MSG_RESULT(["$CXXFLAGS"])
else
    AC_MSG_WARN(["Using g++ specific flags disabled because you don\'t have g++"])
fi
])

dnl #########################################################################
dnl Synopsis:
dnl        AC_PATH_QT
dnl    Purpose:
dnl        To seek and find your Qt installation
dnl Access:
dnl
dnl --with-qt-dir           where the root of qt is installed (/usr/local/qt)    
dnl --with-qt-include       where the qt includes are. (/usr/local/qt/include)
dnl --with-qt-lib           where the qt library is installed. (/usr/local/qt/lib)
dnl
dnl Defines: (to DEFS or config.h)
dnl        None
dnl    Substitutions: (to Makefile from Makefile.in)
dnl        QT_CFLAGS - Path to Qt includes (-Iwhateverincdir)
dnl        QT_LIBS   - Linkpath and libraryname (-Lwhateverlibdir -lqt)
dnl Requirements:
dnl        AC_PATH_XTRA must be called first
dnl        Assumes that libqgl is in the same location as libqt.
dnl Returns:
dnl        values in QT_CFLAGS and QT_LIBS, or configure will stop on error
dnl #########################################################################

AC_DEFUN(AC_PATH_QT,
[
AC_PROVIDE([$0])
AC_REQUIRE_CPP
AC_REQUIRE([AC_PATH_XTRA])

ac_cv_with_qt_dir=
ac_cv_with_qt_lib=
ac_cv_with_qt_inc=
QT_CFLAGS=
QT_LIBS=

AC_ARG_WITH(qt-dir,
    [  --with-qt-dir           where the root of qt is installed (/usr/local/qt) ],
    [  ac_cv_with_qt_dir="$withval" ])

AC_ARG_WITH(qt-include,
    [  --with-qt-include       where the qt includes are. (/usr/local/qt/include) ],
    [  ac_cv_with_qt_inc="$withval" ])
    
AC_ARG_WITH(qt-lib,
    [  --with-qt-lib           where the qt library is installed. (/usr/local/qt/lib)],
    [  ac_cv_with_qt_lib="$withval" ])

dnl ac_cv_with_qt_lib and ac_cv_with_qt_inc are our working variables
dnl set them if they are not set.
if test -n "$ac_cv_with_qt_dir" ; then
  if test ! -n "$ac_cv_with_qt_inc" ; then
    ac_cv_with_qt_inc="${ac_cv_with_qt_dir}/include"
  fi
  if test ! -n "$ac_cv_with_qt_lib" ; then
    ac_cv_with_qt_lib="${ac_cv_with_qt_dir}/lib"
  fi
fi

dnl check for libraries and includes

AC_MSG_CHECKING(["for Qt include paths"])
AC_MSG_RESULT([])
AC_LANG_SAVE
AC_LANG_CPLUSPLUS

SAVE_CPPFLAGS=$CPPFLAGS
for ac_dir in                   \
    $ac_cv_with_qt_inc          \
    ""                          \
    /usr/local/qt/include       \
    /usr/local/include/qt       \
    /usr/share/qt/include       \
    /usr/share/include/qt       \
    /usr/qt/include             \
    /usr/include/qt             \
    /usr/X11R6/include/qt       \
    /usr/include/X11/qt         \
    /usr/local/X11R6/include/qt \
    /usr/local/X11/include/qt   \
    /usr/lib/qt/include		\
    ; \
do
  if test -n "$ac_dir" ; then
    AC_MSG_CHECKING(["in $ac_dir"])
    CPPFLAGS="-I$ac_dir"
  else
    CPPFLAGS=
    AC_MSG_CHECKING(["defaults"])
  fi
  AC_TRY_CPP([#include <qapp.h>],
  [
    AC_MSG_RESULT(["yes"])
    QT_INC_FOUND=yes
    ac_cv_with_qt_inc=$ac_dir
    QT_CFLAGS="-I${ac_dir}"
    break
  ],
  [
    AC_MSG_RESULT(["no"])
  ])
done
if test ! -n $QT_INC_FOUND ; then
  AC_MSG_ERROR(["Qt includes not found"])
fi
CPPFLAGS=$SAVE_CPPFLAGS
unset SAVE_CPPFLAGS
unset QT_INC_FOUND
AC_LANG_RESTORE


AC_MSG_CHECKING(["for Qt library path"])
AC_MSG_RESULT([])

SAVE_LIBS="${LIBS}"
for ac_dir in               \
    $ac_cv_with_qt_lib      \
    ""                      \
    /usr/local/qt/lib       \
    /usr/local/lib/qt       \
    /usr/share/qt/lib       \
    /usr/share/lib/qt       \
    /usr/qt/lib             \
    /usr/lib/qt             \
    /usr/X11R6/lib/qt       \
    /usr/lib/X11/qt         \
    /usr/local/X11R6/lib/qt \
    /usr/local/X11/lib/qt   \
    /usr/X11R6/lib          \
    /usr/lib/X11            \
    /usr/local/X11R6/lib    \
    /usr/local/X11/lib      \
    ; \
do
  if test -n "$ac_dir" ; then
    AC_MSG_CHECKING(["in $ac_dir"])
    LIBS="-L${ac_dir}"  
  else
    AC_MSG_CHECKING(["defaults"])
    LIBS=
  fi
  AC_CHECK_LIB(qt, main,
  [
    QT_LIB_FOUND=yes
    ac_cv_with_qt_lib=$ac_dir
    QT_LIBS="${LIBS} -lqt"
    break
  ],
  [
    AC_MSG_RESULT(["No"])
  ],
  [$X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS])

done
if test ! -n $QT_LIB_FOUND ; then
  AC_MSG_ERROR(["Qt libraries not found"])
fi
LIBS="${SAVE_LIBS}"
unset SAVE_LIBS
unset QT_LIB_FOUND

dnl look for moc, add qtdir/bin to path if qtdir is set
SAVE_PATH="${PATH}"
if test -n $PATH -a -n "$ac_cv_with_qt_dir" ; then
    PATH="${PATH}:$ac_cv_with_qt_dir/bin"
elif test -n "$ac_cv_with_qt_dir" ; then
    PATH="$ac_cv_with_qt_dir/bin"
fi

AC_PATH_PROG(MOC, moc, no)
if test "$MOC" = no; then
    AC_MSG_ERROR("Can\'t find the meta object compiler")
fi
PATH="${SAVE_PATH}"
unset SAVE_PATH


AC_SUBST(QT_LIBS)
AC_SUBST(QT_CFLAGS)

])

dnl #########################################################################
dnl Synopsis:
dnl        AC_PATH_GL
dnl    Purpose:
dnl        To seek and find your Qt installation
dnl Access:
dnl
dnl    --with-gl-dir           where the root of gl is installed (/usr/local/gl)    
dnl --with-gl-include       where the gl includes are. (/usr/local/gl/include)
dnl --with-gl-lib           where the gl library is installed. (/usr/local/gl/lib)
dnl --enable-mesagl         If you always want MesaGL instead of just GL
dnl Defines: (to DEFS or config.h)
dnl        None
dnl    Substitutions: (to Makefile from Makefile.in)
dnl        GL_CFLAGS - Path to Qt includes (-Iwhateverincdir)
dnl        GL_LIBS   - Linkpath and libraryname (-Lwhateverlibdir -lgl)
dnl Requirements:
dnl        AC_PATH_XTRA must be called first, for now
dnl    Returns:
dnl        values in GL_CFLAGS and GL_LIBS, or configure will stop on error
dnl #########################################################################

AC_DEFUN(AC_PATH_GL,
[
AC_PROVIDE([$0])
AC_REQUIRE([AC_PATH_XTRA])

ac_cv_with_gl_dir=
ac_cv_with_gl_lib=
ac_cv_with_gl_inc=
ac_cv_enable_mesagl=

GL_CFLAGS=
GL_LIBS=

AC_ARG_WITH(gl-dir,
    [  --with-gl-dir           where the root of gl is installed (/usr/local/gl) ],
    [  ac_cv_with_gl_dir="$withval" ])

AC_ARG_WITH(gl-include,
    [  --with-gl-include       where the gl includes are. (/usr/local/gl/include) ],
    [  ac_cv_with_gl_inc="$withval" ])
    
AC_ARG_WITH(gl-lib,
    [  --with-gl-lib           where the gl library is installed. (/usr/local/gl/lib)],
    [  ac_cv_with_gl_lib="$withval" ])

AC_ARG_ENABLE(mesagl,
    [  --enable-mesagl         If you always want MesaGL instead of just GL ],
    [  ac_cv_enable_mesagl="$enableval" ],
    [  ac_cv_enable_mesagl="no" ])
    

AC_MSG_CHECKING("we like Mesa GL instead of just OpenGL")
AC_MSG_RESULT("$ac_cv_enable_mesagl")

dnl try to set some paths

AC_MSG_CHECKING("for Gl location")

for ac_dir in                           \
        ${GLDIR}                                \
        ${ac_cv_with_gl_dir}    \
        /usr/gl                         \
        /usr/local/gl                   \
        /usr/share/gl                   \
        /usr/GL                         \
        /usr/local/GL                   \
        /usr/share/GL                   \
        /usr/MesaGL                         \
        /usr/local/MesaGL               \
        /usr/share/MesaGL               \
        ; \
do
        if test -n "$ac_dir" && test -d "$ac_dir"; then
        ac_cv_with_gl_dir="$ac_dir" 
                break;
        fi
done

if test ! -n "$ac_cv_with_gl_dir"; then
        AC_MSG_RESULT("don\'t know...")
else
        AC_MSG_RESULT("$ac_cv_with_gl_dir")
fi
dnl check for libraries and includes
dnl TODO: use the value of $ac_cv_with_gl_inc first if it existed!
 

AC_MSG_CHECKING("for Gl include paths")

AC_TRY_CPP([#include <GL/gl.h>],
[
        AC_MSG_RESULT(["Good, includes in your path"])
],
[
AC_MSG_RESULT(["Not in your path, searching..."])
for ac_dir in                   \
        $ac_cv_with_gl_dir/include      \
        $ac_cv_with_gl_inc              \
    /usr/local/MesaGL/include   \
    /usr/local/include          \
    /usr/share/MesaGL/include   \
    /usr/share/include          \
    /usr/MesaGL/include         \
    /usr/include                \
    /usr/include/GL             \
    /usr/X11R6/include          \
    /usr/include/X11            \
    /usr/local/X11R6/include    \
    /usr/local/X11/include      \
    ; \
do
   AC_MSG_CHECKING(["in $ac_dir"])
   if test -r "$ac_dir/GL/gl.h"; then
      ac_cv_with_gl_inc=$ac_dir
      break
   fi
   AC_MSG_RESULT(["no"])
done

if test -n "$ac_cv_with_gl_inc" ; then
   AC_MSG_RESULT(["yes"])
   GL_CFLAGS="-I${ac_cv_with_gl_inc}"
else
   AC_MSG_ERROR(["Can\'t find the Gl includes"])
fi
])



AC_MSG_RESULT(["+ Gl libraries ${ac_cv_with_gl_lib}, Gl includes ${ac_cv_with_gl_inc}"])

SAVE_LIBS="${LIBS}"



if test ! -n "${ac_cv_with_gl_lib}" && test -n "${ac_cv_with_gl_dir}"; then
    ac_cv_with_gl_lib="${ac_cv_with_gl_dir}/lib"
elif test ! -n "${ac_cv_with_gl_lib}"; then 
    ac_cv_with_gl_lib="."
fi

if test "$ac_cv_enable_mesagl" = no; then
    AC_CHECK_LIB(GL, main,
    [
        if test "${ac_cv_with_gl_lib}" = "."; then
            GL_LIBS="-lGLU -lGL -lXext -lm"
        else
            GL_LIBS="-L${ac_cv_with_gl_lib} -lGLU -lGL -lXext -lm"
        fi
        AC_SUBST(GL_LIBS)
    ],
    [
        GL_LIBS=no
    ],
    [-L$ac_cv_with_gl_lib $X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS -lXext -lm])
fi
if test "$ac_cv_enable_mesagl" = yes || test "${GL_LIBS}" = no; then
    if test "$ac_cv_enable_mesagl" = yes; then
        AC_MSG_WARN(["Will not link with standard Gl, trying to force the Mesa library..."])
    else
        AC_MSG_WARN(["Can\'t not link with Gl, trying the Mesa library..."])
    fi
    AC_CHECK_LIB(MesaGL, main,
    [
        if test "${ac_cv_with_gl_lib}" = "."; then
            GL_LIBS="-lMesaGLU -lMesaGL -lXext -lm"
        else
            GL_LIBS="-L${ac_cv_with_gl_lib} -lMesaGLU -lMesaGL -lXext -lm"
        fi
        AC_SUBST(GL_LIBS)
    ],
    [
        AC_MSG_ERROR(["You need to install \(Mesa\) OpenGL first: can\'t find either the GL or MesaGL library"])
    ], [-L$ac_cv_with_gl_lib $X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS -lXext -lm])
fi

LIBS="${SAVE_LIBS}"

AC_SUBST(GL_CFLAGS)

])

dnl #########################################################################
dnl Synopsis:
dnl        AC_TEST_PTHREAD
dnl
dnl    Purpose:
dnl        To test for posix thread support and find the necessary libs and
dnl	   includes
dnl
dnl Access:
dnl    --disable-threads       always without thread support
dnl Defines: (to DEFS or config.h)
dnl        WITH_PTHREADS
dnl    Substitutions: (to Makefile from Makefile.in)
dnl        THREAD_LIBS   - Linkpath and libraryname (-Lwhateverlibdir -lgl)
dnl    Returns:
dnl
dnl #########################################################################
AC_DEFUN(AC_TEST_PTHREAD,
[
THREAD_LIBS=""

use_pthreads=no
AC_CHECK_HEADERS(pthread.h, [use_pthreads=yes])
if test "$use_pthreads" = "yes"; then
  AC_CHECK_LIB(pthread, pthread_create, THREAD_LIBS="-lpthread")
fi

# on an HP with pthread we need to use -lcma
# on dec alphas we have problems as well
if test "$use_pthreads" = "yes"; then
case $target in
    HP-UX-*.10.*)
	THREAD_LIBS="-lcma"
	;;
    alpha-dec-osf3.2)
        THREAD_LIBS=""
	use_pthreads=no
        ;;
esac
fi

if test "$use_pthreads" = "yes"; then
  AC_DEFINE( WITH_PTHREADS )
fi

AC_SUBST(THREAD_LIBS)

])

AC_DEFUN(AC_ADD_ARGS,[
  ac_configure_args=$ac_configure_args'$1'
])
