/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: ac3dobjects.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 * $LOG:$
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __NFFOBJECTS_H__
#define __NFFOBJECTS_H__

#include "fstream.h"


class G3DShadable;
class G3DGLShader;


class NFFSurface
{
public:
  NFFSurface( ifstream &inpf );

protected:
  float r, g, b, kd, ks, shine, t, ior;
};


/**Dummy class to reference every NFF Object via a common pointer
 */
class NFFObject
{
public:
  NFFObject () { _ok = 1; };

  virtual G3DShadable* buildG3DObject () = 0;
  int  isValid () { return( _ok ); };
  void setSurface( NFFSurface* );

protected:
  int _ok;
};


class NFFCamera : public NFFObject
{
public:
  NFFCamera( ifstream &inpf, char* );

  virtual G3DShadable* buildG3DObject ();
          G3DGLShader* buildG3DCamera ();

protected:
  double fromx, fromy, fromz;
  double atx, aty, atz;
  double upx, upy, upz;
  double zoom;
  double near;

};


class NFFCone : public NFFObject
{
public:
  NFFCone( ifstream &inpf );

  virtual G3DShadable* buildG3DObject ();

protected:
  float bX, bY, bZ, bR, aX, aY, aZ, aR;
};


class NFFLight : public NFFObject
{
public:
  NFFLight( ifstream &inpf, char* );

  virtual G3DShadable* buildG3DObject ();

protected:
  VectorStruct pos;
  float cx, cy, cz;

};


class NFFPPatch : public NFFObject
{
public:
  NFFPPatch( ifstream &inpf );
  ~NFFPPatch();

  virtual G3DShadable* buildG3DObject ();

protected:
  G3DPoint3d  *_vertices;
  G3DVector3d *_normals;

};


class NFFPolygon : public NFFObject
{
public:
  NFFPolygon( ifstream &inpf );
  ~NFFPolygon( );

  virtual G3DShadable* buildG3DObject ();

protected:
  G3DPoint3d *_vertices;

};


class NFFSphere : public NFFObject
{
public:
  NFFSphere( ifstream &inpf );

  virtual G3DShadable* buildG3DObject ();

protected:
  float x, y, z, r;

};


#endif
