#ifndef __IO_RIB_H__
#define __IO_RIB_H__

/*
    Project : "MindsEye I/O Module"
    Copyright 1997. All Rights Reserved.

    FILE:         io_rib.h
    AUTHOR:       Nicholas Yue (nicholas_yue@acm.org)

    $Id: io_rib.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $

    OVERVIEW
    ========
    Class definition for the Renderman Interface Bitstream Format i/o module
*/

#include <fstream.h>
#include <values.h>
#include "io/io_base.h"
#include "3dlib/light.h"
#include "3dlib/g_operator.h"

class G3DScene;

/**IO Module for the RenderMan scene description language.
 */
class IO_RIB : public MEFilter
{
public:
  IO_RIB();
  ~IO_RIB();
  // virtual function
  virtual int  check_header( char *fname );
      
  // pure virtual function
  virtual int getFormatInfo(int type);
protected:
  // virtual function
  int  exitLoad(int result);

  // pure virtual function
  virtual int exitSave(int result);
  virtual int initLoad(const char *fname);
  virtual int initSave(const char *fname, int flag);
  virtual int loadObjects(G3DGroup *);
  virtual int saveObjects(G3DScene *);
protected:
  ifstream  _inpf;
  ofstream  _outpf;
private:
  Param _param[12];
  char filename[256];
  void writeCamera(G3DCamera* cameraRef);
};

class IO_RIB_Lights : public G3D_Operator
{
public:
  IO_RIB_Lights(ofstream& out ) : _outf(out) {};
  virtual bool exec (G3DObject* );
  void resetLightCounter() { _lightCounter = 0; };
  void incrementLightCounter() { _lightCounter++; };
private:
  Param _params[12];
  ofstream& _outf;
  int _lightCounter;
};

class IO_RIB_Geometry : public G3D_Operator
{
public:
  IO_RIB_Geometry(ofstream& out ) : _outf(out) {};
  virtual bool exec (G3DObject* );
private:
  Param _params[12];
  ofstream& _outf;
  //============================
  void writeBlob( G3DMatrix& m );
  void writeBox( G3DMatrix& m );
  void writeCone( G3DMatrix& m);
  void writeCylinder( G3DMatrix& m);
  void writeDifference( G3DMatrix& m);
  void writeDisk( G3DMatrix& m);
  void writeIntersection( G3DMatrix& m);
  void writeNurbsCurve( G3DMatrix& m );
  void writeNurbsSurface( G3DMatrix& m );
  void writePoint( G3DMatrix& m );
  void writePolygon( G3DMatrix& m );
  void writeSphere( G3DMatrix& m );
  void writeTorus( G3DMatrix& m );
  void writeUnion( G3DMatrix& m );
  //============================
  void minmax(const float* input,unsigned int size, float& min,float& max);
  void writeTransformation( G3DMatrix& m );
  void writeShader();
};


/**
 */
class MEFltFactoryRib : public MEFltFactory
{
public:
  MEFltFactoryRib ();
  virtual MEFilter* createFilter ();
  virtual void      writeObject( int id, G3DObject*, ostream& );
};


#endif
