/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: a-list.cpp,v 1.1.1.1 1999/09/09 23:50:16 iamcliff Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <GL/gl.h>
#include <GL/glu.h>

#include "a-list.h"
#include "attribute.h"
#include "3dlib/sceneElement.h"


G3DAttributeList::G3DAttributeList( const G3DAttributeList &orig )
{
  G3DAttribute *src, *src_grp = orig.root_;
  G3DAttribute *tmp, *attr, *grp = NULL;

  root_ = NULL;
  while( src_grp )
    {
      src = src_grp;
      while( src )
	{
	  tmp = src->copy();
	  if( root_ == NULL )
	    root_ = grp = tmp;
	  else if( attr == NULL )
	    {
	      grp->next_ = tmp;
	      grp = tmp;
	    }
	  else
	    attr->next_ = tmp;
	  attr = tmp;
	  src = src->suc_;
	}
      attr    = NULL;
      src_grp = src_grp->next_;
    }
}


G3DAttributeList::~G3DAttributeList()
{
  //TODO release all attributes
}


void
G3DAttributeList::addGroup( G3DAttribute *attrib )
{
  if( root_ == NULL )
    root_ = attrib;
  else
    {
      attrib->next_ = root_->next_;
      root_->next_  = attrib;
    }
}

G3DAttribute*
G3DAttributeList::findGroup( t_type type )
{
  G3DAttribute *tmp = root_;
  while( tmp )
    {
      if( IS_TYPE(tmp->getType(), type) )
	break;
      else
	tmp = tmp->next_;
    }
  return tmp;
}

G3DAListIterator
G3DAttributeList::get( const G3DSceneElement *parent, t_type type,
		       bool inherit ) const
{
  G3DAListIterator it( root_, parent, type, inherit );
  ++it;
  return( it );
}

G3DAttribute*
G3DAttributeList::insert( G3DAttribute *attrib )
{
  G3DAttribute *tmp = findGroup( GET_TYPE(attrib->type_) );

  if( tmp == NULL )
    addGroup( attrib );
  else
    {
      t_type atype = attrib->type_;
      while( tmp )
	{
	  if( tmp->type_ == atype || IS_SUBTYPE(atype, TA_TEXTURE) )
	    { //TODO test if poly
	      if( tmp->isReference() )
		{
		  replace( tmp, attrib );
		  tmp->dispose();
		  break;
		}
	      else
		{
		  tmp->setValues( attrib );
		  delete attrib;  //TODO or attrib->Delete() ??
		  return( tmp );
		}
	    }
	  else if( tmp->suc_ )
	    tmp = tmp->suc_;
	  else
	    {
	      tmp->suc_ = attrib;
	      break;
	    }
	}
    }
  return( attrib );
}

void
G3DAttributeList::remove( t_type )
{
  //TODO
}

void
G3DAttributeList::remove( G3DAttribute *attrib )
{
  G3DAttribute *tmp1 = root_, *tmp2 = root_;
  t_type atype = GET_TYPE(attrib->type_);

  while( tmp2 )
    {
      if( IS_TYPE(tmp2->type_, atype) )
	break;
      tmp1 = tmp2;
      tmp2 = tmp2->next_;
    }

  if( tmp2 == attrib )
    {
      if( tmp2 == root_ )
	{
	  if( tmp2->suc_ )
	    {
	      root_        = tmp2->suc_;
	      root_->next_ = tmp2->next_;
	    }
          else
	    root_ = tmp2->next_;
	}
      else
	{
	  if( tmp2->suc_ )
	    {
	      tmp2->suc_->next_ = tmp1->next_;
	      tmp1->next_       = tmp2->suc_;
	    }
	  else
	    tmp1->next_ = tmp2->next_;
	}	    
    }
  else
    {
      //OK, search in the subgroup
      tmp1 = tmp2->suc_;
      while( tmp1 )
	{
	  if( tmp1 == attrib )
	    {
	      tmp2->suc_ = tmp1->suc_;
	      break;
	    }
	  tmp2 = tmp1;
	  tmp1 = tmp1->suc_;
	}
    }
}

void
G3DAttributeList::replace( G3DAttribute *rem_atr, G3DAttribute *rep_atr )
{
  G3DAttribute *tmp1 = root_, *tmp2 = root_;
  t_type atype = GET_TYPE(rem_atr->type_);

  rep_atr->next_ = rem_atr->next_;
  rep_atr->suc_  = rem_atr->suc_;

  while( tmp2 )
    {
      if( IS_TYPE(tmp2->type_, atype) )
	break;
      tmp1 = tmp2;
      tmp2 = tmp2->next_;
    }

  if( tmp2 == rem_atr )
    {
      if( rem_atr == root_ )
	root_ = rep_atr;
      else
	tmp1->next_ = rep_atr;
    }
  else
    {
      //OK, search in the subgroup
      tmp1 = tmp2->suc_;
      while( tmp1 )
	{
	  if( tmp1 == rem_atr )
	    {
	      tmp2->suc_ = rep_atr;;
	      break;
	    }
	  tmp2 = tmp1;
	  tmp1 = tmp1->suc_;
	}
    }
}


//------------------------------------------------------------


G3DAListIterator&
G3DAListIterator::operator++( )
{
  while( group_ )
    {
      if( marker_ == NULL )
	{
	  if( IS_ATYPE(group_->getType(), type_) )
	    marker_ = group_;
	}
      else
	marker_ = marker_->suc_;
      if( marker_ )
	{
	  t_type type = GET_TNUM(type_);
	  if( type != 0 )
	    {
	      while( marker_ )
		{
		  if( marker_->getType() == type_ )
		    break;
	          else
		    marker_ = marker_->suc_;
		}
	    }
	}
      if( marker_ == NULL )
	group_ = group_->next_;
      else
	return *this;
    }
	    
  if( inherit_ && parent_ )
    return (*this = parent_->getAttribute( type_ ) );
}


//==================================END================================
