/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: anim_spin.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#include "include/config.h"

#include "anim_spin.h"
#include "attribute.h"
#include "t_actions.h"
#include "object.h"


IMPLEMENT_ADYN(G3DAnimationSpin, "anim_spin")


G3DAnimationSpin::G3DAnimationSpin( const G3DAnimationSpin &orig ) :
  G3DAnimation(orig), axis_(orig.axis_), center_(orig.center_)
{
  rotationSpeed_ = orig.rotationSpeed_;
  type_ = TA_ANIM_SPIN;
}


void
G3DAnimationSpin::evaluateAt( G3DObject *obj, float t ) const
{
  G3DMatrix trf, rot;
  float     angle = t * rotationSpeed_ * M_PI * 2;

  obj->setScale( obj->getScale() ); //to force a clean trafo matrix
  obj->getLocalTrafo( trf );
  rot.rotate( axis_, angle );
  rot *= trf;
  obj->setLocalTrafo( rot );

  //G3DQuaternion rot( axis_, angle );
  //rot *= obj->getRotation();
  //obj->setRotation( rot );
/*
  G3DMatrix mat;
  obj->getLocalTrafo( mat );  
  G3DVector3d tmp;
  tmp = mat * center_;
  //cout << "tmp: " << tmp << endl;
  G3DPoint3d orig( obj->getOrigin() );
  obj->setOrigin( (lcenter_ - tmp) + ori );
*/
}

Field G3DAnimationSpin::FieldList_[3] =
{ {"axis", s_vector3}, {"center", s_point3}, {"rotspeed", s_flt} };

const Field*
G3DAnimationSpin::getFieldList () const
{
  return( FieldList_ );
}

uint
G3DAnimationSpin::getParameter( Param *pList )
{
  if( pList )
    {
      pList->vector3 = &axis_;
      pList++;
      pList->point3  = &center_;
      pList++;
      pList->flt     = rotationSpeed_;
    }
  return( 3 );
}

void
G3DAnimationSpin::saveValueAt( const G3DObject*, float )
{
  //Nothing to do here
}

void
G3DAnimationSpin::setParameter( const Param *pList )  //TODO pass obj?
{
  if( pList )
    rotationSpeed_ = pList[2].flt;
}

void
G3DAnimationSpin::setValues( const G3DAttribute* )
{
  //TODO
}


//==================================END===================================
