/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: animation.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#include "include/config.h"

#include "animation.h"
#include "attribute.h"
#include "t_actions.h"
#include "object.h"

G3DAnimation::G3DAnimation( const G3DAnimation &orig )
{
  start_ = orig.start_;
  stop_  = orig.stop_;
}


//--------------------------------------------------------------------


G3DAnimationVis::G3DAnimationVis( const G3DAnimationVis &orig ) :
  G3DAnimation(orig)
{
  struct tvis *last, *pt = orig.root_;
  root_ = NULL;

  while( pt != NULL )
    {
      struct tvis *dup  = new tvis;
      dup->visible = pt->visible;
      dup->timep   = pt->timep;
      if( root_ == NULL )
	root_ = last = dup;
      else
	{
	  last->next = dup;
	  last = dup;
	}
      pt = pt->next;
    }
  last->next = NULL;
}

G3DAnimationVis::~G3DAnimationVis( )
{
  struct tvis *pt = root_;
  while( pt != NULL )
    {
      pt = pt->next;
      delete root_;
      root_ = pt;
    }
}

G3DAttribute*
G3DAnimationVis::copy () const
{
  return( new G3DAnimationVis(*this) );
}

void
G3DAnimationVis::evaluateAt( G3DObject *obj, float t ) const
{
  struct tvis *tmp = root_;
  short state = 1;

  clamp( t );
  while( tmp )
    {
      if( tmp->timep <= t )
	{
	  state = tmp->visible;
	  cout << state << " ";
	  tmp   = tmp->next;
	}
      else
	break;
    }
  if( state )
    {
      ActionData action(A_SHOW);
      obj->doAction( action );
    }
  else
    {
      ActionData action(A_HIDE);
      obj->doAction( action );
    }
}

//TODO
Field G3DAnimationVis::FieldList_[1] = { {"url", s_string} };

const Field*
G3DAnimationVis::getFieldList () const
{
  return( FieldList_ );
}

uint
G3DAnimationVis::getParameter( Param *pList )
{
  if( pList )
    {
      //pList->string  = url_;
    }
  return( 1 );
}

const char*
G3DAnimationVis::getTypeName () const
{
  return( GetTypeName() );
}

const char*
G3DAnimationVis::GetTypeName ()
{
  return( "Anim_Visibility" );
}

void
G3DAnimationVis::saveValueAt( const G3DObject *obj, float t )
{
  struct tvis  *tmp = root_, *tmp2 = NULL;
  short state = (obj->getState() & visible ? 1 : 0 );

  expand( t );
  while( tmp )
    {
      if( tmp->timep < t )
	{
	  tmp2 = tmp;
	  tmp  = tmp->next;
	}
      else if( tmp->timep == t )
	{
	  tmp->visible = state;
	  return;
	}
      else
	break;
    }
  struct tvis *tmp3  = new tvis;
  tmp3->visible = state;
  tmp3->timep   = t;
  tmp3->next    = tmp;
  if( tmp2 )
    tmp2->next = tmp3;
  else
    root_ = tmp3;
}

void
G3DAnimationVis::setValues( const G3DAttribute* )
{
  //TODO
}


//==================================END===================================
