/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: plastic.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * basic attribute classes
 *
 */

#ifndef _PLASTIC_H_
#define _PLASTIC_H_

//#include <string.h>
#include "texture.h"


/**A Description of the visible properties of a surface made of plastic.
  */
class G3DAPlastic : public G3DATexture
{
  DECLARE_ADYN

public:
  G3DAPlastic ();
  G3DAPlastic( const G3DAPlastic &orig );
  ~G3DAPlastic();

    ///Set the corresponding OpenGL state variables.
  virtual void activate () const;
    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
    /**Get a list of params describing the geometry of the object.
       If plist is Null you get just the number of elements.*/
  virtual uint  getParameter( Param* );
  virtual void  setParameter( const Param* );
    ///
  void getAmbientColor( G3DColor &afc ) const { afc = acolor_; };
    ///
  void getDiffuseColor( G3DColor &dfc ) const { dfc = dcolor_; }
    ///
  float getMirrorEffect () { return mirrorEffect_; }
    ///
  float getRoughness () const { return roughness_; }
    ///
  float getSpecRef () const { return specRef_; }
    ///
  void setAmbientColor( const G3DColor &afc ) { acolor_ = afc; }
    ///
  void setDiffuseColor( const G3DColor &dfc ) { dcolor_ = dfc; }
    ///
  void setMirrorEffect( float me ) { mirrorEffect_ = me; }
    ///
  void setRoughness( float rg ) { roughness_ = rg; }
    ///
  void setSpecRef( float sp ) { specRef_ = sp; }

    ///Set the user visible values of the attribute to the values of atr.
  virtual void  setValues( const G3DAttribute *atr );

protected:
  G3DColor acolor_;  //ambient color
  G3DColor dcolor_;  //diffuse color
  float    specRef_;
  float    roughness_;
  float    mirrorEffect_;

  static Field FieldList_[5];

};


#endif
