/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: resol.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "attribute.h"


/**This attribute determens the number and/or size of the polygons
   in a tesselation of a 3d object.
   Quality values have to be in the range [0,1] with 1 being the
   highest quality (i.e. < 1 pixel deviation in the output).
   */
class G3DAResolution : public G3DMonoAttribute
{
  DECLARE_ADYN

public:
  G3DAResolution ()
    { quality_ = 0.5; type_ = TA_RESOLUTION; }
  G3DAResolution( const G3DAResolution &orig ) : G3DMonoAttribute(orig)
    { quality_ = orig.quality_; } 

    ///Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
    /**Get a list of params describing the geometry of the object.
       If plist is Null you get just the number of elements.*/
  virtual uint  getParameter( Param* );
    ///
  float getQuality () const { return quality_; };
    ///
  void  setQuality( float q ) { quality_ = q; };
    ///Set the user visible values of the attribute to the values of atr.
  virtual void  setValues( const G3DAttribute *atr );

protected:
  float    quality_;
     ///1: make an equidistant mesh, even if not necessary
  //int      style_;
  static Field FieldList_[1];
};


#endif


