/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: actions.h,v 1.3 2001/04/01 09:42:28 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef ACTIONS_included
#define ACTIONS_included

#include "t_actions.h"


/// 
class AnimateAction : public GenericAction
{
public:
  /// Constructor.
  AnimateAction ( float atime ) : GenericAction(A_GENERIC_ALL), time_(atime) {}

  ///
  virtual EvalState evaluate ( G3DObject* );

  /// Set the action's time value (?)
  void setTime ( float atime ) { time_ = atime; }

protected:
  float time_;
};

///
class DeleteListAction : public GenericAction
{
public:
  /// Default constructor.
  DeleteListAction () : GenericAction(A_GENERIC_SHAPES) {}

  ///
  virtual EvalState evaluate ( G3DObject * );
};

/// Find the object with the specified name in the scene.
class FindByName : public GenericAction
{
public:
  /// Constructor taken the name of the object to be found.
  FindByName ( const char *name ) : GenericAction(A_GENERIC_ALL)
    { name_ = name;  result_ = 0; }
  
  /// Evaluate the action for a given object.
  virtual EvalState evaluate ( G3DObject* );
  
  /// Return the object found.
  G3DObject* getObject () { return result_; }

 protected:
  const char *name_;
  G3DObject  *result_;
};


///
class SetTimeAction : public GenericAction
{
public:
  /// Constructor.
  SetTimeAction ( float atime ) :
    GenericAction(A_GENERIC_ALL), time_(atime) {}

  /// Evaluate the action for a given object.
  virtual EvalState evaluate ( G3DObject* );

  /// Set the action's time value.
  void setTime ( float atime ) { time_ = atime; }

protected:
  float time_;
};

#endif
