/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: g_array.cpp,v 1.2 2001/07/17 02:34:38 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Description
 * ===========
 * classes for building linked lists
 */


#include "include/config.h"
#include <stdlib.h>
#include <assert.h>

#include "g_array.h"
#include <algo.h>
#include "g_operator.h"
#include "object.h"

void
G3D_Array::doWithConstElements ( G3D_Operator &op ) const
{
  vector<void*>::const_iterator pos = begin();
  while( pos != end() && !op.cexec( (const G3DObject*)(*pos)) )
    ++pos;
}

void
G3D_Array::doWithMElements( G3D_Operator &op )
{
  vector<void*>::iterator pos = begin();
  while( pos != end() && !op.exec( (G3DObject*)(*pos)) )
    ++pos;
}

int
G3D_Array::find ( void *elem ) const
{
  vector<void*>::const_iterator pos;
  pos = ::find (begin(), end(), elem);
  
  return (pos != end()) ? (pos - begin()) : -1;
}

bool
G3D_Array::remove ( G3DObject* obj )
{ 
  vector<void*>::iterator pos = ::find (begin(), end(), (void*)obj);
  if (pos != end()) {
    /*if (clear_) */
    obj->dispose ();
    erase (pos);
    return true;
  }
  return false;
}

void
G3D_Array::push_back ( G3DObject* obj )
{
  obj->addReference ();
  vector<void*>::push_back (obj);
}

void
G3D_Array::reInitialize ()
{
  //TODO this is not thread-safe!!
  /*if (clear_) { */ 
  vector<void*>::iterator pos = begin();
  while (pos != end()) {
    ((G3DObject*)(*pos))->dispose();
    ++pos;
  }
  /* } */
  erase (begin(), end());
}

G3D_Array&
G3D_Array::operator= ( const G3D_Array &src )
{
  cout << "copying object array!" << endl;

  reInitialize ();

  reserve (src.size());
  vector<void*>::const_iterator pos = src.begin();
  while (pos != src.end()) {
    G3DObject * tmp = ((G3DObject*)(*pos))->copy();
    push_back (tmp); // increases the object's ref count to 2
    tmp->dispose (); // decrease the object's ref count to 1
    ++pos;
  }
  return *this;
}

G3DObject*
G3D_Array::operator[]( int i )
{
  return ((G3DObject*)(*(begin() + i)));
}

/*
#ifdef EXPLICIT_TEMPLATES
#ifdef __GNUG__
template void** find<void**, void*>(void**, void**, const void*);
#endif
#endif  //EXPLICIT_TEMPLATES
*/
