/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: g_array.h,v 1.3 2001/07/17 02:34:38 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Description
 * ===========
 * classes for building linked lists
 */

#ifndef __G3D_ARRAY_H_
#define __G3D_ARRAY_H_

#include <vector>

class G3DObject;
class G3D_Operator;

/// An array of pointers.
/** An array of pointers. 
    In the constructor an automatic deletion of the referenced elements
    may be requested when they are removed from the list. Default: is off.
    This feature is the main reason for overloading the base STL class.
*/
class G3D_Array : public vector<void*>
{
public:
  /// Constructor.
  G3D_Array ( bool clear = false )  { clear_ = clear; }

  /// Destructor. Called when the object is destroyed.
  ~G3D_Array () { reInitialize(); }

  /// Apply a given operator for const elements (?)
  void doWithConstElements ( G3D_Operator& ) const;
  
  /// Apply a given operator to the elements.
  void doWithMElements ( G3D_Operator& );

  /// Return index of element in the list, or -1 if not found.
  int  find ( void *elem ) const;
  
  /// Remove all elements from the list.
  void reInitialize ();
  
  /// Remove the element at the given position (?)
  bool remove ( G3DObject* );
  
  void push_back ( G3DObject* );

  /// Set the array's delete mode.
  /** If mode = true delete elements when they are removed from the list.
      Default is false.
  */
  void setDeleteMode ( bool mode ) { clear_ = mode; }

  /// Overloaded assignment operator.
  G3D_Array& operator=( const G3D_Array& );

  /// Overloaded index operator [].
  G3DObject* operator[] ( int );

protected:
  bool clear_;
};

#endif
