/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: h_skeleton.h,v 1.3 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of mesh primitives
 */

#ifndef _HELPER_SKELETON_H_
#define _HELPER_SKELETON_H_

#include "sceneElement.h"


enum G3DSideEnum { aside=0, bside=1 };

class G3DBone;

extern G3DObject* createSkeleton ();


/// A Joint connects 2 elements of a sceleton.
class G3DJoint
{
  friend class G3DSkeleton;

public:
  /// Default conjstructor.
  G3DJoint ();

  /// Copy constructor.
  G3DJoint ( const G3DJoint& );

  /// 
  void addBone ( G3DSideEnum );
  
  ///
  void addBone ( G3DSideEnum, G3DBone* );

  ///
  virtual void adapt ( G3DSideEnum dir, G3DVector3d &force ) = 0;

  ///
  virtual G3DJoint* copy () const = 0;

  ///
  virtual void update ( G3DQuaternion &rot ) = 0;
 
  //virtual void setType( int ) = 0;

  G3DBone    *bones_[2];
  G3DPoint3d position_; //???
  MeFloat    stiffness_;  //0..1
};


///This joint allows roations about an axis.
class G3DJoint1D : public G3DJoint
{
public:
  /// Default constructor.
  G3DJoint1D ();

  /// Copy constructor.
  G3DJoint1D ( const G3DJoint1D& );

  ///
  virtual void adapt ( G3DSideEnum dir, G3DVector3d &force );
  
  ///
  virtual G3DJoint* copy () const;

  ///
  virtual void update ( G3DQuaternion &rot );

protected:
  G3DVector3d axis_;
  float aDefault_, aMin_, aMax_, aCurrent_;
};


/// This joint allows rotations in all directions.
class G3DJoint2D : public G3DJoint
{
public:
  /// Default constructor.
  G3DJoint2D ();

  /// Copy constructor.
  G3DJoint2D ( const G3DJoint2D& );

  ///
  virtual void adapt ( G3DSideEnum dir, G3DVector3d &force );

  ///
  virtual G3DJoint* copy () const;

  ///
  virtual void update ( G3DQuaternion &rot );

protected:
  //axis?
  //min1, min2, max1, max2;
};


/// Base class for bones (?)
class G3DBone
{
  friend class G3DJoint;
  friend class G3DJoint1D;  //TODO this prevents plugin joint types!!
  friend class G3DJoint2D;
  friend class G3DSkeleton;

public:
  /// Default constructor.
  G3DBone ();

  /// Copy constructor.
  G3DBone ( const G3DBone& );

  ///
  void addJoint ( G3DSideEnum, G3DJoint* );

protected:
  G3DVector3d bone_;
  MeFloat     length_;  //of the bone
  G3DJoint    *joints_[2];
};


///
//default: 1 joint + 1 bone
/**A skeleton should not conatin cycles.
 */
class G3DSkeleton : public G3DSceneElement
{
public:
  /// Default constructor.
  G3DSkeleton ();

  //G3DSkeleton( uint nvert , uint nfaces, uint nnormals );

  /// Copy constructor.
  G3DSkeleton ( const G3DSkeleton& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DSkeleton ();

  ///
  void adjust ();

  ///
  virtual G3DObject* copy () const;

  //TODO virtual void       display () const;
  ///
  virtual void display ( int& ) const;
  
  /// Do a given action with this object.
  virtual void doAction ( ActionData& );
  
  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;

  
  /**Copy pointers to the parameter to the list. The order is:
     _rows, _cols, _vertices, _normals.*/
  virtual uint getParameter ( Param* );

  /// Return a Descriptive name of the shape.
  virtual const char* getTypeName () const;
  static  const char* GetTypeName ();

  ///
  void moveTo ( const TVector<MeFloat,3> & );
  
  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();
  
  ///
  virtual MeFloat selectDetails ( const G3DCamera&, const G3DRay&,
				  G3DObject **obj );

  ///
  virtual void setParameter ( const Param *pList );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  void shade () const;

  ///
  virtual void shadeDetails () const;

protected:
  //searchFixPoint
  //updateAngles

protected:
  G3DPoint3d  basis_;  //or a Joint??
  G3DJoint    *root_;  //This is just an organizational thing (aside always 0)
  //G3D_Array *list_;
  //G3DObject   *list_; //list of assigned objects that need update

  //getUniqueName();

  static Field SkeletonFields_[7];
};

#endif
