/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: ha_face.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include "GL/gl.h"
#include "GL/glu.h"
#include "ha_face.h"


G3DFace::G3DFace( G3DPoint3d *vt, uint *faces, uint nf, G3DObject *parent ) :
  G3DObject(parent)
{
  type_     = THA_FACE;

  vertices_ = vt;
  faces_    = new uint[nf];
  memcpy( faces_, faces, nf*sizeof(uint) );
  nfaces_   = nf;
  vertOrig_ = new G3DPoint3d[nf];
  for( int i = nf-1; i >= 0; --i )  //TODO memcpy??
    vertOrig_[i] = vertices_[(*faces) + i];
}

G3DFace::~G3DFace()
{
  delete[] faces_;
  delete[] vertOrig_;
  //TODO if( constraint_ )
// constraint_->dispose();
}

void
G3DFace::doAction( ActionData &action )
{
  if( action.type == A_DESELECT )
    dispose();
  else
    G3DObject::doAction( action );
}

const G3DPoint3d&
G3DFace::getOrigin () const
{
  return( vertOrig_[0] );
}

void
G3DFace::setConstraint( const G3DConstraint* )
{
//c->addReference();
}

void
G3DFace::setOrigin( const TVector<MeFloat,3> &tv )
{
  G3DVector3d delta( vertOrig_[0] - tv );
  for( int i = nfaces_-1; i >= 0; --i )
    vertices_[*faces_ + i] = vertOrig_[i] - delta;

  //TODO apply the constraint
}


//====================================END===============================
