/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997, 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: matrix.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <assert.h>
#include <math.h>

#include "3dlib/g3d_math.h"
#include "matrix.h"

#ifndef M_2PI
#define M_2PI  2*M_PI
#endif

static void swapMat( MeFloat mat[3][3], int c1, int c2 );


//The prototype for the identity matrix
static MeFloat IdentM[] =
  { 1.0, 0.0, 0.0, 0.0,
    0.0, 1.0, 0.0, 0.0,
    0.0, 0.0, 1.0, 0.0,
    0.0, 0.0, 0.0, 1.0 };

inline
void identity( MeFloat *dst )
{
  MeFloat *src = IdentM;

  for( uint i = 16; i > 0; i-- )
    *(dst++) = *(src++);
}


//-----------------------------------------------------------------


G3DMatrix::G3DMatrix ()
{
  identity( _matrix );
}


G3DMatrix::G3DMatrix( const G3DVector3d &dir, const G3DVector3d &up )
{
  G3DVector3d ex, ey, ez;
  MeFloat      *mat = &(_matrix[0]);

  ez = dir * (-1);
  ez.normalize();
  ex = up.cross( ez );
  ex.normalize();
  ey = ez.cross( ex );
  ey.normalize();

  for( int i = 0; i < 3; i++ )
    {
      *(mat++) = ex[i];
      *(mat++) = ey[i];
      *(mat++) = ez[i];
      *(mat++) = 0;
    }
  *(mat++) = 0;
  *(mat++) = 0;
  *(mat++) = 0;
  *mat     = 1;
}


G3DMatrix::G3DMatrix( const TVector<MeFloat,3> &tv )
{
  identity( _matrix );
  
  const MeFloat *src = (const MeFloat*)tv;
  MeFloat *dst       = &(_matrix[12]);

  *(dst++) = *(src++);
  *(dst++) = *(src++);
  *(dst++) = *src;
}

G3DMatrix::G3DMatrix( const G3DQuaternion &quat )
{
  quat.getRotationMatrix( _matrix );
}

G3DMatrix::G3DMatrix( const G3DMatrix &orig )
{
  const MeFloat *src = orig._matrix;
        MeFloat *dst = _matrix;

  for( int i = 16; i > 0; i-- )
    *(dst++) = *(src++);
}

/*
void
G3DMatrix::getAngles( G3DVector3d &v )
{
  if( fabs(_matrix[10]) < 1.0 )
    {
      if( fabs(_matrix[2]) < FG_EPSILON )  // phi3 = 0
	{
	  v[2] = 0.0;
	  v[1] = acos( _matrix[10] );
	  if( _matrix[6] < 0.0 )
	    v[1] = M_2PI - v[1];
	  if( fabs(_matrix[0]) < 1.0 )
	    {
	      if( _matrix[1] >= 0.0 )
		v[0] = acos( _matrix[0] );
	      else
		v[0] = M_2PI - acos( _matrix[0] );
	    }
	  else
	    v[0] = 0.0;
	}
      else if( fabs(_matrix[8]) < FG_EPSILON )  // phi1 = 0
	{
	  v[0] = 0.0;
	  if( _matrix[9] <= 0.0 )
	    v[1] = acos( _matrix[10] );
	  else
	    v[1] = M_2PI - acos( _matrix[10] );
	  if( fabs(_matrix[0]) < 1.0 )
	    {
	      if( _matrix[4] <= 0.0 )
		v[2] = acos( _matrix[0] );
	      else
		v[2] = M_2PI - acos( _matrix[0] );
	    }
	  else
	    v[0] = 0.0;
	}
      else  //all angels != 0
	{
	  MeFloat s2, c1, c2, c3;
	  s2 = sqrt( 1 - _matrix[10]*_matrix[10] );
	  c1 = -_matrix[9] / s2;
	  if( c1 >= 1.0 ) c1 = 1.0;
	  c2 = _matrix[10];
	  c3 = _matrix[6] / s2;
	  if( c3 >= 1.0 ) c3 = 1.0;
	  MeFloat r1 = _matrix[2], r2=_matrix[8];
	  if( r1 > 0.0 )
	    {
	      if( r2 > 0.0 )
		{
		  v[0] = acos( c1 );
		  v[1] = acos( c2 );
		  v[2] = acos( c3 );
		}
	      else
		{
		  v[0] = acos( -c1 );
		  v[1] = M_2PI - acos( c2 );
		  v[2] = M_2PI - acos( -c3 );
		}
	    }
	  else
	    {
	      if( r2 > 0.0 )
		{
		  v[0] = acos( c1 );
		  v[1] = acos( c2 );
		  v[2] = M_2PI - acos( c3 );
		}
	      else
		{
		  v[0] = acos( -c1 );
		  v[1] = M_2PI - acos( c2 );
		  v[2] = acos( -c3 );
		}
	    }
	}
    }
  else   //its a simple rotation around z axis
    {
      if( fabs(_matrix[0]) < 1.0 )
	{
	  v[0] = acos( _matrix[0] );
	  if( _matrix[1] < 0.0 )
	    v[0] = M_2PI - v[0];
	}
      else
	v[0] = 0.0;
      v[1] = 0.0;
      v[2] = 0.0;
    }
}
*/

void
G3DMatrix::getTranslation( G3DVector3d &vec )
{
  vec[0] = _matrix[12];
  vec[1] = _matrix[13];
  vec[2] = _matrix[14];
}

void
swapMat( MeFloat mat[3][3], int c1, int c2 )
{
  MeFloat t;

  for( int i = 0; i < 3; i++ )
    {
      t = mat[c1][i];
      mat[c1][i] = mat[c2][i];
      mat[c2][i] = t;
    }
}

// Gauss-Jordan elimination with partial pivoting
int
G3DMatrix::invert3 ()
{
  MeFloat tmp1[3][3], tmp2[3][3];
  MeFloat *src = _matrix;
  MeFloat pivot;
  int	 col, i1, j;

  for( int n = 0; n < 3; n++ )
    {
      for( int m = 0; m < 3; m++ )
	{
	  tmp1[m][n] = *(src++);
	  if( m == n )
	    tmp2[m][n] = 1.0;
	  else
	    tmp2[m][n] = 0.0;
	}
      src++;
    }
  
  // Loop over cols of a eliminating above and below diag
  for( col = 0; col < 3; col++ )
    {   //Find column with largest pivot in row col among columns col..2
      i1 = col;		    // column with largest pivot candidate
      for( j = col+1; j < 3; j++)
	if( fabs(tmp1[j][col]) > fabs(tmp1[i1][col]) )
	  i1 = j;

      // Swap rows i1 and col in a and b to put pivot on diagonal
      if( i1 != col )
	{
	  swapMat( tmp1, i1, col );
	  swapMat( tmp2, i1, col );
	}

      // Scale row j to have a unit diagonal
      pivot = tmp1[col][col];
      if( pivot == 0. )
	return( 0 );  //singular matrix
      for( int l = 0; l < 3; l++ )
	{
	  tmp1[col][l] /= pivot;
	  tmp2[col][l] /= pivot;
	}

      // Eliminate off-diagonal elems in col j of a, doing identical ops to b
      MeFloat q;
      for( int i = 0; i < 3; i++ )
	{
	  if( i != col )
	    {
	      q = tmp1[i][col];
	      for( int k = 0; k < 3; k++ )
		{
		  tmp1[i][k] -= q * tmp1[col][k];
		  tmp2[i][k] -= q * tmp2[col][k];
		}
	    }
	}
    }

  src = _matrix;
  for( int s = 0; s < 3; ++s )
    {
      for( int t = 0; t < 3; ++t )
	*(src++) = tmp2[t][s];
      src++;
    }
  return( 1 );
}

/*This inversion is done by taking into account the structure of the matrix:.
  It invertes the rotation part and the translation part separately and
  multiplies them.
  TODO check if this is really faster than inverting the 4x4 matrix as a whole.
  */
int
G3DMatrix::invert4 ()
{
  G3DVector3d vec;
  getTranslation( vec );
  int ret = invert3();
  if( ret )
    {
      vec *= -1;
      G3DMatrix trans( vec );
      G3DMatrix rot( *this );
      rot.translate( vec );
      *this = rot;
      *this *= trans;
    }
  return( ret );
}

void
G3DMatrix::makeTrafo( const TVector<MeFloat,3> &a1, const TVector<MeFloat,3> &a2,
		      const TVector<MeFloat,3> &a3, const TVector<MeFloat,3> &b1,
		      const TVector<MeFloat,3> &b2, const TVector<MeFloat,3> &b3)
{
  MeFloat *dst = _matrix;

  *(dst++) = a1.dot(b1);
  *(dst++) = a2.dot(b1);
  *(dst++) = a3.dot(b1);
  dst++;
  *(dst++) = a1.dot(b2);
  *(dst++) = a2.dot(b2);
  *(dst++) = a3.dot(b2);
  dst++;
  *(dst++) = a1.dot(b3);
  *(dst++) = a2.dot(b3);
  *(dst++) = a3.dot(b3);
}

void
G3DMatrix::rotate( const TVector<MeFloat,3> &axis, MeFloat angle )
{
  // u = axis/|axis| = (x,y,z)
  // S = [ 0 -z  y]
  //     [ z  0 -x]
  //     [-y  x  0]
  // M = uu^T + cos(a) * (I - uu^T) + sin(a) * S

  MeFloat ux, uy, uz, sintheta, costheta;

  G3DVector3d tAxis = axis;  
  assert( tAxis.length() != 0.0 );
  tAxis.normalize();
  
  sintheta = sin( angle );
  costheta = cos( angle );
  ux       = tAxis.x();
  uy       = tAxis.y();
  uz       = tAxis.z();

  MeFloat *dst = _matrix;

  *(dst++) = ux*ux + (1. - ux*ux)*costheta;
  *(dst++) = ux*uy*(1 - costheta) + uz*sintheta;
  *(dst++) = ux*uz*(1 - costheta) - uy*sintheta;
  *(dst++) = 0;
  
  *(dst++) = ux*uy*(1 - costheta) - uz*sintheta;
  *(dst++) = uy*uy + (1 - uy*uy)*costheta;
  *(dst++) = uy*uz*(1 - costheta) + ux*sintheta;
  *(dst++) = 0;

  *(dst++) = ux*uz*(1 - costheta) + uy*sintheta;
  *(dst++) = uy*uz*(1 - costheta) - ux*sintheta;
  *(dst++) = uz*uz + (1 - uz*uz)*costheta;  
  *(dst++) = 0;

  *(dst++) = 0;
  *(dst++) = 0;
  *(dst++) = 0;
  *dst     = 1;
}

void
G3DMatrix::rotate( const TVector<MeFloat,3> &tv )
{
  MeFloat c1, c2, c3, s1, s2, s3;
  c1 = cos( tv[0] );
  c2 = cos( tv[1] );
  c3 = cos( tv[2] );
  s1 = sin( tv[0] );
  s2 = sin( tv[1] );
  s3 = sin( tv[2] );

  MeFloat *dst = _matrix;
  *(dst++) = c1*c3 - s1*c2*s3;
  *(dst++) = s1*c3 + c1*c2*s3;
  *(dst++) = s2*s3;
  ++dst;
  *(dst++) = -c1*s3 - s1*c2*c3;
  *(dst++) = -s1*s3 + c1*c2*c3;
  *(dst++) = s2*c3;
  ++dst;
  *(dst++) = s1*s2;
  *(dst++) = -c1*s2;
  *dst     = c2;
}

void
G3DMatrix::rotateX( MeFloat angle )
{
  identity( _matrix );
  _matrix[5]  = cos( angle );
  _matrix[6]  = -sin( angle );
  _matrix[9]  = - _matrix[6];
  _matrix[10] = _matrix[5];
}


void
G3DMatrix::rotateY( MeFloat angle )
{
  identity( _matrix );
  _matrix[0]  = cos( angle );
  _matrix[2]  = -sin( angle );
  _matrix[8]  = - _matrix[2];
  _matrix[10] = _matrix[0];
}


void
G3DMatrix::rotateZ( MeFloat angle )
{
  identity( _matrix );
  _matrix[0] = cos( angle );
  _matrix[1] = -sin( angle );
  _matrix[4] = - _matrix[2];
  _matrix[5] = _matrix[0];
}

G3DMatrix&
G3DMatrix::scale( const TVector<MeFloat,3> &sc )
{
  _matrix[0]  *= sc[0];
  _matrix[5]  *= sc[1];
  _matrix[10] *= sc[2];
  return( *this );
}


void
G3DMatrix::setCol( int i, const TVector<MeFloat,3> &tv )
{
  MeFloat *dst = &(_matrix[i*4]);

  *(dst++) = tv[0];
  *(dst++) = tv[1];
  *dst     = tv[2];
}


void
G3DMatrix::setRow( int i, const TVector<MeFloat,3> &tv )
{
  MeFloat *dst = &(_matrix[i]);

  *dst = tv[0]; dst += 4;
  *dst = tv[1]; dst += 4;
  *dst = tv[2];
}


void
G3DMatrix::setTo( const double *src )
{
  MeFloat *dst = _matrix;

  for( int i = 16; i > 0; i-- )
#ifdef USE_DOUBLE
    *(dst++) = *(src++);
#else
    *(dst++) = MeFloat( *(src++) );
#endif
}

void
G3DMatrix::setTo( const float *src )
{
  MeFloat *dst = _matrix;

  for( int i = 16; i > 0; i-- )
#ifdef USE_DOUBLE
    *(dst++) = double( *(src++) );
#else
    *(dst++) = *(src++);
#endif
}

void
G3DMatrix::setToIdentity ()
{
  identity( _matrix );
}

void
G3DMatrix::translate( MeFloat x, MeFloat y, MeFloat z )
{
  MeFloat *dst = &(_matrix[12]);
  *dst++ += x;
  *dst++ += y;
  *dst   += z;
}


void
G3DMatrix::translate( const TVector<MeFloat,3> &v )
{
  const MeFloat *src = (const MeFloat*)v;
        MeFloat *dst = &(_matrix[12]);
  *(dst++) += *(src++);
  *(dst++) += *(src++);
  *dst     += *src;
}

void
G3DMatrix::transpose3 ()
{
  MeFloat tmp;
  tmp = _matrix[1];
  _matrix[1] = _matrix[4];
  _matrix[4] = tmp;
  tmp = _matrix[2];
  _matrix[2] = _matrix[8];
  _matrix[8] = tmp;
  tmp = _matrix[6];
  _matrix[6] = _matrix[9];
  _matrix[9] = tmp;
}

void
writePovFormat( ostream &os, const G3DMatrix &mat )
{
  const MeFloat *src = mat._matrix;

  os << " matrix <";
  for( int i = 0; i < 4; i++ )
    {
      os << *(src++) << ',';
      os << *(src++) << ',';
      os << *(src++);
      if( i != 3 )
	{
	  os << ',';
	  ++src;
	}
    }
   os << '>' << endl;
}

ostream&
operator<<( ostream& os, const G3DMatrix &mat )
{
  const MeFloat *src = mat._matrix;

  for( int i = 16; i > 0; --i )
    os << *(src++) << ' ';
  
  return( os );
}

istream&
operator>>( istream& is, G3DMatrix &mat )
{
  MeFloat *dst = mat._matrix;

  for( int i = 16; i > 0; --i )
    {
      is >> *(dst++);
      // end of file reached, abort the read
      if( is.eof() || is.fail() )
	break;
    }
  return( is );
}

G3DMatrix&
G3DMatrix::operator=( const G3DMatrix &mat )
{
  setTo( mat._matrix );
  return( *this );
}


G3DMatrix&
G3DMatrix::operator*=( const G3DMatrix &mat )
{
  const MeFloat *src;
        MeFloat *dst = _matrix;
	MeFloat *tmpp, tmp[16];

  //Make a local copy of the matrix.
  tmpp = tmp;
  for( int i = 16; i > 0; i-- )
    *(tmpp++) = *(dst++);

  dst = _matrix;
  for( int j = 0; j < 16; j += 4 )
    {
      for( int i = 0; i < 3; i++ )  //the 4th row is not computed!
	{
	  src = &(mat._matrix[j]);
	  *dst = tmp[i] * *(src++);
	  *dst += tmp[i+4] * *(src++);
	  *dst += tmp[i+8] * *(src++);
	  *dst += tmp[i+12] * *src;
	  dst++;
	  /*
	  *(dst++) = (tmp[i] * *(src++)) + (tmp[i+4] * *(src++))
	             + (tmp[i+8] * *(src++)) + (tmp[i+12] * *src);
		     */
	}
      dst++;  //4th row is always 0 0 0 1 as set by the constructor!
    }
  return( *this );
}

TVector<MeFloat,3>
G3DMatrix::operator*( const G3DPoint3d &v ) const
{
  const MeFloat *src = _matrix;
  const MeFloat *src2 = (const MeFloat*)v;
        MeFloat x, y, z, tmp;

  tmp = *(src2++);
  x   = *(src++) * tmp;
  y   = *(src++) * tmp;
  z   = *(src++) * tmp;
  tmp = *(src2++); src++;
  x  += *(src++) * tmp;
  y  += *(src++) * tmp;
  z  += *(src++) * tmp;
  tmp = *src2; src++;
  x  += *(src++) * tmp;
  y  += *(src++) * tmp;
  z  += *(src++) * tmp;
  src++;
  x  += *(src++);
  y  += *(src++);
  z  += *src;

  TVector<MeFloat,3> tv;
  tv[0] = x;
  tv[1] = y;
  tv[2] = z;
  return( tv );
}

TVector<MeFloat,3>
G3DMatrix::operator*( const TVector<MeFloat,3> &v ) const //G3DVector3d &v )
{
  const MeFloat *src = _matrix;
  const MeFloat *src2 = (const MeFloat*)v;
        MeFloat x, y, z, tmp;

  tmp = *(src2++);
  x   = *(src++) * tmp;
  y   = *(src++) * tmp;
  z   = *(src++) * tmp;
  tmp = *(src2++); src++;
  x  += *(src++) * tmp;
  y  += *(src++) * tmp;
  z  += *(src++) * tmp;
  tmp = *src2; src++;
  x  += *(src++) * tmp;
  y  += *(src++) * tmp;
  z  += *(src++) * tmp;

  TVector<MeFloat,3> tv;
  tv[0] = x;
  tv[1] = y;
  tv[2] = z;
  return( tv );
}

G3DMatrix
G3DMatrix::operator*( const G3DMatrix &mat ) const
{
  const MeFloat *src1, *src2;
        MeFloat *dst, tmp[16];

  dst = tmp;
  src1 = _matrix;
  src2 = mat._matrix;
  for( int j = 0; j < 16; j += 4 )
    {
      for( int i = 0; i < 3; i++ )  //the 4th row is not computed!
	{
	  src2 = &(mat._matrix[j]);
	  *dst = src1[i] * *(src2++);
	  *dst += src1[i+4] * *(src2++);
	  *dst += src1[i+8] * *(src2++);
	  *dst += src1[i+12] * *src2;
	  dst++;
	}
      *(dst++) = 0.0;
    }
  tmp[15] = 1.0;
  G3DMatrix ret;
  ret.setTo( tmp );
  return( ret );
}


//====================================END===============================
