/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: matrix.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __G3D_MATRIX_H_
#define __G3D_MATRIX_H_


#include <stdlib.h>
#include <iostream.h>
#include "g3dvector.h"

class G3DQuaternion;


/** The class implements a 4x4 matrix which combines rotation, scaling
    and translation operations on 3d vectors and points. The 4th component
    of vectors and points is always assumed to be 1.
    The memory layout of the matrix elements follows the OpenGL style:
     ( a0 a4  a8 a12 )
     ( a1 a5  a9 a13 )
     ( a2 a6 a10 a14 )
     ( a3 a7 a11 a15 )
     The data of the class can be passed directly to OpenGL. This is the
     reason that the data area has indeed 16 values.
     */
class G3DMatrix
{
  friend void      writePovFormat( ostream&, const G3DMatrix &mat );
  friend ostream&  operator<<( ostream&, const G3DMatrix &mat );
  friend istream&  operator>>( istream&, G3DMatrix &mat );

public:
    ///create an identity matrix
  G3DMatrix ();
    /**Create a rotation matrix that maps dir onto
       the negative z-axis and up to the y-axis. */
  G3DMatrix( const G3DVector3d &dir, const G3DVector3d &up );
    ///Create a translation matrix.
  G3DMatrix( const TVector<MeFloat,3>& );
    ///Create a rotation matrix.
  G3DMatrix( const G3DQuaternion& );
    ///
  G3DMatrix( const G3DMatrix &orig );

  //TODO MeFloat determinant ();
    ///Calculate the Euler angles. Works only if matrix is pure rotation.
  //void        getAngles( G3DVector3d& );
    ///Set v to the translation part of the matrix
  void        getTranslation( G3DVector3d &v );
    /**Transform only x and y coordinate. This is useful for some
       intersection calculations where z is of no interest.*/
  //void        getxyrot( const G3DVector3d&, MeFloat&, MeFloat& ) const;
    /**Affects only the upper left 3x3 submatrix.
      @return 0 if matrix is singular 1 otherwise. */
  int         invert3 ();
  int         invert4 ();
    /**Make a transformation from system a1,a2,a3 to b1,b2,b3. The second
      system must be orthogonal.*/
  void        makeTrafo(const TVector<MeFloat,3>&, const TVector<MeFloat,3>&,
		        const TVector<MeFloat,3>&, const TVector<MeFloat,3>&,
		        const TVector<MeFloat,3>&, const TVector<MeFloat,3>& );
    ///Build a rotation matrix around axis v.
  void        rotate( const TVector<MeFloat,3>&, MeFloat );  
    ///Make a rotation matrix from the 3 given Euler angles.
  void        rotate( const TVector<MeFloat,3>& );
    ///Make a rotation matrix about the x axis. this must be identity.
  void        rotateX( MeFloat );
    ///
  void        rotateY( MeFloat );
    ///
  void        rotateZ( MeFloat );
    ///This should be only applied to a unit matrix.
  G3DMatrix&  scale( const TVector<MeFloat,3>& );

    ///
  void        setTo( const double* );
    ///
  void        setTo( const float* );
    ///
  void        setToIdentity ();
    ///
  void        setCol( int i, const TVector<MeFloat,3> &tv );
    ///
  void        setRow( int i, const TVector<MeFloat,3> &tv );
    ///
  void        translate( MeFloat, MeFloat, MeFloat );
    ///
  void        translate( const TVector<MeFloat,3>& );
    ///This affects only the upper left 3x3 submatrix
  void        transpose3 ();

  G3DMatrix&         operator=( const G3DMatrix &m );
    ///Matrix multiplication: this = this * m.
  G3DMatrix&         operator*=( const G3DMatrix &m );
    ///Transform the point
  TVector<MeFloat,3> operator*( const G3DPoint3d& ) const;
    ///Transform the vector. This will not apply the translation part.
  TVector<MeFloat,3> operator*( const TVector<MeFloat,3>& ) const;
  G3DMatrix          operator*( const G3DMatrix &m ) const;
                     operator const MeFloat*() const { return(_matrix); };

protected:
  MeFloat  _matrix[16];
};

/*TODO
  //MeFloat&            operator()( uint, uint );

inline MeFloat&
G3DMatrix::operator()( uint i, uint j )
{
  --j;  --i;
  j *= 4;
  j += i;
  assert( j < 16 );
  return( _matrix[j] );
}
*/

#endif
