/*
 * Project : "MindsEye, math lib"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: quaternion.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * Classes for Vectors based on  the "temporary base class idiom".
 * See the article in the german magazine iX issue 11/96 p166.
 *
 */

#ifndef _QUATERNION_H_
#define _QUATERNION_H_


#include <iostream.h>
#include <math.h>

template <class T, int N> class TVector;


/**This is a a representation of a Quaternion. Quaternions are used to
   represent rotations.
   */
class G3DQuaternion
{
    /**Write the components to the ostream, separated by a space.
       The last caracter is a space. */
  friend istream&  operator>>( istream& os, G3DQuaternion& );
  friend ostream&  operator<<( ostream& os, const G3DQuaternion& );

public:
    ///Create unit quaternion without rotation
  G3DQuaternion ();
    ///Create rotation around u with angle a;
  G3DQuaternion( const TVector<MeFloat,3> &u, MeFloat a );
    ///Create rotation x-y-z
  G3DQuaternion( const TVector<MeFloat,3>& );
    ///
  G3DQuaternion( const G3DQuaternion& qat );

  void  conjugate ();
  void  getRotationMatrix( MeFloat data[16] ) const;
  bool  isZero () const;
  void  normalize ();
    ///Decompose this rotation into 3 rotations about the x,y,z axes.
  void  quat2xyz( G3DVector3d &angles ) const;

  G3DQuaternion&     operator=( const G3DQuaternion& );
  TVector<MeFloat,3> operator*( const TVector<MeFloat,3>& ) const;
  G3DQuaternion&     operator*=( const G3DQuaternion &qat );
    ///
  MeFloat            operator[]( int i ) const { return data_[i]; }
    ///Return a const pointer to the data of the vector.
	             operator const MeFloat*() const { return data_; }

protected:
  MeFloat data_[4];

};


#endif
