/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: object.cpp,v 1.5 2002/06/11 09:56:46 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <string.h>
#include <GL/gl.h>

#include "object.h"
#include "tools.h"


//const G3DColor* G3DObject::selectColor_ = NULL;

int G3DObject::obj_count_ = 0;

G3DObject::G3DObject () :
  scale_(1,1,1)
{
  parent_  = 0;
  name_    = 0;
  count_   = 1;
  state_   = visible;
  type_    = 0;
  
  trafo_.setToIdentity (); // ?

  obj_count_++;
  cout << "--> G3DObject constructor (" << obj_count_ << ")" << endl;
}


G3DObject::G3DObject( G3DObject *parent ) :
  scale_(1,1,1)
{
  parent_  = parent;
  name_    = 0;
  count_   = 1;
  state_   = visible;
  type_    = 0;

  obj_count_++;
  cout << "--> G3DObject constructor (" << obj_count_ << ")" << endl;
};


G3DObject::G3DObject( const G3DObject &orig ) :
  trafo_(orig.trafo_), origin_(orig.origin_), scale_(orig.scale_),
  rotation_(orig.rotation_), pivotPoint_(orig.pivotPoint_)
{
  parent_   = 0;  //TODO
  name_     = string_copy( NULL, orig.name_ ); //TODO really?
  count_    = 1; //orig._count;  //TODO no!!
  state_    = orig.state_;
  type_     = orig.type_;

  obj_count_++;
  cout << "--> G3DObject constructor (" << obj_count_ << ")" << endl;
};


G3DObject::~G3DObject ()
{
  obj_count_--;
  cout << "--> G3DObject destructor (" << obj_count_ << ")" << endl;
  if (count_ != 0) 
    cout << "    (but ref count is not 0)" << endl;
  delete[] name_;
}

void
G3DObject::adjust ()
{
}

void
G3DObject::applyTransform ()
{
  trafo_.setToIdentity();
  //TODO
  //reset bounding box
}

G3DObject*
G3DObject::copy () const
{
  return 0;
}

void
G3DObject::dispose ()
{
  cout << "--> G3DObject::dispose" << endl;
  --count_;
  if (count_ <= 0)
    delete this;
}

void
G3DObject::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_HIDE:
      state_ = G3DObjectState( state_ & (~visible) );  break;
    case A_SHOW:
      state_ = G3DObjectState( state_ | visible);      break;
    case A_SELECT:
      state_ = G3DObjectState( state_ | selected );     break;
    case A_DESELECT:
      state_ = G3DObjectState( state_ & (~selected) ); break;
    case A_GENERIC_ALL:
    case A_GENERIC_SHAPES:
    case A_GENERIC_LIGHTS:
      ((GenericAction&)action).evaluate( this );
      break;
    }
}

const G3DQuaternion&
G3DObject::getOrientation () const
{
  return rotation_;
}

const Field*
G3DObject::getFieldList () const
{
  return 0;
}

void
G3DObject::getFunctionList () const
{
}

void
G3DObject::getLocalTrafo( G3DMatrix &m ) const
{
  m = trafo_;
}

const G3DPoint3d&
G3DObject::getOrigin () const
{
  return origin_;
}

uint
G3DObject::getParameter( Param* )
{
  return 0;
}

const G3DPoint3d&
G3DObject::getPivotPoint () const
{
  return pivotPoint_;
}

const G3DVector3d&
G3DObject::getScale () const
{
  return scale_;
}

G3DObject*
G3DObject::getSelectionHandle ()
{
  //TODO increment refCount_ ?
  return this;
}

const char*
G3DObject::GetTypeName ()
{
  return ("Object");
}

const char*
G3DObject::getTypeName () const
{
  return (G3DObject::GetTypeName());
}

void
G3DObject::getWorldTrafo( G3DMatrix &m, bool locale ) const
{
  if( parent_ )
    parent_->getWorldTrafo( m );

  if( locale )
    m *= trafo_;
}

void
G3DObject::rebuildTrafo ()
{
  // trafo_all_in_one = translate * rotate * scale
  trafo_.setToIdentity();
  trafo_.translate( origin_ );

  if (!rotation_.isZero()) {
    G3DMatrix rot( rotation_ );
    trafo_ *= rot;
  }

  G3DMatrix sc;
  sc.scale (scale_);
  trafo_ *= sc;
}

MeFloat
G3DObject::select ( const G3DCamera&, const G3DRay&, G3DPartEnum )
{
  return G3D_INFINITY;
}

void
G3DObject::setLocalTrafo ( const G3DMatrix &m )
{
  trafo_ = m;
}

void
G3DObject::setName( const char* name )
{
  name_ = string_copy( name_, name );
}

void
G3DObject::setOrientation( const G3DQuaternion &quat )
{
  rotation_ = quat;
  rebuildTrafo ();
}

void
G3DObject::setOrientation( const G3DQuaternion &quat,
			   const TVector<MeFloat,3> &pivot,
			   const TVector<MeFloat,3> &orig )
{
  rotation_ = quat;
  rebuildTrafo();
  TVector<MeFloat,3> tmp(orig);   //because orig is const
  setOrigin( tmp - ((trafo_*TVector<MeFloat,3>(pivotPoint_)) - pivot) );
}

void
G3DObject::setOrigin( const TVector<MeFloat,3> &orig )
{
  origin_ = orig;
  rebuildTrafo ();
}

void
G3DObject::setParameter ( const Param* )
{
}

void
G3DObject::setParent ( G3DObject *pt )
{
  /*TODO is this realy useful?
  G3DMatrix mg, mo;
  pt->getWorldTrafo( mg );
  mg.invert4();
  getWorldTrafo( mo );  //w.r.t. the old parent (if any)
  mg *= mo;
  mg.getTranslation( origin_ );
  if( !rotation__.isZero() )
    {
      origin_ += anchor_ * (-1);
      G3DMatrix rot(  );
      G3DPoint3d pc;
      pc = rot * anchor_;
      origin_ += pc;
    }
    */
  parent_ = pt;
}

void
G3DObject::setPivotPoint( const G3DPoint3d &p )
{
  pivotPoint_ = p;
}

void
G3DObject::setScale( const TVector<MeFloat,3> &scale )
{
  scale_ = scale;
  rebuildTrafo();
}

/*
G3DObject& G3DObject::operator=( const G3DObject &a )
{  
  return *this ;
}
*/

//==============================END===================================
