/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: pointstuff.cpp,v 1.2 2002/06/10 09:30:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include "math/g3d_ray.h"
#include "camera.h"
#include <nurbs++/specialType.h>
#include "pointstuff.h"
#include <nurbs++/vector.h>
#include <nurbs++/matrix.h>



int
findNearestPoint( const G3DCamera& cam, const G3DRay &ray,
		  const G3DMatrix &wtrafo, const G3DPoint3d *pt,
		  uint num, MeFloat& radius )
{
//TODO alt: copy ray and transform it into objects system
//problem: distances between different objects not comparable!

  G3DPoint3d ptransformed;
  int    nearest = -1;
  MeFloat dist    = 10E100;
  MeFloat mx = ray.x, my = ray.y;  //devicepoint under mouse:
  MeFloat px, py;  //devicepoint for pt
  MeFloat rad;

  for( uint i = 0; i < num; ++i )
    {
      ptransformed = wtrafo * *(pt++);
      cam.project( ptransformed, &px, &py );
      rad = (mx-px)*(mx-px) + (my-py)*(my-py);
      if( rad <= radius )
	{
	  MeFloat tmp = ray.getDistanceTo( ptransformed );
	  if( (tmp < dist) && (tmp > 0.0) )
	    {
	      dist    = tmp;
	      nearest = i;
	    }
	}
    }
  radius = dist;
  return( nearest );
}

int
findNearestCPointV( const G3DCamera& cam, const G3DRay &ray,
		    const G3DMatrix &wtrafo,
		    const  PLib::Vector<HPoint3Df> &vpt,
		    MeFloat& radius )
{
//TODO alt: copy ray and transform it into objects system
//problem: distances between different objects not comparable!

  G3DPoint3d ptransformed;
  int     nearest = -1;
  MeFloat dist    = 10E100;
  MeFloat mx = ray.x, my = ray.y;  //devicepoint under mouse:
  MeFloat px, py;  //devicepoint for pt
  MeFloat rad;

  for( int i = 0; i < vpt.size(); ++i )
    {
      G3DPoint3d conv( vpt[i].data );
      ptransformed = wtrafo * conv;
      cam.project( ptransformed, &px, &py );
      rad = (mx-px)*(mx-px) + (my-py)*(my-py);
      if( rad <= radius )
	{
	  MeFloat tmp = ray.getDistanceTo( ptransformed );
	  if( (tmp < dist) && (tmp > 0.0) )
	    {
	      dist    = tmp;
	      nearest = i;
	    }
	}
    }
  radius = dist;
  return( nearest );
}

bool
findNearestCPointM( const G3DCamera& cam, const G3DRay &ray,
		    const G3DMatrix &wtrafo,
		    const  PLib::Matrix<HPoint3Df> &mpt,
		    MeFloat& radius, 
		    int &row, int &col )
{
//TODO alt: copy ray and transform it into objects system
//problem: distances between different objects not comparable!

  G3DPoint3d ptransformed;
  int     nearest = -1;
  MeFloat dist    = 10E100;
  MeFloat mx = ray.x, my = ray.y;  //devicepoint under mouse:
  MeFloat px, py;  //devicepoint for pt
  MeFloat rad;

  for( int i = 0; i < mpt.rows(); ++i )
    {
      for( int j = 0; j < mpt.cols(); ++j )
	{
	  G3DPoint3d conv( mpt(i,j).data );
	  ptransformed = wtrafo * conv;
	  cam.project( ptransformed, &px, &py );
	  rad = (mx-px)*(mx-px) + (my-py)*(my-py);
	  if( rad <= radius )
	    {
//cout << rad << endl;
	      MeFloat tmp = ray.getDistanceTo( ptransformed );
	      if( (tmp < dist) && (tmp > 0.0) )
		{
		  dist = tmp;
		  row  = i;
		  col  = j;
		}
	    }
	}
    }
  radius = dist;
  return( (row >= 0) && (col >= 0) );
}


MeFloat
distanceToLine( MeFloat* line, MeFloat mx, MeFloat my, MeFloat rad )
{
  MeFloat s1, s2, p1, p2, dist;
  MeFloat s_length;

  s1   = line[2] - line[0];
  s2   = line[3] - line[1];
  p1   = mx - line[0];
  p2   = my - line[1];
  s_length = s1*s1 + s2*s2;
  if( s_length < FG_EPSILON )
    {
      dist = p1*p1 + p2*p2;
      if( dist <= rad )
	return( dist );
    }
  else
    {
      dist     = (p1 * s2) - (p2 * s1); //distance to infinite line
      dist     = dist * dist / s_length;

      if( dist <= rad )
	{
	  MeFloat d;
	  d = p1*s1 + p2*s2;  //project point to line
	  if( d > -rad && d < s_length + rad )
	    return( dist );
	}
    }
  return( G3D_INFINITY );
}

int
polygonTest( const G3DRay &ray, const G3DMatrix &wtrafo,
	     const G3DPoint3d *pt, uint *index, uint num )
{
  G3DPoint3d ptransformed;
  MeFloat xlast, ylast = 0;
  int wind = 0;

  for( uint i = 0; i <= num; ++i )
    {
      if( i == num )
        index -= num;
      ptransformed = wtrafo * pt[*(index++)];
      //cout << ptransformed[0] << ' ' << ptransformed[1] << "   ";
      if( ylast * ptransformed[1] < 0 )  //TODO what about 0
	{
	  MeFloat f = ylast - ptransformed[1];
          if( fabs( f ) > FG_EPSILON )
	    {
	      MeFloat a = xlast + (ptransformed[0] - xlast)*ylast / f;
	      if( a > 0 )    //shooting a ray to the right
		{
		  //cout << 'a' << a << ' ';
		++wind;
		}
	    }
	}
      xlast = ptransformed[0];
      ylast = ptransformed[1];
    }
  //cout << endl;
  //cout << "w " << wind << endl;
  return( wind & 0x1 );  //even or odd number?
}

/*
void
octree( G3DPoint3d *vert, int n, G3DOctree &ot )
{

sort the individual faces into the given octree
a face may span more than one octree
}
*/

//================================END================================

