/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_geosphere.h,v 1.2 2001/04/01 09:42:28 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_TRISPHERE_H__
#define __S_TRISPHERE_H__

#include "shape.h"

///
/** A sphere is defined by an origin and 3 axes (?)
 */
class G3DGeoSphere : public G3DShape
{
public:
  /// Constructor.
  G3DGeoSphere ( MeFloat r );
  
  /// Constructor.
  G3DGeoSphere ( MeFloat r, const G3DPoint3d &center );

  /// Copy constructor.
  G3DGeoSphere ( const G3DGeoSphere& );

  /// Destructor. called when the object is destroyed.
  virtual ~G3DGeoSphere ();

  ///
  virtual G3DObject* copy () const;

  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;

  
  /**Copy pointers to the geometric data to the list. The order is:
     center, radius*/
  virtual uint getParameter ( Param* );

  /// Return the sphere's position (center ?).
  virtual void getPosition ( G3DPoint3d& ) const;

  /// Return a Descriptive name of the shape.
  virtual const char* getTypeName () const;
  static  const char* GetTypeName ();

  ///
  virtual MeFloat intersect ( G3DRay*, G3DObject** );

  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();

  ///
  virtual void setParameter ( const Param* );
  
  /// Return the sphere's position (center ?)
  virtual void setPosition ( const G3DPoint3d& );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;

protected:
  ///
  void tesselate ();

protected:
  G3DPoint3d  vertices_[14];
  G3DVector3d normals_[14];
  G3DPoint3d  center_;
  MeFloat     radius_;

  static Field FieldList_[2];
};

#endif
