/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_glshapes.cpp,v 1.3 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "s_glshapes.h"


G3DGlStrip::G3DGlStrip ()
{
  nVertices_ = 0;
  vertices_  = NULL;
  normals_   = NULL;
}

G3DGlStrip::G3DGlStrip( const G3DGlStrip &orig )
{
  nVertices_ = orig.nVertices_;
  vertices_  = new G3DPoint3d[nVertices_];
  normals_   = new G3DVector3d[nVertices_];
}

G3DGlStrip::~G3DGlStrip ()
{
  delete[] vertices_;  //TODO call destructors ?
  delete[] normals_;
}

void
G3DGlStrip::setNormal ( uint num, const TVector<MeFloat,3> &normal )
{
  normals_[num] = normal;
  normals_[num].normalize();
}

void
G3DGlStrip::setStripeNumber( uint num )
{
  num *= 2 ;
  num += 2;
  //TODO if( num != nVertices_ )
  delete[] vertices_;
  delete[] normals_;
  if (num > 0) {
    vertices_  = new G3DPoint3d[num];
    normals_   = new G3DVector3d[num];
  }
  else {
    vertices_ = NULL;
    normals_  = NULL;
  }
  nVertices_ = num;
}

void
G3DGlStrip::setVertex ( uint num, const TVector<MeFloat,3> &vertex )
{
  vertices_[num] = vertex;
}


//----------------------------------------------------------------


G3DQuadStrip::~G3DQuadStrip ()
{
}

void
G3DQuadStrip::shade () const
{
  glBegin (GL_QUAD_STRIP);
  for (uint i = 0; i < nVertices_; i++) {
    glNormal3xv( (const MeFloat*)(normals_[i]) );
    // TODO TXTR_COORD(tcx, tcy);
    glVertex3xv( (const MeFloat*)(vertices_[i]) );
    //glNormal3xv( (const MeFloat*)(_normals[i+1]) );
    //glVertex3xv( (const MeFloat*)(_vertices[i+1]) );
  }
  glEnd ();
}


//---------------------------------------------------------------

/*
G3DGLTriFan::G3DGLTriFan( uint num )
{
  _vertices  = NULL;
  _normals   = NULL;
  setTriangleNumber( num );
}


G3DGLTriFan::G3DGLTriFan( const G3DGLTriFan &orig )
{
  _nVertices = orig._nVertices;
  _vertices  = new G3DVector3d[_nVertices];
  _normals   = new G3DVector3d[_nVertices/2];
}


G3DGLTriFan::~G3DGLTriFan()
{
  delete[] _vertices;  //TODO call destructors ?
  delete[] _normals;
}


void
G3DGLTriFan::setTriangleNumber( uint num )
{
  num += 2;
  delete[] _vertices;
  delete[] _normals;
  if( num > 0 )
    {
      _vertices  = new G3DVector3d[num];
      _normals   = new G3DVector3d[num/2];
    }
  else
    {
      _vertices = NULL;
      _normals  = NULL;
    }
  _nVertices = num;
}


void
G3DGLTriFan::setVertex( uint num, const G3DPoint3d &vertex )
{
  _vertices[num] = vertex;
}


void
G3DGLTriFan::setNormal( uint num, const TVector<MeFloat,3> &normal )
{
  _normals[num] = normal;
  _normals[num].normalize();
}


void
G3DGLTriFan::shade () const
{
  glBegin( GL_TRIANGLE_FAN );
  glVertex3xv( (const MeFloat*)(_vertices[0]) );
  for( uint i = 1, j = 0; i < _nVertices; i += 2 )
    {
      glNormal3xv( (const MeFloat*)(_normals[j++]) );
      // TODO TXTR_COORD(tcx, tcy);
      glVertex3xv( (const MeFloat*)(_vertices[i]) );
      glNormal3xv( (const MeFloat*)(_normals[j++]) );
      glVertex3xv( (const MeFloat*)(_vertices[i+1]) );
    }
  glEnd();
}
*/

//-------------------------------------------------------------------


G3DTriStrip::~G3DTriStrip()
{
}

void
G3DTriStrip::shade () const
{
  glBegin( GL_TRIANGLE_STRIP );
  for( uint i = 0; i < nVertices_; i++ )
    {
      glNormal3xv( (const MeFloat*)(normals_[i]) );
      // TODO TXTR_COORD(tcx, tcy);
      glVertex3xv( (const MeFloat*)(vertices_[i]) );
      //glNormal3xv( (const MeFloat*)(normals_[i+1]) );
      //glVertex3xv( (const MeFloat*)(vertices_[i+1]) );
    }
  glEnd();
}


//===============================END===============================
