/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_glshapes.h,v 1.4 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of the primitive shapes used by OpenGL
 */

#ifndef __S_GL_SHAPES_H_
#define __S_GL_SHAPES_H_

#include "shape.h"

/// Base class for OpenGL stripe primitives.
class G3DGlStrip
{
public:
  /// Default constructor.
  G3DGlStrip ();
  
  //G3DTriStrip( uint num, const G3DPoint3d *vertices  );

  /// Copy constructor.
  G3DGlStrip ( const G3DGlStrip& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DGlStrip ();

  /// Set the normal vector of the vertex with a given index.
  void setNormal ( uint num, const TVector<MeFloat,3> &n );

  /// Set the number of stripes. The old date is deleted.
  void setStripeNumber ( uint num );
  
  /// Set the numth vertex to the given value. 
  void setVertex ( uint num, const TVector<MeFloat,3> &v );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
   */
  virtual void shade () const = 0;

protected:
  G3DPoint3d  *vertices_;  //TODO points?
  G3DVector3d *normals_;
  uint        nVertices_;
  //TODO flat_, closed_
};


/// OpenGL QuadStrip.
class G3DQuadStrip : public G3DGlStrip
{
public:
  /// Default constructor.
  G3DQuadStrip () : G3DGlStrip() {}

  /// Copy constructor.
  G3DQuadStrip ( const G3DQuadStrip &orig ) : G3DGlStrip(orig) {}

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DQuadStrip ();

  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
   */
  virtual void shade () const;
};


/**OpenGL Triangle Fan */
/*
class G3DGLTriFan
{
public:
  G3DGLTriFan( uint num );
  G3DGLTriFan( const G3DGLTriFan &);
  ~G3DGLTriFan();

    ///Set the center point of the fan. It is first point in the list.
  //void setCenter( TVector<MeFloat,3>& );
    ///
  void setNormal( uint num, const TVector<MeFloat,3> &normal );
    ///
  void setTriangleNumber( uint );
    ///Set the numth vertex to the given value. 
  void setVertex( uint num, const G3DPoint3d &vertex );
    ///
  void shade () const;

protected:
  G3DVector3d *_vertices;  //TODO points?
  G3DVector3d *_normals;
  uint        _nVertices;
};
*/


/// OpenGL TriangleStrip.
class G3DTriStrip : public G3DGlStrip
{
public:
  /// Default constructor.
  G3DTriStrip () : G3DGlStrip() {}

  /// Copy constructor.
  G3DTriStrip ( const G3DTriStrip &orig ) : G3DGlStrip(orig) {}

  /// Destructor. Called when the object is destroyed.
  ~G3DTriStrip ();

  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
   */
  virtual void shade () const;
};

#endif
