/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_hnurbsurf.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include "s_line.h"
#include "s_hnurbsurf.h"
#include "s_point.h"

/*
void
isoCurves( const PlHNurbsSurface *hs, G3DNurbsCurve &t, G3DNurbsCurve &ta,
	   int lod, const PlVector<float> &ub, const PlVector<float> &vb,
	   PlHNurbsSurface* activePatch );
void
splitAtActiveInU( G3DNurbsCurve &t, G3DNurbsCurve &ta, float u,
		  PlHNurbsSurface *active, int lod );
void
splitAtActiveInV( G3DNurbsCurve &t, G3DNurbsCurve &ta, float v,
		  PlHNurbsSurface *active, int lod );
		  */

//--------------------------------------------------------------------

G3DHNurbsSurface::G3DHNurbsSurface() : 
  PlHNurbsSurface(), G3DNurbs()
{
  type_       = T_NURBS_HSURF;
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const G3DHNurbsSurface& bS ) :
  PlHNurbsSurface(bS), G3DNurbs(bS)
{
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const G3DHNurbsSurface* bS, float us,
				    float ue, float vs, float ve) : 
  PlHNurbsSurface(bS,us,ue,vs,ve), G3DNurbs(*bS)
{
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this; 
}

G3DHNurbsSurface::G3DHNurbsSurface( const PlNurbsSurface& nS ) : 
  PlHNurbsSurface(nS), G3DNurbs()
{ 
  type_       = T_NURBS_HSURF;
  polygon     = new G3DNurbsSpolygon(*this);
  lod         = maxLevel() ; 
  activePatch = this ; 
}


void
G3DHNurbsSurface::addLevel()
{
  // go to the last level
  if(activePatch){
     while(activePatch->firstPatch)
       activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
     activePatch->addPatch(2) ;
  }
}

void
G3DHNurbsSurface::decreaseLevelOfDetail()
{
  --lod ; 
  if(lod<0) lod=0 ; 
  if(activePatch)
    while(activePatch->level > lod)
      activePatch = (G3DHNurbsSurface*)activePatch->base ;
}


int
G3DHNurbsSurface::initClass ()
{
  tesselTolerance_ = 0.1;
  NurbsRenderer_   = gluNewNurbsRenderer();
  if( NurbsRenderer_ )
    {
      gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_SAMPLING_TOLERANCE, 25.0 );
      return( 1 );
    }
  else
    return( 0 );
}

void
G3DHNurbsSurface::modifyPoint( float u, float v, float dx, float dy,
			       float dz )
{
}


void
G3DHNurbsSurface::point( float &u, float &v, int pSize,
			 const G3DColor& colorP, int cp_flag ) const
{
  if( u < U[0] )
    u = U[0];
  if( u > U[U.n-1] )
    u = U[U.n-1];
  if( v < V[0] )
    v = V[0];
  if( v > V[V.n-1] )
    v = V[V.n-1];

  glPointSize( pSize );
  glBegin( GL_POINTS );
  float color[4];
  color[0] = color[1] = color[2] =  0.0;
  color[3] = 1.0;
  glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, color );
  //TODO
  //glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, (const float*)colorP ) ;
  glVertex4fv( pointAt(u, v, levelOfDetail()).data );
  glEnd();

  //TODO
  PlNurbsCurveGL t ;
  t.setObjectColor(colorP,colorP,colorP) ;
  //t.setNurbsRenderer(nurbsRenderer) ;
  t.hideBBox() ;
  t.hideCPoints() ;
  t.hideCpolygon() ;
  t.viewNurbs() ;
  isoCurveU(u,t,levelOfDetail()) ;
  t.glObject() ;
  isoCurveV(v,t,levelOfDetail()) ;
  t.glObject() ;

  if(cp_flag){ // display only the CP affected by the point
    //TODO
    int spanU, spanV ;
    activePatch->findSpan(u,v,spanU,spanV) ;
    int i,j,n ;
    
    PlObjectGL *pnt ;
    pnt = (PlObjectGL*)cpoints.first ;
    n = 0 ;
    while(pnt){
      i = n / activePatch->CtrlP.cols ;
      j = n % activePatch->CtrlP.cols ;
      //cerr << i << ", " << j << " : " ;
      if(i<spanU-activePatch->degreeU || i>spanU)
	pnt->hideObject() ;
      else{
	if(j<spanV-activePatch->degreeV || j>spanV)
	  pnt->hideObject() ;
	else
	  pnt->viewObject() ;
      }
      ++n ;
      pnt = pnt->next() ;  
    }
  }
}

void
G3DHNurbsSurface::resetBoundingBox()
{
  G3DPoint3d min, max;

  min[0] = extremum( 1, coordX );
  min[1] = extremum( 1, coordY );
  min[2] = extremum( 1, coordZ );
  max[0] = extremum( 0, coordX );
  max[1] = extremum( 0, coordY );
  max[2] = extremum( 0, coordZ );

  delete bbox_;
  bbox_ = new G3D_BBAligned( min, max );
}

void
G3DHNurbsSurface::resetCPoints()
{
  if(activePatch)
    {
      PlCPointGL *tempCP;

      cpoints.reset();
      cpoints.setJumpSize( activePatch->ctrlPnts().cols() );

      for(int i=0; i<activePatch->ctrlPnts().rows(); ++i)
	for(int j=0; j<activePatch->ctrlPnts().cols(); ++j)
	  {
	    int mod ;
	    mod = 1 ;
	    if( i==0 || j==0 
	         || i==activePatch->ctrlPnts().rows()-1 
	         || j==activePatch->ctrlPnts().cols()-1 )
	      mod = 0;
	    mod = 1 ; // if we don't use multi-patches it should be safe
	    tempCP = new PlHCPointGL( activePatch->offset(i,j), i, j,
				      activePatch,mod );
	    tempCP->setObjectColor( cpointColorDefault );
	    tempCP->setCurrentColor( cpointActiveColorDefault );
	    if( j==0 && (i!=activePatch->ctrlPnts().rows()-1) )
	      tempCP->setObjectColor( cpoint0ColorDefault );
	    cpoints.add( tempCP );
	  }
    }
}

void
G3DHNurbsSurface::resetPolygon()
{
  int h ;
  if( polygon )
    {
      h = polygon->getState() ;
      delete polygon ;
    }
  polygon = new G3DNurbsSpolygon(*activePatch) ;
  if( h )
    viewCpolygon() ;
  else
    hideCpolygon() ;
}


void
G3DHNurbsSurface::selectHigherLevel()
{
  if( activePatch )
    {
      if( activePatch->level() < levelOfDetail() )
	if( activePatch->firstPatch )
	  activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
    }
}

void
G3DHNurbsSurface::selectHighestLevel()
{
  if(activePatch){
    while(activePatch->level() < levelOfDetail())
      if(activePatch->firstPatch)
	activePatch = (G3DHNurbsSurface*)activePatch->firstPatch ;
  }
}


void
G3DHNurbsSurface::selectLowerLevel()
{
  if( activePatch )
    {
      if( activePatch->base )
	activePatch = (G3DHNurbsSurface*)activePatch->base;
    }
}


void
G3DHNurbsSurface::selectNextPatch()
{
  if(activePatch){
    if(activePatch->nextPatch)
      activePatch = (G3DHNurbsSurface*)activePatch->nextPatch ;
    else
      if(activePatch->base)
	activePatch = (G3DHNurbsSurface*)activePatch->base->firstPatch ;
  }
}


void
G3DHNurbsSurface::selectPrevPatch()
{
  if(activePatch){
    if(activePatch->prevPatch)
      activePatch = (G3DHNurbsSurface*)activePatch->prevPatch ;
    else
      if(activePatch->base){
	activePatch = (G3DHNurbsSurface*)activePatch->base->lastPatch ;
    }
  }
}


void
G3DHNurbsSurface::shade( int ) const
{
  if( NurbsRenderer_ )
    {
    if(0){ // Well this is not necessary anymore with the new method 
      // of creatint a HNURBS surface, I keep it for now but should
      // delete it soon.

      // create iso curves
      G3DNurbsCurve t ;
      G3DNurbsCurve ta ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      ta.setObjectColor(objectColor,selectColor,currentColor) ;
      ta.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      ta.hideBBox() ;
      ta.hideCPoints() ;
      ta.hideCpolygon() ;
      ta.viewNurbs() ;
      if(isSelected()) { t.select() ; ta.select() ; }
      if(isActive()) ta.activate() ;

      Vector_FLOAT ub,vb ;
      ub.resize(0) ;
      vb.resize(0) ;
      isoCurves(this,t,ta,lod,ub,vb,activePatch) ;
    }

    G3DHNurbsSurface* dispPatch ;
    dispPatch = activePatch ;
    int level = levelOfDetail() ;
    if(level<0) 
      level = maxLevel() ;
    while(dispPatch->level() < level)
      dispPatch = (G3DHNurbsSurface*)dispPatch->firstPatch ;
    
    switch(PlNurbsDisplayMode){
    case PLNURBS_DISPLAY_ISOCURVES:
      {
      // create iso curves
      PlNurbsCurveGL t ;
      t.setObjectColor(objectColor,selectColor,currentColor) ;
      t.setNurbsRenderer(nurbsRenderer) ;
      t.hideBBox() ;
      t.hideCPoints() ;
      t.hideCpolygon() ;
      t.viewNurbs() ;
      if(isActive()) t.activate() ;
      if(isSelected()) t.select() ;

      Vector_FLOAT ub ;
      averagingKnots(dispPatch->U,degU,ub) ;

      int i ;
      for(i=0;i<ub.n;++i){
	dispPatch->isoCurveU(ub[i],t,level) ;
	t.glObject() ;
      }
      averagingKnots(dispPatch->V,degV,ub) ;

      for(i=0;i<ub.n;++i){
	dispPatch->isoCurveV(ub[i],t,level) ;
	t.glObject() ;
      }
    }
      break ;
    case PLNURBS_DISPLAY_SHADED:
    case PLNURBS_DISPLAY_HIDDEN_LINES:
    case PLNURBS_DISPLAY_TESSELATION:
    case PLNURBS_DISPLAY_NORMAL:
    default:
      {
      GLenum t = GL_MAP2_VERTEX_4 ; 
      gluNurbsProperty(nurbsRenderer,GLU_DISPLAY_MODE,GLU_OUTLINE_POLYGON) ;
      gluBeginSurface(nurbsRenderer) ;
      glColor() ;
      gluNurbsSurface(nurbsRenderer, dispPatch->U.n, dispPatch->U.memory(), dispPatch->V.n, dispPatch->V.memory(),4,4*dispPatch->P.rows, dispPatch->P[0]->data, dispPatch->degU+1, dispPatch->degV+1, t) ;
      gluEndSurface(nurbsRenderer) ;
      }      
    }
    }
}


//=================================END=================================
