/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_hnurbsurf.h,v 1.3 2002/06/10 09:30:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_HNURBS_SURFACE_H_
#define __S_HNURBS_SURFACE_H_

#include <nurbs++/nurbsS.h>
#include <nurbs++/hnurbsS.h>
#include <GL/glu.h>
#include "s_nurbs.h"



/*  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **
    Class: PlHNurbsSurfaceGL --- an interface to help design with OpenGL 
    Description: This class offers an interface to help design OpenGL 
    applications and use this NURBS library at the same time.
    Created by: Philippe Lavoie (23 September 1997)
    Modified by:
    **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  **  */
/// (?)
class G3DHNurbsSurface : public PlHNurbsSurface, public G3DNurbs
{
public:
  ///
  G3DHNurbsSurface ();

  ///
  G3DHNurbsSurface ( const G3DHNurbsSurface& bS );
#if 0
  // copy constructor with patch information
  // Input: bS --> the object to copy
  // us --> the start of the U parametric patch
  // ue --> the end of the U parametric patch
  // vs --> the start of the V parametric patch
  // ue --> the end of the V parametric patch *
#endif
  G3DHNurbsSurface ( const G3DHNurbsSurface* bS, float us, float ue,
		     float vs, float ve );

  ///
  G3DHNurbsSurface ( const PlNurbsSurfacef& nS );

  /// Adds a level to the Hierarchical surface.
  void addLevel ();

  ///
  G3DObject* copy () const { return (new G3DHNurbsSurface(*this)); }

  ///
  void decreaseLevelOfDetail ();

  ///
  void highestLevelOfDetail () { lod = maxLevel() ; }

  ///
  void increaseLevelOfDetail () { ++lod; if(lod>maxLevel()) lod=maxLevel(); }
  
  ///
  int levelOfDetail () const { return lod; }
  
  ///
  int maxLevelOfDetail () { return maxLevel() ; }
  
  ///
  void modifyPoint ( float u, float v, float dx, float dy, float dz );
  
  /// Draw a point on the surface at coordinates (u,v).
  void point ( float &u, float &v, int pSize, const G3DColor&, int ) const;
  
  ///
  virtual void resetBoundingBox ();

  ///
  void resetCPoints ();

  ///
  void resetPolygon ();
  
  ///
  void resetKnots () {}

  ///
  void setLevelOfDetail ( int l ) { lod = l ; }

  ///
  void selectBasePatch () { activePatch = this; }

  /// Activates the patch at a higher level if available.
  void selectHigherLevel ();

  /// Activates the patch at a lower level if available.
  void selectLowerLevel ();

  /// Activates the next patch at the same level.
  void selectNextPatch ();

  /// Activates the previous patch at the same level.
  void selectPrevPatch ();
  
  ///
  void selectHighestLevel ();

  ///
  virtual void shade ( int ) const;

  ///
  void updateUpToLOD () { updateLevels(lod) ; }
  
  /// Initialise the GLUnurbsObj.
  static int initClass ();

protected:
  G3DHNurbsSurface *activePatch;
  int lod;

private:
  static GLUnurbsObj *NurbsRenderer_;
  static float       tesselTolerance_;
};

#endif
