/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_mesh.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of a mesh primitive
 */

#include "include/config.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include "s_mesh.h"
#include "camera.h"
#include "pointstuff.h"
#include "s_point.h"
#include "ha_face.h"


IMPLEMENT_DYN(G3DMesh, "QuadMesh")



G3DMesh::G3DMesh( MeFloat ) :
  G3DShape()
{
  nVertices_ = 0;
  nFaces_    = 0;
  nNormals_  = 0;
  vertices_  = NULL;
  faces_     = NULL;
  normals_   = NULL;
  smooth_    = false;

  type_ = T_QUADMESH;
}

G3DMesh::G3DMesh( const G3DMesh &orig ) :
  G3DShape( orig )
{
  nVertices_ = orig.nVertices_;
  nFaces_    = orig.nFaces_;
  nNormals_  = orig.nNormals_;
  smooth_    = orig.smooth_;

  vertices_  = new G3DPoint3d[nVertices_];
  memcpy( vertices_, orig.vertices_, sizeof(G3DPoint3d)*nVertices_ );
  faces_     = new uint[nFaces_*4];
  memcpy( faces_, orig.faces_, sizeof(uint)*nFaces_*4 );
  normals_   = new G3DVector3d[nNormals_];
  memcpy( normals_, orig.normals_, sizeof(G3DVector3d)*nNormals_ );  //TODO?
}

G3DMesh::~G3DMesh()
{
  delete[] vertices_;
  delete[] faces_;
  delete[] normals_;
}


void
G3DMesh::adjust ()
{
  if( smooth_ )
    {
      //calcSmoothNormals();  //TODO only region arund selected point?
    }
  G3DShape::adjust();
}

void
G3DMesh::calcNormals ()
{
  if( smooth_ )
    {
      normals_  = new G3DVector3d[nVertices_];
      nNormals_ = nVertices_;
      calcSmoothNormals();
    }
  else
    {
      normals_  = new G3DVector3d[nFaces_];
      nNormals_ = nFaces_;
      //TODO calcFlatNormals();
    }
}

//TODO I'm still looking for a cleaner and shorter(?) implementation
void
G3DMesh::calcSmoothNormals ()
{
  //1st calcualte a normal for each face
  //G3DVector3d *normals = new G3DVector3d[nFaces_];
  G3DVector3d v1, v2, *norm = normals_;
  uint        *face = faces_;
  for( int i = nFaces_; i > 0; --i )
    {
      v1 = vertices_[*(face+1)] - vertices_[*face];
      v2 = vertices_[*(face+3)] - vertices_[*face];
      *norm = v1.cross(v2);
      norm++;
      face += 4;
    }

  //2nd for each vertex avarage the normals of all adjaced faces.
  /*
  fnormals = new G3DVector3d[nfaces];
  fn       = fnormals;
  vert1 = _vertices;
  vert2 = &(_vertices[_cols]);
  for( uint i = 0; i < _rows - 1; i++ )
    {
      for( uint j = 0; j < _cols-1; j++ )
	{
	  v1 = *vert2 - *vert1;
	  v3 = *(vert1++);
	  v2 = *vert2 - *vert1;
	  v3 = v3 - *vert2;
	  n = v3.cross(v1) + v3.cross(v2) + v1.cross(v2);
	  *(fn++) = n.normalize();

	  v1 = *(vert2++);
	  v1 = v1 - *vert2;
	  v3 = *vert1 - *vert2;
	  n = v2.cross(v3) + v2.cross(v1) + v1.cross(v3);
	  *(fn++) = n.normalize();
	}
    }

  G3DVector3d zero(0,0,0);
  G3DVector3d *n1 = _normals;
  for( uint k = _rows*_cols; k > 0; k--)
    *(n1++) = zero;

  fn = fnormals;
  G3DVector3d *n2;
  n1 = _normals;
  n2 = &(_normals[_cols]);
  for( uint l = nfaces; l > 0; l-=2 )
    {
      *(n1++) += *fn;
      *n2     += *fn;
      *n1     += *(fn++);
      *(n2++) += *fn;
      *n1     += *fn;
      *n2     += *(fn++);
    }
  n1 = _normals;
  for( uint m = _rows*_cols; m > 0; m-- )
    {
      n1->normalize();
      n1++;
    }
    */
}

void
G3DMesh::deleteSelectedFace ()
{
  if( selectionType_ == stFace )
    {
      --nFaces_;
      uint nn     = nFaces_ * 4;
      uint *faces = new uint[nn];
      uint index  = selectionIndex_ * 4;
      //TODO test if the vertices are still part of a face
      memcpy( faces, faces_, index*sizeof(uint) );
      memcpy( faces+index, faces_+index+4, (nn-index)*sizeof(uint) );
      delete[] faces_;
      faces_ = faces;
    }
}

void
G3DMesh::doAction( ActionData &action )
{
  G3DShape::doAction( action );
}


Field G3DMesh::MeshFields_[8] =
{ {"vertices", m_point3}, {"array_size", s_number},
  {"num_faces", s_number}, {"faces", m_number}, {"array_size", s_number},
  {"normals", m_vector3}, {"array_size", s_number}, {"smooth", s_logic}
};

const Field*
G3DMesh::getFieldList () const
{
  return( MeshFields_ );
}

uint
G3DMesh::getParameter( Param *pList )
{
  pList->point3  = vertices_;
  pList++;
  pList->number  = nVertices_;
  pList++;
  pList->number  = nFaces_;
  pList++;
  pList->numList = faces_;
  pList++;
  pList->number  = nFaces_ * 4;
  pList++;
  pList->vector3 = normals_;
  pList++;
  pList->number  = nNormals_;
  pList++;
  pList->logic   = smooth_;

  return( 8 );
}

G3DObject*
G3DMesh::getSelectionHandle_ ()
{
  if( selectionType_ == stVertex )
    return(  new G3DVertex(vertices_[selectionIndex_], this) );
  else
    return( new G3DFace(vertices_, faces_+(selectionIndex_*4), 4, this) );
}

void
G3DMesh::resetBoundingBox ()
{
  if( vertices_ )
    {
      G3DPoint3d pmin(*vertices_), pmax(*vertices_);
      G3DPoint3d *vert = vertices_+1;
      for( uint i = nVertices_-1; i > 0; --i )
	{
	  pmin.min( *vert );
	  pmax.max( *(vert++) );
	}
      bbox_.setTo( pmin, pmax );
    }
  else
    bbox_.reset();

  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

MeFloat
G3DMesh::selectDetails( const G3DCamera &cam, const G3DRay &ray )
{
  MeFloat   rad = 5;
  G3DMatrix mw;
  getWorldTrafo( mw );

  selectionIndex_ = findNearestPoint( cam, ray, mw, vertices_,
				      nVertices_, rad );
  if( selectionIndex_ >= 0 )
    {
      selectionType_ = stVertex;
      //cout << "vertex " << rad << endl;
      return( rad );
    }
  else
    {
      //cout << "testing faces" << endl;
      uint *faces = faces_;
      G3DMatrix pj( ray.direction_, cam.getUp() );
      G3DMatrix tr( ray.startpoint_*(-1) );
      pj *= tr;
      //cout << pj * G3DPoint3d( 1, 1, -3 ) << endl;
      //cout << pj * ray.direction_ << "   " << ray.startpoint_ << endl;
      pj *= mw;

      for( uint i = 0; i < nFaces_; ++i )
	{
	  if( polygonTest( ray, pj, vertices_, faces, 4 ) )
	    {
	      selectionType_  = stFace;
	      selectionIndex_ = i;
	      //cout << "pl test: " << i << endl;
	      return 0.0;
	    }
	  faces += 4;
	}
      //findNearestLine else findNearestFace
    }
  return( G3D_INFINITY );
}

void
G3DMesh::setParameter( const Param *plist )
{
  if( plist )
    {
      if( vertices_ != plist->point3 )
	{
	  delete[] vertices_;
	  vertices_  = plist->point3;
	  nVertices_ = (plist+1)->number;
	}
      plist += 2;
      nFaces_ = plist->number;  plist++;
      if( faces_ != plist->numList )
	{
	  delete[] faces_;
	  faces_  = plist->numList;
	}
      plist += 2;
      if( (normals_ != plist->vector3) || normals_ == NULL )
	{
	  delete[] normals_;
	  normals_  = plist->vector3;
	  nNormals_ = (plist+1)->number;
	}
      plist += 2;
      smooth_ = plist->logic;
      if( (normals_ == NULL) && (nFaces_ > 0) )
	calcNormals();

      resetBoundingBox();
    }
}

void
G3DMesh::setVertex( uint r, uint c, TVector<MeFloat,3> )
{
  //TODO
}

void
G3DMesh::shade( int ) const
{
  uint       *face = faces_;

  glDisable(GL_CULL_FACE);  //TODO not if closed
  glBegin( GL_QUADS );
  /*
  for( uint i = 0; i < nFaces_; ++ i )
    {
      if( *facet == SMOOTH )
	{
	}
      else if( *facet == FLAT )
    }
  */
  if( smooth_ )
    {
      //if( *facet == 0 )
      //else use *facet as index into a list of special normals
      for( uint i = 0; i < nFaces_; ++i )
	{
	  // TODO TXTR_COORD(tcx, tcy);
	  // TODO TXTR_COORD(tcx, tcy);
	  glNormal3xv( (const MeFloat*)(normals_[*face]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glNormal3xv( (const MeFloat*)(normals_[*face]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glNormal3xv( (const MeFloat*)(normals_[*face]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glNormal3xv( (const MeFloat*)(normals_[*face]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  /*
	    glBegin( GL_TRIANGLE_STRIP );
	    glNormal3xv(0);
	    glVertex3xv(1);
	    glVertex3xv(2);
	    glVertex3xv(3);
	  */
	}
    }
  else
    {
      for( uint i = 0; i < nFaces_; ++i )
	{
	  glNormal3xv( (const MeFloat*)(normals_[i]) );
	  // TODO TXTR_COORD(tcx, tcy);
	  // TODO TXTR_COORD(tcx, tcy);
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	  glVertex3xv( (const MeFloat*)(vertices_[*(face++)]) );
	}
    }
  glEnd();
  glEnable(GL_CULL_FACE);
}

void
G3DMesh::shadeDetails () const
{
  GLint tmpPolygonMode[2];
  glGetIntegerv( GL_POLYGON_MODE, tmpPolygonMode );
  glDisable( GL_LIGHTING );
  glColor3f(0.0, 0, 0);
  glDepthFunc( GL_LEQUAL );
  glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
  glPolygonOffset( -2.0, -1 );
  glEnable (GL_POLYGON_OFFSET_FILL );

  shade( 0 );

  glDisable( GL_POLYGON_OFFSET_FILL );
  glPolygonMode( GL_FRONT, GLenum(tmpPolygonMode[0]) );
  glPolygonMode( GL_BACK,  GLenum(tmpPolygonMode[1]) );
  glEnable( GL_LIGHTING );
  glDepthFunc( GL_LESS );
}


//==================================END=================================
