/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbscurve.h,v 1.5 2002/06/13 13:14:41 ainvar Exp $
 *

 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_LINE_H_
#define __S_LINE_H_

#include <nurbs++/nurbs.h>
#include <nurbs++/curve.h>
#include <GL/glu.h>
#include "s_nurbs.h"

using PLib::Point3Df;

// TODO is this just a Polyline, useful for other things, too ?
/// This class represents the polygon joining all the control points of the NURBS curve.
/* Created by: Philippe Lavoie (23 September 1997) */
class G3DNurbsCpolygon : public G3DObject
{
public:
  /// Constructor.
  G3DNurbsCpolygon ( PlNurbsCurvef& c ) : curve(c) {}

  /// Destructor. Called when the object is destroyed.
  ~G3DNurbsCpolygon ();
  
  /* objectColor = cPolygonColorDefault;} */

  /// Issue the right OpenGL commands to draw this object on screen.
  void shade () const;

protected:
  PlNurbsCurvef& curve;
};


/// A NURBS curve.
/** This class offers an interface to help design OpenGL 
    applications and use this NURBS library at the same time.

    A nonuniform rational B-spline (NURB) curve is a three-dimensional
    projection of a four-dimensional curve. A NURB curve is specified by its
    order, the number of control points used to define it, the control points
    themselves, and the knots used to define it. Attributes may be applied
    only to the entire NURB curve.
*/
/*  
    Created by: Philippe Lavoie (23 September 1997)
*/
class G3DNurbsCurve : public PlNurbsCurvef, public G3DNurbs
{
  DECLARE_DYN

public:
  /// Constructor.
  G3DNurbsCurve ( MeFloat );

  /// Construcor taking a PlNurbsCurve object.
  G3DNurbsCurve ( const PlNurbsCurvef& nurb );
  
  /// Constructor.
  G3DNurbsCurve ( const Vector_HPoint3Df& P1, const Vector_FLOAT &U1,
                  int degree=3 );

  /// Copy constructor.
  G3DNurbsCurve ( const G3DNurbsCurve& other );

  ///
  virtual void applyTransform ();

  /// Do a given action with this object.
  virtual void doAction ( ActionData& );

  /// Return a list with the names and types of the objects parameter.
  virtual const Field* getFieldList () const;

  ///
  virtual uint getParameter ( Param *list );

  ///
  virtual G3DObject* getSelectionHandle_ ();

  ///
  void modifyPoint ( MeFloat u, MeFloat v, MeFloat dx, MeFloat dy, MeFloat dz);

  /// Utility function to draw the point C(u) on the curve.
  /**utility function to draw the point C(u) on the curve.
     \param {u} the parametric value
     \param {psize} the size of the control points
     \param {colorP} the color of the control points
  */
  void point ( MeFloat &u, MeFloat &v, int pSize,
               const G3DColor& colorP, int ) const;
  
  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();

  ///generate the control point information
  //void resetCPoints();

  ///
  void resetKnots ();
  
  ///
  void resetPolygon () {}

  /// Select details of an object (picking ?).
  virtual MeFloat selectDetails ( const G3DCamera &cam, const G3DRay &ray );

  ///
  virtual void setParameter ( const Param* );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;
  
  ///
  virtual void shadeDetails () const;

  ///Copies curve a into this curve.
  //G3DNurbsCurve& operator= ( const G3DNurbsCurve& a );

  /// Replace the nurbs data structure of this with a.
  G3DNurbsCurve& operator = ( const PlNurbsCurvef& rhs );
  
  /// Initialise the GLUnurbsObj.
  static int initClass ();
  
protected:
  int selectionIndex_;

  static GLUnurbsObj *NurbsRenderer_;
  static float        tesselTolerance_;
  static Field        FieldList_[4];

private:
  static bool _classInitialized;
};

#endif
