/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbssor.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include "s_nurbscurve.h"
#include "s_nurbssor.h"
#include "s_point.h"



/*
void
G3DRotationSpolygon::shade () const
{
  int i, j;

  //TODO glColor() ;
  for( i = 0; i < surface.CtrlP.rows; ++i )
    {
      glBegin( GL_LINE_STRIP );
      for( j = 0; j < surface.CtrlP.cols; ++j )
	glVertex4fv( surface.CtrlP(i, j).data ) ;
      glEnd() ;
    }
  for( j = 0; j < surface.CtrlP.cols; ++j )
    {
      glBegin( GL_LINE_STRIP );
      for( i = 0; i < surface.CtrlP.rows; ++i )
	glVertex4fv( surface.CtrlP(i, j).data );
      glEnd() ;
    }    
}
*/

//--------------------------------------------------------------------

IMPLEMENT_DYN(G3DRotationSurface, "Rotation_surface")


G3DRotationSurface::G3DRotationSurface( const PlNurbsCurvef &curve,
					G3DPoint3d &pos, G3DVector3d &axis,
					MeFloat angle ) :
  G3DNurbsSurface(1), curve_(curve), position_(pos),axis_(axis),angle_(angle)
{
  type_ = T_NURBS_SOR;

  Point3Df p(position_[0], position_[1], position_[2]);
  Point3Df a(axis_[0], axis_[1], axis_[2]);
  makeFromRevolution( curve_, p, a, angle_ );

  resetBoundingBox();
}

void
G3DRotationSurface::adjust ()
{
  Point3Df p(position_[0], position_[1], position_[2]);
  Point3Df a(axis_[0], axis_[1], axis_[2]);
  makeFromRevolution( curve_, p, a, angle_ );

  G3DNurbs::adjust();
}

Field G3DRotationSurface::SorFieldList_[7] =
{ {"degree", s_number}, {"knot_vec", m_flt}, {"knot_num", s_number},
  {"cpt_vec", m_vctrlPt},
  {"position", s_point3}, {"axes", s_vector3}, {"angle", s_flt}
};

const Field*
G3DRotationSurface::getFieldList () const
{
  return( SorFieldList_ );
}

uint
G3DRotationSurface::getParameter( Param *list )
{
  if( list )
    {
      list->number  = curve_.degree();
      list++;
      list->fltList = curve_.knot().memory();
      list++;
      list->number  = curve_.knot().n();
      list++;
      list->vCtrlPt = &(curve_.ctrlPnts());
      list++;
      list->point3  = &position_;
      list++;
      list->vector3 = &axis_;
      list++;
      list->flt     = angle_;
    }
  return( 7 );
}

/* TODO do we need this?
void
G3DRotationSurface::modifyPoint( MeFloat u, MeFloat v, MeFloat dx,
			      MeFloat dy, MeFloat dz )
{
  Point3D delta( dx, dy, dz );
  if( u < KnotU[0] )
    u = KnotU[0];
  if( u > KnotU[KnotU.n-1] )
    u = KnotU[KnotU.n-1];
  if( v < KnotV[0] )
    v = KnotV[0];
  if( v > KnotV[KnotV.n-1] )
    v = KnotV[KnotV.n-1];
  movePoint( u, v, delta );
}
*/

void
G3DRotationSurface::setParameter( const Param *plist )
{
  if( plist )
    {
      Vector<MeFloat> v( plist[1].fltList, plist[2].number );
      delete[] plist[1].fltList;
      PlNurbsCurvef tc( *(plist[3].vCtrlPt), v, plist[0].number );
      delete plist[3].vCtrlPt;
      curve_ = tc;

      position_ = *(plist[4].point3);
      axis_     = *(plist[5].vector3);
      angle_    = plist[6].flt;
      adjust();
    }
}

/*
void
G3DNurbsSurface::shadeDetails () const
{
  glPushName( selName_cpt );
  glPushName( 0 );
  glPointSize( 6 );
  glColor3f(0.2, 0.2, 0.9);

  for( int i = 0; i < P.rows; ++i )
    {
      glLoadName( i );
      glPushName( 0 );
      for( int j = 0; j < P.cols; ++j )
	{
	  glLoadName( j );
	  glBegin( GL_POINTS );
	  glVertex4fv( P(i,j).data );
	  glEnd();
	}
      glPopName();
    }
  glPopName(); glPopName();
  //set color knots.shade( 0 );
  ((G3DNurbsSpolygon*)polygon)->shade( );
}
*/

//=================================END=================================
