/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbssor.h,v 1.4 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_ROTATION_SURFACE_H_
#define __S_ROTATION_SURFACE_H_

#include <nurbs++/nurbs.h>
#include <nurbs++/curve.h>
#include <GL/glu.h>
#include "s_nurbssurf.h"

using PLib::Point3Df;

/**This class offers an interface to help design OpenGL 
   applications and use this NURBS library at the same time.
   Created by: Philippe Lavoie (23 September 1997)
   This Object is not selectable.
*/
/*
class G3DNurbsSpolygon : public G3DObject
{
public:  
  G3DNurbsSpolygon( PlNurbsSurfacef& s ) : surface(s)
    {
      //objectColor = cPolygonColorDefault;
    };

  void shade () const;

protected:
  PlNurbsSurfacef& surface;
};
*/

/// Class representing a NURBS surface of revolution.
/** This class offers an interface to help design OpenGL 
    applications and use this NURBS library at the same time (?)
*/
/*
  Created by: Philippe Lavoie (23 September 1997)
*/
class G3DRotationSurface : public G3DNurbsSurface
{
  DECLARE_DYN

public:
  /// Constructor.
  G3DRotationSurface ( MeFloat s ) : G3DNurbsSurface(s), curve_() {}

  /// Constructor.
  G3DRotationSurface ( const PlNurbsCurvef&, G3DPoint3d &pos, 
		       G3DVector3d &axis, MeFloat angle );

  /// Copy constructor.
  G3DRotationSurface ( const G3DRotationSurface &nS ) :
    G3DNurbsSurface(nS), curve_(nS.curve_), position_(nS.position_),
    axis_(nS.axis_), angle_(nS.angle_) {}
  
  ///
  virtual void adjust ();

  ///
  // virtual void   doAction( ActionData& );

  /// Return the used curve (?)
  /** NOTE: Does not return the curve object taht was rotated itself.
   */
  PlNurbsCurvef& getCurve () { return curve_; }
  
  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;

  ///
  virtual uint getParameter ( Param *list );

  ///
  virtual void setParameter ( const Param *list );
  
  // TODO virtual?
  //void modifyPoint(MeFloat u, MeFloat v, MeFloat dx, MeFloat dy, MeFloat dz);

  ///
  //virtual void shadeDetails () const;
  
private:
  PlNurbsCurvef curve_;
  G3DPoint3d    position_;
  G3DVector3d   axis_;    //TODO
  MeFloat       angle_;

  static Field  SorFieldList_[7];
  //static GLUnurbsObj *NurbsRenderer_;
  //static float       tesselTolerance_;
};

#endif
