/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbssurf.cpp,v 1.4 2000/02/22 21:33:24 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
//#include <qobject.h> // only for ASSERT ()
//#include "s_nurbscurve.h"
#include "s_nurbssurf.h"
#include "s_point.h"
#include "pointstuff.h"

GLUnurbsObj *G3DNurbsSurface::NurbsRenderer_    = NULL;
float        G3DNurbsSurface::tesselTolerance_  = 0.1;
bool         G3DNurbsSurface::_classInitialized = false;

/*TODO
void
splitAtActiveInU( G3DNurbsCurve &t, G3DNurbsCurve &ta, float u,
		  PlHNurbsSurface *active, int lod );
void
splitAtActiveInV( G3DNurbsCurve &t, G3DNurbsCurve &ta, float v,
		  PlHNurbsSurface *active, int lod );
		  */

//--------------------------------------------------------------------

void
G3DNurbsSpolygon::shade () const
{
  int i, j;

  //TODO glColor() ;
  for (i = 0; i < surface.ctrlPnts().rows(); ++i)
    {
      glBegin( GL_LINE_STRIP );
      for (j = 0; j < surface.ctrlPnts().cols(); ++j)
	glVertex3xv (surface.ctrlPnts()(i, j).data) ;
      glEnd () ;
    }
  for (j = 0; j < surface.ctrlPnts().cols(); ++j)
    {
      glBegin (GL_LINE_STRIP);
      for (i = 0; i < surface.ctrlPnts().rows(); ++i)
	glVertex3xv (surface.ctrlPnts()(i, j).data);
      glEnd () ;
    }
}


//--------------------------------------------------------------------

IMPLEMENT_DYN(G3DNurbsSurface, "NURBS_surface")

G3DNurbsSurface::G3DNurbsSurface ( MeFloat ) 
  : PlNurbsSurfacef(), G3DNurbs()
{
  if (!_classInitialized) G3DNurbsSurface::initClass ();
  type_ = T_NURBS_SURF;
  polygon = new G3DNurbsSpolygon (*this);
}

G3DNurbsSurface::G3DNurbsSurface ( const PlNurbsSurfacef& nS ) 
  : PlNurbsSurfacef (nS), G3DNurbs ()
{
  if (!_classInitialized) G3DNurbsSurface::initClass ();
  type_ = T_NURBS_SURF;
  polygon = new G3DNurbsSpolygon (*this);
  resetBoundingBox ();
}

G3DNurbsSurface::G3DNurbsSurface ( int DegU, int DegV, 
                                   const Vector_FLOAT& Uk, 
                                   const Vector_FLOAT& Vk, 
                                   const Matrix_HPoint3Df& Cp ) 
  : PlNurbsSurfacef (DegU,DegV,Uk,Vk,Cp), G3DNurbs ()
{
  if (!_classInitialized) G3DNurbsSurface::initClass ();
  type_ = T_NURBS_SURF;
  polygon = new G3DNurbsSpolygon (*this);
  resetBoundingBox ();
}
 
G3DNurbsSurface::G3DNurbsSurface ( const G3DNurbsSurface& other ) 
  : PlNurbsSurfacef ((PlNurbsSurfacef)other), G3DNurbs (other)
{
  // ASSERT (_classInitialized);
  polygon = new G3DNurbsSpolygon (*this);
  resetBoundingBox ();
}


void
G3DNurbsSurface::applyTransform ()
{
  cout << "G3DNurbsSurface::applyTransform ()" << " not implemented." << endl;
  
  /*TODO
    rx *= M_PI/180.0 ;
    ry *= M_PI/180.0 ;
    rz *= M_PI/180.0 ;
    
    MatrixRT<float> T(rx,ry,rz,tx,ty,tz) ;
    MatrixRT<float> Sx ; 
    Sx.scale(sx,sy,sz) ;
    
    transform(T*Sx) ;
    
    tx = ty = tz = rx = ry = rz = 0 ;
    sx = sy = sz = 1 ;
    resetBoundingBox() ;
  */
}

void
G3DNurbsSurface::doAction ( ActionData &action )
{
  switch (action.type)
    {
    case A_SELECT :
      resetKnots ();
    }
  
  G3DNurbs::doAction (action);
}


Field G3DNurbsSurface::FieldList_[7] =
{ {"u_degree", s_number}, {"v_degree", s_number},
  {"knot_vec_u", m_flt}, {"knot_num_u", s_number},
  {"knot_vec_v", m_flt}, {"knot_num_v", s_number},
  {"ctrl_points", m_mctrlPt}
};

const Field*
G3DNurbsSurface::getFieldList () const
{
  return FieldList_;
}

uint
G3DNurbsSurface::getParameter ( Param *list )
{
  if (list)
    {
      list->number  = degU;
      list++;
      list->number  = degV;
      list++;
      list->fltList = U.memory();
      list++;
      list->number  = U.size();
      list++;
      list->fltList = V.memory();
      list++;
      list->number  = V.size();
      list++;
      list->mCtrlPt = &P;
    }
  return 7;
}

G3DObject*
G3DNurbsSurface::getSelectionHandle_ ()
{
  return (new G3DCPoint(P(selrow_, selcol_), this));
}

int
G3DNurbsSurface::initClass ()
{
  tesselTolerance_ = 0.1;
  NurbsRenderer_ = gluNewNurbsRenderer ();
  if (NurbsRenderer_)
    {
      gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_SAMPLING_TOLERANCE, 25.0 );
      _classInitialized = true;
      return 1;
    }
  else
    return 0;
}

void
G3DNurbsSurface::modifyPoint( MeFloat u, MeFloat v, MeFloat dx,
			      MeFloat dy, MeFloat dz )
{
  Point3Df delta (dx, dy, dz);
  if (u < knotU()[0])
    u = knotU()[0];
  if (u > knotU()[knotU().n()-1])
    u = knotU()[knotU().n()-1];
  if (v < knotV()[0])
    v = knotV()[0];
  if (v > knotV()[knotV().n()-1])
    v = knotV()[knotV().n()-1];
  movePoint (u, v, delta);
}


void
G3DNurbsSurface::point( MeFloat &u, MeFloat &v, int pSize,
			const G3DColor& colorP, int ) const
{
  if (u < U[0])
    u = U[0];
  if (u > U[U.n()-1])
    u = U[U.n()-1];
  if (v < V[0])
    v = V[0];
  if (v > V[V.n()-1])
    v = V[V.n()-1];

  glPointSize (pSize);
  float col[4];
  col[0] = col[1] = col[2] = 0;
  col[3] = 1;
  glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, col);
  //TODO
  //glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION, (const float*)colorP);
  
  glBegin (GL_POINTS);
  glVertex4fv (pointAt(u, v).data);
  glEnd ();

  /* TODO why?
     PlNurbsCurveGL t;
     t.setObjectColor(colorP,colorP,colorP) ;
     //t.setNurbsRenderer(nurbsRenderer) ;
     t.hideBBox() ;
     t.hideCPoints() ;
     t.hideCpolygon() ;
     t.viewNurbs() ;
     isoCurveU(u,t) ;
     t.glObject() ;
     isoCurveV(v,t) ;
     t.glObject() ;
  */
}

void
G3DNurbsSurface::resetBoundingBox ()
{
  G3DPoint3d min, max;

  min[0] = extremum (1, coordX);
  min[1] = extremum (1, coordY);
  min[2] = extremum (1, coordZ);
  max[0] = extremum (0, coordX);
  max[1] = extremum (0, coordY);
  max[2] = extremum (0, coordZ);

  bbox_.setTo (min, max);
  if (parent_)
    ((G3DShape*)parent_)->resetBoundingBox ();
}

/*
void
G3DNurbsSurface::resetCPoints()
{
  G3DCPoint *tempCP ;

  cpoints.reset() ;
  cpoints.setJumpSize(P.cols) ;

  for( int i = 0; i < P.rows; ++i )
    for( int j = 0; j < P.cols; ++j )
      {
	tempCP = new G3DCPoint(P(i,j)) ;
	tempCP->setObjectColor(cpointColorDefault) ;
	tempCP->setCurrentColor(cpointActiveColorDefault) ;
	if( (j == 0) && (i != P.rows-1) )
	  tempCP->setObjectColor(cpoint0ColorDefault) ;
	cpoints.add(tempCP) ;
      }
}
*/

void
G3DNurbsSurface::resetKnots ()
{
  G3DKnot *tempCP ;

  knots_.reInitialize ();
  //knots.setJumpSize(P.cols()) ;

  PlVector_int sU, sV;
  sU.resize (U.n());
  sV.resize (V.n());

  int i,j ;
  
  sU[0] = 0 ;
  for (i = 0; i < U.n()-1; ++i)
    {
      if (U[i+1] >= U[i])
	{
	  sU[i+1] = sU[i]+1 ;
	  sU[i] = 0 ;
	}
      else
	sU[i+1] = 1 ;
    }
  
  sV[0] = 0 ;
  for (i=0;i<V.n()-1;++i)
    {
      if (V[i+1]>=V[i])
	{
	  sV[i+1] = sV[i]+1 ;
	  sV[i] = 0 ;
	}
      else
	sV[i+1] = 1 ;
    }
  
  for (i = 0; i < U.n(); ++i)
    for (j = 0; j < V.n(); ++j)
      {
	//if(sU[i] && sV[i]){
	Point3Df p = pointAt(U[i],V[j]);
	// we must represent multiple knots by only 1 point
	// but with a bigger radius (increase radius by 1)
	tempCP = new G3DKnot (p, i, j);
	//TODO tempCP->setObjectColor( knotColorDefault );
	//TODO tempCP->setCurrentColor( knotActiveColorDefault );
	//if( j == 0 && (i != P.rows-1) )
	//tempCP->setObjectColor( cpoint0ColorDefault );
	tempCP->setPsize (3+maximum(sU[i],sV[j]));
	knots_.push_back (tempCP );
      }
}

MeFloat
G3DNurbsSurface::selectDetails ( const G3DCamera &cam, const G3DRay &ray )
{
  MeFloat   rad = 8;
  G3DMatrix mw;
  getWorldTrafo( mw );
  
  selrow_ = selcol_ = -1;
  if (findNearestCPointM (cam, ray, mw, P, rad, selrow_, selcol_))
    {
      cout << "surf " << rad << endl;
      return rad;
    }
  else
    return G3D_INFINITY;
}

void
G3DNurbsSurface::setParameter( const Param *plist )
{
  if (plist)
    {
      degU = plist->number;           ++plist;
      degV = plist->number;           ++plist;
      
      float *knots = plist->fltList;  ++plist;
      Vector_FLOAT u( knots, plist->number );
      ++plist;
      U  = u;
      delete[] knots;
      
      knots = plist->fltList;  ++plist;
      Vector_FLOAT v (knots, plist->number);
      ++plist;
      V  = v;
      delete[] knots;

      P   = *plist->mCtrlPt;
      delete plist->mCtrlPt;

      resetBoundingBox ();
    }
}

void
G3DNurbsSurface::shade ( int num ) const
{
  if (NurbsRenderer_)
    {
      //TODO get Resolution attribute and set the sampling tolerance
      if (num < 0)
	gluNurbsProperty (NurbsRenderer_, (GLenum)GLU_AUTO_LOAD_MATRIX,
			  GL_FALSE);
      switch (G3DNurbsDisplayMode)
	{
	case 1: //PLNURBS_DISPLAY_ISOCURVES :
	  {
	    // create iso curves
	    /*
	    G3DNurbsCurve t;
	    //t.setObjectColor(objectColor,selectColor,currentColor) ;
	    //t.setNurbsRenderer(nurbsRenderer) ;
	    //t.hideBBox() ;
	    //t.hideCPoints() ;
	    //t.hideCpolygon() ;
	    //t.viewNurbs() ;
	    //if(isActive()) t.activate() ;
	    //if(isSelected()) t.select() ;
	    
	    PlVector_FLOAT ub ;
	    averagingKnots( U, degU, ub );

	    for( int i = 0; i < ub.n(); ++i )
	      {
		isoCurveU( ub[i], t ) ;
		t.shade() ;
	      }
	    averagingKnots(V,degV,ub) ;

	    for( int j = 0; j < ub.n(); ++j )
	      {
		isoCurveV( ub[j], t ) ;
		t.shade() ;
	      }
	      */
	    break;
	  }
	case 2: //PLNURBS_DISPLAY_SHADED :  //TODO generate texture coords
	  {
	    glEnable( GL_AUTO_NORMAL );
  	    glFrontFace( GL_CCW );
  	    glDisable( GL_CULL_FACE );
	    GLenum t = GL_MAP2_VERTEX_4;
	    gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_DISPLAY_MODE,
			      (GLenum)GLU_FILL );
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n(), U.memory(), V.n(), V.memory(),
			     4, 4*P.rows(), P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
            glEnable( GL_CULL_FACE );
  	    glFrontFace( GL_CW );
	    glDisable( GL_AUTO_NORMAL );
	    break;
	  }
	case 3: //PLNURBS_DISPLAY_HIDDEN_LINES:
	  {
	    /*
	    GLenum t = (GLenum)GL_MAP2_VERTEX_4; 
	    gluNurbsProperty( NurbsRenderer_, GLU_DISPLAY_MODE, GLU_FILL );
	  
	    glDisable( GL_LIGHTING );
	    glColor3f(.7f, .7f, .7f);
	  
	    glEnable(GL_STENCIL_TEST);
	    glStencilFunc(GL_ALWAYS, 0, 0); // clear stencil for this object
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n(), U.memory(), V.n(), V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	    glEnable( GL_LIGHTING );

	    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); // turn off color
	    glDisable( GL_DEPTH_TEST );
	    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	    glStencilFunc(GL_ALWAYS, 1, 1);
	    glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
	  
	    gluBeginSurface( NurbsRenderer_ );
	    //glColor() ;
	    gluNurbsSurface( NurbsRenderer_, U.n(), U.memory(), V.n(), V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	  
	    glStencilFunc(GL_EQUAL, 1 , 1);
	    glEnable(GL_DEPTH_TEST);
	    glDepthFunc(GL_LEQUAL);
	    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	  
	    gluBeginSurface( NurbsRenderer_ );
	    //TODO glColor() ;
	    gluNurbsSurface( NurbsRenderer_, U.n(), U.memory(), V.n(), V.memory(),
			     4, 4*P.rows, P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );

	    // clean up state
	    glDisable( GL_STENCIL_TEST );
	    glDepthFunc( GL_LESS );
	    */
	    break;
	  }
	case 4:  //PLNURBS_DISPLAY_TESSELATION:
	  {
	    BasicList<Point3Df> pts ;
	    BasicList<int> connect ;
	    if( tesselTolerance_ < 0.001) 
	      tesselTolerance_ = 0.001 ;
	    tesselate( tesselTolerance_, pts,connect );
	    
	    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	    glBegin(GL_TRIANGLES) ;
	    //TODO glColor() ;
	    connect.goToFirst() ;
	    pts.goToFirst() ;
	    for( int i = 0; i < connect.size(); ++i )
	      {
		if( *(connect[i]->data) >= 0)
		  glVertex3fv(pts[*(connect[i]->data)]->data->data );
	      }
	    glEnd() ;
	    break;
	  }
	  //case PLNURBS_DISPLAY_NORMAL:
	default:
	  {
	    GLenum t = (GLenum)GL_MAP2_VERTEX_4; 
	    gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_DISPLAY_MODE,
			      (GLenum)GLU_OUTLINE_POLYGON );
	    gluBeginSurface( NurbsRenderer_ );
	    gluNurbsSurface( NurbsRenderer_, U.n(), U.memory(), V.n(), V.memory(),
			     4, 4*P.rows(), P[0]->data, degU+1, degV+1, t );
	    gluEndSurface( NurbsRenderer_ );
	  }
	}
      if( num < 0 )
	gluNurbsProperty( NurbsRenderer_, (GLenum)GLU_AUTO_LOAD_MATRIX,
			  GL_TRUE );
    }
}

void
G3DNurbsSurface::shadeDetails () const
{
  glPointSize (4);
  //glColor3f(0.2, 0.2, 0.9);
  glColor3f (1., 0., 0.);
  glBegin (GL_POINTS);
  for (int i = 0; i < P.rows(); ++i)
    {
      for (int j = 0; j < P.cols(); ++j)
	glVertex3xv (P(i,j).data);
    }
  glEnd ();
  //set color knots.shade (0);
  ((G3DNurbsSpolygon*)polygon)->shade ();
}


//---------------------------------------------------------------------

/*
void
splitAtActiveInU( G3DNurbsCurve &t, G3DNurbsCurve &ta, float u,
		  PlHNurbsSurface *active, int lod )
{
  if( lod >= 0 && lod < active->level )
    return ;
  if( u < active->uS || u >active->uE )
    {
      t.shade( 0 );
      return ;
    }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->vS<=c.Knot[0] && active->vE>=c.Knot[c.Knot.n()-1]){
    ta.shade( 0 );
    return ;
  }
  if(active->vS>c.Knot[0] && active->vE < c.Knot[c.Knot.n()-1]){
    // active curve is in the middle
    c.splitAt(active->vS,t,ta) ;
    t.shade( 0 );
    c = ta ;
    c.splitAt(active->vE,ta,t) ;
    t.shade( 0 );
    ta.shade( 0 );
  }
  else{
    if(active->vS>c.Knot[0]) {
      c.splitAt(active->vS,t,ta) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
    else{
      c.splitAt(active->vE,ta,t) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
  }
}


void
splitAtActiveInV( G3DNurbsCurve &t, G3DNurbsCurve &ta, float v,
		  PlHNurbsSurface *active, int lod )
{
  if(lod >= 0 && lod < active->level)
    return ;
  if(v<active->vS || v >active->vE)
    {
      t.shade( 0 );
      return ;
    }
  PlNurbsCurve c ;
  c = t ;
  ta = c ;
  if(active->uS<=c.Knot[0] && active->uE>=c.Knot[c.Knot.n()-1])
    {
      ta.shade( 0 );
      return ;
    }
  if(active->uS>c.Knot[0] && active->uE < c.Knot[c.Knot.n()-1])
    {
      // active curve is in the middle
      c.splitAt(active->uS,t,ta) ;
      t.shade( 0 );
      c = ta ;
      c.splitAt(active->uE,ta,t) ;
      t.shade( 0 );
      ta.shade( 0 );
    }
  else
    {
      if(active->uS>c.Knot[0])
	{
	  c.splitAt(active->uS,t,ta) ;
	  t.shade( 0 );
	  ta.shade( 0 );
	}
      else
	{
	  c.splitAt(active->uE,ta,t) ;
	  t.shade( 0 );
	  ta.shade( 0 );
	} 
    }
}
*/

//=================================END=================================
