/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 2001. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_nurbssurf.h,v 1.4 2002/06/10 09:30:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef __S_NURBS_SURFACE_H_
#define __S_NURBS_SURFACE_H_

#include <nurbs++/nurbsS.h>
#include <nurbs++/hnurbsS.h>
#include <GL/glu.h>
#include "s_nurbs.h"

/// Base class for NURBS surfaces sharing a basic shade method.
/** This class offers an interface to help design OpenGL 
    applications and use this NURBS library at the same time.
    Created by: Philippe Lavoie (23 September 1997)
    This Object is not selectable.
*/
class G3DNurbsSpolygon : public G3DObject
{
public:  
  /// Constructor.
  G3DNurbsSpolygon ( PlNurbsSurfacef& s ) : surface(s) {
    //objectColor = cPolygonColorDefault;
  }
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
      This method is implemented to share common code of subclasses.
  */
  void shade () const;
  
protected:
  PlNurbsSurfacef& surface;
};

///
/** This class offers an interface to help design OpenGL 
    applications and use this NURBS library at the same time.
*/
/* Created by: Philippe Lavoie (23 September 1997) */
class G3DNurbsSurface : public PlNurbsSurfacef, public G3DNurbs
{
  DECLARE_DYN

public:
  /// Constructor.
  G3DNurbsSurface ( MeFloat );
  
  /// Constructor taking a PlNurbsSurface as argument.
  G3DNurbsSurface (const PlNurbsSurfacef& nS);
  
  /// Constructor.
  G3DNurbsSurface ( int DegU, int DegV, const Vector_FLOAT& Uk,
                    const Vector_FLOAT& Vk, const Matrix_HPoint3Df& Cp );

  /// Copy constructor.
  G3DNurbsSurface ( const G3DNurbsSurface& nS );

  /// Apply the object's transform (?)
  void applyTransform ();
  
  /// Do a given action with the object.
  virtual void doAction ( ActionData& );

  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
  
  ///
  virtual uint getParameter ( Param *list );

  ///
  virtual G3DObject* getSelectionHandle_ ();
  
  ///
  void modifyPoint (MeFloat u, MeFloat v, MeFloat dx, MeFloat dy, MeFloat dz);

  /// Draw a point on the Surface at S(u,v).
  /** Draw a point on the Surface at S(u,v).
      Input:  u, v   --> the parametric values
      psize  --> the size of the control points
      colorP --> the color of the control points
  */
  void point ( MeFloat &u, MeFloat &v, int pSize, const G3DColor&, int ) const;
  
  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();

  // void resetCPoints ();

  ///
  void resetPolygon () {}

  ///
  void resetKnots ();

  ///
  virtual MeFloat selectDetails ( const G3DCamera &cam, const G3DRay &ray );

  ///
  virtual void setParameter ( const Param* );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;
  
  ///
  virtual void shadeDetails () const;
  
  /// Initialise the GLUnurbsObj.
  static int initClass ();

private:
  int  selrow_, selcol_;

  static GLUnurbsObj *NurbsRenderer_;
  static float       tesselTolerance_;
  static Field       FieldList_[7];

  static bool _classInitialized;
};

#endif
