/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_point.cpp,v 1.3 2002/06/11 09:56:46 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include "GL/gl.h"
#include "GL/glu.h"

#include "s_point.h"


HPoint3Df dummyCPoint ;
int        pSizeDefault = 4; //TODO class variables?



G3DObject*
G3DKnot::copy () const
{
  return 0;
}

G3DKnot::G3DKnot( const Point3Df& p3d, int kU, int kV ) :
  p(p3d), ku(kU), kv(kV)
{
  type_ = THA_KNOT;
  psize = pSizeDefault;
}


void
G3DKnot::shade () const
{
  glPointSize( psize ) ;
  glBegin(GL_POINTS) ;
  //glColor();
  glVertex3fv(p.data) ;
  glEnd();
}


//------------------------------------------------------------------


G3DObject*
G3DCPoint::copy () const
{
  return 0;
}

void
G3DCPoint::doAction ( ActionData &action )
{
  if( action.type == A_DESELECT ) {
    // delete this;
    dispose ();
  }
  else
    G3DObject::doAction ( action );
}

const G3DPoint3d&
G3DCPoint::getOrigin () const
{
  G3DPoint3d tmp( cpoint_.data );
  point_ = tmp;
//.setTo( cpoint_[0], cpoint_[1], cpoint_[2] );
  return point_;
}

void
G3DCPoint::setOrigin( const TVector<MeFloat,3> &tv )
{
  HPoint3Df tmp( tv[0], tv[1], tv[2], cpoint_.data[3] );
  cpoint_ = tmp;
}



//---------------------------------------------------------------------


G3DVertex::~G3DVertex()
{
  //TODO if( constraint_ )
// constraint_->dispose();
}

void
G3DVertex::doAction( ActionData &action )
{
  if (action.type == A_DESELECT)
    dispose ();
  else
    G3DObject::doAction (action);
}

const G3DPoint3d&
G3DVertex::getOrigin () const
{
  return point_;
}
/*
void
G3DVertex::getPosition( G3DPoint3d &pos ) const
{
  pos = point_;
}
*/

void
G3DVertex::setConstraint( const G3DConstraint* )
{
//c->addReference();
}

void
G3DVertex::setOrigin( const TVector<MeFloat,3> &tv )
{
  //TODO apply the constraint
  point_ = tv;
}


//------------------------------------------------------------------

G3DNormal::G3DNormal( const G3DPoint3d &base, G3DVector3d &nrm,
		      G3DObject *parent ) :
  G3DObject(parent), base_(base), normal_(nrm)
{
  type_ = THA_NORMAL;
};

G3DObject*
G3DNormal::copy () const
{
  return 0;
}

void
G3DNormal::doAction( ActionData &action )
{
  if ( action.type == A_DESELECT ) {
    // delete this;
    dispose ();
  }
  else
    G3DObject::doAction (action);
}

/*
void
G3DNormal::getPosition( G3DPoint3d &pos ) const
{
  pos = base_ + normal_;
}
*/

void
G3DNormal::translate( const TVector<MeFloat,3> &tv )
{
  normal_ += tv;
}


//------------------------------------------------------------------


/*
G3DPoint::~G3DPoint()
{
}


void 
G3DPoint::shade ( int ) const
{
  //glPointSize( psize );
  glBegin( GL_POINTS );
  //glColor();
  glVertex3fv( p.data );
  glEnd() ;
}
*/


//====================================END===============================
