/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: s_point.h,v 1.6 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _POINT_H_
#define _POINT_H_

#include <nurbs++/specialType.h>
#include "object.h"

using namespace PLib;

extern int pSizeDefault;


/// A wrapper object used to control a 4d control Point of a NURBS object.
/** A wrapper object used to control a 4d control Point of a NURBS object.
    Object's of this class can't be created dynamically.
*/
class G3DCPoint : public G3DObject
{
public:
  /// Constructor.
  G3DCPoint ( HPoint3Df &cp, G3DObject *parent ) :
    G3DObject(parent), cpoint_(cp) {
    type_ = THA_CPOINT;
  }

  /// Copy (?)
  virtual G3DObject* copy () const;
  
  /// Do a given action with this object.
  virtual void doAction ( ActionData& );
  
  /// Return the point's origin (?)
  virtual const G3DPoint3d& getOrigin () const;
  
  /// Set the point's origin.
  virtual void setOrigin ( const TVector<MeFloat,3>& );

protected:
  HPoint3Df  &cpoint_ ;
  mutable G3DPoint3d point_;
};


/* TODO
class PlHCPointGL : public G3DCPoint
{
public:
  PlHCPointGL(HPoint3Df& off, int i, int j, PlHNurbsSurfaceGL* hs, int mod): offset(off),s(hs) { type = hcpointObject; psize=pSizeDefault ; i0 = i ; j0 = j ; canModify = mod ;} 
  ~PlHCPointGL() { ; } 

  virtual void glObject() const;

  virtual void modify(const HPoint3Df& v) ;

protected:
  HPoint3Df& offset ;
  PlHNurbsSurfaceGL *s ;
  int i0,j0 ;
  int canModify ;
};
*/


/**This class offers an interface to help design OpenGL 
  applications and use this NURBS library at the same time.
  Created by: Philippe Lavoie (23 September 1997)
*/
/*
class G3DPoint : public G3DObject
{
public:  
  G3DPoint( const Point3D &p3d ) : p(p3d)
    {
      type_ = T_POINT ; psize = pSizeDefault;
    };
  G3DPoint( const G3DPoint &orig ) : G3DObject(orig), p(orig.p)
    {
      psize = orig.psize;
    };
  virtual ~G3DPoint();

    ///
  virtual G3DObject* copy () const { return( new G3DPoint(p) ); };
    ///
  //void  modify( const Point3D &v ) { p += v; };
    ///
  const Point3D& point() const { return p; };
    ///
  void  set( const Point3D &v ) { p = v; };
    ///
  void  setPsize( int s ) { psize = s; };
    ///
  virtual void shade( int ) const;

  MeFloat x() const { return p.x ; }
  MeFloat y() const { return p.y ; }
  MeFloat z() const { return p.z ; }

protected:
  Point3D p;
  int     psize;
};
*/

/// Points on NURBS curves or surfaces corresponding to knot vectors.
/** Objects of this class represent the Points on NURBS curves/surfaces
    corresponding to the values of the knot vector(s).
*/
class G3DKnot : public G3DObject
{
public:
  /// Constructor.
  G3DKnot ( const Point3Df& p3d, int kU, int kV );

  ///
  virtual G3DObject* copy () const;

  /// Return the knot's corresponding u value (?)
  int knotU () { return ku; }

  /// Return the knot's corresponding v value (?)
  int knotV () { return kv; }

  ///
  void modify ( const Point3Df& v ) { p += v ; }

  /// Return the knot's point.
  const Point3Df& point () const { return p; }
  
  /// Set the point value to a given vector.
  void set ( const Point3Df& v ) { p = v ; }

  ///
  void setPsize ( int s ) { psize = s; }

  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  void shade () const;

  /// Return the knot's x coordinate (?)
  MeFloat x () const { return p.x(); }

  /// Return the knot's y coordinate (?) 
  MeFloat y () const { return p.y(); }

  /// Return the knot's z coordinate (?)
  MeFloat z () const { return p.z(); }

protected:
  Point3Df p;
  int ku, kv;
  int psize;
};

/// A wrapper object used to control a normal of a 3d object.
class G3DNormal : public G3DObject
{
public:
  /// Create a handle for normal, situated at the point base.
  G3DNormal ( const G3DPoint3d &base, G3DVector3d &normal, G3DObject *parent );

  ///
  virtual G3DObject* copy () const;

  /// Do a given action with this object.
  virtual void doAction ( ActionData& );
  
  ///
  virtual void translate ( const TVector<MeFloat,3>& );

protected:
  const G3DPoint3d  &base_;
  G3DVector3d       &normal_;

};

class G3DConstraint;

/// Wrapper object used to control a vertex of a 3d object.
class G3DVertex : public G3DObject
{
public:
  /// Constructor.
  G3DVertex( G3DPoint3d &pt, G3DObject *parent ) : G3DObject(parent),point_(pt)
    { type_ = THA_VERTEX; }

  /// Destructor. Called when the object is destroyed.
  ~G3DVertex ();
  
  /// Do a given action with this object.
  virtual void doAction ( ActionData& );

  /// Return origin of local coordinate system.
  virtual const G3DPoint3d& getOrigin () const;

  /// Return the constraint of the vertex (?)
  void setConstraint ( const G3DConstraint* );

  /// Set origin of local coordinate system.
  virtual void setOrigin ( const TVector<MeFloat,3>& );

protected:
  G3DPoint3d &point_;
};

#endif
