/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_polygon.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "s_polygon.h"
#include "s_point.h"
#include "pointstuff.h"


IMPLEMENT_DYN(G3DPolygon, "Polygon")

G3DObject*
cUserPolygon ()
{
  G3DPolygon *tmp = new G3DPolygon;
  tmp->setVertexNumber( 4 );
  tmp->setVertex( 0, G3DPoint3d(-0.5, 0,-0.5) );
  tmp->setVertex( 1, G3DPoint3d( 0.5, 0,-0.5) );
  tmp->setVertex( 2, G3DPoint3d( 0.5, 0, 0.5) );
  tmp->setVertex( 3, G3DPoint3d(-0.5, 0, 0.5) );
  tmp->setNormal( G3DVector3d(0, 1, 0) );
  return tmp;
}


G3DPolygon::G3DPolygon( MeFloat )
{
  type_      = T_POLYGON;
  _nVertices = 0;
  _vertices  = NULL;
}

G3DPolygon::G3DPolygon( const G3DPolygon &orig ) :
  G3DShape( orig )
{
  _nVertices = orig._nVertices;
  _vertices  = new G3DPoint3d[_nVertices];
  //TODO memcpy( _vertices, orig._vertices, sizeof(G3DPoint3d)*_nVertices );
  for( uint i = 0; i < _nVertices; i++ )
    {
      _vertices[i] = orig._vertices[i];
    }
  _normal = orig._normal;
  resetBoundingBox();
}


G3DPolygon::~G3DPolygon()
{
  //TODO what about the G3DPoint3d destructors?
  delete[] _vertices;
}


Field G3DPolygon::FieldList_[3] =
{ {"vertices", m_point3}, {"num_vert", s_number},
  {"normal", s_vector3}
};

const Field*
G3DPolygon::getFieldList () const
{
  return( FieldList_ );
}

uint
G3DPolygon::getParameter( Param *pList )
{
  pList->point3  = _vertices;
  pList++;
  pList->number  = _nVertices;
  pList++;
  pList->vector3 = &_normal;

  return( 3 );
}

G3DObject*
G3DPolygon::getSelectionHandle_ ()
{
  return new G3DVertex( _vertices[selectionIndex_], this );
}

void
G3DPolygon::resetBoundingBox ()
{
  if( _vertices ) //TODO
    {
      G3DPoint3d pmin(*_vertices), pmax(*_vertices);
      G3DPoint3d *vert = _vertices+1;

      for( uint i = _nVertices-1; i > 0; --i )
	{
	  pmin.min( *vert );
	  pmax.max( *(vert++) );
	}
      bbox_.setTo( pmin, pmax );
    }
  else
    bbox_.reset();  //TODO
}

MeFloat
G3DPolygon::selectDetails( const G3DCamera &cam, const G3DRay &ray )
{
  MeFloat   rad = 5;
  G3DMatrix mw;
  getWorldTrafo( mw );

  selectionIndex_ = findNearestPoint( cam, ray, mw, _vertices, _nVertices,
				      rad );
  if( selectionIndex_ >= 0 )
    {
      //TODO normal facing towards us?
      return( rad );
    }
  else
    return( G3D_INFINITY );
}

void
G3DPolygon::setNormal( const TVector<MeFloat,3> &normal )
{
  _normal = normal;
  _normal.normalize();
}


void
G3DPolygon::setParameter( const Param *plist )
{
  if( plist )
    {
//TODO first check if number changed
      if( _vertices != plist->point3 )
	{
	  delete[] _vertices;
	  _vertices = plist->point3;
	}
      ++plist;
      _nVertices = plist->number;

      resetBoundingBox();
    }
}

void
G3DPolygon::setVertex( uint num, const G3DPoint3d &vertex )
{
  //TODO include bbox calculation?
  if( num < _nVertices )
    {
      _vertices[num] = vertex;
      bbox_.addPoint( vertex );  //TODO reset bbox?
    }
}

void
G3DPolygon::setVertexNumber( uint num )
{
  delete[] _vertices;
  if( num > 0 )
    _vertices  = new G3DPoint3d[num];
  else
    _vertices  = NULL;
  _nVertices = num;
}



void
G3DPolygon::shade( int ) const
{
  glDisable(GL_CULL_FACE);  //TODO
  glBegin( GL_POLYGON );
  glNormal3xv( (const MeFloat*)_normal );
  for( uint i = 0; i < _nVertices; i++ )
    {
      glVertex3xv( (const MeFloat*)(_vertices[i]) );
    }
  glEnd();
  glEnable(GL_CULL_FACE);
}

//TODO shadeDetails
// shade to vertices

//===============================END===============================
