/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1999. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_polygon.h,v 1.2 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_POLYGON_H_
#define __S_POLYGON_H_

#include "shape.h"


class G3DA_BBox;

/// Class representing plane and concave polygons.
/** Instaces of this class represent plane and concave polygons.
    It is an OpenGL primitive.
    The vertices are given in clockwise order. That is, the interior
    of the polygon is always to your right side.
*/
class G3DPolygon : public G3DShape
{
  DECLARE_DYN

public:
  ///
  explicit G3DPolygon ( MeFloat s=1 );  //TODO int num );

  /// Copy constructor.
  G3DPolygon ( const G3DPolygon& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DPolygon ();

  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
  
  ///
  virtual uint getParameter ( Param* );

  /// 
  virtual G3DObject* getSelectionHandle_ ();
  
  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();
  
  /// 
  virtual MeFloat selectDetails ( const G3DCamera&, const G3DRay& );
  
  ///
  virtual void setParameter ( const Param* );
  
  /// Set the vertex with a specified index to a the given value. 
  void setVertex ( uint num, const G3DPoint3d &vertex );
  
  /// Set the polygon's normal.
  void setNormal( const TVector<MeFloat,3> &normal );
  
  /// Change the number of vertices. The old ld values are deleted.
  void setVertexNumber ( uint num );
  
  ///
  virtual void shade ( int ) const;

protected:
  G3DPoint3d  *_vertices;
  G3DVector3d _normal;         // TODO a normal for each vertex?
  uint        _nVertices;
  uint        selectionIndex_; // a selected Point

  static Field FieldList_[3];
};

extern G3DObject* cUserPolygon ();

#endif
