/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 2001. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_quadmesh.h,v 1.4 2001/07/15 20:56:26 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 * Declaration of mesh primitives
 */

#ifndef _S_MESH_H_
#define _S_MESH_H_

#include "shape.h"


//Mesh in Generic?

/*
class MeshFaceType
{
protected:
  MeshFaceType () { num = 0; vertices = NULL; };
  ~MeshFaceType() { delete vertices; };

  setTo( const MeshFaceType *orig ) {
    num = orig->num;
    vertices = new uint[num];
    uint *dst = vertices;
    const uint *src = orig.vertices;
    for( int i = num; i > 0; i-- )
      *dst++ = *src++;
  }

  uint num;
  uint *vertices;
};
*/

/// Generic mesh class (?)
/** A mesh is an object defined by a collection of vertices, faces, and
    contours. Meshes may be used to model polyhedra, grids, and other faceted
    objects. A mesh may have a boundary. The term contour is used here to refer
    to a polygonal hole contained in a single face of a mesh. A mesh face
    (or contour) is a list of vertices that defines a polygonal facet. A face
    (or contour) need not be planar, and a contour and its surrounding face need
    not be coplanar; however, rendering of a mesh having a nonplanar face or
    contour, or having a contour not coplanar with its surrounding face, may
    lead to unexpected results.
    
    The specification of a mesh includes an array of vertices and an array of
    faces and contours. The vertices of a mesh are indexed by array position;
    these indices are used to specify the faces and contours of that mesh. Faces
    and contours are also indexed by array position; this index does not
    distinguish between faces and contours. Both of these indices are used in the
    specification of child objects.
    
    Attributes may be attached separately and selectively to the vertices,
    faces, face edges, and corners of a mesh.
*/
class G3DQuadMesh : public G3DShape
{
  DECLARE_DYN

public:
  /// 
  explicit G3DQuadMesh( MeFloat );

  //G3DMesh( uint nvert , uint nfaces, uint nnormals );

  /// Constructor.
  G3DQuadMesh ( const G3DQuadMesh& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DQuadMesh ();
  
  ///
  void adjust ();

  /// This will make a hole in the surface.
  void deleteSelectedFace ();

  /// Do a given action with this object.
  virtual void doAction ( ActionData& );

  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
  
  /**Copy pointers to the parameter to the list. The order is:
     _rows, _cols, _vertices, _normals.*/
  virtual uint getParameter( Param* );

  ///
  virtual G3DObject* getSelectionHandle_ ();
  
  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();
  
  ///
  virtual MeFloat selectDetails ( const G3DCamera&, const G3DRay& );

  ///
  virtual void setParameter ( const Param *pList );

  ///
  void setVertex ( uint r, uint c, TVector<MeFloat,3> );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;
  
  ///
  virtual void shadeDetails () const;

  // void optimize (); This may be a time consuming task

protected:
  ///
  void calcNormals ();
  
  /// 
  void calcSmoothNormals ();

protected:
  G3DPoint3d  *vertices_;
  G3DVector3d *normals_;

  uint wrinkle_;
  enum  {stVertex=0, stFace} selectionType_;
  int   selectionIndex_;
  //enum  selectionType_;
  uint  *faces_; //*facesN_
  uint  nVertices_, nFaces_, nNormals_;
  bool  smooth_;
  //TODO bool closed_;  can only be set if mesh is derived from CSG operation

  static Field MeshFields_[8];
};


extern void uCreateQuadmesh ();

#endif
