/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_reference.h,v 1.2 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __REFERENCE_H_
#define __REFERENCE_H_


#include <stdio.h>
#include "g3d_math.h"
#include "attribute.h"
#include "t_fields.h"
#include "s_object.h"

class ActionData;



/// A reference to a scene element.
class G3DReference : public G3DSceneElement
{
public:
  /// Constructor taking the scene element to reference.
  G3DReference ( const G3DShape* );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DReference ();

  // virtual void adjust ();

  /// Deep copy (?)
  virtual G3DObject* copy () const;  //Dynamic_obj?
  
  /// Display the object on the screen.
  /** Display the referenced object on the screen. This will call the shade()
      function to issue the necessary OGL commands.
  */
  virtual void display ( int& ) const;
  
  /// Do a given action with the referenced object.
  virtual void doAction ( ActionData& );

  /// Get an Attribute by its id number.
  G3DAttribute* getAttribute ( int );

  /// Get an Attribute by its name.
  G3DAttribute* getAttribute ( const char* );
  
  /// Return the referenced object's bounding box.
  virtual const G3D_BBAligned& getBoundingBox () const;
  
  /// Return a list of available parameters (?)
  virtual const Field* getFieldList () const;
  
  /// Get a list of params describing the geometry of the object.
  virtual uint getParameter ( Param* );

  /// Get a Handle for the selected detail.
  virtual G3DObject* getSelectionHandle ();

  /// Return a Descriptive name of the shape.
  virtual const char* getTypeName ();
  
  ///
  //virtual MeFloat intersect ( G3DRay *ray, G3DObject** );
  
  /// Return the number of shape objects contained.
  virtual int numberOfShapes ();

  /// Remove and dispose the given attriubte.
  void removeAttribute ( G3DAttribute* );

  ///
  //virtual void resetBoundingBox ();
  
  ///
  void setAttribute ( G3DAttribute* );

  /// Selection using the OGL picking mechanism
  virtual void select ();

  /// Find detail, widget or tool closest to a given ray.
  virtual MeFloat select ( const G3DCamera&, const G3DRay&, G3DPartEnum );

  /// Find detail closest to a given ray.
  virtual MeFloat selectDetails (const G3DCamera&, const G3DRay& );

  ///
  virtual void shadeDetails () const; //TODO protected?

protected:
  G3DSceneElement *peer_;  //needed for aquiring the attributes of the parent
};

#endif
