/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_scene.cpp,v 1.3 2002/06/12 20:13:18 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"
#include "s_scene.h"
#include "Attributes/texture.h"

const float cb = 0.2;


IMPLEMENT_DYN(G3DScene, "Scene")


G3DScene::G3DScene( MeFloat s ) :
  G3DGroup(s), background_(cb, cb, cb)
{
  type_           = T_SCENE;
  selectChildren_ = true;
  attributes_.insert (new G3DAColor); //TODO set some default value
  
  description_    = 0;
}

G3DScene::G3DScene( const G3DScene &orig ) :
  G3DGroup ( orig ), background_(orig.background_)
{
  if (orig.description_) {
    int l = strlen (orig.description_);
    description_ = new char[l+1];
    strcpy (description_, orig.description_);
  }
  else
    description_ = 0;
}

G3DScene::~G3DScene ()
{
  cout << "--> G3DScene destructor" << endl;

  delete description_;
}

Field G3DScene::SceneFields_[2] =
{ {"background", s_color}, {"description", s_string} };

const Field*
G3DScene::getFieldList () const
{
  return SceneFields_;
}

uint
G3DScene::getParameter ( Param *pList )
{
  if( pList ) {
    pList->color  = &background_;
    pList++;
    pList->string = description_;
  }
  return 2;
}

void
G3DScene::setParameter ( const Param *pList )
{
  if (pList) {
    if (pList[1].string != description_) {
      delete description_;
      description_ = pList[1].string;
    }
  }
}

void
G3DScene::shade ( int lnum ) const
{
  glClearColor( background_.getR(), background_.getG(),
		background_.getB(), background_.getA() );
  glClear( (GLbitfield)GL_COLOR_BUFFER_BIT );
  G3DGroup::shade( lnum );
}


//==================================END==================================
