/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1996 - 2001. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_scene.h,v 1.2 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __SCENE_H_
#define __SCENE_H_


#include "Attributes/color.h"
#include "s_group.h"


/// This is a special group which serves as the root node of a scene.
class G3DScene : public G3DGroup
{
  DECLARE_DYN

public:
  /// Constructor.
  G3DScene ( MeFloat );

  /// Copy constructor.
  G3DScene ( const G3DScene& );

  /// Destructor. Called when the object is destroyed.
  ~G3DScene ();
  
  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;

  /**Get a list of params describing the geometry of the object.
     If plist is Null you get just the number of elements.
  */
  virtual uint getParameter ( Param* );

  // The opposite of getParameter.
  virtual void setParameter ( const Param* );

  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
      Reimplemented to do some special tasks like setting the background.
  */
  virtual void shade ( int ) const;

protected:
  G3DColor background_;  //TODO Attribute?
  char     *description_;

  static Field SceneFields_[2];

  //RefList  _refList;
  //other global settings and rendering hints
};

#endif
