/*
 * Project : "MindsEye, 3D Library"
 * Copyright (c) 1997. All Rights Reserved.
 * Author(s):   Martin Kuball
 *
 * $Id: s_simple.cpp,v 1.5 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * implementation of classes for geometric primitives
 */


#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "s_simple.h"

IMPLEMENT_DYN(G3DS_Box, "Box")
IMPLEMENT_DYN(G3DS_Cone, "Cone")
IMPLEMENT_DYN(G3DS_Cylinder, "Cylinder")
IMPLEMENT_DYN(G3DS_Sphere, "Sphere")


int G3DS_Box::Faces_[6][4] = {
    {4,5,6,7}, {0,1,2,3}, {1,7,6,2},
    {0,3,5,4}, {2,6,5,3}, {0,4,7,1}
  };


G3DS_Box::G3DS_Box( MeFloat s ) :
  _e1(0,s,0),_e2(0,0,s), _e3(s,0,0)
{
  type_ = T_BOX;
  tesselate();
  pivotPoint_ = bbox_.getCenter();
}

G3DS_Box::G3DS_Box( const G3DVector3d &e1, const G3DVector3d &e2,
		    const G3DVector3d &e3 ) :
  _e1( e1 ), _e2( e2 ), _e3( e3 )
{
  type_ = T_BOX;
  tesselate();
  pivotPoint_ = bbox_.getCenter();
}

G3DS_Box::G3DS_Box( const G3DS_Box &orig ) :
  G3DShape( orig )
{
  _e1     = orig._e1;
  _e2     = orig._e2;
  _e3     = orig._e3;
  tesselate();  //TODO copy
}

G3DS_Box::~G3DS_Box () 
{
}

Field G3DS_Box::FieldList_[3] =
{ {"e1", s_vector3}, {"e2", s_vector3}, {"e2", s_vector3} };

const Field*
G3DS_Box::getFieldList () const
{
  return FieldList_;
}

uint
G3DS_Box::getParameter ( Param *pList )
{
  if (pList) {
    pList->vector3 = &_e1;
    pList++;
    pList->vector3 = &_e2;
    pList++;
    pList->vector3 = &_e3;
  }
  return 3;
}

MeFloat
G3DS_Box::intersect( G3DRay *ray, G3DObject **obj )
{
  return 0;
  /*
  MeFloat     tmp[8][2];
  G3DMatrix   *mat = ray->getProjection();
  int         i, *signp, signs[8][2];

  //puts( "inter box" );
  //cout << *mat;
  //cout << "dir: " << ray->_direction << endl;

  signp = &(signs[0][0]);
  for( int j = 0; j < 8; j++ )
    {
      //tmp[j] = 
      mat->getxyrot( _points[j], tmp[j][0], tmp[j][1] );
      cout << " -> " << tmp[j][0] << " " << tmp[j][1] << endl;
      *(signp++) = sign( tmp[j][0] );
      *(signp++) = sign( tmp[j][1] );
    }

  signp = &(signs[0][0]);
  for( int k = 0; k < 16; k++ )
    cout << *(signp++) << " ";
  cout << endl;

  for( i = 0; i < 6; i++ )
    {
      if( signs[Faces_[i][0]][0] + signs[Faces_[i][2]][0] == 0 )
	{
	  if( signs[Faces_[i][1]][1] + signs[Faces_[i][3]][1] == 0 )
	    if( axesTest(tmp, Faces_[i]) ) break;
	}
      else if( signs[Faces_[i][0]][1] + signs[Faces_[i][2]][1] == 0 )
	{
	  if( signs[Faces_[i][1]][0] + signs[Faces_[i][3]][0] == 0 )
	    if( axesTest(tmp, Faces_[i]) ) break;
	}
    }
  if( i < 6 )
    {
      MeFloat dist =(_norm[i].dot(_points[Faces_[i][0]] - ray->_startpoint)) 
	     / (_norm[i].dot(ray->_direction));
      cout << " box hit in face " << i << " with: " << dist << endl;
      *obj = this;
      return dist;
    }
  else
    return( G3D_INFINITY );
*/
}

void
G3DS_Box::resetBoundingBox ()
{
  bbox_.setTo( G3DPoint3d(0.0, 0.0, 0.0), _e1, _e2, _e3 );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Box::setParameter( const Param *pList )
{
  if (pList) {
    _e1 = *(pList->vector3);
    ++pList;
    _e2 = *(pList->vector3);
    ++pList;
    _e3 = *(pList->vector3);
    tesselate();
  }
}

void
G3DS_Box::shade ( int ) const
{
  const int *facep = &(Faces_[0][0]);
  
  glBegin (GL_QUADS);
  for (int i = 0; i < 6; i++) {
    glNormal3xv( (const MeFloat*)(_norm[i]) );
    glVertex3xv( (const MeFloat*)(_points[*(facep++)]) );
    glVertex3xv( (const MeFloat*)(_points[*(facep++)]) );
    glVertex3xv( (const MeFloat*)(_points[*(facep++)]) );
      glVertex3xv( (const MeFloat*)(_points[*(facep++)]) );
  }
  glEnd();
}

void
G3DS_Box::tesselate()
{
  G3DVector3d  t1, t2, t3;

  _points[0].setTo( 0.0, 0.0, 0.0 );
  _points[1] = t1 = _e2;
  t2 = _e2 + _e3;
  _points[2] = t2;
  _points[3] = t3 = _e3;
  _points[4] = _e1;
  _points[5] = t3 + _e1;
  _points[6] = t2 + _e1;
  _points[7] = t1 + _e1;

  _norm[0] = _e2.cross( _e3 );
  _norm[0].normalize();
  _norm[1] = _norm[0] * (-1.0);
  _norm[2] = _e3.cross( _e1 );
  _norm[2].normalize();
  _norm[3] = _norm[2] * (-1.0);
  _norm[4] = _e1.cross( _e2 );
  _norm[4].normalize();
  _norm[5] = _norm[4] * (-1.0);

  resetBoundingBox();
}


//======================================================================

//TODO list of editable points?

G3DS_Cone::G3DS_Cone ( MeFloat s ) :
  _orientation(s,0,0)
{
  type_ = T_CONE;
  _botr = 0.5 * s;
  _topr = 0.0 /* * s */;
  tesselate();
  pivotPoint_ = bbox_.getCenter();
}

G3DS_Cone::G3DS_Cone ( const G3DS_Cone &orig ) :
  G3DShape(orig), _orientation(orig._orientation)
{
  _botr = orig._botr;
  _topr = orig._topr;
  tesselate();
}

G3DS_Cone::~G3DS_Cone ()
{
}


Field G3DS_Cone::FieldList_[3] =
  { {"orientation", s_vector3}, {"r_bottom", s_flt}, {"r_top", s_flt} };

const Field*
G3DS_Cone::getFieldList () const
{
  return FieldList_;
}

uint
G3DS_Cone::getParameter ( Param *pList )
{
  if (pList) {
    pList->vector3 = &_orientation;
    pList++;
    pList->flt     = _botr;
    pList++;
    pList->flt     = _topr;
  }
  return 3;
}

void
G3DS_Cone::resetBoundingBox ()
{
  G3DVector3d ax1, ax2;
  MeFloat     rad;

  rad = max( _botr, _topr );
  ax1 = (_orientation.getOrtho()).normalize() * rad;
  ax2 = _orientation.cross( ax1 );
  ax2.normalize();
  ax2 = ax2 * rad;

  bbox_.setTo( (ax1 + ax2)*(-1), ax1*2, ax2*2, _orientation );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Cone::setParameter ( const Param *pList )
{
  if (pList) {
    _orientation = *(pList->vector3);
    pList++;
    _botr        = pList->flt;
    pList++;
    _topr        = pList->flt;
    tesselate();
  }
}

void
G3DS_Cone::shade ( int lnum ) const
{
  //glDisable( GL_CULL_FACE );
  
  _mantle.shade();
  if( _topr > 0 )
    _top.shade ( lnum );  //TODO _cupFlag
  _bottom.shade ( lnum );
  
  //glEnable( GL_CULL_FACE );
}

void
G3DS_Cone::tesselate ()
{
  G3DVector3d point, norm, nup, major, minor;
  //MeFloat      ratio = _major.length() / _minor.length();
  MeFloat      ratio = _topr / _botr;
  int         j = 0;
  int         slice = 16;

  major = (_orientation.getOrtho()).normalize() * _botr;
  minor = _orientation.cross (major);
  minor.normalize();
  minor *= _botr;

  nup = _orientation;
  nup.normalize();
  nup *= (_botr - _topr) / _orientation.length();

  //TODO slice = getAttribute( RESOLUTION )->getValue();
  _mantle.setStripeNumber (slice);
  if (_topr > 0)
    _top.setVertexNumber (slice);
  _bottom.setVertexNumber (slice);
  MeFloat da = 2 * M_PI / slice;

#if 0
  for (int i = 0; i <= slice; i++) {
    point1 = (major * cos(i*da)) + (minor * sin(i*da));
    point2 = (point1 * ratio) + _orientation;
    normal = point1 + nup;
    _bottom.setVertex (i, point1);
    _top.setVertex (i, point2);
    _mantle.setVertex (i*2, point1);
    _mantle.setVertex (i*2+1, point2);
    _mantle.setNormal (i*2, normal);
    _mantle.setNormal (i*2+1, normal);
  }
#endif
  for (int i = 0; i <= slice; i++) {
    norm = (major * cos(i*da)) + (minor * sin(i*da));
    _mantle.setVertex (j, norm);
    _bottom.setVertex (i, norm);
    
    point = (norm * ratio) + _orientation;
    norm += nup;
    _mantle.setNormal (j++, norm);
    _mantle.setNormal (j, norm);
    _mantle.setVertex (j++, point);
    _top.setVertex (i, point);
  }

  /*TODO remove this.
  point = _origin + major;
  _mantle.setVertex( j, point );
  point = (major * ratio) + _origin + _orientation;
  major += nup;
  _mantle.setNormal( j++, major );
  _mantle.setNormal( j, major );
  _mantle.setVertex( j, point );
  */
  if( _topr > 0 )
    _top.setNormal( _orientation );
  _bottom.setNormal( _orientation * (-1) ); //TODO major.cross(minor)??

  resetBoundingBox();
}



//======================================================================


G3DS_Cylinder::G3DS_Cylinder( MeFloat s ) :
  _orientation(s,0,0), _major(0,s*0.5,0), _minor(0,0,s*0.5)
{
  type_ = T_CYLINDER;
  tesselate();
  pivotPoint_ = bbox_.getCenter();
}

//TODO do we need this?
G3DS_Cylinder::G3DS_Cylinder( G3DVector3d &orient, G3DVector3d &mj,
			      G3DVector3d &mi ) :
  _orientation(orient), _major(mj), _minor(mi)
{
  type_ = T_CYLINDER;
  tesselate();
  pivotPoint_ = bbox_.getCenter();
}

G3DS_Cylinder::G3DS_Cylinder( const G3DS_Cylinder &orig ) :
  G3DShape( orig )
{
  _orientation = orig._orientation;
  _major       = orig._major;
  _minor       = orig._minor;
  tesselate();  //TODO copy
}

G3DS_Cylinder::~G3DS_Cylinder()
{
}


Field G3DS_Cylinder::FieldList_[3] =
{ {"orientation", s_vector3}, {"major_axes", s_vector3},
  {"minor_axes", s_vector3} };

const Field*
G3DS_Cylinder::getFieldList () const
{
  return( FieldList_ );
}

uint
G3DS_Cylinder::getParameter( Param *pList )
{
  if( pList ) {
    pList->vector3 = &_orientation;
    pList++;
    pList->vector3 = &_major;
    pList++;
    pList->vector3 = &_minor;
  }
  return 3;
}

MeFloat
G3DS_Cylinder::intersect( G3DRay*, G3DObject **obj )
{
  /*
  if( bbTest() )
    {
      //make some complicated Calculations
    }
    */
  *obj = this;
  return( 1 );
  //return( -1.0 );
}

void
G3DS_Cylinder::resetBoundingBox ()
{
  bbox_.setTo( (_major+_minor)*(-1), _major*2, _minor*2, _orientation );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Cylinder::setParameter( const Param *pList )
{
  if (pList) {
    _orientation = *(pList->vector3);
    ++pList;
    _major       = *(pList->vector3);
    ++pList;
    _minor       = *(pList->vector3);
    tesselate();
  }
}

void
G3DS_Cylinder::shade ( int lnum ) const
{
  //glDisable( GL_CULL_FACE );
  
  _mantle.shade();
  _top.shade( lnum );  //TODO _cupFlag
  _bottom.shade( lnum );
  
  //glEnable( GL_CULL_FACE );
}

void
G3DS_Cylinder::tesselate ()
{
  G3DVector3d point, norm;
  MeFloat      ratio = _major.length() / _minor.length();
  int         j = 0;
  int         slice = 12;

  //slice = getAttribute( RESOLUTION )->getValue();
  _mantle.setStripeNumber( slice );
  _top.setVertexNumber( slice );
  _bottom.setVertexNumber( slice );
  MeFloat da = 2 * M_PI / slice;

  //TODO _cupFlag
  for( int i = 0; i < slice; i++ )
    {
      point = (_major * cos(i*da)) + (_minor * sin(i*da));
      norm = (_major * (cos( i*da )/ratio))
	 + (_minor * (sin( i*da )*ratio));
      _mantle.setNormal( j, norm );
      _mantle.setVertex( j++, point );
      _bottom.setVertex( i, point );

      point += _orientation;
      _mantle.setNormal( j, norm );
      _mantle.setVertex( j++, point );
      _top.setVertex( i, point );

    }
  _mantle.setNormal( j, _major );
  _mantle.setVertex( j++, _major );
  _mantle.setNormal( j, _major );
  _mantle.setVertex( j, _major + _orientation );

  _top.setNormal( _orientation );
  _bottom.setNormal( _orientation * (-1) );

  resetBoundingBox();
}


//======================================================================


G3DS_Sphere::G3DS_Sphere ( MeFloat diameter )
{
  type_   = T_SPHERE;
  mantle_ = 0;
  radius_ = diameter * 0.5;
  rho_    = M_PI;
  closed_ = true;
  tesselate ();
};

G3DS_Sphere::G3DS_Sphere( const G3DS_Sphere &orig ) :
  G3DShape(orig)
{
  radius_ = orig.radius_;
  rho_    = orig.rho_;
  closed_ = orig.closed_;
  mantle_ = NULL;
  tesselate();
}

G3DS_Sphere::~G3DS_Sphere()
{
  delete[] mantle_;
}


Field G3DS_Sphere::FieldList_[3] =
{ {"radius", s_flt}, {"rho", s_flt}, {"closed", s_logic} };

const Field*
G3DS_Sphere::getFieldList () const
{
  return FieldList_;
}

uint
G3DS_Sphere::getParameter( Param *pList )
{
  if( pList )
    {
      pList->flt   = radius_;
      pList++;
      pList->flt   = rho_;
      pList++;
      pList->logic = closed_; 
    }
  return( 3 );
}

MeFloat
G3DS_Sphere::intersect( const G3DRay &ray, G3DObject **obj )
{
  return 0;
/*
  G3DVector3d pp;
  MeFloat skpp, skpd, skpd2, root;

  pp = _center - ray->startpoint_;
  skpp = pp.dot( pp );
  skpd = pp.dot( ray->direction_ );
  skpd2 = skpd * skpd;

  root = (skpd2 - skpp) + radius_;
  if( root < 0 )
    return( G3D_INFINITY );     //no intersection with the sphere
  else
    {
      *obj = this;
      if( root == 0 )
	return( skpd );
      else
	{
	  root = sqrt( root );
	  if( skpd >= root )
	    return( skpd - root );
          else
	    return( skpd + root );
	}
    }
*/
}

void
G3DS_Sphere::resetBoundingBox()
{
  MeFloat mrd = -radius_;
  //TODO
  bbox_.setTo( G3DPoint3d(mrd,mrd,mrd), G3DPoint3d(radius_,radius_,radius_) );
  if( parent_ )
    ((G3DShape*)parent_)->resetBoundingBox();
}

void
G3DS_Sphere::setParameter( const Param *pList )
{
  if( pList )
    {
      radius_ = pList->flt; pList++;
      rho_    = pList->flt; pList++;
      closed_ = pList->logic;
      tesselate();
    }
}

void
G3DS_Sphere::shade ( int ) const
{
  for( int i = 0; i < stacks_; i++ )
    mantle_[i].shade();
}


void
G3DS_Sphere::tesselate ()
{
  MeFloat x, y, z, nsign = 1.0;  //TODO based on inside/outside Attribute
  MeFloat rho, drho, theta, dtheta;
  G3DQuadStrip *strip;
  int k, slices = 10;
  G3DVector3d normal;
  G3DPoint3d  point;

  stacks_ = 8;
#if 0
  if( rho_ >= M_PI/2 )
    stacks_ = 8;
  else
    stacks_ = 4;
#endif

  delete[] mantle_;
  mantle_ = new G3DQuadStrip[stacks_];
  //if (qobj->Orientation==GLU_INSIDE)
  //nsign = -1.0;
  //else

  //if (rho >= M_PI/2)
  drho = rho_ / (MeFloat)stacks_;
  //else
  //drho = rho_ / ((MeFloat)stacks_ - 1);
  dtheta = 2.0 * M_PI / (MeFloat)slices;
  /*
  normal.setTo( 0, 0, 1 );
  mesh_->setNormal( k, normal *nsign );
  point = normal * radius_;
  mesh_->setVertex( k++, point );
  for( int i = 1; i < stacks_ - 1; ++i )
    {
      rho = i * drho;
      for( int j = 0; j < slices; ++j )
	{
	  theta = j * dtheta;
	  x = cos( theta ) * sin( rho );
	  y = -sin( theta ) * sin( rho );
	  z = cos( rho );
	  normal.setTo( x, y, z );
	  mesh_->setNormal( k, normal * nsign );
	  point.setTo( x, y, z );
	  point *= radius_;
	  mesh_->setVertex( k, point );
	}
    }
  for( int l = 0; l < slices; ++l )
    grid_->setVertex
  */
  // draw intermediate stacks as quad strips
  for (int i = 0; i < stacks_; i++) {
    strip = &(mantle_[i]);
    strip->setStripeNumber (slices);
    rho = i * drho;
    k = 0;
    for (int j = 0; j <= slices; j++) {
      theta = ( (j == slices) ? 0.0 : j * dtheta );
      
      x = cos(theta) * sin(rho);  //TODO might already be calculated
      y = -sin(theta) * sin(rho);
      z = cos(rho);
      normal.setTo (x, y, z);
      strip->setNormal (k, normal * nsign);
      point.setTo (x, y, z);
      point *= radius_;
      strip->setVertex (k++, point);
      
      x = cos (theta) * sin (rho+drho);
      y = -sin (theta) * sin (rho+drho);
      z = cos (rho+drho);
      normal.setTo (x, y, z);
      strip->setNormal (k, normal * nsign);
      point.setTo (x, y, z);
      point *= radius_;
      strip->setVertex (k++, point);
    }
  }
  /* if( rho < M_PI )
     for( int j = 0; j <= slices; j++ )
  */
  resetBoundingBox ();
}

/*TODO
void gluSphere( GLUquadricObj *qobj,
                GLdouble radius, GLint slices, GLint stacks )
{
   GLfloat rho, drho, theta, dtheta;
   GLfloat x, y, z;
   GLfloat s, t, ds, dt;
   GLint i, j, imin, imax;
   GLfloat nsign;

   stacks = 12;
   mantle_ = new G3DGLQuads[12];
   //if (qobj->Orientation==GLU_INSIDE)
   //nsign = -1.0;
   //else
   nsign = 1.0;

   drho = M_PI / (GLfloat) stacks;
   dtheta = 2.0 * M_PI / (GLfloat) slices;

   // texturing: s goes from 0.0/0.25/0.5/0.75/1.0 at +y/+x/-y/-x/+y axis
   // t goes from -1.0/+1.0 at z = -radius/+radius (linear along longitudes)
   // cannot use triangle fan on texturing (s coord. at top/bottom tip varies)

   if (!qobj->TextureFlag)
     {
       // draw +Z end as a triangle fan
       glBegin( GL_TRIANGLE_FAN );
       glNormal3f( 0.0, 0.0, 1.0 );
       TXTR_COORD(0.5,1.0);
       glVertex3f( 0.0, 0.0, nsign * radius );
       for (j=0;j<=slices;j++)
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(drho);
	   y = cos(theta) * sin(drho);
	   z = nsign * cos(drho);
	   if (normals)  glNormal3f( x*nsign, y*nsign, z*nsign );
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
	 glEnd();
     }

   ds = 1.0 / slices;
   dt = 1.0 / stacks;
   t = 1.0;    // because loop now runs from 0
   if (qobj->TextureFlag) {
     imin = 0;
     imax = stacks;
   }
   else {
        imin = 1;
        imax = stacks-1;
      }

   G3DGLQuads &strip;
   // draw intermediate stacks as quad strips
   for( i = imin; i < imax; i++ )
     {
       strip = mantle_[i];
       strip.setStripeNumber( slices );
       rho = i * drho;
       s = 0.0;
       for( j = 0; j < slices; j++ )
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(rho);
	   y = cos(theta) * sin(rho);
	   z = nsign * cos(rho);

	   normal.setTo( x, y, z );
	   strip.setNormal( j, normal * nsign );
	   //TXTR_COORD(s,t);
	   point.setTo( x, y, z );
	   point *= radius;
	   point += _center;
	   strip.setVertex( j, point );

	   x = -sin(theta) * sin(rho+drho);
	   y = cos(theta) * sin(rho+drho);
	   z = nsign * cos(rho+drho);
	   glNormal3f( x*nsign, y*nsign, z*nsign );
	   TXTR_COORD(s,t-dt);
	   s += ds;
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
       t -= dt;
     }

   if (!qobj->TextureFlag)
     {
       // draw -Z end as a triangle fan
       glBegin( GL_TRIANGLE_FAN );
       glNormal3f( 0.0, 0.0, -1.0 );
       TXTR_COORD(0.5,0.0);
       glVertex3f( 0.0, 0.0, -radius*nsign );
       rho = M_PI - drho;
       s = 1.0;
       t = dt;
       for( j = slices; j >= 0; j-- )
	 {
	   theta = (j==slices) ? 0.0 : j * dtheta;
	   x = -sin(theta) * sin(rho);
	   y = cos(theta) * sin(rho);
	   z = nsign * cos(rho);
	   glNormal3f( x*nsign, y*nsign, z*nsign );
	   TXTR_COORD(s,t);
	   s -= ds;
	   glVertex3f( x*radius, y*radius, z*radius );
	 }
     }
}
*/


//==================================END==================================
