/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_torus.h,v 1.3 2001/04/01 09:42:28 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_TORUS_H__
#define __S_TORUS_H__

#include "shape.h"
#include "s_glshapes.h"

/// Torus primitive.
/** A torus is defined by an origin and 2 radii
 */
class G3DTorus : public G3DShape
{
  DECLARE_DYN

public:
  ///
  explicit G3DTorus( MeFloat );
  
  /// Constructor taking two radii and the torus center.
  G3DTorus ( MeFloat r1, MeFloat r2, const G3DPoint3d &center );

  /// Copy constructor.
  G3DTorus ( const G3DTorus& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DTorus ();

  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;
  
  ///
  /** Copy pointers to the geometric data to the paramter list. The order is:
      major radius, minor radius. (?) 
  */
  virtual uint getParameter ( Param* );
  
  /// Find an intersection between the object and a given ray (?)
  virtual MeFloat intersect ( G3DRay*, G3DObject** );

  /// Reset the object's bounding box.
  virtual void resetBoundingBox ();

  ///
  virtual void setParameter( const Param* );
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;

protected:
  ///
  void tesselate ();

protected:
  G3DQuadStrip *_mantle;
  int          _stacks;

  MeFloat    _rad1, _rad2;

  static Field FieldList_[2];
};

#endif
