/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: s_trigrid.h,v 1.2 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __S_TRIGRID_H__
#define __S_TRIGRID_H__

#include "shape.h"
#include "s_glshapes.h"


/// Instances of this class represent rectangular meshes of triangles.
class G3DTriGrid : public G3DShape
{
  DECLARE_DYN

public:
  ///
  explicit G3DTriGrid ( MeFloat );

  // G3DTriGrid( uint, uint );

  /// Copy constructor.
  G3DTriGrid ( const G3DTriGrid& );

  /// Destructor. Called when the object is destroyed.
  virtual ~G3DTriGrid ();

  ///
  void adjust ();
  
  /// Do a given action with this object.
  virtual void doAction ( ActionData& );
  
  /// Return a list with the names and types of the object s parameter.
  virtual const Field* getFieldList () const;

  /**Copy pointers to the parameter to the list. The order is:
     _rows, _cols, _vertices, _normals.*/
  virtual uint       getParameter( Param* ); 
  
  ///
  virtual G3DObject* getSelectionHandle_ ();

  /// reset the object's bounding box.
  virtual void resetBoundingBox ();

  ///
  void resize ( uint, uint );
  
  ///
  virtual MeFloat selectDetails ( const G3DCamera&, const G3DRay& );

  ///
  virtual void setParameter ( const Param *pList );

  ///
  void setVertex ( uint r, uint c, TVector<MeFloat,3> );

  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int ) const;

  ///
  virtual void shadeDetails () const;

protected:
  ///
  void calcSmoothNormals ();

protected:
  uint  rows_, cols_;
  enum  {stVertex=0, stFace} selectionType_;
  int   selectionIndex_;

  G3DPoint3d  *vertices_;
  G3DVector3d *normals_;

  static Field FieldList_[7];
};

#endif
