/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: sceneElement.cpp,v 1.4 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"
#include <string.h>
#include <GL/gl.h>
#include "g_operator.h"
#include "Attributes/texture.h"
#include "sceneElement.h"


class ResetOp : public G3D_Operator
{
public:
  virtual bool exec( G3DObject *obj )
    {
      ((G3DWidget*)obj)->reset();
      return( false );
    }
};


class GSelectDetailOp: public G3D_Operator
{
public:
  GSelectDetailOp( const G3DCamera& cam, const G3DRay& ray ) :
    cam_(cam), ray_(ray)
   { dist_ = G3D_INFINITY;  obj_ = NULL; }

  virtual bool exec( G3DObject* );
  MeFloat      getDistance () { return dist_; }
  G3DObject*   getObject () { return obj_; }

protected:
  const G3DCamera& cam_;
  const G3DRay&    ray_;
  G3DObject        *obj_;
  MeFloat          dist_;
  int num_;
};


//-------------------------------------------------------------


G3DSceneElement::G3DSceneElement () :
  widgets_(true)
{
  //listId_  = 0;
};


G3DSceneElement::G3DSceneElement( const G3DSceneElement &orig ) :
  G3DObject(orig), attributes_(orig.attributes_), widgets_(true)
{
  //The widget list is not duplicated!
  //listId_   = 0;
};


G3DSceneElement::~G3DSceneElement()
{
}

void
G3DSceneElement::addWidget ( G3DWidget *widget )
{
  widgets_.push_back (widget);
}

void
G3DSceneElement::adjust ()
{
  resetBoundingBox();
  ResetOp iter;
  widgets_.doWithMElements( iter );
  //TODO test for animation attributes and evaluate them???
}

void
G3DSceneElement::display( int &num ) const
{
}

void
G3DSceneElement::doAction( ActionData &action )
{
  switch( action.type )
    {
    case A_SELECT :
      //addWidget( new G3DWidget(this) );
      break;
    case A_DESELECT :
      widgets_.reInitialize();
      break;
    case A_FIND_SELECTION :
      ((SelectData&)action).setSelection( this );
      break;
    case A_CENTER_PIVOTPOINT :
      cout << bbox_.getCenter() << endl;
      setPivotPoint( bbox_.getCenter() );
      break;
    }
  G3DObject::doAction( action );
}

G3DAListIterator
G3DSceneElement::getAttribute( t_type type, bool inherit ) const
{
  return( attributes_.get((G3DSceneElement*)parent_, type, inherit) );
}

G3DAListIterator
G3DSceneElement::getAttribute( const char* ) const
{
  return( G3DAListIterator(NULL, this, 0, false) );
}

const G3D_BBAligned&
G3DSceneElement::getBoundingBox () const
{
  return( bbox_ );
}

G3DObject*
G3DSceneElement::getSelectionHandle ()
{
  //TODO increment refCount_ ?
  if( selectedDetail_ )
    {
      if( IS_CLASS(selectedDetail_->getType(), T_SCENE_ELEMENT) )
	return( ((G3DSceneElement*)(selectedDetail_))->getSelectionHandle_() );
      else
	selectedDetail_->getSelectionHandle();
    }
  else
    return NULL;
}

G3DObject*
G3DSceneElement::getSelectionHandle_ ()
{
  return NULL;
}

void
G3DSceneElement::removeAttribute( G3DAttribute *attr )
{
  attributes_.remove( attr );
  attr->dispose();
}

void
G3DSceneElement::select ()
{
  if( state_ & visible )
    {
      //bbox_.intersect( ray, m );
      //TODO bounding box test first?
      glPushMatrix();
      glMultMatrixx( (const MeFloat*)trafo_ );
      int zero = 0;
      display( zero );
      //shade( -1 );  //no details, only the basic shape
      glPopMatrix();
    }
}

bool
GSelectDetailOp::exec ( G3DObject *obj )
{
  MeFloat dist = obj->select( cam_, ray_, allparts );
  if (dist < dist_) {
    dist_ = dist;
    obj_ = obj;
  }
  return false;
}

MeFloat
G3DSceneElement::select( const G3DCamera &cam, const G3DRay &ray,
			 G3DPartEnum flag )
{
  MeFloat dist = G3D_INFINITY;

  selectedDetail_ = NULL;
  if ( flag & details )
    {
      MeFloat rad = selectDetails( cam, ray );
      if( rad < dist )
	{
	  dist = rad;
	  selectedDetail_ = this;
	}
    }
  if( flag & handles )
    {
      GSelectDetailOp sit( cam, ray );
      widgets_.doWithMElements( sit );
      MeFloat rad = sit.getDistance();
      if( rad < dist )
	{
	  dist = rad;
	  selectedDetail_ = sit.getObject();
	}
    }
  //TODO if( flag & tools ) select in Attributes
  return dist;
}

MeFloat
G3DSceneElement::selectDetails( const G3DCamera&, const G3DRay& )
{
  return G3D_INFINITY;
}

void
G3DSceneElement::setAttribute( G3DAttribute *attrib )
{
  attributes_.insert( attrib );
  //TOTO ?? if( _type & T_Group )
  //doAction( retraverse );
}

void
G3DSceneElement::shadeDetails () const
{
}


/*
G3DSceneElement& G3DSceneElement::operator=( const G3DSceneElement &a )
{  
  return *this ;
}
*/

//==============================END===================================
