/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: selectGroup.cpp,v 1.3 2002/06/11 09:56:46 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"

#include "selectGroup.h"
#include "s_group.h"
#include "g_operator.h"
#include "Attributes/reference.h"


class EraseObjectsOp : public G3D_Operator
{
public:
  virtual bool exec ( G3DObject *obj )
  {
    G3DGroup *parent = (G3DGroup*)(obj->getParent());
    if (parent)
      parent->removeObject (obj);
    return false;
  }
};

class SetAttributeOp : public G3D_Operator
{
public:
  SetAttributeOp( G3DAttribute *attr ) { attrib_ = attr; first_ = true; }
  virtual bool exec( G3DObject *obj );

protected:
  G3DAttribute *attrib_;
  bool         first_;
};

bool
SetAttributeOp::exec( G3DObject *obj )
{
  if (obj->getType() & T_SHAPE) {
    if (first_) {
      ((G3DShape*)obj)->setAttribute (attrib_);
      first_ = false;
    }
    else
      ((G3DShape*)obj)->setAttribute (new G3DAReference(attrib_));
  }
  return false;
}

//-------------------------------------------------------------------------


G3DSelectionGroup::G3DSelectionGroup ()
{
  type_ = T_SEL_GROUP;
}

G3DSelectionGroup::G3DSelectionGroup( const G3DSelectionGroup &orig ) :
  G3DObject( orig )
{
  //TODO
}


void
G3DSelectionGroup::doAction ( ActionData &action )
{
  switch (action.type)
    {
    case A_DESELECT:
      sendToAllObjects (action);
      // delete this;
      dispose ();
      break;
    }
}

void
G3DSelectionGroup::eraseObjects ()
{
  EraseObjectsOp op;
  objList_.doWithMElements (op);
  // objList_.setDeleteMode (true);
  objList_.reInitialize ();
  // objList_.setDeleteMode (false);
}

void
G3DSelectionGroup::sendToAllObjects ( ActionData &action )
{
  G3D_SendActionOp it (action);
  objList_.doWithMElements (it);
}

void
G3DSelectionGroup::setAttribute ( G3DAttribute *attrib )
{
  SetAttributeOp it (attrib);
  objList_.doWithMElements (it);
}
