/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: selectGroup.h,v 1.2 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __SELECTION_GROUP_H_
#define __SELECTION_GROUP_H_

#include "object.h"
#include "g_array.h"

class G3DAttribute;

/// Objects of this class store a selection of objects.
/** This object is used to store pointers to selected objects. The
    objects added to this group are not modified. Especially their
    parent is not changed (as would be with a G3DGroup object).
*/
class G3DSelectionGroup : public G3DObject
{
public:
  /// Default constructor.
  G3DSelectionGroup ();

  /// Copy Constructor.
  G3DSelectionGroup ( const G3DSelectionGroup& );

  /// Add a shape to the group.
  void appendObject ( G3DObject *obj ) { objList_.push_back ( obj ); }
  
  /// Do a given action with this object.
  virtual void doAction ( ActionData& );
  
  /// Iterate over the objects.
  void doWithElements ( G3D_Operator &it ) { objList_.doWithMElements ( it ); }
  
  /// Empty the container.
  void eraseObjects ();
  
  /// Return the pos of obj in the List. If it is not in the list return -1.
  int findObject ( G3DObject *obj ) const { return objList_.find (obj); }

  /// Return the shape at position i in the list of shapes.
  G3DObject* getObject ( int i ) { return  (G3DObject*) (objList_[i]); }
  
  /// Return the number of objects contained in this selection.
  int numberOfObjects () const { return ( objList_.size () ); }
  
  /// Remove a shape from the group.
  void removeObject ( G3DObject *obj ) { objList_.remove ( obj ); }
  
  /// Set Attribute for each shape in the list.
  void setAttribute ( G3DAttribute* );
  
protected:
  /// Send an action message to all objects in the selection.
  void sendToAllObjects ( ActionData& );

protected:
  G3D_Array objList_;
};

#endif
