/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: shape.cpp,v 1.3 2002/06/12 20:13:18 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "include/config.h"

#include <string.h>
#include <GL/gl.h>

#include "shape.h"
#include "g_operator.h"
#include "Attributes/texture.h"


G3DShape::~G3DShape ()
{
  if (listId_ > 0)
    glDeleteLists (listId_, 1);
}

void
G3DShape::clDisplay ( int lnum )
{
  if( state_ & visible )
    {
      int tmp = listId_;
      listId_ = 0;
      display( lnum );
      listId_ = tmp;
    }
}

void
G3DShape::compileList( int lnum )
{
  if (listId_ == 0)
    listId_ = glGenLists(1);
  if (listId_ > 0) {
    glNewList (listId_, GL_COMPILE);
    clDisplay (lnum);
    glEndList ();
  }
}

void
G3DShape::deleteList()
{
  if (listId_ > 0) {
    glDeleteLists( listId_, 1 );
    listId_ = 0;
  }
}

void
G3DShape::display( int &num ) const
{
  if( state_ & visible )
    {
      if( !(state_ & selected) && (listId_ > 0) )
	glCallList( listId_ );
      else
	{
	  glPushMatrix();
	  glMultMatrixx( (const MeFloat*)trafo_ );

	  G3DAListIterator it = attributes_.get( (G3DSceneElement*)parent_,
						 TA_TEXTURE, true );
	  if( *it )
	    (*it)->activate();
	  shade( num ); //TODO add layered texture support

	  if ( state_ & selected )
	    {
	      //glDisable( GL_LIGHTING ); TODO ???
	      //glColor4fv( (float*)_selectColor );
	      shadeDetails();
	      G3D_DisplayOp<G3DWidget> dop( 0 );
	      widgets_.doWithConstElements( dop );
	      //glEnable( GL_LIGHTING );
	    }

	  glPopMatrix();
	  //int error = glGetError();
	}
    }
}

int
G3DShape::numberOfShapes ()
{
  return 0;
}

void
G3DShape::select ()
{
  if (state_ & visible) {
    // bbox_.intersect( ray, m );
    // TODO bounding box test first?
    glPushMatrix ();
    glMultMatrixx ((const MeFloat*)trafo_);
    shade (-1);  //no details, only the basic shape
    glPopMatrix ();
  }
}


//==============================END===================================
