/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: shape.h,v 1.3 2001/03/31 15:40:09 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __SHAPE_H_
#define __SHAPE_H_

#include "sceneElement.h"
#include "bounding.h"

/// Abstract base class for all 3d Objects in a scene.
/** Abstract base class for all 3d Objects in a scene. An object may be
    Display lists are associated with individual shapes, not with
    groups of shapes.
    If a shape is selected, an associated display list is ignored.
*/
class G3DShape : public G3DSceneElement
{
protected:
  /// Default constructor.
  G3DShape () { listId_ = 0; }
  
  /// Copy constructor.
  G3DShape ( const G3DShape &other ) : G3DSceneElement(other) { listId_ = 0; }
  
public:
  /// Destructor. Called whem the object is destroyed.
  virtual ~G3DShape ();
  
  /// Wrapper for display() used when compiling a display list.
  /** Wrapper for display() used when compiling a display list. It
      calls display() if the object is not selected.
  */
  void clDisplay ( int lnum );

  /// Build an OpenGL display list.
  /** Build an OpenGL display List. \param {lnum} is the number of active 
      lights.
      It's possible to call the function for individual elements in the
      scene graph.
  */
  virtual void compileList ( int lnum );
  
  /// Delete the display list.
  void deleteList ();
  
  /// Display the object on the screen.
  /**Display the object on the screen. This will call the shade()
     function to issue the necessary OGL commands.
  */
  void display ( int& ) const;
  
  /// Return the number of shapes contained (?)
  virtual int numberOfShapes ();

  /// Perform selection using the OpenGL picking mechanism.
  virtual void select ();
  
  /// Display the shape using OGL functions.
  /** Display the shape using OGL functions.
      \param {lnum} is the number of active lights.
  */
  virtual void shade ( int lnum ) const = 0;

protected:
  int              listId_;
};

#endif
