/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: t_actions.h,v 1.4 2001/07/09 21:50:20 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _T_ACTIONS_H
#define _T_ACTIONS_H

#include <stdlib.h>
#include <GL/gl.h>


//These are just some basic actions. More specific
//actions are defined in the classes where they are
//used.
#define A_HIDE                0
#define A_SHOW                1
#define A_SELECT              2
#define A_DESELECT            3
//A special group action. It tells the group to allow the
//selection of its children.
#define A_TOGGLE_CHILD_ACCESS 4
#define A_FIND_SELECTION      6
// 7
#define A_GENERIC_ALL         8
#define A_GENERIC_SHAPES      9
#define A_GENERIC_LIGHTS      10
//#define A_GROUP               11

#define A_ADD_SHAPE           20
#define A_REM_SHAPE           21

#define A_CENTER_PIVOTPOINT   22


class G3DObject;

/// Base class of all Action.
/** An Action is a message send to an object for evaluation.
    
    pass directly to an object or pass to a group
    trigger actions, collect information
    An Action object is passed through the 
    
    Common to all ActionData objects is a type that tells the object what to
    do with the action object.
*/
class ActionData
{
public:
  /// Constructor.
  ActionData ( int t ) { type = t; }

public:
  int type;
};


enum  EvalState { stopEval=0, continueEval };

/// This Action is passed down the scene graph.
/** This Action is passed down the scene graph. For every object in
    the Graph the evaluate function is called.
*/
class GenericAction : public ActionData
{
public:
  /// Constructor.
  GenericAction ( int t ) : ActionData(t) {}

  ///
  virtual EvalState evaluate( G3DObject* ) = 0;
  
  /// To be used by groups. It must be called after all children have been visited.
  virtual void postAction ( G3DObject* );
};


/// Objects of this class represent the data needed to identify the object picked by the user with the mouse.
class SelectData : public ActionData
{
public:
  /// Constructor.
  SelectData ( GLuint *names ) : ActionData(A_FIND_SELECTION) {
    _num   = (int)*names;
    _names = names + 3;
    _obj   = 0;
  }
  
  /// Get a number from the selection buffer and go to the next one.
  bool getNextNum ( GLuint &num ) {
    if (--_num >= 0) {
      num = *(_names++);
      return true;
    }
    else
      return false;
  }
  
  /// Jump to the next number in the selection buffer.
  void nextNum () { --_num; ++_names; }
  
  /// Get the current number from the selection buffer.
  bool peekNextNum ( GLuint &num ){
    if (_num > 0) {
      num = *_names;
      return true;
    }
    else
      return false;
  }
  
  /// Return the selected object.
  G3DObject* getSelection () { return _obj; }
  
  /// Remember a given object as the one beeing selected.
  void setSelection ( G3DObject *obj ) { _obj = obj; }
  
protected:
   G3DObject *_obj;
   GLuint    *_names;
   int       _num;
};

#endif
