/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: t_objects.h,v 1.2 2000/03/05 23:01:44 crjohnson Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * If you want to register a new object type, send a mail to the
 * MindsEye team.
 *
 */

#ifndef __T_OBJECTS_H__
#define __T_OBJECTS_H__

typedef unsigned int t_type;


//The most significat 4 Bits define the class oft the object
 static const unsigned int  T_SCENE_ELEMENT = 0x10000000;
 static const unsigned int  T_HELPER        = 0x20000000;
 static const unsigned int  T_WIDGET        = 0x30000000;
 static const unsigned int  T_CAMERA        = 0x40000000;
 static const unsigned int  T_ATTRIBUTE     = 0x50000000;
 static const unsigned int  T_HANDLE        = 0x60000000;

#define IS_CLASS(t, cl)    ((t&0xF0000000) == cl)
#define IS_TYPE(t, typ)    ((t&0xFFF00000) == typ)
#define IS_SUBTYPE(t, typ) ((t&0xFFFFF000) == typ)

#define GET_CLASS(t)       (t & 0xF0000000)
#define GET_TYPE(t)        (t & 0xFFF00000)
#define GET_SUBTYPE(t)     (t & 0xFFFFF000)
#define GET_TNUM(t)        (t & 0xFFF)

//-------------------------------------------------------------------

//Scene Elements
 static const unsigned int  T_SHAPE =       (0x100000 | T_SCENE_ELEMENT);
 static const unsigned int  T_LIGHT =       (0x200000 | T_SCENE_ELEMENT);
 static const unsigned int  T_SOUND =       (0x400000 | T_SCENE_ELEMENT);

// shape objects
 static const unsigned int  T_GROUP  =      (0x1000 | T_SHAPE);
 static const unsigned int  T_SIMPLE =      (0x2000 | T_SHAPE);
 static const unsigned int  T_MESH   =      (0x3000 | T_SHAPE);
 static const unsigned int  T_NURBS  =      (0x4000 | T_SHAPE);
 static const unsigned int  T_FLAT   =      (0x5000 | T_SHAPE);
 static const unsigned int  T_COMPOSITE =   (0x6000 | T_SHAPE);
 static const unsigned int  T_MATH =        (0x7000 | T_SHAPE);

//Groups
 static const unsigned int  T_S_GROUP =     (1 | T_GROUP);
 static const unsigned int  T_SCENE =       (2 | T_GROUP);

//simple shapes;
 static const unsigned int  T_BOX	=(1 | T_SIMPLE);
 static const unsigned int  T_CONE	=(2 | T_SIMPLE);
 static const unsigned int  T_CYLINDER =	(3 | T_SIMPLE);
 static const unsigned int  T_SPHERE	= (4 | T_SIMPLE);
 static const unsigned int  T_TORUS	=	(5 | T_SIMPLE);

//mesh shapes
 static const unsigned int  T_QUADMESH =     (1 | T_MESH);
 static const unsigned int  T_TRIMESH  =     (2 | T_MESH);
 static const unsigned int  T_TRIGRID  =     (3 | T_MESH);
 static const unsigned int  T_HIGHFILED =    (4 | T_MESH);

//NURBS shapes
 static const unsigned int  T_NURBS_CURVE =  (1 | T_NURBS);
 static const unsigned int  T_NURBS_SURF =   (2 | T_NURBS);
 static const unsigned int  T_NURBS_HSURF =  (3 | T_NURBS);
 static const unsigned int  T_NURBS_SOR  =   (4 | T_NURBS);

//flat shapes
 static const unsigned int  T_POINT	=	(1 | T_FLAT);
 static const unsigned int 	T_DISK	=	(2 | T_FLAT);
 static const unsigned int  T_POLYGON  =     (3 | T_FLAT);

//composite shapes
 static const unsigned int  T_UNION	=	(1 | T_COMPOSITE);
 static const unsigned int  T_INTERSECTION	=(2 | T_COMPOSITE);
 static const unsigned int  T_DIFFERENCE	=(3 | T_COMPOSITE);
//T_MERGE

//-------------------------------------------------------------------

//Widgets
 static const unsigned int  TW_SHAPE    =     (0x100000 | T_WIDGET);
 static const unsigned int  TW_LIGHT    =     (0x200000 | T_WIDGET);
 static const unsigned int  TW_SOUND    =     (0x400000 | T_WIDGET);
 static const unsigned int  TW_ATTRIB   =     (0x800000 | T_WIDGET);
 static const unsigned int  TW_STYPE    =     0x1000;

//widgets for shapes
 static const unsigned int  TW_SCALE    =     (1 | TW_SHAPE | TW_STYPE);
 static const unsigned int  TW_COORD    =     (2 | TW_SHAPE | TW_STYPE);
 static const unsigned int  TW_ROTATE   =     (4 | TW_SHAPE | TW_STYPE);

//Light widgets
 static const unsigned int  TW_DIRECT_LIGHT = (1 | TW_LIGHT | TW_STYPE);
 static const unsigned int  TW_POINT_LIGHT  = (2 | TW_LIGHT | TW_STYPE);
 static const unsigned int  TW_SPOT_LIGHT   = (3 | TW_LIGHT | TW_STYPE);

//Attribute widgets
 static const unsigned int  TW_ORBIT        = (1 | TW_ATTRIB | TW_STYPE);


//-------------------------------------------------------------------

//Handles are used to manipulate specific parts of an object

 static const unsigned int  T_HTYPE        = 0x101000;
 static const unsigned int  THA_VERTEX     =   (1 | T_HANDLE | T_HTYPE);
 static const unsigned int  THA_NORMAL     =   (2 | T_HANDLE | T_HTYPE);
 static const unsigned int  THA_CPOINT     =   (3 | T_HANDLE | T_HTYPE);
 static const unsigned int  THA_KNOT       =   (4 | T_HANDLE | T_HTYPE);
 static const unsigned int  THA_FACE       =   (5 | T_HANDLE | T_HTYPE);

//The selection group is a sepcial case of a handle
 static const unsigned int  T_SEL_GROUP    = (10 | T_HANDLE | T_HTYPE);


//-------------------------------------------------------------------

// Types of lights
 static const unsigned int  T_LTYPE         =    0x1000;
 static const unsigned int  T_AMBIENT_LIGHT  =   (1 | T_LIGHT | T_LTYPE);
 static const unsigned int  T_DIRECTIONAL_LIGHT = (2 | T_LIGHT | T_LTYPE);
 static const unsigned int  T_POINT_LIGHT	=     (3 | T_LIGHT | T_LTYPE);
 static const unsigned int  T_SPOT_LIGHT	   = (4 | T_LIGHT | T_LTYPE);
 static const unsigned int  T_AREA_LIGHT	   = (5 | T_LIGHT | T_LTYPE);


//-------------------------------------------------------------------

// Types of helpers
 static const unsigned int  T_HETYPE       = 0x101000;
 static const unsigned int  TH_SKELETON    = (1 | T_HELPER | T_HETYPE);
 static const unsigned int  TH_SPACE_WARP  = (2 | T_HELPER | T_HETYPE);


//-------------------------------------------------------------------

/* Cameras */
 static const unsigned int  T_PROJECTION	 = (1 | T_CAMERA | 0x101000);
// static const unsigned int  T_VIEWANGLE


//-------------------------------------------------------------------

// attributes
 static const unsigned int  TA_TEXTURE    = (0x100000 | T_ATTRIBUTE);
 static const unsigned int  TA_ANIMATION  = (0x200000 | T_ATTRIBUTE);
 static const unsigned int  TA_CONSTRAINT = (0x400000 | T_ATTRIBUTE);
 static const unsigned int  TA_MISC       = (0x800000 | T_ATTRIBUTE);
// static const unsigned int  TA_BUMP

 static const unsigned int  TA_ALL_ATTRIBUTES = (TA_TEXTURE | TA_MISC | TA_ANIMATION | TA_CONSTRAINT);

#define  IS_ATYPE(t, tp) ((t&0xFF00000) & tp )


//Textures;
 static const unsigned int  TA_TSIMPLE    = (0x1000 | TA_TEXTURE);
 static const unsigned int  TA_TPROCEDURE = (0x2000 | TA_TEXTURE);
 static const unsigned int  TA_TMAP       = (0x3000 | TA_TEXTURE);

 static const unsigned int  TA_COLOR      = (1 | TA_TSIMPLE);
 static const unsigned int  TA_PLASTIC    = (2 | TA_TSIMPLE);
 static const unsigned int  TA_METAL      = (3 | TA_TSIMPLE);
 static const unsigned int  TA_GLAS       = (4 | TA_TSIMPLE);

 static const unsigned int  TA_IMAGE      = (1 | TA_TMAP);

//animations
 static const unsigned int  TA_ATYPE      = 0x1000;
 static const unsigned int  TA_ANIM_VIS   = (1 | TA_ANIMATION | TA_ATYPE);
 static const unsigned int  TA_ANIM_POS   = (2 | TA_ANIMATION | TA_ATYPE);
 static const unsigned int  TA_ANIM_ROT   = (3 | TA_ANIMATION | TA_ATYPE);
 static const unsigned int  TA_ANIM_SCALE = (4 | TA_ANIMATION | TA_ATYPE);
 static const unsigned int  TA_ANIM_SPIN  = (5 | TA_ANIMATION | TA_ATYPE);

//constraints
 static const unsigned int  TA_CTYPE      = 0x1000;

//misc
 static const unsigned int  TA_MTYPE       =(0x1000);
 static const unsigned int  TA_RESOLUTION  =(1 | TA_MISC | TA_MTYPE);
 static const unsigned int  TA_URL         =(2 | TA_MISC | TA_MTYPE );


#endif
