/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Martin Kuball
 * 
 * $Id: w_dlight.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <stdlib.h>
#include <GL/gl.h>

#include "light.h"
#include "w_dlight.h"
#include "t_actions.h"

const MeFloat Rcm[16] = { 0, 0, -2, 0,  0, 1, 0, 0,  1, 0, 0, 0,  0, 0, 0, 1 };



G3DDirectLightShape::G3DDirectLightShape( G3DLight *parent ) :
  G3DWidget( parent ), cylinder_(0.25), cone_(0.4)
{
  cone_.setOrigin( G3DPoint3d(0.25, 0.0, 0) );
  cone_.setScale( G3DVector3d(0.5, 1, 1) );

  type_ = TW_DIRECT_LIGHT;
  reset();
}


void
G3DDirectLightShape::display( int& ) const
{
  glPushAttrib( GL_CURRENT_BIT | GL_LIGHTING_BIT );
  glDisable( GL_LIGHTING );
  //TODO if( selected ) use select color
  //else
  glColor3fv( (const float*)(((G3DLight*)parent_)->getColor()) );
  glPushMatrix();
  glMultMatrixx( (const MeFloat*)trafo_ );
  glMultMatrixx( Rcm );

  cylinder_.shade( 0 );

  G3DMatrix cm;
  cone_.getLocalTrafo( cm );
  glMultMatrixx( (const MeFloat*)cm );
  cone_.shade( 0 );

  glPopMatrix();
  glPopAttrib();
}

void
G3DDirectLightShape::doAction( ActionData &action )
{
  if( action.type == A_SELECT )
    {
      //TODO change color to select color
    }
  else if( action.type == A_FIND_SELECTION )
    {
      //TODO no handles, yet
      ((SelectData&)action).setSelection( parent_ );
    }
}

void
G3DDirectLightShape::reset ()
{
}


//================================END====================================
