/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: w_rotate.h,v 1.2 2001/04/01 09:42:28 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _ROTATION_WIDGET_H_
#define _ROTATION_WIDGET_H_

#include "widget.h"

class G3DObject;
class G3DShape;
class G3DCamera;
class G3DRay;

/// Widget drawing rotation handles.
/** A Widget object is drawn around a selected Object to visualize its
    bounding box and to supply handle to size or rotate the object (?)
*/
class G3DRotateWidget : public G3DWidget
{
public:
  /// Constructor taling the parent object as argument.
  G3DRotateWidget ( G3DShape* );
  
  /// Display the widget on the screen.
  /** Display the widget on the screen. This will issue the necessary OGL 
      commands.
  */
  virtual void display ( int& ) const;
  
  ///
  void getAxis ( G3DVector3d& ) const;

  /// Return the object's center.
  const G3DPoint3d& getCenter () const { return ocenter_; }

  ///
  void getCircle( G3DVector3d&, G3DVector3d& ) const;

  /// Return origin of local coordniate system.
  virtual const G3DPoint3d& getOrigin () const;

  /// Select details of an object (picking ?).
  virtual MeFloat select ( const G3DCamera&, const G3DRay&, G3DPartEnum );

  /// Set origin of local coordinate system.
  virtual void setOrigin ( const TVector<MeFloat,3>& );

protected:
  ///
  void makeCircle ( const G3DPoint3d&, const G3DVector3d&,
		    const G3DVector3d&, G3DPoint3d* );

  // ?
  ///
  void reset ();

protected:
  G3DPoint3d  ocenter_;
  G3DPoint3d  _vertices[16];
  G3DPoint3d  circle1_[16], circle2_[16], circle3_[16];
  G3DVector3d ce1_, ce2_, ce3_;

  GLuint      selHandle_;
};

#endif
