/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Martin Kuball
 * 
 * $Id: w_scale.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdlib.h>
#include <GL/gl.h>

#include "w_scale.h"
#include "shape.h"
#include "t_actions.h"
#include "pointstuff.h"


G3DScaleWidget::G3DScaleWidget( G3DShape *parent ) :
  G3DWidget( parent )
{
  type_      = TW_SCALE;
  selVertex_ = 0;
  reset();
}

void
G3DScaleWidget::calculateVertices(const G3DPoint3d &origin,const G3DVector3d &e1,
			          const G3DVector3d &e2, const G3DVector3d &e3 )
{
  G3DPoint3d  p, *vert;

  vert    = vertices_;
  *vert++ = p = origin;
  *vert++ = p + e1;
  p += e2;
  *vert++ = p;
  *vert++ = p + e1;
  p += e3;
  *vert++ = p;
  *vert++ = p + e1;
  p = origin + e3;
  *vert++ = p;
  *vert++ = p + e1;
}

void
G3DScaleWidget::display( int& ) const
{
  const G3DPoint3d  *vert = vertices_;
  GLint tmpPolygonMode[2];

  glGetIntegerv( GL_POLYGON_MODE, tmpPolygonMode );
  glDisable( GL_LIGHTING );
  glDisable( GL_CULL_FACE );
  glColor3f(1.0, 0, 0);
  glDepthFunc( GL_LEQUAL );
  glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
  glPolygonOffset( -2.0, -1 );
  //glEnable( GL_POLYGON_OFFSET_FILL );

  glBegin( GL_QUAD_STRIP );
  for( int j = 0; j < 8; j++ )
    {
      glVertex3xv( (const MeFloat*)(*vert++) );
    }
  vert = vertices_;
  glVertex3xv( (const MeFloat*)(*vert++) );
  glVertex3xv( (const MeFloat*)(*vert++) );
  glEnd();

  //glDisable( GL_POLYGON_OFFSET_FILL );
  //glPolygonMode( GLenum(tmpPolygonMode[0]), GLenum(tmpPolygonMode[1]) );
  glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
  glEnable( GL_LIGHTING );
  glEnable( GL_CULL_FACE );
  glDepthFunc( GL_LESS );
}

void
G3DScaleWidget::doAction( ActionData &action )
{
  if( action.type == A_SELECT )
    {
    }
  else if( action.type == A_FIND_SELECTION )
    {
      if( ((SelectData&)action).getNextNum(selVertex_) )
	{
	  ((SelectData&)action).setSelection( this );
	}
     }
}

void
G3DScaleWidget::getFrameAtHandle(G3DVector3d &e1, G3DVector3d &e2, G3DVector3d &e3)
{
  static int basisV[8][3] = { {-1,-1,-1}, {1,-1,-1}, {-1,1,-1}, {1,1,-1},
			      {-1,1,1}, {1,1,1}, {-1,-1,1}, {1,-1,1} };
  G3DPoint3d     origin;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, e1, e2, e3 );
  if( e1.isZero() )
    e1 = e2.cross( e3 );
  if( e2.isZero() )
    e2 = e3.cross( e1 );
  if( e3.isZero() )
    e3 = e1.cross( e2 );
  e1 *= basisV[selVertex_][0];
  e2 *= basisV[selVertex_][1];
  e3 *= basisV[selVertex_][2];
}

void
G3DScaleWidget::getHandlePosition( G3DPoint3d &pos ) const
{
  pos = vertices_[selVertex_];
}

const G3DPoint3d&
G3DScaleWidget::getFixpoint () const
{
  //Get the number of the vertex opposite of the one at the index
  static short oppVert[8] = {5, 4, 7, 6, 1, 0, 3, 2};
  return vertices_[ oppVert[selVertex_] ];
}

void
G3DScaleWidget::reset ()
{
  G3DVector3d    e1, e2, e3;
  G3DPoint3d     origin;
  const G3D_BBAligned &bbox( ((G3DShape*)parent_)->getBoundingBox() );

  bbox.getParameter( origin, e1, e2, e3 );
  calculateVertices( origin, e1, e2, e3 );
}

MeFloat
G3DScaleWidget::select( const G3DCamera &cam, const G3DRay &ray, G3DPartEnum )
{
  MeFloat    dist = G3D_INFINITY;
  MeFloat    rad = 9.0;
  G3DMatrix  mw;
  getWorldTrafo( mw );

  selVertex_ = 0;
  int index = findNearestPoint( cam, ray, mw, vertices_, 8, rad );
  if( index >= 0 )
    {
      selVertex_ = index;
      dist       = rad;
    }
  return( dist );
}


//================================END====================================
