/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: widget.h,v 1.3 2001/04/01 09:42:28 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _WIDGET_H_
#define _WIDGET_H_

#include "object.h"

/// Base class for 3d widgets.
class G3DWidget : public G3DObject
{
public:
  /// Constructor.
  G3DWidget ( G3DObject* );

  ///
  virtual G3DObject* copy () const;

  /// Display the widget on the screen.
  /** Display the widget on the screen. This will issue the necessary OGL 
      commands.
      ...
      The parameter is there to make some template code happy.
  */
  virtual void display ( int& ) const = 0;
  
  /// Return the object's local transform (?)
  virtual void getLocalTrafo ( G3DMatrix &m ) const;

  /// Return the object's orientation.
  virtual const G3DQuaternion& getOrientation () const;
  
  /// Return the origin of local coordinate system.
  virtual const G3DPoint3d& getOrigin () const;
  
  /// Return the object's scale.
  virtual const G3DVector3d& getScale () const;

  /// Return the object's world transformation.
  /** Concatenate all parent transformation matrices up to the root of
      the scene. The last matrix is the one of the object itself, if local.
  */
  virtual void getWorldTrafo ( G3DMatrix &m, bool local=true ) const;

  /// Recalculate the widget from its parent s bounding box.
  virtual void reset ();

  /// Set object orientation.
  /** Set the orientation to q and modify the origin to keep the pivot
      point fixed. The given pivot p (in the parent coordinate system) and
      the origin q are the values for q = (1,0,0,0) 
  */
  virtual void setOrientation ( const G3DQuaternion&,
				const TVector<MeFloat,3>&,
				const TVector<MeFloat,3>&);

  /// Set origin of local coordinate system.
  virtual void setOrigin ( const TVector<MeFloat,3>& );

  /// Set the object's scale.
  virtual void setScale ( const TVector<MeFloat,3>& );
};

#endif
