/*
 * Project : "MindsEye, dialogs"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: renderDlg.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */


#include <qapp.h>
#include <qfiledlg.h>
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
//#include <string.h>

#include "renderDlg.h"
#include "output/renderqueue.h"
#include "output/renderjob.h"
#include "output/smpte-time.h"
#include "kernel/core.h"


#define Inherited MERenderDialogData

//TODO get this from configuration
static const char *ResolutionList[] =
{ "320x240", "640x480", "800x600", "NTSC", "PAL" };

static const char *QueueList[] =
{ "locale BMRT", "locale Povray", 0 };

static const char *FormatList[] =
{ "PPM", "JPEG", "PNG", "MPEG I", "QuickTime", "JPG Sequence",
  "FLI/FLC", "AVI movie", 0 };


//---------------------------------------------------------


MERenderDialog::MERenderDialog () :
  Inherited( NULL, NULL )
{
  queue_ = NULL;
  world_ = NULL;

  setCaption( "MindsEye Render Dialog" );
  resolution_->insertStrList( ResolutionList );
  outputFormat_->insertStrList( FormatList );
  setQueueList();

  fpsField_->setText( "12" );
  shutterTime_->setText( "0.0" );
  startTime_->setEnabled( FALSE );
  stopTime_->setEnabled( FALSE );
}

MERenderDialog::~MERenderDialog()
{
}


void
MERenderDialog::addJob()
{
  if( queue_ )
    {
      QApplication::setOverrideCursor( waitCursor );

      int   w, h;      
      if( sscanf( resolution_->currentText(), "%ix%i", &w, &h ) < 2 )
	return;

      MERenderJob *job = new MERenderJob;
      job->setOutput( 0, fileName_->text() ); //TODO set file type
      job->setWorld( world_ );
      job->setResolution( w, h );
      job->setFrameRate( fpsField_->text() );
      //cout << shutterTime_->text() << endl;
      job->setShutterTime( atof(shutterTime_->text()) );

      if( frames_ == Current )
	job->setTimeInterval( SmpteTime(NULL, 0), SmpteTime(NULL, 0) );
      else if( frames_ == All )
	job->setTimeInterval( SmpteTime(NULL, 0), SmpteTime(NULL, 0) );
      else
	job->setTimeInterval( SmpteTime(startTime_->text(), 40),
			      SmpteTime(stopTime_->text(), 40) );

      queue_->addJob( job );
      world_ = NULL;

      QApplication::restoreOverrideCursor();     
      hide();
    }
}

void
MERenderDialog::browseFileName()
{
  QString str;

//TODO get pattern from file format
  str = QFileDialog::getOpenFileName( NULL, "*" );
  //cout << "file: " << (const char*)str << endl;
  fileName_->setText( str );
 // if( !str.isEmpty() )
}

void
MERenderDialog::closeIt()
{
  close();  //TODO or hide?
}

void
MERenderDialog::formatSelected( int )
{
  puts( "format selected" );
}

void
MERenderDialog::newResolution( int nr )
{
  if( nr == 3 )
    fpsField_->setText( "30 H" );
  else if( nr == 4 )
    fpsField_->setText( "25 H" );
}

void
MERenderDialog::queueSelected( int nr )
{
  queue_ = (MERenderQueue*)((Kernel->getRenderQueues())[nr]);
  if( queue_ )
    queueOptions_->setEnabled( queue_->hasOptionDialog() );
}

void
MERenderDialog::renderCurrent( MWorldObject *w )
{
  world_ = w;
  addJob();
}

void
MERenderDialog::setQueueList()
{
  vector<void*> &queues = Kernel->getRenderQueues();
  for( int i = 0; i < queues.size(); ++i )
    {
      queueSelect_->insertItem( ((MERenderQueue*)queues[i])->getName() );
    }
  if( queues.size() > 0 )
    queueSelected( 0 );
}

void
MERenderDialog::setQueueOptions()
{
  if( queue_ )
    queue_->showOptionDialog();
}

void
MERenderDialog::toggleAllFrames( bool onoff )
{
  if( onoff )
    {
      //startTime__->setText( "1-100" );
      startTime_->setEnabled( FALSE );
      stopTime_->setEnabled( FALSE );
      frames_ = All;
    }
}

void
MERenderDialog::toggleCurrentFrame( bool onoff )
{
  if( onoff )
    {
      startTime_->setEnabled( FALSE );
      stopTime_->setEnabled( FALSE );
      frames_ = Current;
    }
}

void
MERenderDialog::toggleSomeFrames( bool onoff )
{
  if( onoff )
    {
      startTime_->setEnabled( TRUE );
      stopTime_->setEnabled( TRUE );
      frames_ = Some;
    }
}


//================================END================================
