/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: ioModule.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * MIODialog implements a dialog widget for loading and
 * saving 3d data in the various supported formats.
 */

#include "include/config.h"

#include <stdio.h>

#include <qcombo.h>
#include <qfiledlg.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlined.h>
#include <qlistbox.h>
#include <qpushbt.h>
#include <qmessagebox.h>

#include "ioModule.h"
#include "io_base.h"
#include "3dlib/s_scene.h"
#include "kernel/core.h"
#include "kernel/world_obj.h"
#include "kernel/world_list.h"

#include <vector>


static vector<void*> *IoFilterList_ = NULL;

inline
MEFltFactory* getFltFactory( int i )
{
  //i = index in IoFilterList
  return (MEFltFactory*)(*(IoFilterList_->begin() + i));
}


/*---------------------------------------------------------------------------*/

MEIoModule::MEIoModule( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MEIoModule" )
{
  root_        = NULL;
  currentType_ = 0;
  selNum_      = 0;
  strcpy( m_info->module_name, "Dialog module" );

  setCaption( "MindsEye IO Module" );

  QVBoxLayout *gm    = new QVBoxLayout( this, 6 );
  QHBoxLayout *gm_00 = new QHBoxLayout;
  QHBoxLayout *gm_01 = new QHBoxLayout;
  QHBoxLayout *gm_02 = new QHBoxLayout;
  QHBoxLayout *gm_03 = new QHBoxLayout;

  gm->addLayout( gm_00, 1 );
  QLabel *label = new QLabel( "List of loaded scenes", this );
    label->setFixedSize( label->sizeHint() );
    gm_00->addWidget( label, 1 );
    gm_00->addStretch( 100 );

  worldList_ = new QListBox( this );
    worldList_->setMinimumSize( 50, 70 );
    gm->addWidget( worldList_, 100 );
    connect( worldList_, SIGNAL(selected(int)), SLOT(selectWorld(int)) );
    connect( worldList_, SIGNAL(highlighted(int)), SLOT(selectWorld(int)) );

  gm->addLayout( gm_01, 1 );
  label = new QLabel( "Filename:", this );
    label->setFixedSize( 60, 25 );
    gm_01->addWidget( label, 1 );
  fnEdit_ = new QLineEdit( this );
    fnEdit_->setMinimumSize( 50, 25 );
    gm_01->addWidget( fnEdit_, 100 );
  QPushButton *browse = new QPushButton( "browse", this );
    browse->setFixedSize( 50, 25 );
    connect( browse, SIGNAL(clicked()), SLOT( fileDialog()) );
    gm_01->addWidget( browse, 1 );

  gm->addLayout( gm_02, 1 );
  label = new QLabel( "File format:", this );
    label->setFixedSize( 60, 25 );
    gm_02->addWidget( label, 1 );
  QComboBox *combo = new QComboBox( FALSE, this );
    combo->setMinimumSize( 50, 25 );
    combo->insertItem( "automatic" );
    for( int i = 0; i < IoFilterList_->size(); ++i )
      {
	combo->insertItem( getFltFactory(i)->name );
      }
    connect( combo, SIGNAL(activated(int)), SLOT(activated(int)) );
    gm_02->addWidget( combo, 100 );
    
  //>------------- 

  gm->addLayout( gm_03, 1 );
  QPushButton *quitButton = new QPushButton( "Close module", this );
    quitButton->setMinimumSize( 80, 25 );
    connect (quitButton, SIGNAL(clicked()), this, SLOT(closeIt()) );
    gm_03->addWidget( quitButton, 1 );

  gm_03->addStretch( 100 );

  QPushButton *pbutton;
  pbutton = new QPushButton( "load", this );
    pbutton->setMinimumSize( 50, 25 );
    connect( pbutton, SIGNAL(clicked()), this, SLOT(load()) );
    gm_03->addWidget( pbutton, 1 );
  pbutton = new QPushButton( "save", this );
    pbutton->setMinimumSize( 50, 25 );
    connect( pbutton, SIGNAL(clicked()), this, SLOT(save()) );
    gm_03->addWidget( pbutton, 1 );
  pbutton = new QPushButton( "close", this );
    pbutton->setMinimumSize( 50, 25 );
    connect( pbutton, SIGNAL(clicked()), this, SLOT(remove()) );
    gm_03->addWidget( pbutton, 1 );

  //>------------- 

  setWorldList();
  connect( Kernel, SIGNAL(worldListChanged()), SLOT(setWorldList()) );
  resize( 300, 250 );
  gm->activate();
}

MEIoModule::~MEIoModule ()
{
}

/*--------------------------------------------------------------------------*/

int
MEIoModule::AddFltFactory( MEFltFactory *ff )
{
  IoFilterList_->push_back( (void*)ff );
}

void
MEIoModule::activated( int nr )
{
  currentType_ = nr;
}

void
MEIoModule::closeIt()
{
  close( );
}

void
MEIoModule::load()
{
  const char* fname = fnEdit_->text();

  if( *fname != '\0' )
    {
      MEFilter *ioObj = GetIoFilter( currentType_, fname );
      if( ioObj == NULL )
	return;

      MWorldList &worlds = Kernel->getWorldList();
      if( root_ == NULL )
	{
	  G3DScene     *scene = new G3DScene(1);
	  MWorldObject *world = new MWorldObject( scene );
	  if( ioObj->file_load( world, fname ) == READ_ERROR )
	    {
	      delete world;
	      return;
	    }
	  const char *tmp = strrchr( fname, '/' );
	  if( tmp == NULL )
	    tmp = fname;
	  else
	    tmp++;
	  scene->setName( tmp );
	  world->setFileName( fname );
	  Kernel->addWorld( world );
	}
      else
	{
	  ioObj->import( root_, fname );
	  emit objectsImported();  //TODO
	}
    }
}

void
MEIoModule::fileDialog( )
{
  QString str;
  const char *extension;

  if( currentType_ == 0 )
    extension = "*";
  else
    extension = getFltFactory(currentType_-1)->fextension;

  str = QFileDialog::getOpenFileName( NULL, extension );
  if( !str.isEmpty() )
    fnEdit_->setText( str );
}

MEFilter*
MEIoModule::GetIoFilter( int type, const char *str )
{
  //type = index of filter in combo-box
  if( type == IO_AUTOMATIC && str )
    {
      const char *ctype = strrchr( str, '.' );
      if( ctype == NULL )
	return( NULL );
      ctype++;
      for( int i = 0; i < IoFilterList_->size(); i++ )
	{
	  if( strcmp(ctype, getFltFactory(i)->fextension + 2) == 0 )
	    {
	      type = i;
	      break;
	    }
	}
    }
  else
    --type;

  if( type < IoFilterList_->size() )
    return( getFltFactory(type)->createFilter() );
  else
    return( NULL );
}

MEFilter*
MEIoModule::GetIoFilter( int typeId )
{
  for( int i = 0; i < IoFilterList_->size(); i++ )
    {
      if( getFltFactory(i)->getID() == typeId )
        return( getFltFactory(i)->createFilter() );
    }
  return NULL;
}

void
MEIoModule::InitStandardFilter ()
{
  if( IoFilterList_ == NULL )
    {
      IoFilterList_ = new vector<void*>;

      IoFilterList_->push_back( getFltFactoryMe3d() );
      IoFilterList_->push_back( getFltFactoryPov() );  //TODO
    }
}

void
MEIoModule::remove( )
{
  if( selNum_ >= 0 )
    {
      if( QMessageBox::information(this, "MindsEye", "Really remove scene?",
				   "Yes", "No", NULL, 1, 0) == 0 )
	{
	  Kernel->removeWorld( selNum_ );
	  /* TODO
cout << "remove scene" << endl;
	  MWorldList &worlds  = Kernel->getWorldList();
	  MWorldObject *world = worlds[selNum_];
	  //if( world->isEdited() )
	  */
	}
    }
}

void
MEIoModule::save()
{
  const char* fname = fnEdit_->text();
  //TODO test if file exists
  if( *fname != '\0' )
    {
      MEFilter *ioObj = GetIoFilter( currentType_, fname );
      if( ioObj == NULL )
	return;

      MWorldList &worlds = Kernel->getWorldList();
      if( worlds.size() >= selNum_ )
	{
	  ioObj->file_save( worlds[selNum_], fname, 0 );
	  const char *tmp = strrchr( fname, '/' );
	  if( tmp == NULL )
	    tmp = fname;
	  else
	    tmp++;
	  ((worlds[selNum_])->getScene())->setName( tmp );
	  (worlds[selNum_])->setFileName( fname );
	  Kernel->addWorld( NULL );  //TODO
	}
      delete ioObj;
    }
}

void
MEIoModule::saveWorld( MWorldObject *world )
{
  const char *fname = world->getFileName();
  MEFilter *ioObj = GetIoFilter( IO_AUTOMATIC, fname );  //TODO automatic?

  if( fname && ioObj )
    ioObj->file_save( world, fname, 0 );

  delete ioObj;
}

void
MEIoModule::selectWorld( int num )
{
  if( worldList_->currentItem() != num )
    worldList_->setCurrentItem( num );
  else
    {
      MWorldList &worlds = Kernel->getWorldList();
      const char *fname = worlds[num]->getFileName();
      if( fname )
	fnEdit_->setText( fname );
      selNum_ = num;
    }
}

void
MEIoModule::setWorldList()
{
  MWorldList &worlds = Kernel->getWorldList();
  worldList_->clear();
  worldList_->setAutoUpdate( FALSE );

  MWorldList::iterator wld = worlds.begin();
  while( wld != worlds.end() )
    worldList_->insertItem( ((MWorldObject*)(*(wld++)))->getName() );

  worldList_->setAutoUpdate( TRUE );
  worldList_->repaint();
  selNum_ = 0;
}


//===================================END===============================
