/*
 * Project : "MindsEye, IO library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: ioModule.h,v 1.2 2001/07/09 21:50:20 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __IO_DIALOGS__
#define __IO_DIALOGS__

#include "include/config.h"
#include "kernel/module.h"
#include "kernel/world_list.h"
#include "io_filter.h"

class QLineEdit;
class QListBox;
class G3DGroup;
class MWorldObject;


/**The class provides an interface for storing and accesing IO filter.
   There are some build-in standard filter like the MindsEye format and
   nff.
   Objects of this class provide a GUI to access the IO filter to load and
   store 3d data.
 */
class MEIoModule : public MEModule
{
  Q_OBJECT

public:
  ///
  MEIoModule ( int, int );
  ~MEIoModule ();

  static int AddFltFactory ( MEFltFactory* );
  ///
  static MEFilter* GetIoFilter ( int, const char* );
  static MEFilter* GetIoFilter ( int );
  static void      InitStandardFilter ();
  ///Save World to the file given by world->getFileName ()
  static  void      saveWorld ( MWorldObject* );
  /**Root must be set if you want to import objects from a file.
     The root will become the parent of all objects in the file.*/
  void setRoot ( G3DGroup *root ) { root_ = root; }

public slots:
  void selectWorld ( int );
  
signals:
  ///
  void objectsImported ();
 
private slots:
  ///
  void activated ( int );
  
  ///
  void  closeIt();
  /**Create an io object of the selected type. Call the
     load or save method based on the value of _action.*/
  void load ();
  void remove ();
  void save ();
  void fileDialog ();
  ///Set the list of loaded scenes.
  void setWorldList ();
  
protected:
  QLineEdit *fnEdit_;
  QListBox  *worldList_;
  G3DGroup  *root_;
  int       currentType_;
  int       selNum_;
};

#endif
