/*
 * Project : "MindsEye, 3D library"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: io_base.h,v 1.3 2001/07/09 21:50:20 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERWIE
 * =======
 * Definition of the base class of the IO module
 *
 */

#ifndef __IO_BASE_H_
#define __IO_BASE_H_

#include "io_filter.h"
#include <iosfwd>
#include <map>

class MWorldObject;
class G3DObject;
class G3DGroup;
class G3DScene;
class MEFilter;

typedef MEFilter* THookFkt( void*, ostream& );

/**
The types of the objects that could be written by a filter is not know a priori.
Therfore this class provides a mechanism to add functions that write objects
in a special format.
*/
class MEFltFactory
{
public:
  MEFltFactory() {}
  virtual MEFilter* createFilter() = 0;
  void      addObjectHook( int, THookFkt );
  int       getID () const { return id_; }
  virtual void writeObject ( int id, G3DObject*, ostream& ) = 0;

public:
  const char *name;
  const char *fextension;

protected:
  int id_;
  map<int, void*> objectHooks_;
  //hookList_[id] = func;

};

//ME3DFilter
/**Base class for all classes that implement loading and saving
   of a 3D file format.
*/
class MEFilter
{
protected:
  ///
  MEFilter () {
    factory_    = NULL;
    shapeCount_ = 0;
    lightCount_ = 0;
  }
  
public:
  virtual ~MEFilter ();

  /// TODO return Version if possible?
  virtual int check_header ( char *fname ) = 0;
  
  ///
  int file_load ( MWorldObject*, const char *fname );
  
  ///Save a scene to a file. The flag has additional format information.
  int file_save ( MWorldObject*, const char *fname, int flag );
  
  /// TODO return FormatInfo
  virtual int getFormatInfo ( int type ) = 0;
  
  /// Import objects from a file. They will be added as children of root.
  int import ( G3DGroup *root, const char *fname );
  
  void setFactory ( MEFltFactory* ff ) { factory_ = ff; }
  
protected:
  ///Add default light source and camera if not defined in the loaded file
  virtual int exitLoad ( int );
  
  ///
  virtual int exitSave ( int ) = 0;
  
  ///Open the file and test the format.
  virtual int initLoad ( const char *fname ) = 0;
  
  ///Open the file.
  virtual int initSave ( const char *fname, int ) = 0;
  
  ///
  virtual int loadObjects ( G3DGroup* ) = 0;
  
  ///
  virtual int saveObjects ( G3DScene* ) = 0;

protected:
  MWorldObject *world_;
  MEFltFactory *factory_;
  int  shapeCount_, lightCount_;
};

#endif
