/*
 * Project : "MindsEye"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: core.cpp,v 1.14 2002/06/12 20:13:18 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include "dialogs/about/about_dlg.h"
#include "dialogs/plconfig/plconfig.h"
#include "io/ioModule.h"
#include "material/m_mat.h"
#include "modeler/modeler.h"
//#include "network/m_network.h"
#include "shaper/shaper.h"
#include "treeedit/scene_graph.h"
#include "viewer/m_viewer.h"
#include "include/version.h"
#include "3dlib/t_objects.h"
#include "3dlib/object.h"

#ifdef OCCTEST
#include "occtest/occtestwidget.h"
#endif

#include "world_obj.h"
#include "core.h"
#include "console.h"
#include "kernelWidget.h"
#include "world_list.h"
#include "plugin/dll.h"
#include "initialize.h"
#include <qgl.h>
#include <qwindowsstyle.h>
// #include <qplatinumstyle.h>

static void scanRegion ( pair<MmumapIItr,MmumapIItr> &r, QWidget *p,
                         vector<void*> &list );


//The global visible instance of the kernel
MCore *Kernel = 0;


/*---------------------------------------------------------------------------*/
MCore::MCore ( int argc, char **argv )  //TODO no arguments
{ 
  // init kernel vars
  the_console    = 0;
  clipboard_     = 0;
  worldList_.push_back (new MWorldObject);

  //TODO move this to setup setup_modules() ?
  for (int i = 0; i < NR_MODULES; i++)
    {
      modules [i].load_type      = LOAD_ON_CALL; // cool if user can change it!
      modules [i].the_module     = 0   ;
      modules [i].the_module_inf = 0;
      modules [i].toggled        = false;
    }
  //TODO it should be possible to start more than one instance of a module

  kernel_blocked = false;// now we're ready :)
}

MCore::~MCore ()
{
  puts( "delete mcore" );

  // ---------- shutdown running modules:
  
  for( int i = 1; i < NR_MODULES; i++ ) { // skip console
    if (modules[i].toggled == true) {
      printf ("Killing module: [%s]\n",modules[i].the_module_inf->module_name);
      delete modules[i].the_module;
      delete modules[i].the_module_inf;
    }
  }

  // ---------- ?

  delete pluginMgr_;

  //TODO delete message_system;

  // ---------- delete worlds:

  while (worldList_.size () != 0) {
    delete worldList_[0];
  }
}

/*--------------------------------------------------------------------------*/

void
MCore::addAttributeFactory ( const char *name, AcreateFunc *func )
{
  typename2AttrFactory_[name] = (void *) func;
}

void
MCore::addIoFltFactory ( MEFltFactory *ff )
{
  MEIoModule::AddFltFactory (ff);
}

void
MCore::addMaterialWidget ( TMatWidget *widget )
{
  MEMaterialEditor::AddMaterialWidget (widget);
}

void
MCore::addObjectCategory ( const char* name )
{
  objectCategories_.push_back (name);
}

void
MCore::addPaletteWidget ( uint type, void *func )
{
  paletteWidgets_.insert( pair<uint, void*>(type, func) );
  //TODO what about replace? and what to replace if > 1 entry?
}

void
MCore::addObjectFactory ( const char *name, OcreateFunc *func )
{
  typename2ObjFactory_[name] = (void *) func;
}

void
MCore::addObjectName ( const char *cat, const char *desc, const char *type )
{
  category2Desc_.insert( pair<const char*, const char*>(cat, desc) );
  desc2Typename_[desc] = type;
}

void
MCore::addRenderQueue ( MERenderQueue *queue )
{
  renderQueues_.push_back( queue );  //TODO
}

void
MCore::addScriptingHook ( ExecuteScriptFunc *f )
{
  execScriptFunc_ = f;
}

void
MCore::addWorld ( MWorldObject* data )
{
  the_console->m_puts ("A world was loaded");
  if (data)
    worldList_.push_back (data);
  emit worldListChanged();
}

/*--------------------------------------------------------------------------*/

void
MCore::close_module ( int module_id )
{
  if (modules[module_id].load_type == LOAD_ON_CALL) {
    the_console->m_printf ("Module [%s] is shutting down",
			   modules [module_id].the_module_inf->module_name);
    delete modules[module_id].the_module;
    modules[module_id].the_module     = 0;
    modules[module_id].the_module_inf = 0;
    modules[module_id].toggled        = false;
  }
  else {
    the_console->m_printf("Module [%s] is closing",
			  modules [module_id].the_module_inf->module_name);
    modules[module_id].the_module->hide ();
    modules[module_id].toggled = false;
  }
}

G3DAttribute*
MCore::createAttribute ( const char *type )
{
  map<const char*, void*, ltstr>::iterator itr
    = typename2AttrFactory_.find (type);
  
  if (itr != typename2AttrFactory_.end())
    return ((AcreateFunc*)((*itr).second))();
  else
    return 0;
}

G3DObject*
MCore::createObject ( const char *type, MeFloat scale )
{
  map<const char*, void*, ltstr>::iterator itr
    = typename2ObjFactory_.find (type);
  if (itr != typename2ObjFactory_.end())
    return (((OcreateFunc*)((*itr).second))(scale));
  else
    return 0;
}

const char**
MCore::getAttributeNames ()
{
  const char **ret = new (const char*)[typename2AttrFactory_.size()+1];
  const char **tmp = ret;
  map<const char*, void*, ltstr>::iterator it =
    typename2AttrFactory_.begin();
  while( it != typename2AttrFactory_.end() )
    *(tmp++) = (*(it++)).first;
  *tmp = 0;
  return ret;
}

vector<const char*>&
MCore::getObjectCategories ()
{
  return objectCategories_;
}

pair<MmumapItr, MmumapItr>
MCore::getObjectList ( const char *category ) const
{
  return category2Desc_.equal_range( category );
}

const char*
MCore::getObjTypename ( const char *desc )
{
  map<const char*, const char*, ltstr>::iterator itr =
    desc2Typename_.find( desc );
  if( itr != desc2Typename_.end() )
    return( (const char*)(itr->second) );
  else
    return NULL;
}

void
MCore::getPaletteWidgets ( uint type, QWidget *p, vector<void*> &list )
{
  pair<MmumapIItr, MmumapIItr> region =
    paletteWidgets_.equal_range( GET_CLASS(type) );
  scanRegion( region,  p, list );
  region = paletteWidgets_.equal_range( GET_TYPE(type) );
  scanRegion( region,  p, list );
  region = paletteWidgets_.equal_range( GET_SUBTYPE(type) );
  scanRegion( region,  p, list );
  region = paletteWidgets_.equal_range( type );
  scanRegion( region, p, list );
}

void
scanRegion( pair<MmumapIItr,MmumapIItr> &r, QWidget *p, vector<void*> &list )
{
  typedef void PaletteFrameFactory( vector<void*>&, QWidget* );

  MmumapIItr &itr = r.first;
  while( itr != r.second ) {
    PaletteFrameFactory *pff;
    pff = (PaletteFrameFactory*)((*itr).second);
    pff( list, p );
    ++itr;
  }
}

vector<void*>&
MCore::getRenderQueues ()
{
  return renderQueues_;
}

MCore*
MCore::loadKernel ( int argc, char **argv )
{
  if (Kernel == 0)
    Kernel = new MCore (argc, argv);
  
  // go through the various setup stages
  Kernel->setup_app (argc, argv);
  // Kernel->setup_gl (); // ?
  Kernel->setup_chooser ();  //TODO setup_modules() ?
  Kernel->setup_console ();  //  "
  Kernel->setup_sns ();
  Kernel->setup (); 
  
  return Kernel;
}

void
MCore::createNewScene ()
{
  MWorldObject *tmp = new MWorldObject;
  addWorld (tmp);
  //TODO return tmp ?
}

const char*
MCore::executeScript ( const char *sc )
{
  return execScriptFunc_ ? execScriptFunc_ (sc) : 0;
}

MEModule*
MCore::getModule ( int id )
{
  if ((id >= 0) && (id < NR_MODULES)) {
    if( modules[id].the_module == NULL )
      toggle_module (id);
    return( modules[id].the_module );
  }
  return 0;
}

void
MCore::importObjects ( G3DGroup *grp )
{
  toggle_module( MODULE_LOAD );
  ((MEIoModule*)(modules[MODULE_LOAD].the_module))
    ->setRoot( grp );
}

void
MCore::loadScene ()
{
  (getModule (MODULE_LOAD))->raise();
  //toggle_module( MODULE_LOAD );
  //TODO return the new scene?
}

void
MCore::removeWorld ( int index )
{
  the_console->m_puts( "removing world" );
  if (index < worldList_.size()) {
    MWorldObject *w = worldList_[index];
    //delete w; //TODO
    //worldList_.remove( index );
    //emit worldListChanged();
  }
}

void
MCore::saveScene ( int index )
{
  MWorldObject *world = worldList_[index];
  if (world && world->getFileName())
    MEIoModule::saveWorld (worldList_[index]);
  else
    saveSceneAs (index);
}

void
MCore::saveSceneAs ( int index )
{
  //MEIoModule *tmp = (MEIoModule*)(getModule(MODULE_SAVE));
  MEIoModule *tmp = dynamic_cast<MEIoModule*>(getModule(MODULE_SAVE));
  tmp->raise ();
  if (tmp)
    tmp->selectWorld (index);
}

void
MCore::setClipboard ( G3DObject *obj )
{
  if( clipboard_ )
    clipboard_->dispose();
  clipboard_ = obj;
}

void
MCore::shutdown ()
{
  // delete this; // not very elegant.
  qApp->exit_loop /* quit */ ();
}

/*
void
MCore::process_message (struct message *m)
{
  switch (m->message_id)
    {
    case K_MESS_DATAINFO :
      break;
    case K_MESS_MODULES :
      break;
    case K_MESS_STATUS :
      the_console->m_printf( "I'm feeling fine Dave." );
      break; 
    case K_MESS_WHO :
      the_console->m_printf( "Multiuser mode not implemented, yet. Sorry!" );
      break; 
    case K_MESS_QUIT :
      shutdown();
      break;
    case K_MESS_LOADSCENE :
      toggle_module( MODULE_LOAD );
      break;
    case K_MESS_SAVESCENE :
      toggle_module( MODULE_SAVE );
      ((MEIoModule*)(modules[MODULE_SAVE].the_module))
	->selectWorld( int(m->contents) );
      break;
    case K_MESS_IMPORTOBJ :
      toggle_module( MODULE_LOAD );
      ((MEIoModule*)(modules[MODULE_LOAD].the_module))
	->setRoot( (G3DGroup*)(m->contents) );
      break;
    case K_MESS_NEWSCENE :
      {
	MWorldObject *tmp = new MWorldObject;
	addWorld( tmp );
	break;
      }
    default:
      the_console->m_printf ("[Kernel]: Unrecoqnized message id: [%d]",
			     m->message_id);
      break;
    }
}
*/

int MCore::run ()
{
  //TODO test if setup was successful?

#ifdef WITH_KERNEL_WIDGET
  kernel_widget->show();
#endif
  //TODO show only if user wants to
  //toggle_module( MODULE_CONSOLE );
  the_console->m_printf ("Entering message loop........" );
  
  // int result = mindseye_app->exec();
  mindseye_app->enter_loop();
  
  return 0 /* result */;
}

/*---------------------------------------------------------------------------*/

void
MCore::load_module ( int module_id )
{
  //TODO QApplication::setOverrideCursor( waitCursor );

  if (kernel_blocked == false) {  //TODO is this thread safe?
    kernel_blocked = true;
    
    switch (module_id)
      {
      case MODULE_CONFIG :
	// modules[module_id].the_module = new MEPlConfigDlg
	break;   
      case MODULE_SCRIPT :
	break; 
      case MODULE_TREEDLG :
	{
	  MSceneGraphDlg *dlg =
	    new MSceneGraphDlg( module_id,
				modules [module_id].load_type );
	  modules[module_id].the_module = dlg;
	  break;
	}
      case MODULE_MODEL :
	{
	  MModeler *modeler = 
 	    new MModeler( module_id,
			    modules [module_id].load_type );
	  modules[module_id].the_module = modeler;
	  break;
	}
#ifdef OCCTEST
      case MODULE_OCCTEST :
	{
	  occTestWidget = new OccTestWidget( module_id, modules[module_id].load_type);
	  occTestWidget->setGeometry(0,0,600,600);
	  occTestWidget->show();
	  break;
	}
#endif
      case MODULE_MATERIAL :
	modules[module_id].the_module
	  = new MEMaterialEditor( module_id,
				  modules [module_id].load_type );
	break; 
      case MODULE_VIEWER :
	modules[module_id].the_module
	  = new MEViewer (module_id,
			  modules [module_id].load_type);
	break; 
	/*
	  case MODULE_NET :
	  modules[module_id].the_module
	  = new MNetwork (module_id,
	  modules [module_id].load_type);
	  break; 
	*/
      case MODULE_LOAD :
      case MODULE_SAVE :
	modules[module_id].the_module
	  = new MEIoModule (module_id,
			    modules [module_id].load_type);
	break;
      case MODULE_HELP :
	break;
      case MODULE_SHAPER :
	modules[module_id].the_module =
	  new ME2DShaper( module_id, modules[module_id].load_type );
 	break;
      }
    
    kernel_blocked=FALSE;
  }
  
  //TODO QApplication::restoreOverrideCursor();
}

int
MCore::setup ()
{
  the_console->m_printf ("Booting MindsEye v%d.%d.%d ...", MAJOR,MINOR,PATCH);

 // booting message subsystem
  /*TODO
  message_system = new MMessageServer( DEFAULT_QSIZE );
  connect (message_system,
	   SIGNAL (console_output (const char *)),
	   the_console,
	   SLOT (m_puts (const char *)));
  the_console->m_printf ("Message subsystem ready,setting queuesize to [%d]",
			 DEFAULT_QSIZE);
  */
 // message system ready

  the_console->m_printf ("Setting up 3d class library and plugins");
  init3dObjects (this);
  init3dAttributes (this);
  initPaletteWidgets (this);
  MEIoModule::InitStandardFilter();
  initRenderQueues (this);  //TODO move this to 
  MEMaterialEditor::InitDefaultWidgets();

  pluginMgr_ = new MEPluginManager;
  //TODO pluginMgr_->loadDllsFromDir( "system dir", "" );
  //TODO each directory has a loadfile
  pluginMgr_->loadDllsFromDir ("../plugins", "");  //user specified dir
  pluginMgr_->print ();
  
  the_console->m_puts (CopyrightMessage);
  return 0;
}

int
MCore::setup_app ( int argc,char **argv )
{
  mindseye_app = new QApplication (argc, argv);
  mindseye_app->setColorMode (QApplication::CustomColors);
  
  // >------------ set font style
  
  app_font = new QFont ("Helvetica", 10);
  app_font->setBold (false);
  
  // >------------ set widget style to "Windows"
  
  mindseye_app->setFont (*app_font, true);
  QWindowsStyle * style = new QWindowsStyle ();
  mindseye_app->setStyle (style);
  
  // >------------ QApplication for MindsEye should be setup now
  
  return 0;
}

int
MCore::setup_gl ()
{
  cout << "setup gl" << endl;
  
  static QWidget *dummyGL = new QGLWidget (0);
  return 0;
}

int
MCore::setup_chooser ()
{
#ifdef WITH_KERNEL_WIDGET
  kernel_widget = new MKernelWidget( 0, 0 );
  mindseye_app->setMainWidget( kernel_widget );
#else
  kernel_widget = NULL;
#endif
  return 0;
}

int
MCore::setup_console ()
{
  //modules[0].the_module = TODO
  the_console = new MKernelConsole (0, 0);
  the_console->initScripting();  //TODO
#ifndef WITH_KERNEL_WIDGET
  mindseye_app->setMainWidget( the_console );
#endif
  return 0;
}

int
MCore::setup_sns ()
{
#ifdef WITH_KERNEL_WIDGET
  connect( kernel_widget, SIGNAL (toggle_module (int)),
	   this, SLOT (toggle_module (int)) );
#endif
  return 0;
}

void
MCore::toggle_module ( int module_id )
{
  if ((module_id == MODULE_CONSOLE) && (the_console != 0) ) {
    // The console is the only main part which does not use the module
    // services. This is because it should always be available
    // Hmm, point being ?
    if (modules[0].toggled == true) {
      the_console->hide (); 
      modules[0].toggled = false;
    }
    else {
      the_console->show ();
      modules[0].toggled = true;
    }
  }
  else if (module_id == MODULE_ABOUT) {
    MAboutDialog *tmp = new MAboutDialog ();
    tmp->show ();
  }
  else if (module_id == MODULE_CONFIG) {
    MEPlConfigDlg *tmp = new MEPlConfigDlg;
    tmp->show ();
  }
  else {
    // this is the place where the actual behaviour of the modules
    // is determined.
    if (modules[module_id].toggled == TRUE) {
      close_module( module_id );
    }
    else {
      if (modules[module_id].load_type == LOAD_ON_CALL) {
	load_module (module_id);
	if (modules[module_id].the_module == 0)
	  return;
	
	modules[module_id].the_module_inf
	  = modules[module_id].the_module->get_module_info();
	//TODO move this into MModule?
	the_console->m_printf("Module [%s] v%d.%d.%d is running",
			      modules[module_id].the_module_inf->module_name,
			      modules[module_id].the_module_inf->major_version,
			      modules[module_id].the_module_inf->minor_version,
			      modules[module_id].the_module_inf->patch_level);
	connect( modules[module_id].the_module,
		 SIGNAL(close_me(int)),
		 this, SLOT(close_module(int)) );
      }
      else {
	the_console->m_printf( "Module [%s] v%d.%d.%d is shown",
			       modules[module_id].the_module_inf->module_name,
			       modules[module_id].the_module_inf->major_version,
			       modules[module_id].the_module_inf->minor_version,
			       modules[module_id].the_module_inf->patch_level);
      }
      modules[module_id].toggled = TRUE; 
      modules[module_id].the_module->show();
    } 
  } 
}

//====================================END=============================
