/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997. All Rights Reserved.
 * Author(s):  F. Meier, M. Kuball
 * 
 * $Id: init_obj.cpp,v 1.4 2001/07/17 02:34:38 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include "3dlib/light.h"
#include "3dlib/s_group.h"
#include "3dlib/s_scene.h"
#include "3dlib/s_simple.h"
#include "3dlib/s_trigrid.h"
//#include "3dlib/s_geosphere.h"
#include "3dlib/s_nurbscurve.h"
#include "3dlib/s_mesh.h"
#include "3dlib/s_nurbssurf.h"
#include "3dlib/s_nurbssor.h"
#include "3dlib/s_polygon.h"
#include "3dlib/s_torus.h"
#include "3dlib/h_skeleton.h"

#include "shaper/shaper.h"

#include "core.h"
#include "initialize.h"

#include <qgl.h>

//G3DShape* instead?
G3DObject* createCurve2 ( MeFloat );
G3DObject* createTrigrid ( MeFloat );
G3DObject* meshFromShaper ( MeFloat );
G3DObject* polygonFromShaper ( MeFloat );

#define OBJ_CLASSES 6
static const char *ObjCategories[OBJ_CLASSES] = 
{ "simple",
  "nurbs",
  "user defined",
  "special",
  "lights",
  "helpers"
};

static const char *str1 = "uCurve";
static const char *str2 = "uTrigrid";
static const char *str3 = "uPolygon";
static const char *str4 = "uMesh";


#define ADD_OBJ(class_name, cat, desc) \
  kernel->addObjectFactory(class_name::GetTypeName(),class_name::factory );\
  kernel->addObjectName( cat, desc, class_name::GetTypeName() );

#define ADD_OBJ_FAC(class_name) \
  kernel->addObjectFactory(class_name::GetTypeName(),class_name::factory );


void
init3dObjects ( MCore *kernel )
{
  for( int i = 0; i < OBJ_CLASSES; ++i )
    Kernel->addObjectCategory (ObjCategories[i]);
  
  const char *cat = ObjCategories[0];
  ADD_OBJ (G3DS_Box, cat, "Box")
  ADD_OBJ (G3DS_Cone, cat, "Cone")
  ADD_OBJ (G3DS_Cylinder, cat, "Cylinder")
  // ADD_OBJ_FAC( G3DGeoSphere )
  ADD_OBJ_FAC (G3DGroup)
  ADD_OBJ_FAC (G3DMesh)
  ADD_OBJ_FAC (G3DNurbsCurve)
  ADD_OBJ_FAC (G3DNurbsSurface)
  ADD_OBJ_FAC (G3DPolygon)
  ADD_OBJ_FAC (G3DRotationSurface)
  ADD_OBJ (G3DS_Sphere, cat, "Sphere")
  ADD_OBJ_FAC (G3DScene)
  ADD_OBJ (G3DTorus, cat, "Torus")
  ADD_OBJ_FAC (G3DTriGrid)

  kernel->addObjectFactory (str4, meshFromShaper);
  kernel->addObjectName (cat, "mesh from 2dshaper", str4);
  kernel->addObjectFactory (str3, polygonFromShaper);
  kernel->addObjectName (cat, "Polygon from 2dshaper", str3);
  kernel->addObjectFactory (str2, createTrigrid);
  kernel->addObjectName (cat, "Trigrid", str2);
  kernel->addObjectFactory (str1, createCurve2);
  kernel->addObjectName (ObjCategories[1], "curve from 2dshaper", str1);

  // cat = ObjCategories[];
  // ADD_OBJ_FAC( G3DSkeleton );

  cat = ObjCategories[4];
  ADD_OBJ_FAC (G3DAmbientLight)
  ADD_OBJ (G3DDirectionalLight, cat, "directional light")
  ADD_OBJ (G3DPointLight, cat, "point light")
}


G3DObject*
createCurve2 ( MeFloat )
{
  ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule(MODULE_SHAPER));
  return tmp ? tmp->getNurbsCurve() : 0; // TODO: or via emit signal ?
}

G3DObject*
createTrigrid ( MeFloat scale )
{
  G3DTriGrid *tmp = new G3DTriGrid (scale);
  tmp->resize (6, 5);
  return tmp;
}

G3DObject*
meshFromShaper ( MeFloat )
{
  ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule(MODULE_SHAPER));
  return tmp ? tmp->getMesh() : 0; 
}

G3DObject*
polygonFromShaper ( MeFloat )
{
  ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule(MODULE_SHAPER));
  return tmp ? tmp->getPolygon() : 0;
}
