/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen
 * 
 * $Id: kernelWidget.cpp,v 1.5 2002/06/17 18:18:14 ainvar Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * The is the main view of the MindsEye kernel, this widgets signals
 * if modules have to be loaded and executed.
 */

#include "include/config.h"
#include <qapp.h>
#include <qpushbt.h>
#include <qbttngrp.h>
#include "kernelWidget.h"
#include "include/m_vars.h"
#include "core.h"

#ifdef WITH_KMAP_ICONS
#include <qpixmap.h>
#include "icons/mk_icons.h"
#include "icons/mb_icons.h"
#endif


MKernelWidget::MKernelWidget( QWidget *parent=0, const char *name=0 ) :
  QWidget(parent, name)
{
  setCaption ("MindsEye Kernel");
  init();

#ifdef WITH_KMAP_ICONS
  QPixmap background;
  background.loadFromData( backgr_png_data, backgr_png_len );
  setBackgroundPixmap( background );
  resize( 343,185 );
  setMinimumSize( 343,185 );
  setMaximumSize( 343,185 );
#else
  setMinimumSize( 96,298 );
  setMaximumSize( 96,298 );
  resize( 96,298 );
#endif
}

void
MKernelWidget::init ()
{
  QButtonGroup *bgroup = new QButtonGroup;
  connect( bgroup, SIGNAL(clicked(int)), this, SLOT(toggleModule(int)) );

  QPushButton* pbtn;

#ifdef WITH_KMAP_ICONS
  QPixmap my_pix;

  pbtn = new QPushButton( this, "PushButton_1" );
  my_pix.loadFromData(load_png_data,load_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_LOAD  );

  pbtn = new QPushButton( this, "PushButton_2" );
  my_pix.loadFromData(save_png_data,save_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_SAVE  );

  pbtn = new QPushButton( this, "PushButton_3" );
  my_pix.loadFromData(modeler_png_data,modeler_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_4" );
  my_pix.loadFromData(d_tree_png_data,d_tree_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 1, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_TREEDLG  );

  pbtn = new QPushButton( this, "PushButton_5" );
  my_pix.loadFromData(material_png_data,material_png_len);
  //pbtn->setPixmap( my_pix );
  pbtn->setText( "Shaper" );
  pbtn->setGeometry( 45, 120, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_SHAPER );

  pbtn = new QPushButton( this, "PushButton_6" );
  my_pix.loadFromData(material_png_data,material_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MATERIAL );

  pbtn = new QPushButton( this, "PushButton_7" );
  my_pix.loadFromData(viewer_png_data,viewer_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 150, 120, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_VIEWER );

  pbtn = new QPushButton( this, "PushButton_8" );
  my_pix.loadFromData(console_png_data,console_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 41, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_CONSOLE );

  pbtn = new QPushButton( this, "PushButton_9" );
  my_pix.loadFromData(config_png_data,config_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_CONFIG );

  pbtn = new QPushButton( this, "PushButton_10" );
  my_pix.loadFromData(title_png_data,title_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 45, 1, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_ABOUT );

  pbtn = new QPushButton( this, "PushButton_11" );
  my_pix.loadFromData(help_png_data,help_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 256, 160, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_MODEL  );



// quit
  pbtn = new QPushButton( this, "PushButton_12" );
  my_pix.loadFromData(quit_png_data,quit_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 1, 160, my_pix.width (),my_pix.height ());
  connect (pbtn, SIGNAL (clicked()), Kernel /*qApp*/, 
	   SLOT (shutdown()/*quit()*/));

#ifdef OCCTEST
// open cascade test
  pbtn = new QPushButton( this, "PushButton_13" );
  my_pix.loadFromData(occ_png_data,occ_png_len);
  pbtn->setPixmap( my_pix );
  pbtn->setGeometry( 255, 80, my_pix.width (),my_pix.height ());
  bgroup->insert( pbtn, MODULE_OCCTEST);
#endif


#else

  pbtn = new QPushButton( this, "PushButton_1" );
  pbtn->setGeometry( 0, 0, 96, 24 );
  pbtn->setText( "Load" );
  bgroup->insert( pbtn, MODULE_LOAD  );

  pbtn = new QPushButton( this, "PushButton_2" );
  pbtn->setGeometry( 0, 24, 96, 24 );
  pbtn->setText( "Save" );
  bgroup->insert( pbtn, MODULE_SAVE  );

  pbtn = new QPushButton( this, "PushButton_3" );
  pbtn->setGeometry( 0, 50, 96, 24 );
  pbtn->setText( "Modeler" );
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_4" );
  pbtn->setGeometry( 0, 74, 96, 24 );
  pbtn->setText( "Hierarchy Editor" );
  bgroup->insert( pbtn, MODULE_TREEDLG  );

  pbtn = new QPushButton( this, "PushButton_5" );
  pbtn->setGeometry( 0, 98, 96, 24 );
  pbtn->setText( "Shaper" );
  bgroup->insert( pbtn, MODULE_SHAPER );

  pbtn = new QPushButton( this, "PushButton_6" );
  pbtn->setGeometry( 0, 124, 96, 24 );
  pbtn->setText( "Materials" );
  bgroup->insert( pbtn, MODULE_MATERIAL );

  pbtn = new QPushButton( this, "PushButton_7" );
  pbtn->setGeometry( 0, 148, 96, 24 );
  pbtn->setText( "Viewer" );
  bgroup->insert( pbtn, MODULE_VIEWER );

  pbtn = new QPushButton( this, "PushButton_8" );
  pbtn->setGeometry( 0, 174, 96, 24 );
  pbtn->setText( "Console" );
  bgroup->insert( pbtn, MODULE_CONSOLE );

  pbtn = new QPushButton( this, "PushButton_9" );
  pbtn->setGeometry( 0, 198, 96, 24 );
  pbtn->setText( "Preferences" );
  bgroup->insert( pbtn, MODULE_CONFIG );

  pbtn = new QPushButton( this, "PushButton_10" );
  pbtn->setGeometry( 0, 224, 96, 24 );
  pbtn->setText( "About" );
  bgroup->insert( pbtn, MODULE_ABOUT );

  pbtn = new QPushButton( this, "PushButton_11" );
  pbtn->setGeometry( 0, 248, 96, 24 );
  pbtn->setText( "Help" );
  bgroup->insert( pbtn, MODULE_MODEL  );

  pbtn = new QPushButton( this, "PushButton_12" );
  pbtn->setGeometry( 0, 274, 96, 24 );
  pbtn->setText( "Quit" );
  connect( pbtn, SIGNAL (clicked()), qApp, SLOT (quit()) );

#ifdef OCCTEST
  pbtn = new QPushButton( this, "PushButton_13" );
  pbtn->setGeometry( 0, 300, 96, 24 );
  pbtn->setText( "occtest" );
  bgroup->insert( pbtn, MODULE_OCCTEST);
#endif

#endif
}

void
MKernelWidget::toggleModule( int id )
{
  emit toggle_module( id );
}

 
//===================================END=================================
