/*
 * Project : "MindsEye"
 * Copyright 1996 - 1998. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: m_mat.cpp,v 1.4 2001/07/20 12:11:06 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"

//#include <qbitmap.h>
#include <qframe.h>
#include <qlayout.h>
//#include <qgrpbox.h>
#include <qlabel.h>
#include <qlined.h>
#include <qcombo.h>
#include <qpushbt.h>

#include <vector>

#include "3dlib/Attributes/texture.h"
#include "m_mat.h"
#include "mat_widget.h"
//#include "kernel/m_core.h"
#include "include/cursors.h"
#include "m_matv.h"
#include "previewer.h"

vector<void*> MEMaterialEditor::MaterialWidgets_;

/*---------------------------------------------------------------------------*/

MEMaterialEditor::MEMaterialEditor( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MaterialWidget")
{
  resize (400, 350);
  createPreviews ();
  label_ = new QLabel ("Choose a material type", this);
    label_->setAlignment (AlignHCenter|AlignVCenter);
    label_->setFixedSize (label_->sizeHint());
  selType_ = new QComboBox (this);
    selType_->setMinimumSize (40, 24);
    connect( selType_, SIGNAL(activated(int)), SLOT(typeSelected(int)) );
    //vector<void*> &widgets = Kernel->getAttributeWidgets();
    for (int i = 0; i < MaterialWidgets_.size(); ++i)
      selType_->insertItem( ((TMatWidget*)MaterialWidgets_[i])->name);

  closeButton_ = new QPushButton ("Close", this, "Close");
    closeButton_->setMinimumSize (80, 25);
    connect( closeButton_, SIGNAL(clicked()), SLOT(closeIt()) );

  gmMain_    = 0;
  gmh1_      = 0;
  gmh2_      = 0;
  matWidget_ = 0;
  createLayout (0);
  active_ = 0;

  setActivePreview (0);
  setCaption ("MindsEye Material editor");
}

MEMaterialEditor::~MEMaterialEditor ()
{
}

/*---------------------------------------------------------------------------*/

void
MEMaterialEditor::closeIt ()
{
  close ();
}

void
MEMaterialEditor::createLayout ( int id )
{
  cout << "create Layout " << id << endl;

  //delete gmh1_;
  //delete gmh2_;
  delete gmMain_;
  delete matWidget_;

  cout << "...deleted old stuff." << endl;

  gmMain_ = new QVBoxLayout( this, 4, 4 );
  gmMain_->addWidget (previewFrame_, 0);

  gmh1_ = new QHBoxLayout ();
  gmMain_->addLayout (gmh1_, 0);

  gmh1_->addWidget (label_, 2);
  //gmh1_->addStretch (2);
  gmh1_->addWidget (selType_, 96);

  //vector<void*> &widgets = Kernel->getAttributeWidgets();
  MWCreator     *mwFunc;
  mwFunc     = ((TMatWidget*)MaterialWidgets_[id])->factory;
  matWidget_ = mwFunc (this);
  matWidget_->setMinimumSize (10, matWidget_->height());
  matWidget_->show ();
  connect( matWidget_, SIGNAL(changed(const G3DATexture*)),
	   SLOT(setMaterial(const G3DATexture*)) );
  gmMain_->addWidget (matWidget_, 0);

  gmMain_->addStretch (100);

  gmh2_ = new QHBoxLayout ();
  gmMain_->addLayout (gmh2_, 0);
  gmh2_->addStretch (100);
  gmh2_->addWidget (closeButton_, 0);

  gmMain_->activate ();
  resize (width(), 350);  //TODO
  update ();
}

void
MEMaterialEditor::createPreviews ()
{
 //>------- create cursors for the viewports
  //QBitmap *cb1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_cur_bits, FALSE);
  //QBitmap *cm1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_msk_bits, FALSE);
  //QCursor *hand_cursor = new QCursor (*cb1,*cm1);

  printf("creating previews\n");

  previewer_ = new MRaytracingPreviewer (60, 60);
  {
    G3DAColor deftexture;
    previewer_->renderPixmapWith (&deftexture);
  }
  previewFrame_ = new QFrame (this, 0);
  //previewFrame_->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
  previewFrame_->setMinimumSize (100, 100);
  
  QGridLayout *gm = new QGridLayout (previewFrame_, 2, 5, 4, 4);
  gm->addRowSpacing (0, 70);
  gm->addRowSpacing (1, 20);
  
  QLineEdit *name;
  for (int i = 0; i < 5; ++i) {
    preView_[i] = new MEMaterialViewport (i, previewer_, previewFrame_);
    connect ( preView_[i], SIGNAL(activated(int)),
	      SLOT(setActivePreview(int)) );
    gm->addWidget (preView_[i], 0, i);
    
    name = new QLineEdit (previewFrame_);
    name->setText ("Material");  //TODO connect
    gm->addWidget (name, 1, i);
  }
  gm->activate ();
  
  active_ = 0;
  preView_[0]->setFrameStyle (QFrame::Panel | QFrame::Plain); 
}

void
MEMaterialEditor::editMaterial (G3DAttribute*)
{
}

const G3DATexture*
MEMaterialEditor::getMaterial ( int i ) const
{
  if ((i < 0) || (i > 4))
    i = 0;
  return (preView_[i]->getMaterial());
}

struct module_info*
MEMaterialEditor::get_module_info ()
{
  // enter this module's specific information and give it back to the kernel
  strcpy (m_info->module_name, "Material Editor");
  return m_info;
}

void
MEMaterialEditor::setActivePreview ( int num )
{
  if (num != active_) {
    const char *atName = preView_[num]->getMaterial()->getTypeName();
    cout << atName << endl;
    //vector<void*> &widgets = Kernel->getAttributeWidgets();
    //int i;
    TMatWidget* tmwidget;
    for (int i = 0; i < MaterialWidgets_.size(); ++i) {
      tmwidget = (TMatWidget*)MaterialWidgets_[i];
      if (tmwidget) {
        cout << tmwidget->aname << endl;
        if(tmwidget->aname == atName) {
          createLayout (i);
          selType_->setCurrentItem (i);
          break;
        }
      }
    }
    cout << "emitting materialChanged" << endl;
    emit materialChanged (preView_[num]->getMaterial());
    preView_[active_]->setFrameStyle (QFrame::Panel | QFrame::Sunken);
    preView_[active_]->repaint (false);
    active_ = num;
    preView_[num]->setFrameStyle (QFrame::Panel | QFrame::Plain);
    preView_[num]->repaint (false);
  }
}

void
MEMaterialEditor::setMaterial ( const G3DATexture *material )
{
  preView_[active_]->setMaterial (material);
}

void
MEMaterialEditor::setPreviewer ()
{
  /*
  delete previewer_;
  previewer_ = x;
  for( int i = 0; i < 5; ++i )
    {
      preView_[i]->setPreviewer( previewer_ );
    }
  */
}

void
MEMaterialEditor::typeSelected ( int id )
{
  createLayout (id);
  //setMaterial ( matWidget_->getMaterial() );
}


//----------------------------------------------------------------------


void
MEMaterialEditor::AddMaterialWidget ( TMatWidget *widget )
{
  if (widget) {
    cout << "adding material" << endl;
    MaterialWidgets_.push_back (widget);
  }
}


#include "matw_generic.h"
#include "matw_mtl.h"
#include "matw_plastic.h"

//TODO move this inot matw_*.cpp
QFrame* createGenericWidget( QWidget* );
QFrame* createMetalWidget( QWidget* );
QFrame* createPlasticWidget( QWidget* );


void
MEMaterialEditor::InitDefaultWidgets ()
{
  TMatWidget *pair;

  printf("material editor : initializing default widgets\n");

  pair = new TMatWidget;
  pair->name    = MEMatWidgetGeneric::GetDescription();  //TODO
  pair->aname   = MEMatWidgetGeneric::GetAttributeType();
  pair->factory = createGenericWidget;
  cout << "adding material" << endl;
  MaterialWidgets_.push_back (pair);

  pair = new TMatWidget;
  pair->name    = MEMatWidgetMetal::GetDescription();
  pair->aname   = MEMatWidgetMetal::GetAttributeType();
  pair->factory = createMetalWidget;
  cout << "adding material" << endl;
  MaterialWidgets_.push_back (pair);

  pair = new TMatWidget;
  pair->name    = MEMatWidgetPlastic::GetDescription();
  pair->aname   = MEMatWidgetPlastic::GetAttributeType();
  pair->factory = createPlasticWidget;
  cout << "adding material" << endl;
  MaterialWidgets_.push_back (pair);
}


QFrame*
createGenericWidget ( QWidget *parent )
{
  return new MEMatWidgetGeneric (parent);
}

QFrame*
createMetalWidget ( QWidget *parent )
{
  return new MEMatWidgetMetal(parent);
}

QFrame*
createPlasticWidget ( QWidget *parent )
{
  return new MEMatWidgetPlastic(parent);
}


//====================================END===================================
