/*
 * Project : "MindsEye"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): M. van Velsen, M. Kuball
 *
 * $Id: m_mat.h,v 1.3 2001/07/12 00:47:11 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __MMATERIAL_H__
#define __MMATERIAL_H__

#include <qwidget.h>
#include <vector>
#include "kernel/module.h"

class MEMaterialViewport;
class MEPreviewRenderer;
class G3DAttribute;
class G3DATexture;
class TMatWidget;

class QComboBox;
class QFrame;
class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;


/**The material editor consists of 5 preview widgets and a widget
  to edit the texture. The edited texture affecs the selected (active)
  preview widget only.
  */
class MEMaterialEditor : public MEModule
{
  Q_OBJECT

public:
  MEMaterialEditor ( int, int );
  virtual ~MEMaterialEditor ();
  
  void                editMaterial ( G3DAttribute* );
  struct module_info* get_module_info ();
  const G3DATexture*  getMaterial ( int i ) const;
  
  static void         AddMaterialWidget ( TMatWidget* );
  static void         InitDefaultWidgets ();
  
signals:
  void materialChanged ( const G3DATexture* );

public slots:
  ///User has selected a preview widget
  void setActivePreview ( int );

protected slots:
  void closeIt ();
  void setMaterial ( const G3DATexture* );
  void setPreviewer ();
  void typeSelected ( int );

private:        
   void createLayout ( int );
   void createPreviews ();

private:
   MEMaterialViewport *preView_[5];
   QFrame             *matWidget_;
   MEPreviewRenderer  *previewer_;

   QFrame      *previewFrame_;
   QVBoxLayout *gmMain_;
   QHBoxLayout *gmh1_, *gmh2_;
   QLabel      *label_;
   QComboBox   *selType_;
   QPushButton *closeButton_;

   int         active_;    //number of active preview widget

   static vector<void*> MaterialWidgets_;
};

#endif
