/*
 * Project : MindsEye, material editor
 * Copyright (c) 1996 - 1998
 * Author(s): M. van Velsen, M. Kuball
 * 
 * $Id: previewer.cpp,v 1.5 2002/06/10 09:30:14 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"

#include "3dlib/Attributes/texture.h"
#include "previewer.h"

#include <qimage.h>
#include "../raytracer/tplain.hh"
#include "../raytracer/sphere.hh"
#include "../raytracer/light.hh"
#include "../raytracer/tchecked.hh"
#include "../raytracer/plane.hh"


#include <math.h>

#define DEG_TO_RAD(deg) (deg*M_PI/180)
#define RAD_TO_DEG(rad) (rad*180/M_PI)

//The quad object is created by the first invocation of the constructor
//and kept until the termination of the program.
GLUquadricObj *MEOglPreviewer::QuadObj_ = 0;


MEPreviewRenderer::~MEPreviewRenderer ()
{
}


//-------------------------------------------------------------------


MEOglPreviewer::MEOglPreviewer ( int w, int h, QWidget *parent ) 
  : QGLPixmap (w, h, parent)
{
  width_  = w;
  height_ = h;
  if (QuadObj_ == 0) {
    QuadObj_ = gluNewQuadric();
    gluQuadricDrawStyle (QuadObj_, (GLenum)GLU_FILL);
    gluQuadricNormals   (QuadObj_, (GLenum)GLU_SMOOTH);
  }
}

MEOglPreviewer::~MEOglPreviewer ()
{
}


const QPixmap*
MEOglPreviewer::getPixmap () const
{
  return this;
}

void
MEOglPreviewer::init ()
{
  glViewport (0, 0, width_, height_);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();
  
  if (width_ <= (height_ * 2))
    glOrtho ( -6.0, 6.0, 
	      -3.0*((GLfloat)height_*2)/(GLfloat)width_, 
	      3.0*((GLfloat)height_*2)/(GLfloat)width_, 
	      -10.0, 10.0);
  else
    glOrtho ( -6.0*(GLfloat)width_/((GLfloat)height_*2), 
	      6.0*(GLfloat)width_/((GLfloat)height_*2), 
	      -3.0, 3.0, 
	      -10.0, 10.0);
  
  glMatrixMode (GL_MODELVIEW);
}

void
MEOglPreviewer::renderPixmapWith ( const G3DATexture *mat )
{
  if (mat == 0) {
    cout << "MSimplePreviewRenderer::renderPixmapWith: materisl is 0!" << endl;
    return; // TODO: use check macro.
  }
  
  cout << "MSimplePreviewRenderer::renderPixmapWith: trying to render."<< endl;

  /*
  if (!valid)
    return;
  */

  makeCurrent ();
  cout << "render pix" << endl;
  init ();
  GLfloat position[]       = { 0.0, 3.0, 2.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };

  glDisable (GL_DEPTH_TEST);
  glEnable (GL_CULL_FACE);
  glEnable (GL_LIGHTING);
  glEnable (GL_LIGHT0);
  
  glLightfv (GL_LIGHT0, GL_POSITION, position);
  glLightModelfv (GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

  glClearColor (0.0, 0.1, 0.1, 0.0);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPushMatrix ();
  glTranslatef (0.0, 0.0, 0.0);
  mat->activate ();
  showSphere (4.5);
  glPopMatrix ();
  
  glFlush ();
  int error;
  while (1) {
    error = glGetError();
    if (error == GL_NO_ERROR)
      break;
    fprintf (stderr, "GL error: %s\n", gluErrorString((GLenum)error));
  }
  ///save( "test.bmp", "BMP" ); //"XPM" XXX.
}

/*
void
MEOglPreviewer::resize( int width, int height )
{ 
  init (width, height);
}
*/

void
MEOglPreviewer::showSphere (GLdouble radius) const
{
  gluSphere (QuadObj_, radius, 32, 32);
}

// ---------- MRaytracingPreviewer:

MRaytracingPreviewer::MRaytracingPreviewer ( int w, int h )
{
  width_ = w, height_ = h;
  init ();
}

MRaytracingPreviewer::~MRaytracingPreviewer ()
{
  delete pixmap_;
  delete renderer_;
}

const QPixmap* MRaytracingPreviewer::getPixmap () const
{
  return pixmap_;
}

void MRaytracingPreviewer::renderPixmapWith ( const G3DATexture* mat )
{
  renderer_ = new Scene ();
  
  TPlain ball, shiny, black, white;
  ball.surface.color = rgb(0.7, 0.2, 0.1);
  black.surface.color=BLACK;
  white.surface.color=WHITE;
  shiny.surface.diffuse=0.6;
  shiny.surface.reflect=0.4;
  shiny.surface.specular=1.0;
  shiny.surface.hardness=100;
  // ball.surface.refract=0.4;
  // ball.surface.transparency=0.4;
  // ball.surface.ior=1.4;
 

  const G3DAColor * material = dynamic_cast<const G3DAColor *>(mat);
  if (material) {
    G3DColor ambient, diffuse, specular;
    float specular_exp, transparency;
    
    material->getAmbientColor (ambient);
    material->getDiffuseColor (diffuse);
    material->getSpecularColor (specular);
    specular_exp = material->getSpecularExp ();
    transparency = material->getTransparency ();

    ball.surface.color = rgb (diffuse.getR(), diffuse.getG(), diffuse.getB());
    ball.surface.transparency = 1.0 - transparency;
  }
  
  TChecked floor;
  floor.texture_a=&white;
  floor.texture_b=&black;
  
  ball.slave = floor.slave = &shiny;
 
  Sphere s1 (Vector(0,0,1), 2.2);
  s1.texture = &ball;
  
  Plane p1 (Vector(0,1,0), -1.301);
  p1.texture = &floor;

  s1.set_child (&p1);

  renderer_->set_objects (&s1);
  
  Vector camera_pos (0, 2, -4);
  
  Light l1(camera_pos, GREY), 
    l2(Vector(0,100,0),WHITE), 
    l3(Vector(0,0,0.5),WHITE);
  l1.set_child(&l2);
  l2.set_child(&l3);
  
  renderer_->set_lights (&l1);
 
  renderer_->set_bg_color (BLACK);
  renderer_->set_position (camera_pos);
  renderer_->set_aspect (1.0);
  renderer_->set_direction (rotate(Z, X, DEG_TO_RAD(22)));
  renderer_->set_up (rotate(Y*0.5, X, DEG_TO_RAD(20)));//*(128.0/1024.0));
  renderer_->set_brightness (1.0);

  renderer_->set_resolution (width_, height_);
  renderer_->set_anti_aliasing (false);
  renderer_->render ();
    
  QImage * image = new QImage (width_, height_, 32);
  
  uchar * src = renderer_->get_image().get_data(), * dst;
  for (int y = 1; y < height_; y++) {
    dst = image->scanLine(y);
    for (int x = 0; x < width_; x++) {
      *dst++ = *src++;
      *dst++ = *src++;
      *dst++ = *src++;
      *dst++;
    }
  }
  
  pixmap_->convertFromImage (*image);

  delete renderer_;
  renderer_ = 0;
}

void MRaytracingPreviewer::init ()
{
  renderer_ = 0;
  pixmap_ = new QPixmap (width_, height_);
}

