/*
 * Project : MindsEye, material editor
 * Copyright (c) 1996 - 1998
 * Author(s): M. van Velsen, M. Kuball
 * 
 * $Id: previewer.h,v 1.5 2002/06/10 09:30:14 ainvar Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __PREVIEWER_H__
#define __PREVIEWER_H__

#include <GL/gl.h>
#include <GL/glu.h>
#include "qglpix.h"
#include "qwidget.h"
#include "../raytracer/scene.hh"


class G3DATexture;

class MEPreviewRenderer
{
public:
  MEPreviewRenderer () {}
  virtual ~MEPreviewRenderer ();
  
  virtual const QPixmap* getPixmap () const = 0;
  virtual void renderPixmapWith ( const G3DATexture* ) = 0;
};

typedef MEPreviewRenderer MPreviewRenderer;

/**Preview the choosen material.
 */
class MEOglPreviewer : public QGLPixmap, public MEPreviewRenderer
{
public:
  MEOglPreviewer ( int, int, QWidget *parent );
  virtual ~MEOglPreviewer ();

  virtual const QPixmap* getPixmap () const;
  virtual void renderPixmapWith ( const G3DATexture* );

protected: 
  void init ();
  void showSphere ( GLdouble ) const;
  
private:
  int width_, height_;

  /**The quad object is created by the first invocation of the constructor
     and kept until the termination of the program.*/
  static GLUquadricObj *QuadObj_;
};

/**Preview the choosen material using the internal preview raytracer.
 */
class MRaytracingPreviewer : public MPreviewRenderer
{
public:
  MRaytracingPreviewer ( int, int );
  virtual ~MRaytracingPreviewer ();
  
  virtual const QPixmap* getPixmap () const;
  virtual void renderPixmapWith ( const G3DATexture* );
  
protected: 
  void init ();
  
private:
  int width_, height_;
  Scene * renderer_; // the raytracer scene.
  QPixmap * pixmap_;
};

#endif
