/*
 * Project : "MindsEye"
 * Copyright  1998. All Rights Reserved.
 * Author(s): M. Kuball
 *
 * $Id: qglpix.cpp,v 1.3 2001/07/12 00:47:11 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */


#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include "qglpix.h"
#include <iostream.h>
#include <stdio.h>

QGLPixmap::QGLPixmap ( int w, int h, QPaintDevice *pd )
  : QPixmap (w, h, -1)
{
  paintDevice = pd;
  valid = chooseContext ();
  //TODO cout << "glpix: " << depth() << endl;
}


QGLPixmap::~QGLPixmap ()
{
  if (!valid) return;
  
  glXDestroyContext (paintDevice->x11Display(), (GLXContext)cx);
  glXDestroyGLXPixmap (paintDevice->x11Display(), (GLXPixmap)pixmap_);
  vi = 0;
  cx = 0;
}

void*
QGLPixmap::chooseVisual ()
{
  int spec[40] = { GLX_LEVEL, 0, GLX_BUFFER_SIZE, 16, GLX_RGBA, 
		   GLX_RED_SIZE, 4, GLX_GREEN_SIZE, 4, GLX_BLUE_SIZE, 4,
		   GLX_ALPHA_SIZE, 0, None };

#if 0
  int spec[40];
  int i = 0;
  
  spec[i++] = GLX_LEVEL;
  spec[i++] = 0;
  
  spec[i++] = GLX_DEPTH_SIZE;
  spec[i++] = 0;

  cout << "QGLPixmap::chooseVisual: depth is" << depth() << "." << endl;
  
  if (depth() > 8) {
    spec[i++] = GLX_RGBA;
    //    spec[i++] = GLX_RED_SIZE;
    //    spec[i++] = 1;
    //    spec[i++] = GLX_GREEN_SIZE;
    //    spec[i++] = 1;
    //    spec[i++] = GLX_BLUE_SIZE;
    //    spec[i++] = 1;
  }
  else {
    spec[i++] = GLX_BUFFER_SIZE;
    spec[i++] = 8;
  }
  spec[i] = None;
#endif

  void *result = glXChooseVisual (paintDevice->x11Display(),
				  paintDevice->x11Screen(), spec); 
  
  int error;
  while (1) {
    error = glGetError();
    if (error == GL_NO_ERROR)
      break;
    fprintf (stderr, "GL error: %s\n", gluErrorString((GLenum)error));
  }
  
  return result;
}

bool
QGLPixmap::chooseContext ()
{
  vi = chooseVisual();
  if (vi == 0) {
    cout << "QGLPixmap::chooseContext: choosing visual failed!" << endl;
    return false;
  }

  cout << "visual OK" << endl;

  pixmap_ =  (void*)glXCreateGLXPixmap (paintDevice->x11Display(),
                                        (XVisualInfo *)vi, hd);
  cx = glXCreateContext (paintDevice->x11Display(), (XVisualInfo *)vi,
                         None, true);
  return cx != 0;
}

void QGLPixmap::makeCurrent ()
{
  if (!valid ) return;
  
  glXMakeCurrent (paintDevice->x11Display(), (GLXPixmap)pixmap_,
                  (GLXContext)cx );
}
