/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: animWidget.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

//#define red qt_red
//#define black qt_black
#include <algo.h>

#include <qchkbox.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpopmenu.h>
#include <qlayout.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qmsgbox.h>
#include <qpixmap.h>

#include "3dlib/shape.h"
//#include "3dlib/s_scene.h"
#include "3dlib/actions.h"
#include "3dlib/Attributes/animation.h"
#include "3dlib/Attributes/a-list.h"
#include "vlayout/m_viewm.h"
#include "kernel/world_obj.h"
#include "animWidget.h"
#include "modeler.h"
#include "icons/modeler/grip.xpm"

#ifdef WITH_PTHREADS
#include <pthread.h>

static void animationProc( void* );
#endif


class SaveValueAction : public GenericAction
{
public:
  SaveValueAction( float atime ) :
    GenericAction(A_GENERIC_ALL), time_(atime) {}
  virtual EvalState evaluate( G3DObject* );

protected:
  const float time_;
};

//------------------------------------------------------------------


MEGrip::MEGrip( QWidget *parent ) :
  QLabel( parent )
{
  QPixmap my_pix( (const char**)grip ) ;
  setPixmap( my_pix );
}

void
MEGrip::mouseReleaseEvent( QMouseEvent* )
{
  emit showDialog();
}

//------------------------------------------------------------------


MEAnimationDialog::MEAnimationDialog( MEAnimationTool *tool )
{
  setCaption( "MindsEye - Animation control" );

  QBoxLayout *gm = new QVBoxLayout( this, 4, 4 );
  QHBoxLayout *gm1 = new QHBoxLayout();
  gm->addLayout( gm1, 1 );

  QLabel *lb = new QLabel( "start: ", this );
    lb->setMinimumSize( 30, 20 );
    gm1->addWidget( lb, 1 );
  startEdit_ = new QLineEdit( this );
    startEdit_->setMinimumSize( 35, 20 );
    gm1->addWidget( startEdit_, 20 );
  gm1->addStretch( 10 );
  lb = new QLabel( "stop: ", this );
    lb->setMinimumSize( 30, 20 );
    gm1->addWidget( lb, 1 );
  stopEdit_ = new QLineEdit( this );
    stopEdit_->setMinimumSize( 35, 20 );
    gm1->addWidget( stopEdit_, 20 );
  gm1->addStretch( 10 );
  lb = new QLabel( "current:", this );
    lb->setMinimumSize( 40, 20 );
    gm1->addWidget( lb, 1 );
  currentEdit_ = new QLineEdit( this );
    currentEdit_->setMinimumSize( 35, 18 );
    gm1->addWidget( currentEdit_, 20 );
  gm->addStretch( 10 );

  slider_ = new QSlider( QSlider::Horizontal, this );
  slider_->setMinimumSize( 10, 20 );
  gm->addWidget( slider_, 1 );

  gm->addStretch( 10 );

  QHBoxLayout *gm2 = new QHBoxLayout();
  gm->addLayout( gm2, 1 );
  QPushButton *play = new QPushButton( "play", this );
    play->setMinimumSize( 60, 20 );
    connect( play, SIGNAL(clicked()), tool, SLOT(play()) );
    gm2->addWidget( play, 1 );
  QPushButton *stop = new QPushButton( "stop", this );
    stop->setMinimumSize( 60, 20 );
    connect( stop, SIGNAL(clicked()), tool, SLOT(stop()) );
    gm2->addWidget( stop, 1 );
  QPushButton *pkf = new QPushButton( "pkf", this );
    pkf->setMinimumSize( 60, 20 );
    connect( pkf, SIGNAL(clicked()), tool, SLOT(prevKf()) );
    gm2->addWidget( pkf, 1 );
  QPushButton *nkf = new QPushButton( "nkf", this );
    nkf->setMinimumSize( 60, 20 );
    connect( nkf, SIGNAL(clicked()), tool, SLOT(nextKf()) );
    gm2->addWidget( nkf, 1 );
  gm2->addStretch( 100 );

  gm->addStretch( 10 );

  QHBoxLayout *gm3 = new QHBoxLayout();
  gm->addLayout( gm3, 1 );
  QPushButton *btn = new QPushButton( "close", this );
    btn->setMinimumSize( 60, 20 );
    gm3->addWidget( btn, 1 );
    connect( btn, SIGNAL(clicked()), SLOT(closeIt()) );
  gm3->addStretch( 100 );

  resize( 260, 150 );
  gm->activate();

  //TODO add a keyframe checkbox
  //add a button to set default values for start and stop
  // how to read start and stop?
  //while playing the animation, disable all input fields
}

void
MEAnimationDialog::closeEvent( QCloseEvent *e )
{
  emit hideDialog();
  QWidget::closeEvent( e ); 
}

void
MEAnimationDialog::closeIt()
{
  hide();
  emit hideDialog();
}


//------------------------------------------------------------------


MEAnimationTool::MEAnimationTool( MEStatusBar *sbar, QObject *modeler )
{
  animFrame_ = new QFrame( sbar ); //MEAnimationFrame( sbar );
    animFrame_->resize( 90, 22 );
    //connect( animFrame_, SIGNAL(showDialog()), SLOT(showDialog()) );
    sbar->addAnimationWidget( animFrame_ );
    sbar->toggleAnimationWidget( true );
  MEGrip *grip = new MEGrip( animFrame_ );
    grip->setGeometry( 2, 1, 10, 20 );
    connect( grip, SIGNAL(showDialog()), SLOT(showDialog()) );
  wTimeField_ = new QLineEdit( animFrame_ );
    wTimeField_->setMaxLength( 16 );
    wTimeField_->setText( "0.0" );
    wTimeField_->setGeometry( 12, 2, 30, 18 );
    connect( wTimeField_, SIGNAL(returnPressed()), SLOT(setTime()) );
  wKeyframeToggle_ = new QCheckBox( "Key", animFrame_ );
    wKeyframeToggle_->setGeometry( 46, 1, 42, 20 );
    connect( wKeyframeToggle_, SIGNAL(toggled(bool)),
	     SLOT(setKeyframe(bool)) );

  animDialog_ = new MEAnimationDialog( this );
  connect( animDialog_, SIGNAL(hideDialog()), SLOT(hideDialog()) );
   
  connect( modeler, SIGNAL(createPopup(G3DObject*,QPopupMenu*)),
	   SLOT(createPopup(G3DObject*,QPopupMenu*)) );

  sbar_   = sbar;
  world_  = NULL;
  connect( modeler, SIGNAL(worldChanged(QObject*)), SLOT(setWorld(QObject*)) );
}

void
MEAnimationTool::activate()
{
  //popup a dialog: select the attributes you wish to animate
  //animationDlg_->show( _world->getSelection() );
  //TODO add animation attribute
  G3DObject *sel = world_->getSelection();
  if( IS_CLASS(sel->getType(), T_SCENE_ELEMENT) )
    {
      G3DAListIterator it =
	((G3DSceneElement*)sel)->getAttribute( TA_ANIMATION );
      if( *it )
	{
	  if( QMessageBox::information( 0, "MindsEye", 
			       "The object already has animation attributes.\n"
			       "Do you want to reset them?",
			       QMessageBox::Yes, QMessageBox::No )
	      == QMessageBox::No )
	    return;
	}
      ((G3DSceneElement*)sel)->setAttribute( new G3DAnimationVis(time_) );
    }
}

void
animationProc( void *arg )
{
  ((MEAnimationTool*)arg)->runAnimation();
}

void
MEAnimationTool::createPopup( G3DObject *obj, QPopupMenu *menu )
{
 //TODO animation of/off
  if( IS_CLASS(obj->getType(), T_SCENE_ELEMENT) )
    menu->insertItem( "animate", this, SLOT(activate()) );
}

void
MEAnimationTool::getAnimationData ()
{
}

void
MEAnimationTool::hideDialog()
{
  sbar_->toggleAnimationWidget( true );
  animFrame_->show();
}

void
MEAnimationTool::nextKf()
{
  //TODO goto next keyframe
}

void
MEAnimationTool::play()
{
#ifdef WITH_PTHREADS
  //test if already running or use toggle Button?
  //Create the render thread
  /*
  pthread_t thread;
  pthread_create( &thread, NULL, animationProc, this );
  pthread_detach( thread );
  */
#endif

  connect( world_, SIGNAL(timerDone()), SLOT(timerDone()) );
  world_->startTimer();

}

void
MEAnimationTool::prevKf()
{
  //TODO goto previous keyframe
}

void
MEAnimationTool::runAnimation ()
{
  //G3DScene *scene = activeJob_->getScene();
  float tb, te = 1, dt = 0.1;
  SetTimeAction action( tb );  //TODO use a special action
  while( 1 );
  {
    tb = 0;
    while( tb <= te )
      {
	action.setTime( tb );
	world_->doWithScene( action );
	tb += dt;
      }
  }
}

void
MEAnimationTool::setKeyframe( bool flag )
{
  if( flag == TRUE )
    {
      keyList_.insert( time_ );
    }
  else
    {
      keyList_.erase( time_ );
      //TODO remove this time frame in all Attributes?
    }
}

void
MEAnimationTool::showDialog()
{
  animFrame_->hide();
  sbar_->toggleAnimationWidget( false );
  animDialog_->show();
}


EvalState
SaveValueAction::evaluate( G3DObject *obj )
{
  if( IS_CLASS(obj->getType(), T_SCENE_ELEMENT) )
    {
      G3DSceneElement  *sel = (G3DShape*)obj;
      G3DAListIterator attr = sel->getAttribute( TA_ANIMATION );
      while( *attr != NULL )
	{
	  ((G3DAnimation*)(*attr))->saveValueAt( sel, time_ );
	  ++attr;
	}
    }
  return( continueEval );
}

void
MEAnimationTool::setTime( )
{
  float t;
  t = atof( wTimeField_->text() );

  if( ::find(keyList_.begin(),keyList_.end(),time_) != keyList_.end() )
    {
      SaveValueAction action1( time_ );
      world_->doWithScene( action1 );
    }

  SetTimeAction action2( t );
  world_->doWithScene( action2 );
 
  //TODO don't forget the camera!
  time_ = t;
  if( ::find(keyList_.begin(),keyList_.end(),time_) != keyList_.end() )
    wKeyframeToggle_->setChecked( TRUE );
  else
    wKeyframeToggle_->setChecked( FALSE );

  world_->update();
}

void
MEAnimationTool::setWorld( QObject *world )
{
  world_ = (MWorldObject*)world;
  //if( world )
  //TODO get the Animation Context of the scene

  time_       = 0;  //TODO get time from scene
  wTimeField_->setText( "0" );

  //connect( world_, SIGNAL(changed()), this, SLOT(getAnimationData()) );
  //getAnimationData();
}

void
MEAnimationTool::stop()
{
  //  timer_->stop();
}

void
MEAnimationTool::timerDone()
{
  cout << "timer" << endl;
#ifdef WITH_PTHREADS
  //wait for the thread before further processing
  //tell the thread to start a new turn
#endif
  //setTime(); fr alle nicht  //it's not realtime
  //AnimateAction animate( );
  //world_->doWithScene( animate );
  //world_->update( NULL );
}


//===================================END=================================
