/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: animWidget.h,v 1.2 2002/06/17 20:28:25 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _ANIMATION_WIDGET_H_
#define _ANIMATION_WIDGET_H_

#define red stl_red
#define black stl_black
#include <set>
#undef red
#undef black

//#include <qframe.h>
#include <qlabel.h>

class MModeler;
class MWorldObject;
class MEAnimationTool;
class MEStatusBar;
class QLineEdit;
class QCheckBox;
class G3DObject;
class QPopupMenu;
class QSlider;


struct lflt
{
  bool operator()(float f1, float f2) const
  {
    return( f1 < f2 );
  }
};


class MEGrip : public QLabel
{
 Q_OBJECT

public:
  MEGrip( QWidget *parent );

signals:
  void showDialog();

protected:
  void mouseReleaseEvent( QMouseEvent* );
};



/**A Widget which features controls to animate a scene. This includes
  a Field to set the time.
 */
class MEAnimationDialog: public QWidget
{
  Q_OBJECT

public:
  MEAnimationDialog( MEAnimationTool* );

signals:
  void hideDialog();

protected:
  void closeEvent( QCloseEvent *e );

protected slots:
  void closeIt();

protected:
  QSlider  *slider_;
  QLineEdit *startEdit_, *stopEdit_, *currentEdit_, *fpsEdit_;
  
};


/**
 */
class MEAnimationTool: public QObject
{
  Q_OBJECT

public:
  MEAnimationTool( MEStatusBar *sbar, QObject* );

  void runAnimation ();

protected slots:
    ///Prepare an object for animation.
  void activate();
    ///
  void createPopup( G3DObject*, QPopupMenu* );
  void hideDialog();
  void nextKf();
  void play();
  void prevKf();
  void setKeyframe( bool );
  void setTime( );
  void setWorld( QObject* );
  void showDialog();
  void stop();
  void timerDone();

protected:
  void getAnimationData ();

protected:
  QFrame    *animFrame_;
  QWidget   *animDialog_;
  QLineEdit *wTimeField_;
  QCheckBox *wKeyframeToggle_;

  MWorldObject *world_;
  MEStatusBar  *sbar_;

  float            time_;
  set<float, lflt> keyList_;

};

#endif
