/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: aspectFrame.cpp,v 1.1.1.1 1999/09/09 23:57:25 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlined.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

#include "aspectFrame.h"
#include "dialogs/vedit.h"
#include "modeler.h"
#include "3dlib/camera.h"
#include "kernel/world_obj.h"


MEAspectFrame::MEAspectFrame( QObject *modeler, QWidget *p ) :
  MEPaletteFrame( "Aspect Ratio", modeler, p)
{
  modeler_     = (MModeler*)modeler;
  aspectRatio_ = 1;

  int y = tSize;

  QLabel *lb = new QLabel( "Set preferred aspect ratio", this );
    lb->setGeometry( 2, y, 140, 18 );
    y += 16;
  lb = new QLabel( "for output (width / height)", this );
    lb->setGeometry( 2, y, 140, 18 );
    y += 18;

  xratio_ = new QLineEdit( this );
    xratio_->setGeometry( 4, y, 28, 20 );
  lb = new QLabel( "  / ", this );
    lb->setGeometry( 32, y, 20, 20 );
  yratio_ = new QLineEdit( this );
    yratio_->setGeometry( 52, y, 28, 20 );
    y += 22;

  showSRegion_ = new QCheckBox( "show secure Region", this );
    showSRegion_->setGeometry( 4, y, 120, 20 );
    connect( showSRegion_, SIGNAL(toggled(bool)), SLOT(toggleRegion(bool)) );
    y += 24;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry( 4, y, 30, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(setAspectRatio()) );
    y += 22;

  visHeight_ = y;
  resize( 100, y );
}

void
MEAspectFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
}

void
MEAspectFrame::setAspectRatio( )
{
  float x, y;
  x = atof( xratio_->text() );
  if( x <= 0 )
    {
      x = 1;
      xratio_->setText( "1" );
    }
  y = atof( yratio_->text() );
  if( y <= 0 )
    {
      y = 1;
      yratio_->setText( "1" );
    }
  aspectRatio_ = x/y;
  if( camera_ )
    {
      camera_->setAspectRatio( aspectRatio_ );
      ((MWorldObject*)world_)->updateCamera();
    }
}

void
MEAspectFrame::setCamera( G3DCamera *cam )
{
  camera_ = cam;
  if( camera_ )
    camera_->setAspectRatio( aspectRatio_ );
}

void
MEAspectFrame::setWorld( QObject *obj )
{
  if( world_ )
    world_->disconnect( this );

  world_  = obj;
//TODO get aspect Ratio from scene
  xratio_->setText( "1" );
  yratio_->setText( "1" );

  connect( world_, SIGNAL(cameraActivated(G3DCamera*)),
	   SLOT(setCamera(G3DCamera*)) );
}

void
MEAspectFrame::toggleRegion( bool val)
{
  modeler_->setShowSRegion( val );
  ((MWorldObject*)world_)->updateCamera();
}

void
MEAspectFrame::updateView()
{
}


//=================================END====================================
