/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: attrib_widget.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbutton.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qkeycode.h>

#include "3dlib/sceneElement.h"
#include "3dlib/Attributes/attribute.h"
#include "kernel/core.h"
#include "dialogs/listview.h"
#include "attrib_widget.h"
#include "paletteFrame.h"
#include "modeler.h"

#define GET_WIDGET(i) ((MEPaletteFrame*)(widgets_[i]))


class MEAttributeItem : public QListViewItem
{
public:
  MEAttributeItem( QListViewItem *root, G3DAttribute *attr ) :
	QListViewItem(root, attr->getTypeName()) { attr_ = attr; }

  G3DAttribute *attr_;
};


//------------------------------------------------------------------


MEAttributeWidget::MEAttributeWidget( MModeler *modeler, QWidget *parent ) :
 MeTabChild(parent) // : QWidget(parent)
{
  //setFrameStyle( WinPanel | Raised );
  model_ = modeler;
  int y = 2;

  QLabel *lb = new QLabel( "Available Attributes:", this );
    lb->setGeometry( 1, y, 120, 18 );
    y += 18;
  attribTypes_ = new QListBox( this );
    attribTypes_->setGeometry( 0, y, 140, 70 );
    y += 72;
    const char **attributeNames = Kernel->getAttributeNames();
    attribTypes_->insertStrList( attributeNames );
    delete[] attributeNames;

  addButton_ = new QPushButton( "add Attribute", this );
    addButton_->setGeometry( 1, y, 80, 18 );
    addButton_->setEnabled( false );
    y += 22;
    connect( addButton_, SIGNAL(clicked()), SLOT(addAttribute()) );

  lb = new QLabel( "Attributes of selected shape:", this );
    lb->setGeometry( 1, y, 145, 18 );
    y += 18;
  attribView_ = new MEListView( this );
    attribView_->addColumn( "" );
    attribView_->header()->hide();
    attribView_->setSorting( -1 );
    //attribView_->setColumnWidthMode( 0, QListView::Manual );
    //attribView_->setRootIsDecorated( true );
    attribView_->setTreeStepSize( 20 );
    attribView_->setGeometry( 0, y, 100, 120 );
    connect( attribView_, SIGNAL(currentChanged(QListViewItem*)),
	     SLOT(setCurrent(QListViewItem*)) );

  yWidgets_   = y + 120 + 4;
  widgets_.reserve( 10 );
  world_      = NULL;
  connect( modeler, SIGNAL(worldChanged(QObject*)), SLOT(setWorld(QObject*)) );
}

void
MEAttributeWidget::addAttribute( )
{
  if( object_ )
    {
      int index = attribTypes_->currentItem();
      if( index >= 0 )
	{
	  G3DAttribute *attr =
	    Kernel->createAttribute( attribTypes_->text(index) );
	  object_->setAttribute( attr );
	  insertListItem( attr );
	}
    }
}

void
MEAttributeWidget::insertListItem( G3DAttribute *attr )
{
  t_type atype = attr->getType();

  if( IS_TYPE(atype, TA_TEXTURE) )
    new MEAttributeItem( listRoot1_, attr );
  else if( IS_TYPE(atype, TA_CONSTRAINT) )
    new MEAttributeItem( listRoot3_, attr );
  else if( IS_TYPE(atype, TA_ANIMATION) )
    new MEAttributeItem( listRoot2_, attr );
  else
    new MEAttributeItem( listRoot4_, attr );
}

void
MEAttributeWidget::keyPressEvent( QKeyEvent *e )
{
  if( e->key() == Key_Delete )
    {
      removeAttribute();
    }
  else
    e->ignore();
}

void
MEAttributeWidget::makeRootItems ()
{
  listRoot4_ = new QListViewItem( attribView_, "miscellaneous" );
  listRoot4_->setOpen( true );
  listRoot2_ = new QListViewItem( attribView_, "animation" );
  listRoot2_->setOpen( true );
  listRoot3_ = new QListViewItem( attribView_, "constraints" );
  listRoot3_->setOpen( true );
  listRoot1_ = new QListViewItem( attribView_, "material" );
  listRoot1_->setOpen( true );
}

void
MEAttributeWidget::relayout()
{
  resizeEvent( NULL );
}

void
MEAttributeWidget::removeAttribute()
{
  if( currentItem_ )
    {
      QListViewItem *tmp = currentItem_->parent();
      //TODO if( tmp != attribView_ )  //don't remove the root
	{
	  object_->removeAttribute( currentItem_->attr_ );
	  tmp->removeItem( currentItem_ );
	  setCurrent( NULL );
	}
    }
}

void
MEAttributeWidget::resizeEvent( QResizeEvent* )
{
  attribTypes_->resize( width(), attribTypes_->height() );
  addButton_->resize( width(), addButton_->height() );
  //attribView_->setColumnWidth( 0, width()-2 );
  attribView_->resize( width(), attribView_->height() );

  int y = yWidgets_;
  for( int i = 0; i < widgets_.size(); ++i )
    {
      MEPaletteFrame *tmp;
      tmp = GET_WIDGET(i);
      tmp->setGeometry( 0, y, width(), tmp->height() );
      y += tmp->height() + 2;
    }
  psize_ = y;
  //emit setPageSize( y );
}

void
MEAttributeWidget::setCurrent( QListViewItem *item )
{
  for( int i = 0; i < widgets_.size(); ++i )
    {
      delete GET_WIDGET(i);
    }
  widgets_.erase( widgets_.begin(), widgets_.end() );

  if( item && (item->parent() != NULL) )
    {
      currentItem_ = (MEAttributeItem*)item;
      uint type    = currentItem_->attr_->getType();
      Kernel->getPaletteWidgets( type, this, widgets_ );
      for( int i = 0; i < widgets_.size(); ++i )
	{
	  MEPaletteFrame *tmp;
	  tmp = GET_WIDGET(i);
	  tmp->setWorld( world_ );
	  tmp->setObject( object_ );
	  tmp->move( 2000, 0 );  //prevents unnecessary redraws
	  tmp->show();
	  connect( tmp, SIGNAL(sizeChanged()), SLOT(relayout()) );
	}
    }
  else
    currentItem_ = NULL;
  resizeEvent( NULL );
}

void
MEAttributeWidget::setObject( G3DObject *obj )
{
  for( int i = 0; i < widgets_.size(); ++i )
    {
      delete GET_WIDGET(i);
    }
  widgets_.erase( widgets_.begin(), widgets_.end() );

  if( obj && IS_CLASS(obj->getType(), T_SCENE_ELEMENT) )
    {
      object_ = (G3DSceneElement*)obj;
      updateView();
      addButton_->setEnabled( true );
    }
  else
    {
      attribView_->clear();
      object_ = NULL;
      addButton_->setEnabled( false );
    }
}

void
MEAttributeWidget::setWorld( QObject *world )
{
  if( world_ )
    world_->disconnect( this );
  world_ = world;

  setObject( NULL );

  connect( world, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)) );
  connect( world, SIGNAL(objectChanged()),//TODO maybe add signal attribChanged
	   SLOT(updateView()) );
}

void
MEAttributeWidget::updateView()
{
  if( object_ )
    {
      attribView_->clear();
      makeRootItems();
      //get attributes from object and sort them into listRoots
      G3DAListIterator iterator = object_->getAttribute(TA_ALL_ATTRIBUTES);
      while( *iterator )
	{
	  if( !(*iterator)->isReference() )
	    {
	      insertListItem( *iterator );
	      ++iterator;
	      /*
		num   = (*iterator)->getParameter( params_ );
		flist = (*iterator)->getFieldList();
		*/
	    }
	}
    }
  //else TODO disable view
}

//===================================END=================================
