/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: camCtrlFrame.cpp,v 1.1.1.1 1999/09/09 23:57:29 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qpixmap.h>
#include <qlabel.h>
#include "vlayout/cam_controler.h"
#include "camCtrlFrame.h"
#include "3dlib/camera.h"
#include "kernel/world_obj.h"
#include "icons/modeler/c1.xpm"
#include "icons/modeler/c2.xpm"
#include "icons/modeler/c3.xpm"
#include "icons/modeler/c7.xpm"


class METoggleButton : public QPushButton
{
public:
  METoggleButton( const char**, QWidget* );
};

METoggleButton::METoggleButton( const char**pix, QWidget *p ) :
  QPushButton( p )
{
  setPixmap( QPixmap(pix) );
  setToggleButton( true );
}


//----------------------------------------------------------------


MECamCtrlFrame::MECamCtrlFrame( QObject *modeler, QWidget *p ) :
  MEPaletteFrame("Navigation Control", NULL, p)
{
  int y = tSize;

  connect( modeler, SIGNAL(cameraControlerChanged(MECameraControler*)),
	   SLOT(setControler(MECameraControler*)) );

  QButtonGroup *bg = new QButtonGroup();
  bg->setExclusive( true );
  connect( bg, SIGNAL(pressed(int)), SLOT(setMode(int)) );

  btn_[MECameraControler::Fly] = new METoggleButton( (const char**)c1, this );
  btn_[MECameraControler::Fly]->setGeometry( 5, y, 25, 25 );
  bg->insert( btn_[MECameraControler::Fly], MECameraControler::Fly );

  btn_[MECameraControler::Orbit] = new METoggleButton( (const char**)c2,this );
  btn_[MECameraControler::Orbit]->setGeometry( 35, y, 25, 25 );
  bg->insert( btn_[MECameraControler::Orbit], MECameraControler::Orbit );

  btn_[MECameraControler::Rotate] = new METoggleButton((const char**)c3, this);
  btn_[MECameraControler::Rotate]->setGeometry( 65, y, 25, 25 );
  bg->insert( btn_[MECameraControler::Rotate], MECameraControler::Rotate );

  btn_[MECameraControler::Zoom] = new METoggleButton( (const char**)c7,this );
  btn_[MECameraControler::Zoom]->setGeometry( 95, y, 25, 25 );
  bg->insert( btn_[MECameraControler::Zoom], MECameraControler::Zoom );
  y += 28;

  QLabel *lb = new QLabel( "fly speed: ", this );
  lb->setGeometry( 2, y, 60, 18 );
  y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
MECamCtrlFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  //position_->resize( width()-4, 20 );
  //target_->resize( width()-4, 20 );
}

void
MECamCtrlFrame::setMode( int id )
{
  if( camControler_ )
    camControler_->setMode( id );
}

void
MECamCtrlFrame::setControler( MECameraControler *cc )
{
  camControler_ = cc;
  btn_[cc->getMode()]->setOn( true );
}

void
MECamCtrlFrame::updateView()
{
  //Do nothing
}


//=================================END====================================
