/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: cameraFrame.cpp,v 1.1.1.1 1999/09/09 23:57:30 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlined.h>

#include "cameraFrame.h"
#include "dialogs/vedit.h"
#include "3dlib/camera.h"

static char *modes[] = 
{ "top", "bottom", "front", "back", "left", "right" };


MECameraFrame::MECameraFrame( QObject *modeler, QWidget *p ) :
  MEPaletteFrame( "Camera Parameter", modeler, p)
{
  camera_ = NULL;
  int y   = tSize;

  QLabel *lb = new QLabel( "Mode: ", this );
  lb->setGeometry( 2, y, 47, 18 );
  modeLabel_ = new QLabel( this );
  modeLabel_->setGeometry( 50, y, 80, 18 );
  y += 20;

  lb = new QLabel( "Position (x,y,z)", this );
  lb->setGeometry( 2, y, 100, 18 );
  y += 18;
  position_ = new MEVectorEdit( this );
  position_->move( 2, y );
  y += 20;

  lb = new QLabel( "Look at (x,y,z)", this );
  lb->setGeometry( 2, y, 100, 18 );
  y += 18;
  target_ = new MEVectorEdit( this );
  target_->move( 2, y );
  y += 20;

  lb = new QLabel( "field of view (degree in y)", this );
  lb->setGeometry( 2, y, 125, 18 );
  y += 18;
  fov_ = new QLineEdit( this );
  fov_->setGeometry( 4, y, 50, 20 );
  
  y += 22;
  visHeight_ = y;
  resize( 100, y );
}

void
MECameraFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  position_->resize( width()-4, 20 );
  target_->resize( width()-4, 20 );
}

void
MECameraFrame::setCamera( G3DCamera *cam )
{
  camera_ = cam;
  updateView();
}

void
MECameraFrame::setWorld( QObject *obj )
{
  if( world_ )
    world_->disconnect( this );

  world_  = obj;
  connect( world_, SIGNAL(cameraActivated(G3DCamera*)),
	   SLOT(setCamera(G3DCamera*)) );
  connect( world_, SIGNAL(cameraChanged()),
	   SLOT(updateView()) );
}

void
MECameraFrame::updateView()
{
  if( camera_ )
    {
      int t = camera_->getViewMode();
      if( t == G3DCamera::Perspective )
	modeLabel_->setText( "perspective" );
      else
	modeLabel_->setText( modes[t - G3DCamera::Top] );

      position_->setVector( camera_->getPosition() );
      target_->setVector( camera_->getLookAt() );
      QString str;
      str.setNum( camera_->getFOV(), 'g', 2 );
      fov_->setText( str );
    }
}


//=================================END====================================
