/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: cameraPool.cpp,v 1.2 2000/02/11 22:41:11 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombo.h>
#include "kernel/world_list.h"
#include "kernel/world_obj.h"
#include "3dlib/camera.h"
#include "cameraPool.h"


MECameraPool::MECameraPool (QObject *modeler, QWidget *p) :
  MEPaletteFrame ("Camera Pool", modeler, p)
{
  int y = tSize;
  
  QLabel *lb = new QLabel ("Choose a camera", this);
  lb->setGeometry (2, y, 130, 20);
  y += 20;

  names_ = new QComboBox (true, this);
  names_->move (2, y);
  names_->setInsertionPolicy (QComboBox::NoInsertion);
  connect (names_, SIGNAL(activated(int)), SLOT(cameraSelected(int)));
  connect (names_, SIGNAL(activated(const QString&)),
           SLOT(cameraSelected(const QString&)));
  y += 24;
  
  camera_ = 0;
  
  QPushButton *btn = new QPushButton ("add", this);
    btn->setGeometry (2, y, 45, 20);
    connect (btn, SIGNAL(clicked()), SLOT(addCamera()));
  btn = new QPushButton ("del", this);
    btn->setGeometry (52, y, 45, 20);
    connect( btn, SIGNAL(clicked()), SLOT(deleteCamera()) );
  btn = new QPushButton ("restore", this);
    btn->setGeometry (97, y, 45, 20);
    //connect( btn, SIGNAL(clicked()), SLOT(deleteCamera()) );
  y += 22;

  visHeight_ = y;
  resize (100, y);
}


void
MECameraPool::addCamera ()
{
  G3DCamera *cam = ((MWorldObject*)world_)->getActiveCamera();
  cam = new G3DCamera( *cam );
  cam->setName( "test" );
  ((MWorldObject*)world_)->addCamera( cam );
  ((MWorldObject*)world_)->setActiveCamera( cam );
  /*
  char      str[20];
  G3DCamera *camera = new G3DCamera;
  sprintf( str, "Camera %i", current_ + 1 );
  camera->setName( str );
  */
}

void
MECameraPool::cameraSelected (const QString& n)
{
  G3DCamera *cam = ((MWorldObject*)world_)->getActiveCamera();
  if (strcmp(cam->getName(), n.ascii()) != 0)
    {
      cam->setName (n.ascii());
      setCameraList ();
    }
}

void
MECameraPool::cameraSelected (int index)
{
  //cout << index << endl;
  if( index < ((MWorldObject*)world_)->getNumberOfCameras() )
    {
      MCameraList &cameras = ((MWorldObject*)world_)->getCameras();
      G3DCamera *cam = cameras[index];
      ((MWorldObject*)world_)->setActiveCamera( cam );
    }
}

void
MECameraPool::deleteCamera()
{
  cout << names_->currentItem() << endl;
  //Search for cam in camera list
}

void
MECameraPool::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  names_->resize( width()-4, 20 );
}

void
MECameraPool::setCamera( G3DCamera *cam )
{
  camera_ = cam;
  updateView();
}

void
MECameraPool::setCameraList( )
{
  MCameraList &cameras = ((MWorldObject*)world_)->getCameras();
  names_->clear();

  MCameraList::iterator cam = cameras.begin();
  while( cam != cameras.end() )
    {
      names_->insertItem( ((G3DCamera*)(*cam))->getName() );
      ++cam;
    }
  //cameraSelected( 0 ); //TODO
}

void
MECameraPool::setWorld( QObject *obj )
{
  if( world_ )
    world_->disconnect( this );

  world_ = obj;
  connect( world_, SIGNAL(cameraActivated(G3DCamera*)),
	   SLOT(setCamera(G3DCamera*)) );
  connect( world_, SIGNAL(cameraListChanged()), SLOT(setCameraList()) );

  setCameraList();
}

void
MECameraPool::updateView()
{
  //TODO
  if( camera_ )
    {
      names_->setEditText( camera_->getName() );
    }
}


//=================================END====================================
