/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: create_widget.cpp,v 1.2 2001/07/17 02:34:38 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbt.h>
#include <qbttngrp.h>
#include <qcombo.h>
#include <qlistbox.h>

#include "3dlib/object.h"
#include "kernel/core.h"
#include "create_widget.h"
#include "nurbsTools.h"
#include "modeler.h"


/*
There are two ways to creat objects:
  from an internal template
  from existing objects via an algorithm
*/

MECreateWidget::MECreateWidget( MModeler *model, QWidget *parent ) :
  MeTabChild(parent) // QWidget(parent)
{
  model_     = model;
  workScale_ = 1.0;

  vector<const char*> &objCategories = Kernel->getObjectCategories();
  objClasses_  = new QComboBox( this );
    objClasses_->setGeometry( 0, 2, 10, 24 );
    vector<const char*>::const_iterator itr = objCategories.begin();
    for( ; itr != objCategories.end(); ++itr )
      objClasses_->insertItem( *itr );
    connect( objClasses_, SIGNAL(activated(int)), SLOT(useObjClass(int)) );

  objTypes_ = new QListBox( this );
    objTypes_->setGeometry( 0, 28, 10, 100 );
    connect( objTypes_, SIGNAL(highlighted(int)), SLOT(useObjType(int)) );

  nurbsFrame_ = new MENurbsTools( model, this );
  nurbsFrame_->move( 0, 130 );
  connect( nurbsFrame_, SIGNAL(sizeChanged()), SLOT(relayout()) );

  connect( model, SIGNAL(setWorkScale(MeFloat)),
	   SLOT(setWorkScale(MeFloat)) );

  yWidgets_ = 130;
  objClasses_->setCurrentItem (0);
  useObjClass (0);
}

void
MECreateWidget::relayout()
{
  resizeEvent (0);
}

void
MECreateWidget::resizeEvent( QResizeEvent* )
{
  objClasses_->resize( width()-1, objClasses_->height() );
  objTypes_->resize( width()-1, objTypes_->height() );
  nurbsFrame_->resize( width(), nurbsFrame_->height() );

  psize_ = yWidgets_ + nurbsFrame_->height() + 2;
  //emit setPageSize( y );
}

void
MECreateWidget::setWorkScale ( MeFloat s )
{
  workScale_ = s;
  useObjType (typeNr_);
}

void
MECreateWidget::useObjClass( int nr )
{
  objTypes_->setAutoUpdate (FALSE);
  objTypes_->clear ();

  const char *cname = objClasses_->text (nr);
  pair<MmumapItr,MmumapItr> const &rng = Kernel->getObjectList( cname );
  MmumapItr itr = rng.first;
  for( ; itr != rng.second; ++itr )
    objTypes_->insertItem( (*itr).second );

  objTypes_->setAutoUpdate (TRUE);
  objTypes_->repaint ();
}

void
MECreateWidget::useObjType ( int nr )
{
  typeNr_ = nr;
  G3DObject *obj = 0;
  
  const char *tname = Kernel->getObjTypename (objTypes_->text(nr));
  if (tname != 0)
    obj = Kernel->createObject (tname, workScale_);
  
  model_->setNewObject (obj);
  if (obj)
    obj->dispose ();
}
