/*
 * Project : "MindsEye, NURBS editor"
 * Copyright: 1998. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: edit_widget.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbt.h>
#include <qlined.h>
#include <qlabel.h>

#include "3dlib/object.h"
#include "3dlib/actions.h"
#include "3dlib/s_scene.h"
#include "kernel/world_obj.h"
#include "kernel/core.h"
#include "paletteFrame.h"
#include "trafoFrame.h"
#include "edit_widget.h"


MEEditWidget::MEEditWidget( QObject *modeler, QWidget *parent ) :
  MeTabChild(parent) // QWidget(parent)
{
  int y = 2;
  QLabel *lb = new QLabel( "object name", this );
    lb->setGeometry( 2, y, 90, 20 );
  setBtn_ = new QPushButton( "set", this );
    setBtn_->setGeometry( 0, y, 25, 20 );
    connect( setBtn_, SIGNAL(clicked()), SLOT(setObjectName()) );
  findBtn_ = new QPushButton( "find", this );
    findBtn_->setGeometry( 0, y,25, 20 );
    connect( findBtn_, SIGNAL(clicked()), SLOT(findObject()) );
  y += 20;
  name_ = new QLineEdit( this );
    name_->move( 0, y );
  y += 20;
  lb = new QLabel( "type", this );
    lb->setGeometry( 1, y, 40, 20 );
  typeName_ = new QLabel( this );
    typeName_->move( 50, y );
    typeName_->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    y += 24;

  //add some default widgets
  yWidgets_ = y;
  trafoFrame_ = new METrafoFrame( modeler, this );
  trafoFrame_->move( 0, y );
  connect( trafoFrame_, SIGNAL(sizeChanged()), SLOT(relayout()) );

  //TODO Modifikatorstapel

  widgets_.reserve( 10 );
  world_ = NULL;
  connect( modeler, SIGNAL(worldChanged(QObject*)), SLOT(setWorld(QObject*)) );
}

void
MEEditWidget::deleteWidgets ()
{
  for( int i = 0; i < widgets_.size(); ++i )
    {
      delete GET_WIDGET(i);
    }
  widgets_.erase( widgets_.begin(), widgets_.end() );
}

void
MEEditWidget::findObject()
{
  FindByName iter( name_->text() );
  world_->doWithScene( iter );
  if( iter.getObject() )
    world_->selectObject( iter.getObject() );

}

void
MEEditWidget::relayout()
{
  resizeEvent( NULL );
}

void
MEEditWidget::resizeEvent( QResizeEvent* )
{
  name_->resize( width(), 20 );
  setBtn_->move( width()-50, setBtn_->y() );
  findBtn_->move( width()-25, findBtn_->y() );
  typeName_->resize( width()-50, 20 );

  trafoFrame_->resize( width(), trafoFrame_->height() );
  int y = yWidgets_ + trafoFrame_->height() + 2;

  for( int i = 0; i < widgets_.size(); ++i )
    {
      MEPaletteFrame *tmp;
      tmp = GET_WIDGET(i);
      tmp->setGeometry( 0, y, width(), tmp->height() );
      y += tmp->height() + 2;
    }
  psize_ = y;
  //emit setPageSize( y );
}

void
MEEditWidget::setObjectName()
{
  if( object_ )
    {
      object_->setName( name_->text() );
      world_->update();
    }
}

void
MEEditWidget::setObject( G3DObject *obj )
{
  if( obj == NULL )
    obj = world_->getScene();
  //if( obj-getType() != object_->getType() )
    
  deleteWidgets();
  object_ = obj;
  updateView();

  Kernel->getPaletteWidgets( object_->getType(), this, widgets_ );
  for( int i = 0; i < widgets_.size(); ++i )
    {
      MEPaletteFrame *tmp;
      tmp = GET_WIDGET(i);
      tmp->setWorld( world_ );
      tmp->setObject( object_ );
      tmp->move( 2000, 0 );  //prevents unnecessary redraws
      tmp->show();
      connect( tmp, SIGNAL(sizeChanged()), SLOT(relayout()) );
    }
  resizeEvent( NULL );
}

void
MEEditWidget::setWorld( QObject *world )
{
  if( world_ )
    world_->disconnect( this );
  world_ = (MWorldObject*)world;
//TODO 
  setObject( world_->getScene() );

  connect( world, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)) );
  connect( world, SIGNAL(objectChanged()),
	   SLOT(updateView()) );
  /*TODO
  connect( world, SIGNAL(sceneChanged(G3DGroup*)),
	   SLOT(updateGL()) );
	   */
}

void
MEEditWidget::updateView()
{
  if( object_ )
    {
      name_->setText( object_->getName() );
      typeName_->setText( object_->getTypeName() );
    }
  else
    {
      name_->setText( "" );
      typeName_->setText( "no selection" );
    }
}


//===================================END=================================
