/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: genericFrame.cpp,v 1.1.1.1 1999/09/09 23:57:38 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include "generic.h"
#include "dialogs/vedit.h"
#include "3dlib/object.h"


METrafoFrame::METrafoFrame( QObject *modeler, QWidget *p ) :
  MEPaletteFrame( "Transformation", modeler, p)
{
  int y = tSize;

  QLabel *lb = new QLabel( "Scale", this );
  lb->setGeometry( 2, y, 50, 20 );
  y += 20;
  scale_ = new MEVectorEdit( this );
  scale_->move( 2, y );
  y += 20;

  lb = new QLabel( "Rotation", this );
  lb->setGeometry( 2, y, 50, 20 );
  y += 20;
  rotation_ = new MEVectorEdit( this );
  rotation_->move( 2, y );
  y += 20;

  lb = new QLabel( "Position", this );
  lb->setGeometry( 2, y, 50, 20 );
  y += 20;
  position_ = new MEVectorEdit( this );
  position_->move( 2, y );
  y += 22;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 40, 18 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
METrafoFrame::applyValues()
{
  G3DVector3d vec;
  scale_->getVector( vec );
  object_->setScale( vec );
  position_->getVector( vec );
  object_->setPosition( vec );
}

void
METrafoFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  scale_->resize( width()-4, 20 );
  rotation_->resize( width()-4, 20 );
  position_->resize( width()-4, 20 );
}

void
METrafoFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();

  QString str;
  const Field *flist;
  Param params[10];
  int num = object_->getParameter( params );
  flist = object_->getFieldList();
  for( int i = 0; i < num; ++i )
    {
      lb = new QLabel( flist->name, this );
      lb->setGeometry( 2, y, 70, 20 );
      switch( flist->type )
	{
	case s_point3:
	  edit = new MEVectorEdit( this ); break;
	  edit->setVector();
	  inf >> *(params->point3);  break;
	case s_vector3:
	      inf >> *(params->vector3);  break;
	    case s_ctrlPt:
	      inf >> *(params->ctrlPt);  break;
	    case s_color:
	      inf >> *(params->color); break;
	    case s_flt:
	      inf >> params->flt; break;
	    case s_number:
	      inf >> params->number; break;
	    case s_logic:
	      inf >> params->logic; break;
	      //TODO case n_string:
	      // inf >> params->string; break;
	    case s_string:
      
      str.setNum( params[1].flt, 'g', 4 );
      radius_->setText( str );
      
    }
}

void
METrafoFrame::setWorld( QObject *obj )
{
  MEPaletteFrame::setWorld( obj );
  connect( world_, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)) );
}

void
METrafoFrame::updateView()
{
  if( object_ )
    {
      scale_->setVector( object_->getScale() );
      //rotation_->setVector(
      position_->setVector( object_->getOrigin() );
    }
  else
    {
      scale_->erase();
      rotation_->erase();
      position_->erase();
    }
}


//=================================END====================================
