/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: lightFrame.cpp,v 1.2 2002/06/16 01:52:19 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>
#include <qcheckbox.h>

#include "lightFrame.h"
#include "3dlib/light.h"
#include "kernel/world_obj.h"
// #include "dialogs/color/colorpicker.h"
#include <qcolordialog.h>
#include "factories.h"
#include <vector>


void
createLightWidget( vector<void*> *list, QWidget *p )
{
  list->push_back( (void*)(new MELightFrame(p)) );
}


//--------------------------------------------------------------------


//general attributes:
//cast shadow
//fall off
//color
//intensity

MELightFrame::MELightFrame( QWidget *p ) :
  MEPaletteFrame( "Light Parameter", NULL, p)
{
  int y = tSize + 2;

  colorBtn_ = new QPushButton( "color", this );
    colorBtn_->setGeometry( 4, y, 10, 20 );
    connect( colorBtn_, SIGNAL(clicked()), SLOT(chooseColor()) );
    y += 22;

  QLabel *led = new QLabel( "Intensity [W m^-2 sr^-1]", this );
    led->setGeometry( 2, y, 125, 20 );
    y += 18;

  intensity_ = new QLineEdit( this );
    intensity_->setGeometry( 4, y, 40, 20 );
    y += 22;

  shadows_ = new QCheckBox( "cast shadows", this );
    shadows_->setGeometry( 4, y, 100, 20 );
    y += 18;

  enabled_ = new QCheckBox( "enabled", this );
    enabled_->setGeometry( 4, y, 100, 20 );
    y += 22;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 40, 18 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
MELightFrame::applyValues ()
{
  Param params[10];

  object_->getParameter( params );
  params[1].flt = atof( intensity_->text() );
  object_->setParameter( params );
  if( world_ != NULL )
    ((MWorldObject*)world_)->update();
}

void
MELightFrame::chooseColor()
{
  QColor oldColor (color_.red(), color_.green(), color_.blue());
  
  QColor newColor = QColorDialog::getColor (oldColor, 0, "");
  
  if (newColor.isValid()) {
    colorBtn_->setPalette (QPalette(newColor));
    color_.setTo (newColor.red(), newColor.green(), newColor.blue());
    object_->setColor (color_);
  }
}

void
MELightFrame::resizeEvent ( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent (e);
  colorBtn_->resize( width()-8, colorBtn_->height() );
  //intensity_->resize( width()-114, intensity_->height() );
}

void
MELightFrame::setObject ( G3DObject *obj )
{
  object_ = (obj->getType() & T_LIGHT) ? (G3DLight*)obj : 0;
  updateView ();
}

void
MELightFrame::updateView ()
{
  if (0 != object_) {
    Param params[10];
    object_->getParameter (params);
    
    color_ = *(params->color);
    QColor   col( color_.red(), color_.green(), color_.blue() );
    colorBtn_->setPalette( QPalette(col) );
    
    QString str;
    str.setNum( params[1].flt, 'g', 3 );
    intensity_->setText( str );
    shadows_->setChecked( params[2].logic );
    enabled_->setChecked( params[3].logic );
  }
}

//=================================END====================================
