/*
 * Project : "MindsEye, modeler"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: materialFrame.cpp,v 1.3 2001/07/15 20:56:27 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include "kernel/core.h"
#include "kernel/world_obj.h"
#include "material/m_mat.h"
#include "3dlib/shape.h"
#include "3dlib/selectGroup.h"
#include "3dlib/Attributes/texture.h"
#include "modeler.h"
#include "materialFrame.h"
#include "factories.h"

//static MEMaterialFrame instance_;

void
createMaterialWidget( vector<void*> *list, QWidget *p )
{
  //if( instance_ == NULL )
  //instance_ = new MEMaterialFrame(p)
  list->push_back ((void*)(new MEMaterialFrame(p)));
}


MEMaterialFrame::MEMaterialFrame ( QWidget *p ) :
  MEPaletteFrame ( "Material", 0, p)
{
  QButtonGroup *bgroup = new QButtonGroup;
  connect (bgroup, SIGNAL(clicked(int)), this, SLOT(setMaterial(int)));

  int y = tSize;

  /* button to invoke the material editor for the current object's material. */
  QPushButton *btmp;
  btmp = new QPushButton ("edit current material", this);
  btmp->setGeometry (2, y, 125, 20);
  connect( btmp, SIGNAL(clicked()), SLOT(editMaterial()) );
  y += 20;
  
  /* label for the group of buttons below. */
  QLabel *lb = new QLabel ("Set from material editor:", this);
  lb->setGeometry( 2, y, 140, 18 );
  y += 18;

  /* five buttons to set materials 1 through 5 from the material editor. */
  static const char * captions[5] = {"m1", "m2", "m3", "m4", "m5"};
  for (int i=0; i < 5; i++) {
    btmp = new QPushButton (captions[i], this);
    btmp->setGeometry (2 + i*25, y, 25, 20);
    bgroup->insert (btmp);
  }

  visHeight_ = y + 22;
  resize (120, y);
}

void
MEMaterialFrame::editMaterial ()
{
  MEMaterialEditor *matedit =
    (MEMaterialEditor*)Kernel->getModule( MODULE_MATERIAL );  //TODO force
  if (object_ && matedit && IS_TYPE(object_->getType(), T_SHAPE)) {
    G3DAListIterator iterator =
      ((G3DShape*)object_)->getAttribute(TA_TEXTURE);
    if (*iterator) 
      matedit->editMaterial (*iterator);
  }
}

#if 0
void
MMaterialFrame::editMaterial ()
{
  if (object_ != 0 && IS_TYPE (object_->getType(), T_SHAPE) {
    MMaterialEditor *materialEditor = Kernel->getModule("MaterialEditor");
    if (materialEditor != 0) {
      ((G3DShape*)object_)->getAttribute(TA_TEXTURE);
      //...
}
#endif

void
MEMaterialFrame::setMaterial ( int nr )
{
  MEMaterialEditor *matedit =
    (MEMaterialEditor*)Kernel->getModule( MODULE_MATERIAL );

  if (object_ && matedit)
    {
      G3DATexture *mat = (G3DATexture*)((matedit->getMaterial(nr))->copy ());
      
      if (object_->getType() == T_SEL_GROUP)
	{
	  ((G3DSelectionGroup*)object_)->setAttribute (mat);
	  ((MWorldObject*)world_)->update (0);
	}
      else if (IS_TYPE(object_->getType(), T_SHAPE))
	{
	  ((G3DShape*)object_)->setAttribute (mat);
	  ((MWorldObject*)world_)->update ();
	}
      else
	delete mat;
    }
}

void
MEMaterialFrame::setObject ( G3DObject *obj )
{
  if (obj && (IS_TYPE(obj->getType(), T_SHAPE)
	      || obj->getType() == T_SEL_GROUP))
    {
      object_ = obj;
      //addButton_->setEnabled (true);
    }
  else
    object_ = NULL;
}


void
MEMaterialFrame::setWorld ( QObject *obj )
{
  MEPaletteFrame::setWorld( obj );
  connect (obj, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)));
}

void
MEMaterialFrame::updateView ()
{
}

//===================================END=================================
