/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: nurbsFrame.cpp,v 1.1.1.1 1999/09/09 23:57:49 iamcliff Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <stdio.h>
#include <qpushbt.h>
//#include <qradiobt.h>
#include <qpixmap.h>
//#include <qkeycode.h>
#include <qbttngrp.h>
//#include <qlabel.h>
//#include <qlayout.h>

#include "modeler/modeler.h"
#include "nurbsTools.h"
#include "nurbsedit/shaper.h"
#include "nurbsedit/circleWidget.h"
#include "nurbsedit/SkinSurf.h"
#include "nurbsedit/SorWidget.h"
#include "nurbsedit/SweepWidget.h"
#include "nurbsedit/ExtrudeWidget.h"
#include "kernel/m_core.h"


MENurbsTools::MENurbsTools( QObject *model, QWidget *parent ) :
  MEPaletteFrame( "Generate Surface", model, parent )
{
  _model = (MModeler*)model;

  int y = tSize;

  //QBoxLayout  *layout = new QVBoxLayout( this, 4 );
  QPixmap     pixmap;
  QPushButton *btmp;

  /*
  QLabel *label = new QLabel( "Generate surface", this );
    label->setGeometry( 2, 2, 140, 20 );
  */

  genSurfGroup = new QButtonGroup(); //"Generate surface",this) ;
    //genSurfGroup->setGeometry( 120, 45 );
    //genSurfGroup->setFrameStyle( NoFrame );
    connect( genSurfGroup, SIGNAL(clicked(int)), SLOT(genSurf(int)) );
    //layout->addWidget( genSurfGroup );
  btmp = new QPushButton("SOR", this ); //genSurfGroup);
    pixmap.load("icons/nurbsedit/sor.pnm");
    btmp->setPixmap(pixmap) ;
    btmp->setGeometry( 0, y, 35, 30 );
    genSurfGroup->insert( btmp );
  btmp = new QPushButton("Skin", this ); //genSurfGroup) ;
    btmp->setGeometry(35,y,35,30) ;
    genSurfGroup->insert( btmp );
  btmp = new QPushButton("Sweep", this ); //genSurfGroup) ;
    btmp->setGeometry(70,y,35,30) ;
    genSurfGroup->insert( btmp );
  btmp = new QPushButton("Extrude", this ); //genSurfGroup) ;
    btmp->setGeometry(105,y,35,30) ;
    genSurfGroup->insert( btmp );

  skinS    = new SkinSurfWidget;
  surfOfR  = new SurfOfRevWidget;
  sweepS   = new SweptSurfWidget;
  extrudeS = new ExtrudeSurfWidget;

  visHeight_ = y + 32;
  resize( 100, visHeight );
}


MENurbsTools::~MENurbsTools()
{
  delete extrudeS;
  //delete interpWindow;
  delete skinS;
  delete surfOfR;
  delete sweepS;
}


/* TODO
void NurbsWidget::adjustEditBtn(NurbsEditMode mode){
  if(presentMode != mode){
    switch(presentMode){
    case edit_location: editTransformation->setOn(FALSE) ; break ;
    case edit_scale: editScale->setOn(FALSE) ; break ;
    case edit_cp: editCP->setOn(FALSE) ; break ;
    case edit_knots: nurbsO->setDisplayKnots(0) ;
      viewKnotBtn->setOn(FALSE) ; break ;
    case edit_param: selectUVBtn->setOn(FALSE) ; break ;
    case edit_point: editUVBtn->setOn(FALSE) ; break ;
      break ;
    default:
       ;
    }
    switch(mode){
    case edit_location: editTransformation->setOn(TRUE) ; break ;
    case edit_scale: editScale->setOn(TRUE) ; break ;
    case edit_cp: editCP->setOn(TRUE) ; break ;
    case edit_knots: viewKnotBtn->setOn(TRUE) ; break ;
    case edit_param: selectUVBtn->setOn(TRUE) ; break ;
    case edit_point: editUVBtn->setOn(TRUE) ; break ;
      break ;
    default:
       ;
    }
  }
  presentMode = mode ;
}
*/

void MENurbsTools::affectMode( int )
{
  /* TODO
  int affect_flag = 0 ;
  switch(a)
    {
    case 0:
      affect_flag = PLNURBS_FLAGS_AFFECT_ACTIVE_ONLY ; break ;
    case 1:
      affect_flag = PLNURBS_FLAGS_AFFECT_SELECTED_ONLY ; break ;
    case 2:
      affect_flag = PLNURBS_FLAGS_AFFECT_ALL ; break ;
    default:
      ;
    }
    */
  //nurbsO->setAffectFlag(affect_flag) ;
  //nurbsO->setTransformFlag(affect_flag) ;
  //nurbsO->updateDisplayFlags() ;
  
  //nurbsO->updateGL() ;
}

void MENurbsTools::drawCurve()
{
  //interpWindow->show() ;
}

void MENurbsTools::editMode(int a)
{
  /*
  NurbsEditMode current_mode = nurbsO->editMode() ;
  switch(a)
    {
    case 1: break ;
    case 0: 
      if(current_mode == edit_location)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_location) ; 
      break ;
    case 2: 
      if(current_mode == edit_cp)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_cp) ; 
      break ;
    case 3: 
      if(current_mode == edit_scale)
	nurbsO->setEditMode(no_edit) ; 
      else
	nurbsO->setEditMode(edit_scale) ; 
      break ;
    default:
      ;
    }
  nurbsO->updateGL() ;
  */
}

/*
void MENurbsTools::genCurve(int c)
{
  switch(c)
    {
    case 0:
      circleWindow->show();
      break;
    case 1:
      {
	ME2DShaper *tmp = (ME2DShaper*)(Kernel->getModule( MODULE_SHAPER ));
	if( tmp )
	  {
	    G3DObject *obj = tmp ->getObject();
	    if( obj )
	      _model->setNewObject( obj );
	  }
	break;
      }
    }
}
*/

void
MENurbsTools::genSurf(int s)
{
  switch(s)
    {
    case 0:
      surfOfR->show( _model->getWorld() );  break;
    case 1:
      skinS->show( _model->getWorld() );    break;
    case 2:
      sweepS->show( _model->getWorld() );   break;
    case 3:
      extrudeS->show( _model->getWorld() ); break;
    }
}

void
MENurbsTools::knotOp(int o)
{
  /*
  switch(o)
    {
    case 0: 
      if(nurbsO->editMode() == edit_knots)
	{
	  nurbsO->setDisplayKnots(0) ; 
	  nurbsO->setEditMode(no_edit) ;
	}
      else
	{
	  nurbsO->setDisplayKnots(1) ; 
	  nurbsO->setEditMode(edit_knots) ; 
	}
      break ;
    case 1:
      nurbsO->insertKnot(paramU,paramV,useU,useV) ;  //TODO no break???
    case 2:
      nurbsO->deleteKnot() ;
    }
  nurbsO->updateDisplayFlags() ;
  nurbsO->updateGL() ;
  */
}

/*
int MENurbsTools::load(const char* filename)
{
  if(!nurbsO->load(filename,1)){  
    cerr << "Couldn't load nurbs objects from " << filename << endl ;
    return 0 ;
  }
  return 1 ;
}
*/


void
MENurbsTools::makeBtnGroup( QButtonGroup *btnGroup, int num, char **names )
{
  QPushButton *btmp;
  QPixmap pixmap;
  char    pname[40];
  int     x = 0, y = 15;

  for( int i = 0; i < num; i++ )
    {
      btmp = new QPushButton( *names++, btnGroup );
      if( *names )
	{
	  sprintf( pname, "icons/nurbsedit/%s.pnm", *names++ );
	  pixmap.load( pname );
	  btmp->setPixmap( pixmap );
	}
      btmp->setGeometry( x, y, 30, 30 );
      if( i == 0 )
	btmp->setOn(TRUE);
      if( x < 90 )
	x += 30;
      else
	{
	  x = 0; y += 30;
	}
      btmp->setToggleButton( TRUE );
    }
}

/*
void NurbsWidget::paramOp(int o){
  switch(o){
  case 0: 
    if(nurbsO->editMode() == edit_param)
      nurbsO->setEditMode(no_edit) ; 
    else
      nurbsO->setEditMode(edit_param) ; 
    nurbsO->updateGL() ;
    break ;
  case 1: 
    if(nurbsO->editMode() == edit_point)
      nurbsO->setEditMode(no_edit) ; 
    else
      nurbsO->setEditMode(edit_point) ; 
    nurbsO->updateGL() ;
    break ;
  case 2: useU = !useU; break ;
  case 3: useV = !useV; break ;
  case 4: setParam->show() ; break ;// set the parameters value
  }
}
*/

void MENurbsTools::preferenceWindow()
{
  //prefW->show() ;
}


void MENurbsTools::selectMode(int a)
{
  /*
  switch(a)
    {
    case 0: nurbsO->selectAll() ; break ;
    case 1: nurbsO->selectNone() ; break ;
    }
  nurbsO->updateGL() ;
  */
}

/*
void MENurbsTools::setU(float u)
{
  paramU = u ;
  char txt[10] ;
  sprintf(txt,"%2.4f", paramU) ;
  uLabel->setText(txt) ;
}

void MENurbsTools::setV(float v){
  paramV = v ;
  char txt[10] ;
  sprintf(txt,"%2.4f",paramV) ;
  vLabel->setText(txt) ;
}

void MENurbsTools::setUV(float u, float v)
{
  //nurbsO->setUV(u,v) ;
}
*/

void MENurbsTools::setSurfaceView(int a)
{
  /*TODO
  switch(a)
    {
    case 0:
      G3DNurbsDisplayMode = PLNURBS_DISPLAY_NORMAL ; break ;
    case 1:
      G3DNurbsDisplayMode = PLNURBS_DISPLAY_ISOCURVES ; break ;
    }
    */
}

void MENurbsTools::viewMode(int a)
{
  switch(a)
    {
    case 0:
      //nurbsO->setDisplayObject(!nurbsO->displayObject()) ;
      break ;
      //case 1: nurbsO->setDisplayPoints(!nurbsO->displayPoints()) ;  break ;
      //case 2: nurbsO->setDisplayPolygon(!nurbsO->displayPolygon());   break ;
    case 3:
      emit info(); break;
      //_model->info(); break; // weight
      //case 4: nurbsO->setDisplayAxis(!nurbsO->displayAxis()) ; break ;
      //case 5: nurbsO->setDisplayCenter(!nurbsO->displayCenter()) ; break ;
      //case 6: nurbsO->resetView() ; break ;
      //case 7: nurbsO->setDisplayBBox(!nurbsO->displayBBox()) ; break ;
    default:
      ;
    }
  //nurbsO->updateDisplayFlags();
  //nurbsO->updateGL();
}

/*
int MENurbsTools::save(const char* filename,int saveMode){
  return nurbsO->save(filename,saveMode) ;
}
*/



/*
PreferenceWidget::PreferenceWidget( QWidget *parent, const char *name )
  : QWidget(parent,name)
{
  initMetaObject() ;
  setMinimumSize(300,200) ;
  setMaximumSize(300,200) ;
  hide() ;

  QLabel *surfaceLabel = new QLabel( "View surface as", this );

  surfaceLabel->move(10,30) ;

  QButtonGroup *surfaceView = new QButtonGroup(this) ;

  surfaceView->setFrameStyle(QFrame::NoFrame) ;
  surfaceView->setGeometry(110,30,90,60) ;

  QRadioButton *normalS = new QRadioButton("normal",surfaceView) ;
  QRadioButton *isocurveS = new QRadioButton("iso curves",surfaceView) ;

  normalS->setGeometry(5,5,80,15);
  isocurveS->setGeometry(5,20,80,15) ;

  PlNurbsDisplayMode = PLNURBS_DISPLAY_ISOCURVES ;
  isocurveS->setChecked(TRUE) ;

  QPushButton *okButton = new QPushButton("Set",this) ;
  okButton->setGeometry(100,170,50,25) ;

  //TODO connect with SIGNAL?
  connect(surfaceView,SIGNAL(clicked(int)),SLOT(newSurfaceView(int))) ;
  connect(okButton,SIGNAL(clicked()), SLOT(hideWindow())) ;
}

void PreferenceWidget::hideWindow()
{
  hide() ;
}

void PreferenceWidget::newSurfaceView(int a)
{
  emit changeSurfaceView(a) ;
}


*/
