/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: nurbsTools.cpp,v 1.2 2001/07/17 02:34:38 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpushbt.h>
#include <qpixmap.h>
#include <qbttngrp.h>

#include "modeler.h"
#include "nurbsTools.h"
#include "nurbsedit/circleWidget.h"
#include "nurbsedit/SkinSurf.h"
#include "nurbsedit/SorWidget.h"
#include "nurbsedit/SweepWidget.h"
#include "nurbsedit/ExtrudeWidget.h"
#include "kernel/core.h"


MENurbsTools::MENurbsTools( MModeler *model, QWidget *parent ) :
  MEPaletteFrame( "Generate Surface", model, parent )
{
  _model = model;

  int y = tSize;

  //QBoxLayout  *layout = new QVBoxLayout( this, 4 );
  QPixmap     pixmap;
  QPushButton *btmp;

  /*
  QLabel *label = new QLabel( "Generate surface", this );
    label->setGeometry( 2, 2, 140, 20 );
  */

  genSurfGroup = new QButtonGroup(); //"Generate surface",this) ;
    //genSurfGroup->setGeometry( 120, 45 );
    //genSurfGroup->setFrameStyle( NoFrame );
    connect( genSurfGroup, SIGNAL(clicked(int)), SLOT(genSurf(int)) );
    //layout->addWidget( genSurfGroup );
  btmp = new QPushButton ("SOR", this); //genSurfGroup);
    pixmap.load ("icons/nurbsedit/sor.pnm");
    btmp->setPixmap (pixmap) ;
    btmp->setGeometry (0, y, 35, 30);
    genSurfGroup->insert (btmp);
  btmp = new QPushButton ("Skin", this); //genSurfGroup) ;
    btmp->setGeometry(35,y,35,30) ;
    genSurfGroup->insert( btmp );
  btmp = new QPushButton("Sweep", this ); //genSurfGroup) ;
    btmp->setGeometry(70,y,35,30) ;
    genSurfGroup->insert( btmp );
  btmp = new QPushButton("Extrude", this ); //genSurfGroup) ;
    btmp->setGeometry(105,y,35,30) ;
    genSurfGroup->insert( btmp );

  skinS    = new SkinSurfWidget;
  surfOfR  = new SurfOfRevWidget;
  sweepS   = new SweptSurfWidget;
  extrudeS = new ExtrudeSurfWidget;

  visHeight_ = y + 35;
  resize( 100, visHeight_ );
}


MENurbsTools::~MENurbsTools()
{
  delete extrudeS;
  //delete interpWindow;
  delete skinS;
  delete surfOfR;
  delete sweepS;
}


void
MENurbsTools::genSurf(int s)
{
  switch(s)
    {
    case 0:
      surfOfR->show (_model->getWorld());  break;
    case 1:
      skinS->show (_model->getWorld());    break;
    case 2:
      sweepS->show (_model->getWorld());   break;
    case 3:
      extrudeS->show (_model->getWorld()); break;
    }
}

void
MENurbsTools::resizeEvent ( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent (e);
  //scale_->resize( width()-4, 20 );
  //rotation_->resize( width()-4, 20 );
  //position_->resize( width()-4, 20 );
}

/*
void
MENurbsTools::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
}
*/
void
MENurbsTools::setWorld( QObject *obj )
{
  /*
  MEPaletteFrame::setWorld( obj );
  connect( world_, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)) );
  */
}

void
MENurbsTools::updateView()
{
  /*
  if( object_ )
    {
      scale_->setVector( object_->getScale() );
      G3DVector3d rot;
      const G3DQuaternion &quat = object_->getOrientation();
      cout << quat  << endl;
      quat.quat2xyz(rot);
      //(object_->getRotation()).quat2xyz( rot ); //deconstruct(rot);
      rot[0] = rad2deg( rot[0] );
      rot[1] = rad2deg( rot[1] );
      rot[2] = rad2deg( rot[2] );
      rotation_->setVector( rot );
      position_->setVector( object_->getOrigin() );
    }
  else
    {
      scale_->erase();
      rotation_->erase();
      position_->erase();
    }
  */
}


//=================================END====================================
