/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: plightFrame.cpp,v 1.1.1.1 1999/09/09 23:57:53 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>
#include <qslider.h>

#include "plightFrame.h"
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "factories.h"
#include <vector>


void
createPLightWidget( vector<void*> *list, QWidget *p )
{
  list->push_back( (void*)(new MEPointLightFrame(p)) );
}



MEPointLightFrame::MEPointLightFrame( QWidget *p ) :
  MEPaletteFrame( "PointLight Paramter", NULL, p)
{
  int y = tSize;

  QLabel *lb = new QLabel( "falloff: ", this );
    lb->setGeometry( 2, y, 40, 20 );
    y += 20;

  slider_ = new QSlider( QSlider::Horizontal, this );
    slider_->setRange( 0, 100 );
    slider_->setGeometry( 4, y, 120, 18 );
    connect( slider_, SIGNAL(valueChanged(int)), SLOT(setFalloff(int)) );
    y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
MEPointLightFrame::applyValues( )
{
  /*
  Param params[10];
  const char *str;
  MeFloat l;

  object_->getParameter( params );

  object_->setParameter( params );
  if( world_ != NULL )
    ((MWorldObject*)world_)->update();
*/
}

void
MEPointLightFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  slider_->resize( width()-8, slider_->height() );
}

void
MEPointLightFrame::setFalloff( int index )
{ 
  /*This nice algorithm is based on a description in
    "QuickDraw 3D Preliminary User Interface Guideline"
  */
  MeFloat f = MeFloat(index) / 100.0;
  MeFloat k = 1/(1 - 0.75*f);
  MeFloat p = 1.0; // distance between light and light-at position
  MeFloat c0, c1, c2;

  if( k <= 2.0 )
    { 
      c0 = 2 - k;
      c1 = (k-1)/p;
      c2 = 0;
    } 
  else
    { 
      c0 = 0; 
      c1 = (4 - k) / (2*p);
      c2 = (k - 2) / (2*p*p);
    }

  Param params[10];
  object_->getParameter( params );
  params[4].flt  = c0;
  params[5].flt  = c1;
  params[6].flt  = c2;
  object_->setParameter( params );
  object_->adjust();
  if( world_ != NULL )
    ((MWorldObject*)world_)->update();
}

void
MEPointLightFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
}

void
MEPointLightFrame::updateView()
{
  if( object_ )
    {
      Param params[10];
      MeFloat c0, c2, k, f, p = 1.0;

      object_->getParameter( params );

      c0 = params[4].flt;
      c2 = params[6].flt;
      if( c2 > 0 )
	k = (c2 * 2*p*p) + 2;
      else //c0 > 0
	k = 2 - c0;
      f = (1 - 1/k) * 4.0 / 3.0;
      slider_->setValue( int(f*100) );
    }
}


//=================================END====================================
