/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: polygonFrame.cpp,v 1.1.1.1 1999/09/09 23:57:54 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>

#include "polygonFrame.h"
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "factories.h"
#include <vector>


void
createPolygonWidget( vector<void*> *list, QWidget *p )
{
  //TODO createShapeFrame( list, p );
  //createMaterialWidget( list, p );
  list->push_back( (void*)(new MEPolygonFrame(p)) );
}

//--------------------------------------------------------------------


MEPolygonFrame::MEPolygonFrame( QWidget *p ) :
  MEPaletteFrame( "Polygon Parameter", NULL, p)
{
  int y = tSize;

  QLabel *lb = new QLabel( "Nr. of vertices", this );
  lb->setGeometry( 2, y, 96, 20 );
  nVertices_ = new QLineEdit( this );
  nVertices_->move( 78, y );
  y += 20;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 100, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
MEPolygonFrame::applyValues( )
{
  Param params[4];
  const char *str;

  object_->getParameter( params );

  str    = nVertices_->text();
  int nv = atoi( str );
  if( nv < 3 )
    nv = 3;

/*
  object_->setParameter( params );
  object_->adjust();
  if( world_ != NULL )
    ((MWorldObject*)world_)->update();
*/
}

void
MEPolygonFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  nVertices_->resize( width()-100, 20 );
}

void
MEPolygonFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
}

void
MEPolygonFrame::updateView()
{
  if( object_ )
    {
      QString str;
      Param params[4];
      object_->getParameter( params );

      str.setNum( params[1].number );
      nVertices_->setText( str );
      
    }
  else
    {
      nVertices_->setText( "" );
    }
}


//=================================END====================================
