/*
 * Project : "MindsEye"
 * Copyright (c) 1996 - 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: sceneInfo.cpp,v 1.3 2002/06/16 01:52:19 ainvar Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <qlabel.h>
#include <qmultilinedit.h>
#include <qpushbutton.h>
#include <qcolordialog.h>
// #include "dialogs/color/colorpicker.h"
#include <qcolordialog.h>
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "factories.h"
#include "sceneInfo.h"
#include <vector>


void
createSceneInfo( vector<void*> *list, QWidget *p )
{
  list->push_back( (void*)(new MESceneInfo(p)) );
}

//--------------------------------------------------------------------


MESceneInfo::MESceneInfo( QWidget *parent ) :
  MEPaletteFrame( "Scene Parameter", NULL, parent )
{
  int y = tSize;

    //This one is used to add some informative text to the scene
  infoEditor_ = new QMultiLineEdit( this, NULL ); //TODO
  infoEditor_->setGeometry( 2, y, 100, 40 );
  infoEditor_->setText( "Enter a description" );
  y += 42;

  colButton_ = new QPushButton( "Background", this );
  connect( colButton_, SIGNAL(clicked()), SLOT(setBgColor()) );
  colButton_->setGeometry( 4, y, 100, 20 );
  y += 22;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(4, y, 40, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 22;

  visHeight_ = y;
  resize( 100, y );
}

void
MESceneInfo::applyValues( )
{
  Param params[4];
  scene_->getParameter( params );
  //*(params->color) = bgColor_;

  const QString &text = infoEditor_->text();
  //cout << text << endl;
  int l = text.length(); //strlen( text );
  if( (params[1].string == NULL) || (l > strlen(params[1].string)) )
    params[1].string = new char[l+1];
  strcpy( params[1].string, text );

  scene_->setParameter( params );
  if( world_ != NULL )
    ((MWorldObject*)world_)->update();
}

void
MESceneInfo::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );  
  infoEditor_->resize( width()-4, infoEditor_->height() );
  colButton_->resize( width()-8, colButton_->height() );
}

void
MESceneInfo::setBgColor()
{
  QColor oldColor (bgColor_.red(), bgColor_.green(), bgColor_.blue());
  
  QColor newColor = QColorDialog::getColor (oldColor, 0, "");
  
  if (newColor.isValid()) {
    colButton_->setPalette (QPalette(newColor));
    bgColor_.setTo (newColor.red(), newColor.green(), newColor.blue());
  }
}

void
MESceneInfo::setObject( G3DObject *obj )
{
  scene_ = obj;
  updateView();
}

void
MESceneInfo::updateView( )
{
  if( scene_ )
    {
      Param params[4];
      scene_->getParameter( params );
      bgColor_ = *(params->color);
      infoEditor_->setText( params[1].string );
      
      QColor col( bgColor_.red(), bgColor_.green(), bgColor_.blue() );
      colButton_->setPalette( QPalette(col) );
    }
  else
    infoEditor_->setText( "" );
}

//=====================================END================================
