/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: sphereFrame.cpp,v 1.1.1.1 1999/09/09 23:57:59 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>

#include "sphereFrame.h"
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "factories.h"

void
createSphereWidget( vector<void*> *list, QWidget *p )
{
  list->push_back( (void*)(new MESphereFrame(p)) );
}

MESphereFrame::MESphereFrame( QWidget *p ) :
  MEPaletteFrame( "Sphere Paramter", NULL, p)
{
  int y = tSize;
  
  QLabel *lb = new QLabel( "Radius", this );
    lb->setGeometry( 2, y, 75, 20 );
  radius_ = new QLineEdit( this );
    radius_->move( 78, y );
    y += 20;

  lb = new QLabel( "section", this );
    lb->setGeometry( 2, y, 60, 20 );
  section_ = new QLineEdit( this );
    section_->move( 78, y );
    y += 20;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 30, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 22;

  visHeight_ = y;
  resize( 100, y );
}

void
MESphereFrame::applyValues()
{
  Param params[4];
  const char *str;
  MeFloat l;
  object_->getParameter( params );

  str = radius_->text();
  l   = atof( str );
  if( l < 0 )
    l = 0;
  params[0].flt = l;

  str = section_->text();
  l = atof( str );
  if( l >= 1 || l <= 0 )
    l = M_PI;
  else
    l = M_PI * l;
  params[1].flt = l;

  object_->setParameter( params );
  if( world_ ) //TODO
    ((MWorldObject*)world_)->update();
}

void
MESphereFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  radius_->resize( width()-80, 20 );
  section_->resize( width()-80, 20 );
}

void
MESphereFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
  //connect( model
}

void
MESphereFrame::updateView()
{
  if( object_ )
    {
      QString str;
      Param params[4];
      object_->getParameter( params );

      str.setNum( params[0].flt, 'g', 4 );
      radius_->setText( str );
      str.setNum( params[1].flt / M_PI, 'g', 4 );
      section_->setText( str );
    }
  else
    {
      radius_->setText( "" );
      section_->setText( "" );
    }
}


//=================================END====================================
