/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: spinFrame.cpp,v 1.1.1.1 1999/09/09 23:58:00 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>

#include "spinFrame.h"
#include "3dlib/shape.h"
#include "3dlib/Attributes/anim_spin.h"
#include "kernel/world_obj.h"
#include "dialogs/vedit.h"


MESpinFrame::MESpinFrame( QWidget *p ) :
  MEPaletteFrame( "Spin Parameter", NULL, p)
{
  int y = tSize;

  QLabel *lb = new QLabel( "axis", this );
    lb->setGeometry( 5, y, 75, 18 );
    y += 18;
  axis_ = new MEVectorEdit( this );
    axis_->move( 2, y );
    y += 20;

  lb = new QLabel( "center", this );
    lb->setGeometry( 5, y, 75, 18 );
    y += 18;
  center_ = new MEVectorEdit( this );
    center_->move( 2, y );
    y += 22;

  lb = new QLabel( "rotations / s", this );
    lb->setGeometry( 5, y, 92, 20 );
    speed_= new QLineEdit( this );
    speed_->setGeometry( 97, y, 10, 20 );
    y += 22;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 30, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 23;

  visHeight_ = y;
  resize( 100, y );
}

void
MESpinFrame::applyValues()
{
  Param params[4];
  attrib_->getParameter( params );

  axis_->getVector( *(params[0].vector3) );
  center_->getVector( *(params[1].point3) );
  params[2].flt = atof( speed_->text() );

  attrib_->setParameter( params );
  object_->adjust();  //TODO from setParameter?
  if( world_ )
    ((MWorldObject*)world_)->update();
}

void
MESpinFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  axis_->resize( width()-4, axis_->height() );
  center_->resize( width()-4, center_->height() );
  speed_->resize( width() - 100, speed_->height() );
}

void
MESpinFrame::setObject( G3DObject *obj )
{
  object_ = (G3DShape*)obj;
  G3DAListIterator iterator = object_->getAttribute( TA_ANIM_SPIN );
  attrib_ = *iterator;

  updateView();
}

void
MESpinFrame::updateView()
{
  QString str;
  Param params[4];
  attrib_->getParameter( params );

  axis_->setVector( *(params[0].vector3) );
  center_->setVector( *(params[1].point3) );
  str.setNum( params[2].flt, 'g', 4 );
  speed_->setText( str );
}


//---------------------------------------------------------------------

#include "factories.h"
#include <vector>

void
createSpinWidget( vector<void*> *list, QWidget *p )
{
  list->push_back( (void*)(new MESpinFrame(p)) );
}


//=================================END====================================
