/*
 * Project : "MindsEye"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s):   Philippe Lavoie, Martin Kuball
 * 
 * $Id: tabwidget.cpp,v 1.5 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qpainter.h>
#include <qscrollbar.h>
#include <qtabbar.h>
#include "tabwidget.h"

//TODO
#define WIDGET_HEIGHT 2000

MeTabChild::MeTabChild( QWidget *parent ) :
  QWidget(parent)
{
  pos_   = 0;
  psize_ = 0;
}


MeTabWidget::MeTabWidget( QWidget *parent, const char *name) :
  QWidget(parent,name) 
{
  setMinimumSize (200, 200);
  size_ = 0;
  tab_  = new QTabBar( this, "tab" );
  connect( tab_, SIGNAL(selected(int)), SLOT(showPage(int)) );

  scroller_ = new QScrollBar( QScrollBar::Vertical, this );
  connect( scroller_, SIGNAL(valueChanged(int)), SLOT(scroll(int)) );
  scrollerWidth_ = 8;
}

int
MeTabWidget::addTab( int mode, MeTabChild* w, const QString &name )
{
  QTab *t = new QTab;
  t->label = name;
  w->id_   = tab_->addTab( t );
  w->mode_ = mode;
  widgets_[size_++] = w;

  w->setGeometry( 0, tab_->height()+2,
		  width()-scrollerWidth_-1, WIDGET_HEIGHT );
  //connect( w, SIGNAL(setPageSize(int)), SLOT(setPageSize(int)) );

  tab_->setTabEnabled( w->id_, TRUE );
  tab_->raise();
}

int
MeTabWidget::current () const
{
  int id = tab_->currentTab();
 
  if( id >= 0 )
    {
      for( int i = 0; i < size_; ++i )
	{
	  if( widgets_[i]->id_ == id )
	    return( i );
	}
    }
  return 0;
}

void
MeTabWidget::paintEvent( QPaintEvent* )
{
  if(!tab_)
    return ; 

  const int r = width() - 1 - scrollerWidth_;
  const int l = 0; 
  const int t = tab_->height() - 5;
  const int b = height() - 1; 

  QPainter p ; 
  p.begin(this);
  p.setPen(white);
  p.drawLine( l, t, r - 1, t );
  p.end();
}

void
MeTabWidget::resizeEvent( QResizeEvent * )
{
  //cout << "tabw resizeEvent" << endl;
  tab_->resize( width(), tab_->height() );
  int page = height() - tab_->height() - 2;
  scroller_->setGeometry( width() - scrollerWidth_, tab_->height()+2,
			  scrollerWidth_, page );
  setScroller();
  //setPageSize( widgets_[current()].psize );

  /*
  //TODO this is only necessary once
  for( int i = 0; i < size_; ++ i )
    {
      widgets_[i].w->move( 0, tab_->height()+2 - widgets_[i].pos );
      widgets_[i].w->resize( width()-scrollerWidth_-1, WIDGET_HEIGHT );
    }
*/
}

void
MeTabWidget::scroll( int val )
{
  widgets_[current()]->pos_ = val;
  widgets_[current()]->move( 0, tab_->height()+2-val );
}
/*
void
MeTabWidget::setCurrent( int id )
{
  tab_->setCurrentTab( id );
  showPage( id );
}
*/
void
MeTabWidget::setScroller () //PageSize( int h )
{
  int i = current();
  int page = height() - tab_->height() - 2;
  int h = widgets_[i]->psize_;
  if( page > h )
    page = h;

  //cout << "ps " << h << " " << page << endl;
  scroller_->setRange( 0, h - page );
  scroller_->setSteps( 5, page );
  scroller_->setValue( widgets_[i]->pos_ );
}

void
MeTabWidget::show()
{
  tab_->setFocus(); 
  QSize min( tab_->sizeHint() );
  tab_->resize( min ); 
  QWidget::show(); 
}

void
MeTabWidget::showPage( int id )
{
cout << "show page" << endl;
  if( tab_->currentTab() != id )
    {
      tab_->setCurrentTab( id );
      return ; // no need to go through the list since
               // showPage will be called again
    }

cout << "show page 2" << endl;
  // go through the list and show the widget with the proper id
  int mode;
  for( int i = 0; i < size_; ++i )
    {
      if( widgets_[i]->id_ == id )
	{
	  widgets_[i]->show();
	  mode = widgets_[i]->mode_;
	  setScroller();
	  //setPageSize( widgets_[i].psize );
	  //scroller_->setValue( widgets_[i].pos );
	}
      else
        widgets_[i]->hide();
    }
  emit selected( mode ); 
}


//=================================END===================================
