/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: torusFrame.cpp,v 1.2 2001/07/09 21:50:20 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qlabel.h>
#include <qlined.h>
#include <qpushbutton.h>

#include "torusFrame.h"
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "factories.h"
#include <vector>


void
createTorusWidget( vector<void*> *list, QWidget *p )
{
  //Kernel->getPaletteWidget( G3DShape::GetTypeName(), list, p );
  //createShapeWidget( list, p );
  list->push_back( (void*)(new METorusFrame(p)) );
}


//------------------------------------------------------------------------


METorusFrame::METorusFrame( QWidget *p ) :
  MEPaletteFrame( "Torus Paramter", NULL, p)
{
  int y = tSize;
  
  QLabel *lb = new QLabel( "Major radius", this );
    lb->setGeometry( 2, y, 75, 20 );
    radius1_ = new QLineEdit( this );
    radius1_->move( 78, y );
    y += 20;

  lb = new QLabel( "Minor radius", this );
    lb->setGeometry( 2, y, 75, 20 );
    radius2_ = new QLineEdit( this );
    radius2_->move( 78, y );
    y += 20;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 30, 20 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 22;

  visHeight_ = y;
  resize( 100, y );
}

void
METorusFrame::applyValues()
{
  Param params[4];
  const char *str;
  MeFloat l;
  object_->getParameter( params );

  str = radius1_->text();
  l   = atof( str );
  if( l < 0 )
    l = 0;
  params[0].flt = l;

  str = radius2_->text();
  l   = atof( str );
  if( l < 0 )
    l = 0;
  params[1].flt = l;

  object_->setParameter( params );
  object_->adjust();  //TODO from setParameter?
  if( world_ )
    ((MWorldObject*)world_)->update();
}

void
METorusFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  radius1_->resize( width()-80, 20 );
  radius2_->resize( width()-80, 20 );
}

void
METorusFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
}

void
METorusFrame::updateView()
{
  if( object_ )
    {
      QString str;
      Param params[4];
      object_->getParameter ( params );

      str.setNum( params[0].flt, 'g', 4 );
      radius1_->setText( str );
      
      str.setNum( params[1].flt, 'g', 4 );
      radius2_->setText( str );
    }
  else
    {
      radius1_->setText( "" );
      radius2_->setText( "" );
    }
}


//=================================END====================================
