/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: trafoFrame.cpp,v 1.1.1.1 1999/09/09 23:58:04 iamcliff Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "include/config.h"
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include "trafoFrame.h"
#include "dialogs/vedit.h"
#include "3dlib/object.h"
#include "3dlib/g3d_math.h"
#include "kernel/world_obj.h"

static const char *pivotTypes[] = { "local", "parent", NULL };


METrafoFrame::METrafoFrame( QObject *modeler, QWidget *p ) :
  MEPaletteFrame( "Transformation", modeler, p)
{
  pivotType_ = 0;
  int y = tSize;

  QLabel *lb = new QLabel( "Position", this );
  lb->setGeometry( 2, y, 70, 20 );
  y += 18;
  position_ = new MEVectorEdit( this );
  position_->move( 2, y );
  y += 22;

  lb = new QLabel( "Scale (x-y-z)", this );
  lb->setGeometry( 2, y, 90, 20 );
  y += 18;
  scale_ = new MEVectorEdit( this );
  scale_->move( 2, y );
  y += 24;

  lb = new QLabel( "Pivot Point", this );
  lb->setGeometry( 2, y, 70, 20 );
  cbPivot_ = new QComboBox( this );
  cbPivot_->setGeometry( 74, y, 10, 20 );
  cbPivot_->insertStrList( pivotTypes );
  connect( cbPivot_, SIGNAL(activated(int)), SLOT(setPivotType(int)) );
  y += 20;
  pivot_ = new MEVectorEdit( this );
  pivot_->move( 2, y );
  y += 22;

  lb = new QLabel( "Rotation (x-y-z)", this );
  lb->setGeometry( 2, y, 90, 20 );
  y += 18;
  rotation_ = new MEVectorEdit( this );
  rotation_->move( 2, y );
  y += 22;

  QPushButton *btn = new QPushButton( "set", this );
    btn->setGeometry(2, y, 40, 18 );
    connect( btn, SIGNAL(clicked()), SLOT(applyValues()) );
    y += 20;

  visHeight_ = y;
  resize( 100, y );
}

void
METrafoFrame::applyValues()
{
  G3DVector3d vec;
  scale_->getVector( vec );
  object_->setScale( vec );

  rotation_->getVector( vec );
  vec[0] = deg2rad( vec[0] ) / 2;
  vec[1] = deg2rad( vec[1] ) / 2;
  vec[2] = deg2rad( vec[2] ) / 2;
  object_->setOrientation( G3DQuaternion(vec), object_->getPivotPoint(),
			   object_->getOrigin() );

  position_->getVector( vec );
  object_->setOrigin( vec );

  if( pivotType_ == 0 )
    {
      pivot_->getVector( vec );
      object_->setPivotPoint( vec );
    }
  else
    {
      G3DMatrix trafo;
      object_->getLocalTrafo( trafo );
      trafo.invert4();
      G3DPoint3d pnt;
      pivot_->getVector( pnt );
      object_->setPivotPoint( trafo*pnt );
    }

  object_->adjust();
  if( world_ != NULL ) //TODO
    ((MWorldObject*)world_)->update();

}

void
METrafoFrame::resizeEvent( QResizeEvent *e )
{
  MEPaletteFrame::resizeEvent( e );
  scale_->resize( width()-4, 20 );
  rotation_->resize( width()-4, 20 );
  pivot_->resize( width()-4, 20 );
  cbPivot_->resize( width()-76, 20 );
  position_->resize( width()-4, 20 );
}

void
METrafoFrame::setObject( G3DObject *obj )
{
  object_ = obj;
  updateView();
}

void
METrafoFrame::setPivotType( int t )
{
  pivotType_ = t;
  if( pivotType_ == 0 )
    pivot_->setVector( object_->getPivotPoint() );
  else
    {
      G3DMatrix trafo;
      object_->getLocalTrafo( trafo );
      pivot_->setVector( trafo*(object_->getPivotPoint()) );
    }
}

void
METrafoFrame::setWorld( QObject *obj )
{
  MEPaletteFrame::setWorld( obj );
  connect( world_, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(setObject(G3DObject*)) );
}

void
METrafoFrame::updateView()
{
  if( object_ )
    {
      scale_->setVector( object_->getScale() );
      G3DVector3d rot;
      const G3DQuaternion &quat = object_->getOrientation();
      //cout << "update " << quat  << endl;
      quat.quat2xyz(rot);
      //(object_->getRotation()).quat2xyz( rot ); //deconstruct(rot);
      rot[0] = rad2deg( rot[0] );
      rot[1] = rad2deg( rot[1] );
      rot[2] = rad2deg( rot[2] );
      rotation_->setVector( rot );
      setPivotType( pivotType_ );
      position_->setVector( object_->getOrigin() );
    }
  else
    {
      scale_->erase();
      rotation_->erase();
      position_->erase();
    }
}


//=================================END====================================
