/*
 * Project : "MindsEye, NURBS editor"
 * Copyright 1996, 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: ExtrudeWidget.cpp,v 1.3 2002/06/10 09:30:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <qpushbt.h>
#include <qlabel.h>
#include <qlined.h>
#include <nurbs++/specialType.h>

#include "kernel/world_obj.h"
#include "3dlib/t_objects.h"
#include "3dlib/s_nurbssurf.h"
#include "3dlib/s_nurbscurve.h"
#include "ExtrudeWidget.h"
#include "3dlib/math/tools.h"



ExtrudeSurfWidget::ExtrudeSurfWidget( QWidget *parent, const char *name ) :
  QWidget(parent, name)
{
  //initMetaObject() ;
  setMinimumSize(150,160) ;
  setMaximumSize(150,160) ;
  _model = NULL;

  QLabel *label1 = new QLabel("Extruding a curve",this) ;
  label1->setGeometry(10,10,120,15) ;

  QLabel *label2 = new QLabel("Direction X:",this) ;
  label2->setGeometry(20,40,75,15) ;
  QLabel *label3 = new QLabel("Direction Y:",this) ;
  label3->setGeometry(20,60,75,15) ;
  QLabel *label4 = new QLabel("Direction Z:",this) ;
  label4->setGeometry(20,80,75,15) ;
  QLabel *label5 = new QLabel("Length:",this);
  label5->setGeometry(50,100,45,15) ;

  QLineEdit *editTx = new QLineEdit(this) ;
  QLineEdit *editTy = new QLineEdit(this) ;
  QLineEdit *editTz = new QLineEdit(this) ;
  QLineEdit *editLength = new QLineEdit(this) ;

  editTx->setGeometry(100,40,40,20) ;
  editTy->setGeometry(100,60,40,20) ;
  editTz->setGeometry(100,80,40,20) ;
  editLength->setGeometry(100,100,40,20) ;

  editTx->setText("1") ;
  editTy->setText("0") ;
  editTz->setText("0") ;
  editLength->setText("1") ;
  tx = 1 ;
  ty = 0 ;
  tz = 0 ;
  length = 1 ;

  connect(editTx,SIGNAL(textChanged(const QString&)), 
          SLOT(editTx(const QString&))) ;
  connect(editTy,SIGNAL(textChanged(const QString&)),
          SLOT(editTy(const QString&))) ;
  connect(editTz,SIGNAL(textChanged(const QString&)),
          SLOT(editTz(const QString&))) ;
  connect(editLength,SIGNAL(textChanged(const QString&)),
          SLOT(editLength(const QString&))) ;
  
  QPushButton *okBtn = new QPushButton("Ok",this) ;
  QPushButton *cancelBtn = new QPushButton("Cancel",this) ;
  
  okBtn->setGeometry(20,130,60,20) ;
  cancelBtn->setGeometry(80,130,60,20) ;

  connect(okBtn,SIGNAL(clicked()),SLOT(extrudeSurface())) ;
  connect(cancelBtn,SIGNAL(clicked()),SLOT(hideWindow())) ;
}

void
ExtrudeSurfWidget::editLength(const QString& d)
{
  length = float( atof(d.ascii()) );
}

void
ExtrudeSurfWidget::editTx(const QString& d)
{
  tx = float( atof(d.ascii()) );
}

void
ExtrudeSurfWidget::editTy(const QString& d)
{
  ty = float( atof(d.ascii()) );
}

void
ExtrudeSurfWidget::editTz(const QString& d)
{
  tz = float( atof(d.ascii()) );
}

void
ExtrudeSurfWidget::extrudeSurface()
{
  G3DObject *obj;
  if( _model )
    {
      obj = _model->getSelection();
      if( obj && (obj->getType() == T_NURBS_CURVE) )
	{
	  G3DNurbsSurface *s = new G3DNurbsSurface(1);

	  PlNurbsCurvef n1( *(G3DNurbsCurve*)obj );
	  MatrixRT<float> trafo;
	  G3DMatrix       tobj;
	  obj->getLocalTrafo( tobj );
	  convertMatrix( trafo, tobj );
	  n1.transform( trafo );
	  PlNurbsCurvef n2( n1 );
	  
	  Point3Df dir( tx, ty, tz );
	  dir *= (length / norm(dir));

	  MatrixRT<float> Tx ;
	  n2.transform( Tx.translate(dir.x(),dir.y(),dir.z()) );
	  PlNurbsCurveArrayf ca;
	  ca.resize( 2 );
	  ca[0] = n1;
	  ca[1] = n2;
	  s->skinU( ca, 1 );

	  _model->addObject( s );
	}
    }
  hide() ;
}

void
ExtrudeSurfWidget::hideWindow()
{
  hide() ;
}

void
ExtrudeSurfWidget::show( MWorldObject *world )
{
  _model = world;
  QWidget::show();
}


//==================================END==============================
