//occtestviewer.cpp

// (C) 2000  C R Johnson   iamcliff@ix.netcom.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU  General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//                                                                
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details. 
//                                      
// You should have received a copy of the GNU General Public License 
// along with this software (see COPYING); if not, write to the 
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "occtestviewer.h"
//#include <GL/gl.h>
//#include <GL/glu.h>
//#include <GL/glx.h>
#include<iostream.h>
#include<qapplication.h>
#include<V3d_OrthographicView.hxx>
#include<TopoDS_Solid.hxx>
#include<BRepPrimAPI_MakeBox.hxx>
#include<AIS_Shape.hxx>
#include<AIS_Trihedron.hxx>
#include<Geom_Axis2Placement.hxx>
#include<BRepOffsetAPI_MakeEvolved.hxx>
#include<BRepBuilderAPI_MakePolygon.hxx>

static const Standard_Real twistFudge = .1;
static const Standard_Real rotateFudge = .02;

//----------------------Constructor----------------------------------------

OccTestViewer::OccTestViewer(QWidget* parent, const char* name)
	: QGLWidget(QGLFormat(DoubleBuffer|DirectRendering),parent,name)
	,haveTrihedron(false),haveBox(false),inViewManipulation(false)
	,speedFactor(2.),zoomFactor(.05),inMotion(false),haveSweep(false)
{
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::OccTestViewer()" << endl;
#endif

	setMinimumSize(500,475);
	setMaximumSize(1600,1200);
	occTestDevice=new Graphic3d_GraphicDevice("");
	occTestWindow=new Xw_Window(occTestDevice,winId());

#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::OccTestViewer()" << endl;
#endif

}

//--------------------------------------------------------------


void OccTestViewer::SetContext(const Handle(AIS_InteractiveContext)& aContext) 
{
	occTestContext = aContext;
}

//--------------------------------------------------------------
void OccTestViewer::SetView(const Handle(V3d_View)& aView) 
{ 
	occTestView=aView; 
	occTestView->SetWindow(occTestWindow); 
}
//--------------------------------------------------------------
void OccTestViewer::paintEvent ( QPaintEvent * ) 
{ 
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::paintEvent()" << endl;
#endif

//if (!occTestView.IsNull()) 
//{ 
//	cout << "occTestView is not null" << endl; 
//	occTestView->Redraw();
//}

#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::paintEvent()" << endl << endl;
#endif
} 

//--------------------------------------------------------------
void OccTestViewer::resizeEvent ( QResizeEvent * qre ) 
{ 
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::resizeEvent()" << endl;
#endif
	if(!isValid())
	{
		cerr << " not a valid opengl context" << endl;
		return;
	}

#ifndef WNT // flush X event first 
	QApplication::flushX(); 
#endif 
	if (!occTestView.IsNull()) // tests the _Handle_ for undefined address
		occTestView->MustBeResized();  

#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::resizeEvent()" << endl << endl;
#endif
}

//------------------------------------------------------------
void OccTestViewer::InitOccTestViewer()
{
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::InitOccTestViewer()" << endl;
#endif
	TCollection_ExtendedString Name("OccTestViewer"); 
	TCollection_AsciiString Domain("freeengineer.org");
	Handle(V3d_Viewer) aViewer= new V3d_Viewer(
		GraphicDevice(),
		Name.ToExtString(),
		Domain.ToCString(),
		1000.0,
		V3d_XposYnegZpos,
		Quantity_NOC_CADETBLUE3,
		V3d_WIREFRAME,
		V3d_GOURAUD,
		V3d_ASAP,
		Standard_True,
		Standard_True,
		V3d_TEX_ALL
	);
	aViewer->SetDefaultLights();
	aViewer->SetLightOn();
	aViewer->SetDefaultVisualization(V3d_WIREFRAME);
	Handle(V3d_OrthographicView) aView=new V3d_OrthographicView(aViewer); 
	aView->SetDegenerateModeOn();
	SetView(aView);
	Handle(AIS_InteractiveContext)aContext = new AIS_InteractiveContext(aViewer);
	SetContext(aContext);

	setMouseTracking(true);
//	SetViewer(aViewer);
#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::InitOccTestViewer()" << endl << endl;
#endif
}
//
//--------------------------------------------------------------
void OccTestViewer::Trihedron()
{ 
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::TurnRed()" << endl;
#endif
	if(!occTestWindow->IsMapped())occTestWindow->Map();

	if(!haveTrihedron)
	{
		gp_Pnt p(0,0,0);
		gp_Dir z(0,0,1);
		gp_Dir x(1,0,0);
		Handle(Geom_Axis2Placement) ax = new Geom_Axis2Placement(p,z,x);
		Handle(AIS_Trihedron) tx = new AIS_Trihedron(ax);
		occTestContext->Display(tx);
		haveTrihedron=true;
	}
		
#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::TurnRed()" << endl;
#endif
}
//--------------------------------------------------------------
void OccTestViewer::Box()
{ 
#ifdef TEXT_DEBUG
cout << " ->OccTestViewer::TurnWhite()" << endl;
#endif

	if(!occTestWindow->IsMapped())occTestWindow->Map();


	if(!haveBox)
	{

		Standard_Real l = 100;
		Standard_Real w = 200;
		Standard_Real h = 300;

		TopoDS_Solid S = BRepPrimAPI_MakeBox(l,w,h);
		Handle(AIS_Shape) anAis = new AIS_Shape(S);
		occTestContext->Display(anAis);
		haveBox=true;
	}

#ifdef TEXT_DEBUG
cout << "<- OccTestViewer::TurnWhite()" << endl;
#endif
}
//--------------------------------------------------------------
void OccTestViewer::FitAll()
{ 
	if(!occTestWindow->IsMapped())occTestWindow->Map();
	occTestView->FitAll(0.05,true);
}
//--------------------------------------------------------------
void OccTestViewer::Evolve()
{

if(!haveSweep)
{
	BRepBuilderAPI_MakePolygon P;
	P.Add(gp_Pnt(0.,0.,0.));
	P.Add(gp_Pnt(200.,0.,0.));
	P.Add(gp_Pnt(200.,200.,0.));
	P.Add(gp_Pnt(0.,200.,0.));
	P.Add(gp_Pnt(0.,0.,0.));
	TopoDS_Wire W = P.Wire();
	TopoDS_Wire wprof = BRepBuilderAPI_MakePolygon(gp_Pnt(0.,0.,0.),gp_Pnt(-60.,-60.,-200.));
	TopoDS_Shape S = BRepOffsetAPI_MakeEvolved(W,wprof,GeomAbs_Arc,Standard_True,Standard_False,Standard_True,0.0001);
	
	Handle(AIS_Shape) anAis = new AIS_Shape(W);
	occTestContext->Display(anAis);
	Handle(AIS_Shape) anotherAis = new AIS_Shape(wprof);
	occTestContext->Display(anotherAis);
 	Handle(AIS_Shape) stillAnotherAis = new AIS_Shape(S);
	occTestContext->Display(stillAnotherAis);
	haveSweep=true;
	}
} 
//--------------------------------------------------------------
void OccTestViewer::Shading()
{
	for(occTestContext->InitCurrent();occTestContext->MoreCurrent();occTestContext->NextCurrent())
	{
// this is just to test that the selected object is being processed
		occTestContext->SetColor(occTestContext->Current(),Quantity_NOC_BLUE1,Standard_False);
// here's where we set up shading. 
		occTestContext->SetDisplayMode(occTestContext->Current(),AIS_Shaded,Standard_False);
	}
//	occTestView->SetDegenerateModeOff();
	occTestContext->UpdateCurrentViewer();

//	cout << "OccTestViewer::Shading: DegenerateMode is ";
//	if(occTestView->DegenerateModeIsOn())cout << "on" << endl;
//	else cout << "off" << endl;

	
}
//--------------------------------------------------------------
void OccTestViewer::Wire()
{
	for(occTestContext->InitCurrent();occTestContext->MoreCurrent();occTestContext->NextCurrent())
		occTestContext->SetDisplayMode(occTestContext->Current(),AIS_WireFrame,Standard_False);
//	occTestView->SetDegenerateModeOff();
	occTestContext->UpdateCurrentViewer();

//	cout << "OccTestViewer::Wire: DegenerateMode is ";
//	if(occTestView->DegenerateModeIsOn())cout << "on" << endl;
//	else cout << "off" << endl;
}
//--------------------------------------------------------------
void OccTestViewer::HLR()
{
	for(occTestContext->InitCurrent();occTestContext->MoreCurrent();occTestContext->NextCurrent())
		occTestContext->SetDisplayMode(occTestContext->Current(),AIS_ExactHLR,Standard_False);
//	occTestView->SetDegenerateModeOn();
	occTestContext->UpdateCurrentViewer();

//	cout << "OccTestViewer::HLR: DegenerateMode is ";
//	if(occTestView->DegenerateModeIsOn())cout << "on" << endl;
//	else cout << "off" << endl;
}

//--------------------------------------------------------------
void OccTestViewer::mouseMoveEvent( QMouseEvent * qme )
{
// if control key is down, is a motion command
	Standard_Integer x = qme->x();
	Standard_Integer y = qme->y();
	Standard_Integer dx = x - lastx; 
	Standard_Integer dy = y - lasty;
	switch(qme->state())
	{
	case (ControlButton | RightButton): // rotate
	{
		occTestView->Rotate(dx*speedFactor*rotateFudge,-dy*speedFactor*rotateFudge,0);
		if(!inMotion)inMotion = true;
		break;
	}
	case (ControlButton | MidButton):	// pan
	{
// get width size in Window system
		V3d_Coordinate xc,yc; occTestView->Center(xc,yc);	
      Standard_Integer Xc,Yc; occTestView->Convert(xc,yc,Xc,Yc);
// set the new center
		occTestView->SetCenter(Xc - dx,Yc - dy);
	
		if(!inMotion)inMotion = true;
		break;
	}
	case (ControlButton | LeftButton): // zoom
	{
		if(dx > 0 )
			occTestView->Panning(0,0,1 + zoomFactor );
		else  
			occTestView->Panning(0,0,1 - zoomFactor );
		if(!inMotion)inMotion = true;
		break;
	}
	case (ControlButton | LeftButton | MidButton ):	// spin 
	{
		cout << "shift-left-mid" << endl;
		Quantity_PlaneAngle twist  = occTestView->Twist() ;
		if(dx > 0 )
			occTestView->SetTwist(twist + speedFactor * twistFudge);
		else  
			occTestView->SetTwist(twist - speedFactor * twistFudge);
		if(!inMotion)inMotion = true;
		break;
	}
	default:
		occTestContext->MoveTo(qme->x(),qme->y(),occTestView);
		break;
	}
	lastx = x;
	lasty = y;
}
//--------------------------------------------------------------
void OccTestViewer::mousePressEvent( QMouseEvent * qme )
{
cout << "mousePressEvent" << endl;
	lastx = qme->x();
	lasty = qme->y();
//GLWidget::mousePressEvent(qme);
}
//--------------------------------------------------------------
void OccTestViewer::mouseReleaseEvent( QMouseEvent * qme )
{
#ifdef TEXT_DEBUG
cout << "mouseReleaseEvent" << endl;
#endif
	if(inMotion)inMotion = false;
	else
		{
		switch (qme->state())
		{

		case (LeftButton):
			occTestContext->Select();
			break;
		case (LeftButton | ShiftButton):
			occTestContext->ShiftSelect();
			break;
		}
	}
//QGLWidget::mouseReleaseEvent(qme);
}
//--------------------------------------------------------------
void OccTestViewer::keyPressEvent( QKeyEvent * qke )
{
cout << "key press event" << endl;
	switch (qke->key())
	{
	case Key_F11:
cout << "Key_F11" << endl;
		speedFactor *= .8;
		zoomFactor *= .8;
	
		break;
	case Key_F12 :
cout << "Key_F12" << endl;
		speedFactor *= (1/.8);
		zoomFactor *= (1/.8);
		break;
	}
}

