/*
 * Project : "MindsEye, render queue"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: renderjob.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdlib.h>
#include "renderjob.h"
#include "3dlib/s_scene.h"
#include "kernel/world_obj.h"


MERenderJob::MERenderJob () :
  startTime_(NULL,0), stopTime_(NULL,0)
{
  world_ = NULL;
  fname_ = NULL;
  worldIsCopy_ = false;
}

MERenderJob::~MERenderJob ()
{
  delete[] fname_;
  if( worldIsCopy_ )
    {
      delete world_;
      delete camera_;
    }
}


void
MERenderJob::copyWorld ()
{
  G3DScene *scene = (G3DScene*)(world_->getScene()->copy());
  //set time to 0
  camera_  = world_->getActiveCamera();  //TODO always != NULL ?
  camera_  = new G3DCamera( *camera_ );  //TODO use copy()
  world_   = new MWorldObject( scene );
  world_->setActiveCamera( camera_ );
  worldIsCopy_ = true;
}

G3DScene*
MERenderJob::getScene ()
{
  return( world_->getScene() );
}

void
MERenderJob::setFrameRate( const char *text )
{
  fps_ = atoi( text );
  if( fps_ <= 0 )
    fps_ = 1;
  else if( fps_ > 1000 )
    fps_ = 1000;
}

void
MERenderJob::setOutput( const char* format, const char *fname )
{
  delete[] fname_;
  fname_ = new char[strlen(fname)+1];
  strcpy( fname_, fname );
}

void
MERenderJob::setResolution( uint w, uint h )
{
  w_ = w;
  h_ = h;
}

void
MERenderJob::setTimeInterval( const SmpteTime &t1, const SmpteTime &t2 )
{
  startTime_ = t1;
  if( startTime_ > t2 )
    stopTime_  = t1;
  else
    stopTime_  = t2;
}

void
MERenderJob::setShutterTime( float )
{
}

void
MERenderJob::setWorld( MWorldObject *wl )
{
  world_ = wl;  //TODO make a copy ??
}


//=================================END=================================
