/*
 * Project : "MindsEye, render queue"
 * Copyright (c) 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: renderqueue.h,v 1.3 2002/06/16 20:50:39 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */

#ifndef __RENDER_QUEUE_H_
#define __RENDER_QUEUE_H_

#include "include/config.h"
#include <deque>

#ifdef WITH_PTHREADS
#include <pthread.h>

//pthread_mutex_t child;
//pthread_cond_t  cldterm;
//pid_t           pid_ready;

#else
typedef char pthread_mutex_t;
#endif

typedef unsigned int uint;


/**This lets the compiler keep track of all open mutexes. He will take
   care of releasing them if we leave the block.
   */
class Mutex
{
public:
  Mutex( pthread_mutex_t *mutex )
#ifdef WITH_PTHREADS
    { mutex_ = mutex;  pthread_mutex_lock( mutex_ ); }
#else
    {}
#endif
#ifdef WITH_PTHREADS
  ~Mutex()
    { pthread_mutex_unlock( mutex_ ); }

protected:
  pthread_mutex_t *mutex_;
#endif
};



/**
 */
class MERenderHost
{
public:
  MERenderHost ();

  //how to handle failures
protected:
  const char *name_; // full name of the host
  //p_id  p_id;      // id of the child process
  uint retrycount_;
  uint removed_;     // dont use it
  //pointer to frame??
};


class MERenderJob;

//What if multiple MEs are running in a Network: central Queue?
/**Base class for all Render queues.
   A render queue is an object that consists of a queu of jobs and a driver
   for the specific rendering program (e.g. povray, or bmrt).
 */
class MERenderQueue
{
public:
  MERenderQueue ();
  virtual ~MERenderQueue ();

  void         addHost ( const char* name );
  void         addJob ( MERenderJob* );
  const char*  getName () { return (const char*)name_; }
  virtual bool hasOptionDialog ();
  virtual bool prepareJob ( MERenderJob* ) = 0;
  void         renderJob ();
  virtual void renderFrame ( int ) = 0;
  void         removeHost ( const char* name );
  void         setName ( const char *name );
  virtual void showOptionDialog () = 0;

protected:
  //typedef pthread_mutex_t Lock;
  /*
  void deleteHost();
  void lock( Lock* );
  void unlock( Lock* );
  */

protected:
  char *name_;  //a descriptive name of the queue

  int flags_; //z.b. rendern im Hintergrund, ...
  //vector<MERenderHost> hostList_;
  deque<MERenderJob*>  jobQueue_;
  MERenderJob          *activeJob_;

  //char *command_;  //how to call the render engine
  int  freeHosts_;  //number of hosts waiting for a job

  //post-function

  pthread_mutex_t mt_jobQueue_; //queueMod_, entryMutex_;
#ifdef WITH_PTHREADS
  pthread_cond_t  cnd_jobQueue_; //entryCond_;
#endif
};

#endif
