#ifndef RGB
#define RGB

#include "defines.hh"

#define BLACK (rgb(0, 0, 0))
#define WHITE (rgb(1, 1, 1))
#define GREY (rgb(0.5, 0.5, 0.5))

#define RED (rgb(1, 0, 0))
#define GREEN (rgb(0, 1, 0))
#define BLUE (rgb(0, 0, 1))

#define YELLOW (rgb(1, 1, 0))
#define CYAN (rgb(0, 1, 1))
#define MAGENTA (rgb(1, 0, 1))

class rgb
{
 public:
  rgb ()                               { red=green=blue=0, valid=true; }
  rgb ( double r, double g, double b ) { red=r, green=g, blue=b, valid=true; }
  rgb ( bool v )                       { red=green=blue=0, valid=v; }

  void clip ();

  double red, green, blue;

  bool valid;
  
  void full_dump ();

  rgb& operator += ( const rgb& rhs );
  rgb& operator -= ( const rgb& rhs );
  rgb& operator /= ( double rhs );
  rgb& operator *= ( double rhs );
  rgb& operator *= ( const rgb& rhs );

protected:
private:
};

rgb operator+(const rgb& a, const rgb& b);
rgb operator-(const rgb& a, const rgb& b);
rgb operator*(const rgb& a, const rgb& b);
rgb operator*(const rgb& a, double b);
rgb operator*(double b, const rgb& a);
rgb operator/(const rgb& a, double b);

inline rgb& rgb::operator += ( const rgb& rhs )
{
  red += rhs.red, green += rhs.green, blue += rhs.blue;
  return *this;  
}

inline rgb& rgb::operator -= ( const rgb& rhs )
{
  red -= rhs.red, green -= rhs.green, blue -= rhs.blue;
  return *this;  
}

inline rgb& rgb::operator *= ( const rgb& rhs )
{
  red *= rhs.red, green *= rhs.green, blue *= rhs.blue;
  return *this;  
}

inline rgb& rgb::operator /= ( double rhs )
{
  red /= rhs, green /= rhs, blue /= rhs;
  return *this;  
}

inline rgb& rgb::operator *= ( double rhs )
{
  red *= rhs, green *= rhs, blue *= rhs;
  return *this;  
}

inline void rgb::clip ()
{
 if (red<0)
  red=0;
 else
  if (red>1)
   red=1;

 if (green<0)
  green=0;
 else
  if (green>1)
   green=1;

 if (blue<0)
  blue=0;
 else
  if (blue>1)
   blue=1;
}

inline rgb operator+(const rgb& a, const rgb& b)
{
  return rgb (a.red + b.red, a.green + b.green, a.blue + b.blue);
}

inline rgb operator-(const rgb& a, const rgb& b)
{
  return rgb (a.red - b.red, a.green - b.green, a.blue - b.blue);
}

inline rgb operator*(const rgb& a, const rgb& b)
{
  return rgb (a.red * b.red, a.green * b.green, a.blue * b.blue);
}

inline rgb operator*(const rgb& a, double b)
{
  return rgb (a.red * b, a.green * b, a.blue * b);
}

inline rgb operator*(double b, const rgb& a)
{
  return rgb (a.red * b, a.green * b, a.blue * b);
}

inline rgb operator/(const rgb& a, double b)
{
  return rgb (a.red / b, a.green / b, a.blue / b);
}

#endif
