#ifndef SCENE
#define SCENE

#include "object.hh"
#include "ray.hh"
#include "rgb.hh"
#include "rgbbuffer.hh"

class Scene
{
public:
  Scene ();
  Scene ( List* objects, List* lights );
  
  // Scene:
  void set_objects ( List* objects );
  void set_lights ( List* lights );
  List* get_objects () { return objects; }
  List* get_lights () { return lights; }

  void render ();
  
  // Camera:
  void set_position ( const Vector& pos ) { position = pos; }
  const Vector& get_position () const { return position; }
  
  void set_direction ( const Vector& dir );
  
  void set_up ( const Vector& u );

  void set_aspect ( double asp );
  
  void set_anti_aliasing ( bool aa ) { anti_aliasing = aa; }
  bool get_anti_aliasing () const { return anti_aliasing; }

  void set_resolution ( int x, int y ) { image.set_size (x, y); }
  int get_xres () const { return image.get_width(); }
  int get_yres () const { return image.get_height(); }
  
  void set_bg_color ( const rgb& bg_col ) { bg_color = bg_col; }
  
  void set_ray_threshold ( double rt ) { ray_threshold = rt; }
  
  void set_brightness ( double b ) { brightness = b; }
  void set_gamma ( double g ) { gamma = 1.0/g; }
  
  void set_dof_quality ( int quality ) { dof_quality = quality; }
  void set_dof_focus ( double focus ) { dof_focus=focus; }
  void set_dof_aperture ( double aperture ) { dof_aperture = aperture; }
  
  RGBBuffer& get_image () { return image; }

protected:
  rgb trace_ray ( Ray& ray );
  rgb point ( int x, int y, int depth );
  rgb pinpoint ( int x, int y );
  void calc_side ();
  void find_closest ( Ray& ray, double* closest, Vector* isect, Object** contact, rgb* filter);
  
  rgb bg_color;
  List *objects;
  List *lights;
  RGBBuffer image;
  bool anti_aliasing;
  Vector position, direction, up, side;
  double aspect, ray_threshold, brightness, xs, ys, gamma;
  double dof_aperture, dof_focus;
  int ray_tally, dof_quality;
  
private:
};

void destroy_scene ( Scene *s );

#endif
