#include <iostream.h>

#include "sphere.hh"

Sphere::Sphere ()
{
  set_center (Vector(0,0,0));
  set_radius (1);
  norm = new Vector;
  (*norm) = Vector (0,0,0);
  ins = new bool;
  (*ins) = false;
}

Sphere::Sphere ( const Vector& c, double r )
{
  set_center (c);
  set_radius (r);
  norm = new Vector;
  *norm = Vector (0,0,0);
  ins = new bool;
  *ins = false;
}

Sphere::~Sphere ()
{
  delete norm;
  delete ins;
}

Vector Sphere::normal () const
{
  return *norm;
}

bool Sphere::inside () const
{
  return *ins;
}

/*
  Possible optimizations:
  ok - Don't compute (*ins) if we've got no intersection.
     - Assume (*ins) iff we found one interection in front and one behind.
     - Is it possible to avoid sqrt computation for spheres no closer then
       closest if we pass a "squared" value from previous "closest" tests?
       This would only optimize the case where a sphere is behind another
       sphere in our ray's path.
     - build a function taking an array of lines to test a sphere against.
     - 'decompose' Vector. (less readable, though.)
     - use register double.
     - only pass back t1 and *ins, possibly inside 'struct Line'.
     - In an assembler version we can reuse some of the variables.
       We can try if the compiler reassigns registers after going out of scope
       inside a function using { ... }.
*/

#ifndef rdouble
#define rdouble register double
#endif

//Version 1 by Kai
Vector Sphere::_intersection ( const Line& line ) const
{
  double b, c;        // as in ax + bx + c
  rdouble oneovera;   // 1/a, a as in ax + bx + c
  double p2, q;       // as in x + px + q; p2 is just p/2
  rdouble xs, ys, zs; // line.start (x, y, z) triple
  rdouble xd, yd, zd; // line.direction (x, y, z) triple
  double p2p2;        // p/2 * p/2
  rdouble t1, t2;     // parameter of the line
  double root, root2; // the "root" in the quadratic "formula" and root

  // the lines start is translated to match that coordinate system
  // in which the sphere has it's center at (0, 0, 0)
  Vector start=line.get_start()-center;
  Vector direction=line.get_direction();
  Vector retval;

  *ins = false; // if the ray doesn't intersect the sphere this is implied.
  
  // direction must not be the null vector !
  xs=start.x, ys=start.y, zs=start.z;
  xd=direction.x, yd=direction.y, zd=direction.z;
  
  // from solving the sphere equation sqrt(x + y + z) = r:
  oneovera= 1.0/(xd*xd + yd*yd + zd*zd);
  b=2.0*(xs*xd + ys*yd + zs*zd);
  c=xs*xs + ys*ys + zs*zs - radius*radius;

  //for solving the quadratic equation w/ the "easy to remember" formula
  p2=.5*(b*oneovera); // = 0.5 * b/a
  p2p2=p2*p2;
  q=c*oneovera; // = c/a

  // "no solution", so there is no intersection:
  if (p2p2 - q < .0) {
    retval.valid=false;
    return retval;
  }
  
  root2=p2p2-q;
  root=sqrt(root2);
  t1=-p2+root;
  t2=-p2-root;

  if (t1<.0) {
    if (t2<.0) {
      retval.valid=false;
      return retval; // no intersection in front of us (break)
    }
    // *ins = true;
    t1 = t2;
  }
  else
    if (t2>=.0) { // t2 is not behind us (ok)
      if (t2<t1) // t2 is closer than t1
	t1=t2;
    }
  //else
  //  *ins = true;
      
  
  retval=line.get_start() + direction*t1;
  retval.valid=true;
  
  //Normal for sphere is simple so work it out now
  *norm = retval - center;
  
  //If the ray started from inside the sphere then invert the normal
  *ins = (start.magnitude() < radius) ? true : false;
  if (*ins) {
    *norm *= -1.0;
  }
  
  return retval;
}
