#include "surface.hh"

Surface::Surface ()
{
  normal.valid=false;
  color.valid=false;
  diffuse=specular=hardness=transparency=reflect=refract=ior=-1;
}

//************
//* Addition *
//************

inline Vector add_vect ( const Vector& a, const Vector& b)
{
  //Nice and simple logic really..  :)
  Vector retval=( (a.valid)?( (b.valid)?(a+b):(a) ):( (b.valid)?(b):(Vector(false)) ) );
  return retval;
}

inline rgb add_rgb ( const rgb& a, const rgb& b )
{
  rgb retval=( (a.valid)?( (b.valid)?(a+b):((a)) ):( (b.valid)?(b):(rgb(false)) ) );
  return retval;
}

inline double add_dbl ( double a, double b )
{
  return ((!(a<0))?( (!(b<0))?(a+b):(a) ):( (!(b<0))?(b):(-1)));
}

//******************
//* Multiplication *
//******************

inline Vector mult_vect ( const Vector& a, const Vector& b)
{
  //Nice and simple logic really..  :)
  Vector retval=( (a.valid&&b.valid)?(a*b):(Vector(false)) );
  return retval;
}

inline rgb mult_rgb ( const rgb& a, const rgb& b )
{
  //Nice and simple logic really..  :)
  rgb retval=( (a.valid&&b.valid)?(a*b):(rgb(false)) );
  return retval;
}

inline double mult_dbl ( double a, double b )
{
  //Nice and simple logic really..  :)
  return ( ((!(a<0))&&(!(b<0)))?(a*b):(-1) );
}

Surface operator+ ( const Surface& a, const Surface& b )
{
  Surface c (0); // create a surface uninitialized

  c.normal   = add_vect (a.normal, b.normal);
  c.color    = add_rgb (a.color, b.color);
  c.diffuse  = add_dbl (a.diffuse, b.diffuse);
  c.specular = add_dbl (a.specular, b.specular);
  c.hardness = add_dbl (a.hardness, b.hardness);
  c.transparency = add_dbl (a.transparency, b.transparency);
  c.reflect  = add_dbl (a.reflect, b.reflect);
  c.refract  = add_dbl (a.refract, b.refract);
  c.ior      = add_dbl (a.ior, b.ior);
  
  return c;
}

Surface& Surface::operator+= ( const Surface& rhs )
{
  normal   = add_vect (normal, rhs.normal);
  color    = add_rgb (color, rhs.color);
  diffuse  = add_dbl (diffuse, rhs.diffuse);
  specular = add_dbl (specular, rhs.specular);
  hardness = add_dbl (hardness, rhs.hardness);
  transparency = add_dbl (transparency, rhs.transparency);
  reflect  = add_dbl (reflect, rhs.reflect);
  refract  = add_dbl (refract, rhs.refract);
  ior      = add_dbl (ior, rhs.ior);
  
  return *this;
}

Surface operator* ( const Surface& a, const Surface& b )
{
  Surface c (0); // create a surface uninitialized

  c.normal    = mult_vect (a.normal, b.normal);
  c.color     = mult_rgb (a.color, b.color);
  c.diffuse   = mult_dbl (a.diffuse, b.diffuse);
  c.specular  = mult_dbl (a.specular, b.specular);
  c.hardness  = mult_dbl (a.hardness, b.hardness);
  c.transparency = mult_dbl (a.transparency, b.transparency);
  c.reflect   = mult_dbl (a.reflect, b.reflect);
  c.refract   = mult_dbl (a.refract, b.refract);
  c.ior       = mult_dbl (a.ior, b.ior);
  
  return c;
}

Surface operator* ( const Surface& a, double b )
{
  Surface c (0); // create a surface uninitialized

  c.normal    = ( (a.normal.valid&&(!(b<0)))?(a.normal*b):(Vector(false)) );
  c.color     = ( (a.color.valid&&(!(b<0)))?(a.color*b):(rgb(false)) );
  c.diffuse   = mult_dbl (a.diffuse, b);
  c.specular  = mult_dbl (a.specular, b);
  c.hardness  = mult_dbl (a.hardness, b);
  c.transparency = mult_dbl (a.transparency, b);
  c.reflect   = mult_dbl (a.reflect, b);
  c.refract   = mult_dbl (a.refract, b);
  c.ior       = mult_dbl (a.ior, b);
 
  return c;
}


Surface& Surface::operator*= ( const Surface& rhs )
{
  normal    = mult_vect (normal, rhs.normal);
  color     = mult_rgb (color, rhs.color);
  diffuse   = mult_dbl (diffuse, rhs.diffuse);
  specular  = mult_dbl (specular, rhs.specular);
  hardness  = mult_dbl (hardness, rhs.hardness);
  transparency = mult_dbl (transparency, rhs.transparency);
  reflect   = mult_dbl (reflect, rhs.reflect);
  refract   = mult_dbl (refract, rhs.refract);
  ior       = mult_dbl (ior, rhs.ior);
  
  return *this;
}

Surface& Surface::operator*= ( double rhs )
{
  normal    = ( (normal.valid&&(!(rhs<0)))?(normal*rhs):(Vector(false)) );
  color     = ( (color.valid&&(!(rhs<0)))?(color*rhs):(rgb(false)) );
  diffuse   = mult_dbl (diffuse, rhs);
  specular  = mult_dbl (specular, rhs);
  hardness  = mult_dbl (hardness, rhs);
  transparency = mult_dbl (transparency, rhs);
  reflect   = mult_dbl (reflect, rhs);
  refract   = mult_dbl (refract, rhs);
  ior       = mult_dbl (ior, rhs);
  
  return *this;
}
