/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: objects.h,v 1.2 2002/06/16 20:50:39 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _2D_OBJECTS_H_
#define _2D_OBJECTS_H_

//#include "3dlib/s_point.h"
//#include "3dlib/s_line.h"
#include "3dlib/g3d_math.h"
#include <vector>

#define POINT(i) (*((G3DPoint3d*)(points_[i])))

enum EnumInterpolation { Interpolate = 0, LeastSquares, Approximate };


class G3DNurbsCurve;
class G3DObject;
class QPen;


/**Objects of this class represent an object in the 2D shaper.
 */
class MELineObject //: public ME2dShape
{
public:
  MELineObject ()
    { curve_ = NULL; closed_ = false; showLine_ = true; }
  MELineObject( G3DNurbsCurve *c);
  ~MELineObject();

  void  appendPoint( G3DPoint3d *p ) { points_.push_back((void*)p); }
  void  closeLine () { closed_ = true; }
  void  computeCurve( int, int, int, float );
  void           deleteCurve ();
  void           draw( QPainter*, float, QPen& );
  G3DObject*     extrudePolyLine ();
  G3DObject*     getCurve ();
  G3DPoint3d&    getPoint( int );
  void           movePoint( int, const TVector<MeFloat,3>& );
  vector<void*>& pointList () { return points_; }
  int            numberOfPoints () { return points_.size(); }
  MeFloat        select ( MeFloat mx, MeFloat my );
  MeFloat        selectPoint ( MeFloat mx, MeFloat my, int& );

protected:
  vector<void*> points_;
  G3DNurbsCurve *curve_;
  bool          closed_;
  bool          showLine_;
};


#endif
