/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: shaper.cpp,v 1.4 2000/03/07 23:01:19 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <qpainter.h>
#include <qlabel.h>
#include <qbttngrp.h>
//#include <qradiobt.h>
#include <qpushbt.h>
#include <qlined.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qcombo.h>
#include <qpopmenu.h>

#include "3dlib/object.h"
#include "viewport.h"
#include "shaper.h"
#include "grid2d.h"
#include "objects.h"


static const char *InterpolationMethods[3] =
{ "Interpolation", "Least squares", "Approximation" };


ME2DShaper::ME2DShaper ( int module_id, int load_t ) 
  : MEModule(module_id, load_t, "ME2DShaper")
{
  ipMode_  = Interpolate;
  degree_  = 3;
  apError_ = 1.5;
  nCP_     = 8;

  QBoxLayout *gm = new QVBoxLayout (this, 0, 2);
  gm->setMenuBar (makeMenuBar());
  gm->addWidget (makeToolbar(), 0);

  drawArea_ = new MEDrawingWidget (shapes_, this);
  connect (drawArea_, SIGNAL(positionChanged(float,float)),
	   SLOT( showPosition(float,float)));
  connect (drawArea_, SIGNAL(showPopup(int,int)),
	   SLOT( showPopup(int,int)));
  gm->addWidget (drawArea_, 100);

  previewLabel =  new QLabel (this);
  previewLabel->setMinimumSize (100, 20);
  gm->addWidget (previewLabel, 0);

  gm->activate ();
  resetValues ();
  makePopup ();
  setMinimumSize (200, 200);
  resize (440, 400);
}

ME2DShaper::~ME2DShaper ()
{
  resetValues ();
}


void
ME2DShaper::addDrawing ()
{
  shapes_.push_back (new MELineObject);
  drawArea_->setCurrent (shapes_.size()-1);
  drawArea_->setEditMode (MEDrawingWidget::FreeHand);
}

void
ME2DShaper::addPolyLine ()
{
  shapes_.push_back (new MELineObject);
  drawArea_->setCurrent (shapes_.size()-1);
  drawArea_->setEditMode (MEDrawingWidget::AppendPoint);
}

void
ME2DShaper::closeLine ()
{
  MELineObject *tmp = drawArea_->getSelectedShape();
  if (tmp) {
    tmp->closeLine ();
    drawArea_->update ();
  }
}

void
ME2DShaper::computeCurve ()
{
  MELineObject *tmp = drawArea_->getSelectedShape();
  if (tmp) {
    degree_ = atoi (editDeg_->text());
    if (degree_ < 3)
      degree_ = 3;
    else if (degree_ > 10)
      degree_ = 10;
    //degEdit_->setText()
    tmp->computeCurve (ipMode_, degree_, nCP_, apError_);
    drawArea_->update ();
  }
}

void
ME2DShaper::deleteCurve ()
{
  MELineObject *tmp = drawArea_->getSelectedShape();
  if (tmp) {
    tmp->deleteCurve();
    drawArea_->update();
  }
}

/*
void
ME2DShaper::editDegree (const QString& txt)
{ 
  degree = (int) atof (txt.ascii()); 
  modify ();
}

void
ME2DShaper::editError (const QString& txt)
{
  error = (float) atof (txt.ascii()); 
  modify();
}

void
ME2DShaper::editNofCP (const QString& txt)
{
  nCP = (int) atof (txt.ascii()); 
  modify();
}
*/

G3DObject*
ME2DShaper::getMesh ()
{
  MELineObject *line = drawArea_->getSelectedShape();
 
  return (line) ? line->extrudePolyLine () : 0;
}

G3DObject*
ME2DShaper::getPolygon ()
{
  return 0; // TODO.
}

G3DObject*
ME2DShaper::getNurbsCurve ()
{
  MELineObject *line = drawArea_->getSelectedShape();
  if (line) {
    return line->getCurve();  //TODO copy();
  }
  else
    return 0;
}

void
ME2DShaper::hideWindow ()
{ 
  hide ();
}

void
ME2DShaper::interpMode ( int a )
{
  ipMode_ = a;
  MELineObject *tmp = drawArea_->getSelectedShape();
  if (tmp->getCurve())
    computeCurve ();
}

QMenuBar*
ME2DShaper::makeMenuBar ()
{
  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR(file);
  //TODO load image as background
  //file->insertItem ("load", this, SLOT(loadScene()), CTRL+Key_L);
  //file->insertItem ("save", this, SLOT(saveScene()), CTRL+Key_S);
  //file->insertItem ("save Selection", this,SLOT(exportObjects()), CTRL+Key_E);
  //file->insertSeparator ();
  file->insertItem ("close shaper", this, SLOT(hideWindow()), CTRL+Key_C);

  QPopupMenu *edit = new QPopupMenu;
  CHECK_PTR(edit);
  edit->insertItem ("add polyline", this, SLOT(addPolyLine()));
  edit->insertItem ("add drawing", this, SLOT(addDrawing()));
  edit->insertItem ("plot function");
  //edit->insertItem ("undo",  this, SLOT(newScene()), CTRL+Key_U);
  //edit->insertItem ("scene properties", this, SLOT(sceneProperties()));


  QPopupMenu *options = new QPopupMenu;
  CHECK_PTR(options);
  options->insertItem ("show Control Pts", this, SLOT(toggleCPts()));
  options->insertItem ("grid settings...", this, SLOT(gridDialog()));

  QMenuBar *menu = new QMenuBar (this);
  menu->setFrameStyle (QFrame::WinPanel | QFrame::Raised);
  CHECK_PTR(menu);
  menu->insertItem ("&File", file);
  menu->insertItem ("&Edit", edit);
  menu->insertItem ("&Options", options);

  return (menu);
}

void
ME2DShaper::makePopup ()
{
  popup_ = new QPopupMenu;
  popup_->insertItem ("compute curve", this, SLOT(computeCurve()));
  popup_->insertItem ("delete curve",  this, SLOT(deleteCurve()));
  popup_->insertSeparator ();
  popup_->insertItem ("close Line", this, SLOT(closeLine()));
}

QFrame*
ME2DShaper::makeToolbar ()
{
  QFrame *toolbar = new QFrame (this);
    toolbar->setMinimumSize (24, 100);
    //toolbar->setFrameStyle( QFrame::Panel | QFrame::Raised );
    //toolbar->setLineWidth (2);

  QBoxLayout *gm = new QHBoxLayout (toolbar, 3, 0);
  QPushButton *clearBtn = new QPushButton ("Clear", toolbar);
    clearBtn->setMinimumSize (50, 20);
    connect( clearBtn,   SIGNAL(clicked()), SLOT(resetValues())) ;
    gm->addWidget (clearBtn, 0);

  gm->addSpacing (10);

  QComboBox *cbox = new QComboBox (toolbar);
    cbox->setMinimumSize (100, 20);
    cbox->insertStrList (InterpolationMethods, 3);
    gm->addWidget (cbox, 0);
    connect( cbox, SIGNAL(activated(int)), SLOT(interpMode(int)));

  QLabel *degL = new QLabel ("degree ", toolbar);
    degL->setMinimumSize (degL->sizeHint());
    gm->addWidget (degL, 0);
  editDeg_ = new QLineEdit (toolbar) ;
    editDeg_->setMinimumSize (40, 20);
    gm->addWidget (editDeg_, 0);

  gm->addStretch (100);
  gm->activate ();
  return toolbar;

/*
  connect (editDeg, SIGNAL(textChanged(const QString&)),
           this, SLOT(editDegree(const QString&)) );


  QLabel *cpL = new QLabel("Number of control points\nfor least squares approx",btnBox);
  cpL->setGeometry(5,130,150,35) ;
  editNCP = new QLineEdit(btnBox) ;
  editNCP->setGeometry(155,140,30,20) ;

  QLabel *errL = new QLabel("Accepted pixel error\nfor approximation",btnBox) ;
  errL->setGeometry(5,160,130,35);
  editErr = new QLineEdit(btnBox);
  editErr->setGeometry(135,170,50,20);

  connect( editNCP,SIGNAL(textChanged(const QString&)),
	   this, SLOT(editNofCP(const QString&)) );
  connect( editErr,SIGNAL(textChanged(const QString&)),
	   this, SLOT(editError(const QString&)) );
	   */
}
/*
void
ME2DShaper::modify()
{ 
  modified = 1;
}
*/

void
ME2DShaper::resetValues ()
{
  degree_   = 3 ;
  nCP_      = 8 ;
  apError_  = 1.5 ;

  drawArea_->setCurrent (-1);
  for ( int i = 0; i < shapes_.size(); ++i) {
    delete (MELineObject*)(shapes_[i]);
  }
  shapes_.clear ();

  shapes_.push_back (new MELineObject);
  drawArea_->setCurrent (shapes_.size()-1);
  drawArea_->setEditMode (MEDrawingWidget::AppendPoint); // XXX : ok ?
  
  drawArea_->update ();
  /*
    char txt[64] ;
    sprintf( txt, "%d", degree );
    editDeg->setText(txt) ;
    sprintf(txt,"%d",nCP) ;
    editNCP->setText(txt) ;
    sprintf( txt, "%.3f", error );
    editErr->setText( txt );
  */
}

void
ME2DShaper::gridDialog ()
{
  ME2DGridDialog *gridDlg = new ME2DGridDialog();
  connect( gridDlg, SIGNAL(setGridMode(EnumGridMode,float,float)),
	   drawArea_, SLOT(setGrid(EnumGridMode,float,float)) );
  gridDlg->show();
}

void
ME2DShaper::setObject ( G3DObject *obj )
{
  if( obj->getType() == T_NURBS_CURVE ) {
    //TODO copy or reference?
    shapes_.push_back( new MELineObject((G3DNurbsCurve*)obj) );
  }
}

void
ME2DShaper::showPopup ( int x, int y )
{
  popup_->move( x, y );
  popup_->show();
}

void
ME2DShaper::showPosition ( float x, float y )
{
  char text[50];

  sprintf( text, "(%.3g,  %.3g)", x, y );
  previewLabel->setText( text );
}

void
ME2DShaper::toggleCPts ()
{
}


//====================================END==========================
