/*
 * Project : "MindsEye, gui widgets"
 * Copyright (c) 1995 - 1998. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: shaper.h,v 1.3 2000/02/26 16:39:21 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _2D_SHAPER_H_
#define _2D_SHAPER_H__

#include <qwidget.h>
#include "kernel/module.h"
#include <vector>

class QLabel;
class QLineEdit;
class QMenuBar;
class QPopupMenu;
class QFrame;
class G3DObject;
class MEDrawingWidget;
class MELineObject;


/**A Widget to draw a flat 3d NURBS curve. The curve is build from
  points enterd by the user using some interpolation technique.
 */
class ME2DShaper: public MEModule
{
  Q_OBJECT

public:
  ///
  ME2DShaper ( int module_id, int load_t );
  virtual ~ME2DShaper ();

  G3DObject* getMesh ();
  G3DObject* getNurbsCurve ();
  G3DObject* getPolygon ();
  ///Let the user edit an object.
  void       setObject ( G3DObject* );

signals:
  void newNurbs ( G3DObject* );

public slots:
  void interpMode ( int );
  /*
  void computeCurve (MELineObject*);

  void editDegree (const QString&);
  void editNofCP (const QString&);
  void editError (const QString&);

  void preview ();
  void modify ();
  */

protected:
  //void resizeEvent( QResizeEvent* );
  QMenuBar* makeMenuBar ();
  void      makePopup ();
  QFrame*   makeToolbar();

protected slots:
  void addPolyLine ();
  void addDrawing ();
  void closeLine ();
  void computeCurve ();
  void deleteCurve ();
  void gridDialog ();
  void hideWindow ();
  void resetValues ();
  void showPopup ( int, int );
  void showPosition ( float, float );
  void toggleCPts ();

protected:
  int   ipMode_;
  int   degree_, nCP_;
  int   modified;
  float apError_;

  vector<void*> shapes_;

  MEDrawingWidget *drawArea_;
  QLabel          *previewLabel;
  QLineEdit       *editDeg_, *editNCP_, *editErr_;
  QPopupMenu      *popup_;

};


#endif
