/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: viewport.cpp,v 1.3 2000/02/29 20:31:17 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <qpainter.h>
#include <qpixmap.h>
//#include <qlabel.h>
//#include <qbttngrp.h>
//#include <qradiobt.h>
//#include <qpushbt.h>
//#include <qlined.h>
#include "viewport.h"
#include "objects.h"

MEDrawingWidget::MEDrawingWidget ( vector<void*> &shapes, QWidget *parent )
  : QFrame(parent), shapes_(shapes)
{
  //setMinimumSize(100,100) ;
  setFrameStyle (WinPanel | Sunken);
  setLineWidth (2);
  
  //preview = new PlNurbsListGL ;
  stepx_       = stepy_ = 0.04;
  scale_       = 200;
  editMode_    = EditCurve;
  gridType_    = Cartesian;
  activeShape_ = 0;
  offscreen_ = 0;
  ignoreFollowingLeave_ = damaged_ = false;
}

MEDrawingWidget::~MEDrawingWidget ()
{
}


void
MEDrawingWidget::drawContents ( QPainter *direct )
{
  QPainter *p = 0, *buffered = 0;
  
  if (offscreen_ == 0) {
    p = direct;
  }
  else if (damaged_) {
    p = buffered = new QPainter ((QPaintDevice *) offscreen_);
    buffered->setBackgroundColor(palette().normal().button());
    buffered->eraseRect(contentsRect());
  }
  if (p) {
    drawGrid (p);
    p->translate (offset_.x(), offset_.y());
    drawObjects (p);
    drawCurrent (p);
    damaged_ = false;
  }
  if (offscreen_ != 0) {
    direct->drawPixmap(0, 0, *offscreen_);
  }
  delete buffered;
}

void
MEDrawingWidget::drawCrossWire ( QPainter *p, int x, int y )
{
  if (gridType_ == Cartesian) {
    p->drawLine (-width()/2, y, width()/2, y);
    p->drawLine (x, height()/2, x, -height()/2);
  }
  else if( gridType_ == Polar ) {
    int r = int (sqrt((x*x) + (y*y)));
    p->drawEllipse (-r, -r, 2*r, 2*r);
  }
}

void
MEDrawingWidget::drawGrid ( QPainter *p )
{
  // BUG : the grid doesn't match the mouse constraint grid.
  
  if (gridType_ == Cartesian) {
    QRect rect = contentsRect();
    int l = rect.left(), r = rect.right();
    int t = rect.top(), b = rect.bottom();
    p->setPen (palette().normal().mid());
    while ((l += 10) < r) {
      p->drawLine (l, t, l, b);
    }
    l = rect.left(), r = rect.right();
    while ((t += 10) < b) {
      p->drawLine (l, t, r, t);
    }
  }
}

void
MEDrawingWidget::drawObjects ( QPainter *p )
{
  QPen pen1 (black), pen2 (red);
  
  for (int i = 0; i < shapes_.size(); ++i) {
    MELineObject *obj = (MELineObject*)(shapes_[i]);
    obj->draw (p, scale_, obj != activeShape_ ? pen1 : pen2);
  }
}

// method name ?
void
MEDrawingWidget::drawCurrent ( QPainter *p )
{
  float x = mousePt_[0], y = mousePt_[1];
  
  if (hasCross_) {
    if (editMode_ == AppendPoint) {  //there is at least 1 point
      p->setPen (blue);
      int mx =  int (x * scale_), my = -int (y * scale_);
      if (hasPoint_) { //draw line from last point to cursor
        G3DPoint3d &endpt = activeShape_->getPoint (-1);
        p->drawLine (int(endpt[0]*scale_), -int(endpt[1]*scale_), mx, my);
      }
      drawCrossWire (p, mx, my);
    }
    else if (editMode_ == FreeHand) { //-> stop nach releaseEvent
      // TODO.
    }
    else { //if (editMode_ == EditCurve)
      if (activeShape_ != 0) { 
        if ((mButton_ == MidButton) && (activePoint_ >= 0)) {
          float dx = x - mouseX_, dy = y - mouseY_;
          mouseX_ = x, mouseY_ = y;
          activeShape_->movePoint (activePoint_, G3DVector3d(dx, dy, 0));
          // damage ();
        }
        
        // TODO: else move curve.
      }
    }
  }
}

void
MEDrawingWidget::damage ()
{
  damaged_ = true;
  repaint (contentsRect(), offscreen_ ? false : true );
}

/*
G3DObject*
MEDrawingWidget::getObject ()
{
  return (active_) ? active_->curve()->copy() : 0;
}
*/

QPixmap *
MEDrawingWidget::setOffscreen ( bool b )
{
  if (b) {
    if (offscreen_ == 0) {
      offscreen_ = new QPixmap (width(), height());
      damaged_ = true;
    }
  }
  else {
    delete offscreen_;
    offscreen_ = 0;
  }
  
  return offscreen_;
}

void
MEDrawingWidget::enterEvent ( QEvent * )
{
  setOffscreen (true);
}

void
MEDrawingWidget::leaveEvent ( QEvent * )
{
  if (ignoreFollowingLeave_) {
    ignoreFollowingLeave_ = false;
  }
  else {
    setOffscreen (false);
  }
  hasCross_ = false;
  damage ();
}

void
MEDrawingWidget::mouseMoveEvent ( QMouseEvent * mouse )
{
  float x = float(mouse->x() - offset_.x()) / scale_;
  float y = float(offset_.y() - mouse->y()) / scale_;
  round (x, y);
  
  mousePt_[0] = x, mousePt_[1] = y;
  
  // TODO: damage only what needs to be redone.

  if (!hasCross_) 
    hasCross_ = true;
  damage ();
}

void
MEDrawingWidget::mousePressEvent( QMouseEvent *mouse )
{
  mButton_ = mouse->button();
  
  float x = float(mouse->x() - offset_.x()) / scale_;
  float y = float(offset_.y() - mouse->y()) / scale_;
  round (x, y);
  
  mousePt_[0] = x, mousePt_[1] = y;
  
  if (mouse->button() == LeftButton) {
    setOffscreen ();
    if (editMode_ == AppendPoint) {
      if (!hasPoint_) {
        activeShape_->appendPoint (new G3DPoint3d(mousePt_));
        hasPoint_ = true;
      }
      else {
        G3DPoint3d &endpt = activeShape_->getPoint (-1);
        if (mousePt_ != endpt) {
          activeShape_->appendPoint (new G3DPoint3d(mousePt_));
        }
      }
      damage ();
    }
    else {  //select
      for (int i = 0; i < shapes_.size(); ++i) {
        MELineObject *obj = (MELineObject*)(shapes_[i]);
        MeFloat d = obj->select (x, y);
        if (sqrt(d) * scale_ < 3) {
          activeShape_ = obj;
          damage ();
          break;
        }
      }
    }
  }
  else if (mouse->button() == MidButton) {
    if (editMode_ == AppendPoint) {
      //TODO if points.size < 2 delete
      editMode_ = EditCurve;
      setMouseTracking (false);
      damage ();
      // repaint ();
    }
    else { //if( editMode_ == EditCurve )
      int index;
      activePoint_ = -1;
      for (int i = 0; i < shapes_.size(); ++i) {
        MELineObject *obj = (MELineObject*)(shapes_[i]);
        MeFloat d = obj->selectPoint( x, y, index );
        if ( d * scale_ < 3 ) {
          activeShape_ = obj;
          activePoint_ = index;
          mouseX_ = x, mouseY_  = y;
          cout << "point grapped" << endl;
          //repaint();
          break;
        }
      }
    }
  }
  else if (mouse->button() == RightButton) {
    ignoreFollowingLeave_ = true;
    QPoint new_coors = mapToGlobal (mouse->pos());
    emit showPopup(new_coors.x(), new_coors.y());
  }
}

void
MEDrawingWidget::mouseReleaseEvent ( QMouseEvent *mouse )
{
  float x = float(mouse->x() - offset_.x()) / scale_;
  float y = float(offset_.y() - mouse->y()) / scale_;
  round (x, y);
  mousePt_[0] = x, mousePt_[1] = y;
  
  if (mouse->button() == LeftButton) {
    if ((editMode_ == AppendPoint) && hasPoint_) {
      if (mousePt_ != (activeShape_->getPoint (-1))) {
        activeShape_->appendPoint (new G3DPoint3d(mousePt_));
      }
    }
    damage ();
  }
}

void
MEDrawingWidget::resizeEvent ( QResizeEvent *e )
{
  QFrame::resizeEvent (e);
  offset_.setX (width()/2);
  offset_.setY (height()/2);
  
  // (XXX).
  setOffscreen (false);
  setOffscreen ();
}

void
MEDrawingWidget::round ( float &x, float &y )
{
  if (gridType_ == Cartesian) { 
    int tmp;
    if( x >= 0 )
      tmp = int((x / stepx_) + 0.5);
    else
      tmp = int((x / stepx_) - 0.5);
    x  = tmp * stepx_;
    if( y >= 0 )
      tmp = int((y / stepy_) + 0.5);
    else
      tmp = int((y / stepy_) - 0.5);
    y  = tmp * stepy_;
    emit positionChanged( x, y );
    /*
      int tmp;
      tmp = int((x / stepx_) + (x >= 0) ? 0.5 : -0.5);
      x = tmp * stepx_;
      tmp = int((y / stepy_) + (y >= 0) ? 0.5 : -0.5);
      y = tmp * stepy_;
      emit positionChanged (x, y);
    */
  }
  else if (gridType_ == Polar) {
    double radius = sqrt((x*x) + (y*y));
    double angle  = acos( x / radius );
    
    int irad = int((radius / stepx_) + 0.5);
    radius   = irad * stepx_;
    if (radius < stepx_)
      radius = stepx_;
    
    int iang;
    if (y < 0)
      angle = 2*M_PI - angle;
    iang = int((angle / stepy_) + (angle >= 0) ? 0.5 : -0.5);
    angle = iang * stepy_;
    
    x = float(radius * cos (angle)), y = float(radius * sin (angle));
    emit positionChanged (radius, rad2deg(angle));
  }
}

void
MEDrawingWidget::setCurrent ( int index )
{
  if (index >= 0) {
    activeShape_ = (MELineObject*)(shapes_[index]);
  }
  else {
    activeShape_ = 0;
    hasCross_ = hasPoint_ = false;
  }
}

void
MEDrawingWidget::setEditMode ( MEDrawingWidget::EditMode mode )
{
  //TODO finish previous mode
  editMode_ = mode;
  if (editMode_ == AppendPoint) {
    //ELineObject *obj = new MELineObject;
    //objects_.insert( objects_.end(), obj );
    //active_ = obj;
    setMouseTracking (true);
    hasCross_ = hasPoint_ = false;
  }
}

void
MEDrawingWidget::setGrid ( EnumGridMode mode, float step_a, float step_b )
{
  //TODO finish previous mode
  gridType_ = mode;

  stepx_ = step_a;
  stepy_ = step_b;
}


//====================================END==========================
