/*
 * Project : "MindsEye, 3D library"
 * Copyright (c) 1995 - 1997. All Rights Reserved.
 * Author(s): P. Lavoie, M. Kuball
 *
 * $Id: viewport.h,v 1.3 2000/03/07 23:01:19 ainvar Exp $
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _ne_InterpWidget_h_
#define _ne_IntergWidget_h_

#include "include/config.h"
#include <qframe.h>
#include <qpoint.h>
#include "grid2d.h"
#include "3dlib/g3d_math.h"
#include <vector>

class QPainter;
class MELineObject;


/**A canvas widget to draw a NURBS curve. This is used by 
   CurveInterpWidget.
   */
class MEDrawingWidget: public QFrame
{
  Q_OBJECT

public:
  enum EditMode { AppendPoint, FreeHand, EditCurve };

  MEDrawingWidget ( vector<void*>&, QWidget *parent=0 );
  virtual ~MEDrawingWidget ();

  MELineObject* getSelectedShape () { return activeShape_; }

  signals:
  void positionChanged ( float, float );
  void showPopup ( int, int );

public slots:
  void setCurrent ( int );
  void setEditMode ( EditMode );
  void setGrid ( EnumGridMode, float, float );

protected:
  void drawContents  ( QPainter* );
  void drawCrossWire ( QPainter *p, int, int );
  void drawGrid      ( QPainter *p );
  void drawObjects   ( QPainter *p );
  void drawCurrent   ( QPainter *p );

  void damage ();

  QPixmap * setOffscreen ( bool = true );

  void mouseMoveEvent    ( QMouseEvent * );
  void mousePressEvent   ( QMouseEvent * );
  void mouseReleaseEvent ( QMouseEvent * );
  void resizeEvent       ( QResizeEvent* );
  void enterEvent        ( QEvent * );
  void leaveEvent        ( QEvent * );
  void round ( float &x, float &y );
  
protected:
  vector<void*> &shapes_;
  MELineObject  *activeShape_;
  int            activePoint_;

  QPixmap       *offscreen_;
  bool           damaged_;
  bool           ignoreFollowingLeave_;

  int   selected_, mButton_;
  int   mod_curve;
  float stepx_, stepy_;
  float scale_;
  float mouseX_, mouseY_;
  bool  hasCross_, hasPoint_;

  EnumGridMode gridType_;
  EditMode editMode_;

  G3DPoint3d mousePt_;
  QPoint     offset_;
};


#endif
