/*
 * Project : "MindsEye"
 * Copyright (c) 1997. All Rights Reserved.
 * Atuhor(s): M. Kuball
 *
 * $Id: scene_graph.cpp,v 1.3 2002/06/12 20:13:18 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * A dialog  for viewing and editing the scene graph. It's based on
 * ktreelisttest from Keith Brown.
 *
 */


//#include <qchkbox.h>
//#include <qgrpbox.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpopmenu.h>
#include <qcombo.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qkeycode.h>

#include "3dlib/sceneElement.h"
#include "3dlib/s_scene.h"
//TODO #include "3dlib/camera.h"
#include "kernel/world_obj.h"
#include "kernel/core.h"
#include "dialogs/listview.h"
#include "scene_graph.h"

#include "viewitem.h"
#include "insert_op.h"


MSceneGraphDlg::MSceneGraphDlg( int module_id, int load_t ) :
  MEModule(module_id, load_t, "MSceneGraphDialog" )
{
  setMinimumSize( 200, 300 );

  selectedItem_ = 0;
  listRoot_     = 0;

  QVBoxLayout *gm    = new QVBoxLayout (this, 6);
  QHBoxLayout *gm_00 = new QHBoxLayout;
  QHBoxLayout *gm_01 = new QHBoxLayout;
  QHBoxLayout *gm_02 = new QHBoxLayout;
  QHBoxLayout *gm_03 = new QHBoxLayout;
  gm->addLayout (gm_03, 3);
  gm->addLayout (gm_00,100);
  gm->addLayout (gm_01, 3);
  gm->addLayout (gm_02, 3);
  //gm->addSpacing( 5 );

  QLabel *lb = new QLabel( "Scene:", this );
    lb->setAlignment( AlignHCenter|AlignVCenter );
    lb->setFixedSize( lb->sizeHint() );
    gm_03->addWidget( lb, 1 );
  worldSelector_ = new QComboBox( this );
    worldSelector_->setMinimumSize( 40, 20 );
    connect( worldSelector_, SIGNAL(highlighted(int)), SLOT(setWorld(int)) );
    gm_03->addWidget( worldSelector_, 100 );

  treeView_ = new MEListView( this );
  //treeView_->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
  //treeView_->viewport()->setFocusPolicy( ClickFocus );
    treeView_->addColumn ("type");
    treeView_->addColumn ("name");
    //treeView_->header()->hide();
    treeView_->setSorting (-1);
    treeView_->setMultiSelection( false );  //TODO
    //treeView_->setColumnWidthMode( 0, QListView::Manual );
    //treeView_->setRootIsDecorated( true );
    treeView_->setTreeStepSize( 15 );
    treeView_->setAllColumnsShowFocus( true );
    treeView_->setMinimumSize( 100, 100 );
    connect (treeView_, SIGNAL(currentChanged(QListViewItem*)),
	     SLOT(setCurrent(QListViewItem*)) );
    connect (treeView_, SIGNAL(selectionChanged(QListViewItem*)),
	     SLOT(setSelection(QListViewItem*)));
    gm_00->addWidget( treeView_, 70 );

  QListBox *lbox = new QListBox( this );
    lbox->setMinimumSize( 50, 100 );
    gm_00->addWidget( lbox, 30 );

  QLabel *label = new QLabel("Name:", this);
    label->setAlignment( AlignHCenter|AlignVCenter );
    label->setFixedSize( label->sizeHint() );
    gm_01->addWidget( label, 1 );
  _nameEdit = new QLineEdit( this ); //TODO use this for value editing
    _nameEdit->setMinimumSize( 40, 20 );
    gm_01->addWidget( _nameEdit, 100 );
  QPushButton *pbtn = new QPushButton( "set", this );
    pbtn->setMinimumSize( 28, 20 );
    gm_01->addWidget( pbtn, 1 );
    connect( pbtn, SIGNAL(clicked()), SLOT(changeName()) );
  QPushButton *fbtn = new QPushButton( "find", this );
    fbtn->setMinimumSize( 30, 20 );
    gm_01->addWidget( fbtn, 1 );
    //connect( pbtn, SIGNAL(clicked()), SLOT(changeName()) );

  //gm_01->addSpacing( 5 );

  pbtn = new QPushButton( "Close", this );
  pbtn->setMinimumSize( 65, 22 );
  connect( pbtn, SIGNAL(clicked()), this, SLOT(closeIt()) );

  gm_02->addWidget (pbtn, 0);
  gm_02->addStretch (100);

  gm->activate ();
  resize (300, 300);

  world_ = 0;
  setWorldList ();
  connect( Kernel, SIGNAL(worldListChanged()), SLOT(setWorldList()) );
}


void
MSceneGraphDlg::addSubItem()
{
  //TODO only add as child if a group otherwise add to parent
  //get currentItem
  //go up while not group

  //MTreeListItem *tmp = (MTreeListItem*)(_outlineTree->getCurrentItem());
  /*
  if( tmp && tmp->isGroup() )
    {
      parent = tmp->getGroup();
      parent->append( new MShapeItem(new G3DGroup) );
    }
  _nameEdit->setText("");
  */
}


void MSceneGraphDlg::changeName ()
{
  MeViewItem *citem = (MeViewItem*)(treeView_->currentItem());
  if( _nameEdit->text() && treeView_->currentItem() ) {
    G3DObject *obj = citem->getObject();
    if (obj) {
      obj->setName (_nameEdit->text());
      citem->update ();
    }
  }
}

void
MSceneGraphDlg::closeIt ()
{
  close ();
}

void
MSceneGraphDlg::itemDown()
{
  /*
  int index = _outlineTree->currentItem();
  if( index >= 0 )
    _outlineTree->lowerItem(index);
  */
}

// indents the current item one level, splitting it into a new branch
void 
MSceneGraphDlg::itemIn ()
{
  //_outlineTree->split( _outlineTree->currentItem() );
}

// outdents the current item one level, making it a sibling of its parent
void
MSceneGraphDlg::itemOut ()
{
  //_outlineTree->join( _outlineTree->currentItem() );
}

// moves the current item up one slot in its branch
void 
MSceneGraphDlg::itemUp ()
{
  /*
    //TODO reorder the objects in the scene graph
    int index = _outlineTree->currentItem();
    if( index >= 0 )
    _outlineTree->raiseItem(index);
  */
}

void
MSceneGraphDlg::keyPressEvent( QKeyEvent *e )
{
  //cout << "SGD key press" << endl;
  
  switch (e->key())
    {
    case Key_Delete: 
      removeObject();  
      break;
    case Key_S:
      if (selectedItem_) 
	selectedItem_->select (world_);
      break;
    default :
      e->ignore();
    }
}

QPopupMenu*
MSceneGraphDlg::makePopup ()
{
  QPopupMenu *popup = new QPopupMenu;
  CHECK_PTR (popup);

  popup->insertItem ("delete",    this, SLOT(removeObject()));
  popup->insertItem ("add",       this, SLOT(addSubItem()));
  popup->insertItem ("inspect",   this, SLOT(showProperties()));
  popup->insertSeparator ();
  popup->insertItem ("set Focus", this, SLOT(setFocus()));
  popup->insertItem ("show");
  
  return popup;
}

void
MSceneGraphDlg::objectSelected ( G3DObject * obj )
{
  cout << "--> MSceneGraphDlg::objectSelected (" 
       << (const char *)(obj ? "ok" : "null") << ")" << endl;

  //TODO set selection
  if (obj) {
    MeViewItem * li = findItem (obj);
    if (li) {
      treeView_->setSelected (li, true);
    }
  }
  else { // Don't know whether an empty selection is even possible ?
    QListViewItem * sel = treeView_->selectedItem();
    if (sel) {
      treeView_->setSelected (sel, false);
    }
  }
}

// removes the current item from the tree
void
MSceneGraphDlg::removeObject()
{
  //TODO do not remove scene object
  ASSERT (world_ != 0); // needed ?
  if (selectedItem_) {
    selectedItem_->remove (world_);
  }
  
  /*
    if (selectedItem_) {
    QListViewItem *tmp = selectedItem_->parent();
    if (tmp) {
    selectedItem_->remove (world_);
    // tmp->takeItem (selectedItem_);
    }
    }
  */
}

void
MSceneGraphDlg::setCurrent ( QListViewItem *item )
{
  cout << "MSceneGraphDlg::setCurrent ("
       << (const char *)(item ? "ok" : "null") << ")" << endl;

  if (item) {
    MeViewItem *tmp = (MeViewItem*)item;
    selectedItem_ = tmp;
    G3DObject  *obj = tmp->getObject();
    if (obj) {
      _nameEdit->setText( obj->getName() );
      //TODO show properties
    }
  }
  else
    selectedItem_ = 0;
  //TODO disable edit field?
}

void
MSceneGraphDlg::setSelection ( QListViewItem *item )
{
  cout << "MSceneGraphDlg::setSelection ("
       << (const char *)(item ? "ok" : "null") << ")" << endl;
  
  if (item) {
    MeViewItem *tmp = (MeViewItem*)item;
    selectedItem_ = tmp; // ?
    G3DObject *obj = tmp->getObject ();
    if (obj) {
      ASSERT (world_);
      world_->selectObject (obj);
    }
  }
  else {
    selectedItem_ = 0; // ?
  }
}

void
MSceneGraphDlg::setFocus ()
{
  /*
    MTreeListItem *tmp = (MTreeListItem*)(_outlineTree->currentItem());
    tmp->object->getPosition();
    _world->getActiveCamera()
    cam->setFocus( pos );
  */
}

void
MSceneGraphDlg::setWorldList ()
{
  MWorldList &worlds = Kernel->getWorldList();
  worldSelector_->clear ();
  
  bool gotIt = false;
  MWorldList::iterator wld = worlds.begin();
  while (wld != worlds.end()) {
    if (*wld == world_)
      gotIt = true;
    worldSelector_->insertItem (((MWorldObject*)(*wld))->getName());
    ++wld;
  }
  if (!gotIt)
    setWorld (0);
}

void
MSceneGraphDlg::setWorld ( int i )
{
  if (world_)
    disconnect (world_);

  MWorldList &worlds = Kernel->getWorldList();
  world_ = worlds[i];
  ASSERT(world_ != 0);
  scene_ = world_->getScene();

  connect( world_, SIGNAL(objectSelected(G3DObject*)),
	   SLOT(objectSelected(G3DObject*)) );
  connect( world_, SIGNAL(sceneChanged(G3DGroup*)),
	   SLOT(updateScene(G3DGroup*)) );

  //TODO  ?? _worldSelector->repaint();
  updateScene (0);
}

void
MSceneGraphDlg::showProperties ()
{
//get Property dialog from Kernel
}

// toggles expand button drawing on/off
void MSceneGraphDlg::toggleExpandButtons ( bool on )
{
  //_outlineTree->setExpandButtonDrawing( on );
}

// toggles tree branch drawing on/off
void MSceneGraphDlg::toggleTree ( bool on )
{
  //_outlineTree->setTreeDrawing( on );
}

void
MSceneGraphDlg::updateScene( G3DGroup* )
{
  //_outlineTree->setAutoUpdate( FALSE );
  treeView_->clear ();

  //MSceneItem *tmp = new MSceneItem( scene_ );
  ASSERT (scene_ != 0);
  listRoot_ = new MeViewItem (treeView_, scene_);
  CHECK_PTR (listRoot_);
  listRoot_->setOpen (true);
  trace (scene_, listRoot_);

  treeView_->triggerUpdate ();
}

/** Finds the item corresponding to a given G3DObject.
    (Maybe a G3DTraversal (an "Op") should be used for this ??)
    Returns : The corresponding MeViewItem or 0.
**/
MeViewItem *
MSceneGraphDlg::findItem ( G3DObject * obj )
{
  ASSERT (listRoot_ != 0);
  
  QListViewItemIterator it (listRoot_);
  for (; it.current(); ++it) {
    if (((MeViewItem *)(it.current()))->getObject() == obj) 
      break;
  }
  
  return (MeViewItem *)(it.current());
}

void
trace ( G3DGroup *group, MeViewItem *start )
{
  //MShapeItem *tmpa = new MShapeItem( "Attributes" );
  //start->appendChild( tmpa );

  CHECK_PTR (group); // use ASSERT since this is an internal function ?
  CHECK_PTR (start); // ? 

  if (group->numberOfLights() > 0) {
    MeViewItem *tmpl = new MeViewItem (start, "Lights");
    CHECK_PTR (tmpl);
    InsertOp iop (tmpl);
    ((G3DGroup*)group)->doWithLights (iop); 
  }
  if (group->numberOfHelper() > 0) {
    MeViewItem *tmph = new MeViewItem (start, "Helper");
    CHECK_PTR (tmph);
    InsertOp iop (tmph);
    ((G3DGroup*)group)->doWithHelper (iop);
  }
  //TODO Sound
  if (group->numberOfShapes() > 0) {
    //TODO ?
    //MeViewItem *tmps = new MeViewItem (start, "Shapes");
    InsertOp iop (start);
    ((G3DGroup*)group)->doWithShapes (iop);
  }
}


//==================================END==================================
