/*
 * Project : "MindsEye"
 * Copyright (c) 1997 - 1998. All Rights Reserved.
 * Atuhor(s): M. Kuball
 *
 * $Id: scene_graph.h,v 1.3 2002/06/16 20:50:39 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * A dialog  for viewing and editing the scene graph. It's based on
 * ktreelisttest from Keith Brown.
 *
 */

#ifndef _SCENE_DLG_H
#define _SCENE_DLG_H

//#include <qapp.h>
#include <stdio.h>

#include "kernel/module.h"
#include "kernel/world_list.h"


class QListView;
class QListViewItem;
class G3DObject;
class G3DGroup;
class G3DScene;
class MeViewItem;
class MWorldObject;
class QPopupMenu;
class QComboBox;
class QLineEdit;


/**A dialog with a tree list of the scene graph.
 */
class MSceneGraphDlg : public MEModule
{
  Q_OBJECT

public:
  MSceneGraphDlg( int, int );
    
protected:
  virtual void keyPressEvent ( QKeyEvent * e );  
  QPopupMenu*  makePopup ();

protected slots:
    ///
  void addSubItem();
    ///Change the name of the current object to the name in the text field.
  void changeName();
    ///Close the Dialog.
  void closeIt();
    ///Collapses the tree globally to the next level.
  //void collapse();
    ///Expands the tree globally to the next level.
  //void expand();
  void setCurrent ( QListViewItem* );
  void setSelection ( QListViewItem* );
    ///Moves the current item down one slot in its branch.
  void itemDown();
  void itemIn();
  void itemOut();
  void itemUp();
    ///
  void objectSelected ( G3DObject* );
  void removeObject ();
    ///
  void setFocus ();
  void setWorld ( int );
    ///
  void setWorldList ();
    ///
  void showProperties ();
  void toggleExpandButtons ( bool on );
  void toggleTree ( bool on );
    /**The part of the scene below the group has changed.
      Update the tree list.*/
  void updateScene( G3DGroup* );
  MeViewItem * findItem ( G3DObject* );
  
protected:
  ///The scene which will be displayed in the outlineTree widget.
  G3DScene      *scene_;  //TODO do we need this?
  ///
  QListView     *treeView_;
  ///
  MeViewItem    *selectedItem_;
  MeViewItem    *listRoot_;
  QLineEdit     *_nameEdit;
  QComboBox     *worldSelector_;
  ///The world object the displayed scene is part of.
  MWorldObject  *world_;
};


#endif  _SCENE_DLG_H
