/*
 * Project : "MindsEye"
 * Copyright (c) 1997. All Rights Reserved.
 * Atuhor(s): M. Kuball
 *
 * $Id: viewitem.cpp,v 1.3 2001/07/17 02:34:38 ainvar Exp $
 *
 *
 * COPYRIGHT
 * ========= 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 * A TreeList Item for viewing and editing the scene graph. It's based on
 * ktreelist from Keith Brown.
 *
 */

#include "include/config.h"
#include <qstring.h>
#include "3dlib/object.h"
#include "kernel/world_obj.h"
#include "viewitem.h"


MeViewItem::MeViewItem ( QListViewItem *p, G3DObject *obj ) :
  QListViewItem(p, obj->getTypeName() )
{
  cout << "list view item created"<< endl;
  if( obj->getName() )
    setText( 1, obj->getName() );
  object_ = obj;
  object_->addReference();
}

MeViewItem::MeViewItem ( QListViewItem *p, const char *name ) :
  QListViewItem(p, name )
{
  object_ = 0;
}

MeViewItem::MeViewItem ( QListView *p, G3DObject *scn ) :
  QListViewItem(p, scn->getTypeName() )
{
  if( scn->getName() )
    setText( 1, scn->getName() );
  object_ = scn;
  object_->addReference();
}

MeViewItem::~MeViewItem ()
{
  cout << "--> MeViewItem destructor" << endl;

  if (object_)
    object_->dispose ();
}

void
MeViewItem::remove ( MWorldObject *world )
{
  cout << "--> MeViewItem::remove (" << (const char *)(world ? "ok" : "null")
       << ")" << endl;

  if (object_) {
    ASSERT (world != 0);
    world->deleteObject (object_);
  }
}

void
MeViewItem::select ( MWorldObject *world )
{
  cout << "-->MeViewItem::select (" << (const char *)(world ? "ok" : "null")
       << ")" << endl;

  if (object_) {
    ASSERT (world != 0);
    world->selectObject (object_);
  }
}

void
MeViewItem::update ()
{
  cout << "--> MeViewItem::update ()" << endl;

  setText (0, object_->getTypeName());
  if (object_->getName())
    setText (1, object_->getName());
  else
    setText (1, " ");
}

void
MeViewItem::activate ()
{
  cout << "--> MeViewItem::activate ()" << endl;
}

//======================================END==================================
