/*
 * Project : MindsEye
 * Copyright (c) 1996, 1997, 1998
 * Author(s): M. Kuball, F. Meier
 * 
 * $Id: cam_controler.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   shapeList_.append( obj );

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H

#include <qbitmap.h>
#include <qcursor.h>
#include <qkeycode.h>
#include <qevent.h>
#include <GL/gl.h>
#include "include/cursors.h"
#include "3dlib/camera.h"
#include "kernel/world_obj.h"
#include "cam_controler.h"


MECameraControler::MECameraControler ()  
{
  mode_ = Orbit;
  dynamicSpeedMultiplier_ = 1.0;

//TODO
  QBitmap cb1(CURSOR_X, CURSOR_Y, hand_open_cur_bits, FALSE);
  QBitmap *cm1 = new QBitmap(CURSOR_X, CURSOR_Y, hand_open_msk_bits, FALSE);
  panCursor_   = new QCursor( cb1, *cm1 );
}

MECameraControler::~MECameraControler()
{
  delete panCursor_;
}

void 
MECameraControler::cameraInteraction( QMouseEvent *e, int mbtn )
{
	if( ( mbtn & LeftButton ) && ( mbtn & MidButton)) // left and middle = zoom
		dollyCamera( e );	
	else if( ( mbtn & LeftButton ) && ( mbtn & RightButton)) // left and right = roll
		rollCamera( e );
	else if( mbtn & LeftButton ) // bitwise exclusive OR will tell if a button is down
		orbitCamera( e );
	else
		tuckCamera( e );
}


void
MECameraControler::evaluateInput( QMouseEvent *e, int mbtn )
{
  if( camera_ )
    {
      if( camera_->isParallelMode() )
	{
	  if( mbtn & LeftButton )
	    tuckCamera( e );
	  else
	    fovCamera( e, LeftButton );
	}
      else
	{
	  switch( mode_ )
	    {
	    case Fly :
	      flyCamera( e );
	      break;
	    case Orbit :
	      cameraInteraction( e, mbtn );
	      //break;
	      //orbitCamera(e);
	      break;
	    case Rotate :
	      rotateCamera(e);
	      break;
	    case Tuck :
	      tuckCamera(e);
	      break;
	    case Roll :
	      rollCamera(e);
	      break;
	    case Zoom :
	      fovCamera(e, mbtn);
	      break;
	    }
	    vx_ = e->x();  vy_ = e->y();
	    camera_->mouse2view( vx_, vy_ );
	}
    }
}

void
MECameraControler::flyCamera( QMouseEvent *e )
{
  xspeed_ = (_oldx - e->x()) * 0.1 ;
  yspeed_ = (e->y() - _oldy) * 0.01 ;
  char buffer[40];
  sprintf( buffer, "forward speed: %.3f angular speed: %.3f",
	   yspeed_, xspeed_ );
  emit setInfoText( buffer );
}

void 
MECameraControler::fovCamera( QMouseEvent *e, int mbtn )
{
  int yDiff = e->y() - _oldy;
  if( yDiff != 0 )
    {
      if( mbtn & LeftButton )
	{
	  if( camera_->isParallelMode() )
	    camera_->setParallelScale( _old_value + yDiff*0.1* dynamicSpeedMultiplier_ );
          else
	    // add some amount to original fov value
	    camera_->setFOV( _old_value + yDiff * 0.5* dynamicSpeedMultiplier_ );
	}
      else  //dolly camera
	{
	  camera_->dolly( yDiff * 0.01 * dynamicSpeedMultiplier_);
	  _oldy = e->y();
	}
    }
}

void MECameraControler::dollyCamera(QMouseEvent *e)
{
  int yDiff = e->y() - _oldy;

  if( yDiff != 0 )
    {
      camera_->dolly( yDiff * 0.01 * dynamicSpeedMultiplier_ );
      _oldx = e->x(); _oldy = e->y();
    }
}

QCursor*
MECameraControler::getCursor ()
{
  return panCursor_;
  //TODO return cursor shape depending on the mode
}

t_type
MECameraControler::getType ()
{
  return camera_->getType();
}

void 
MECameraControler::orbitCamera( QMouseEvent *e )
{
  int     xDiff, yDiff;
  MeFloat x, y;

  xDiff = e->x() - _oldx;
  yDiff = e->y() - _oldy;

  if( xDiff != 0 )
    {
      x = xDiff * 0.5 * dynamicSpeedMultiplier_; // 0.5 degrees per pixel
      camera_->azimuth( deg2rad(x) );
    }
  if( yDiff != 0 )
    {
      y = yDiff * 0.5 * dynamicSpeedMultiplier_;
      camera_->elevation( deg2rad(y) );
    }
  _oldx = e->x();
  _oldy = e->y();
}

void 
MECameraControler::rollCamera (QMouseEvent *e )
{
  int xDiff = e->x() - _oldx;
  if( xDiff != 0 )
    { 
      MeFloat x = -xDiff * 0.5 * dynamicSpeedMultiplier_;
      camera_->roll( deg2rad(x) );
      _oldx = e->x(); _oldy = e->y();
    }
}

void 
MECameraControler::rotateCamera( QMouseEvent *e )
{
  bool    doUpdate = FALSE;
  int     xDiff, yDiff;

  xDiff = e->x() - _oldx;
  yDiff = e->y() - _oldy;

  if( xDiff != 0 )
    {
      camera_->yaw( deg2rad(xDiff*1.5)* dynamicSpeedMultiplier_ );
      doUpdate = TRUE;
    }
  if (yDiff != 0)
    {	 
      camera_->pitch(deg2rad(yDiff*1.5)* dynamicSpeedMultiplier_);
      doUpdate=TRUE;
    }
  if (doUpdate) {
    _oldx = e->x(); _oldy = e->y();
  }
}

void
MECameraControler::setMode( int m )
{
  mode_ = m;
}

void
MECameraControler::showFeedback ()
{
  //TODO camera_->setIdentityProjection()
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho( -1, 1, -1, 1, -1, 1 );
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();

  glLineWidth( 1.5 );
  glDisable( GL_LIGHTING );

  if( mode_ == Fly )
    {
      MeFloat ox = _oldx, oy = _oldy;
      camera_->mouse2view( ox, oy );
      glBegin( GL_LINES );
      glVertex3f( ox, oy, 0 );
      glVertex3f( vx_, vy_, 0 );
      glEnd();
    }
}

void
MECameraControler::start( QMouseEvent *e )
{
  _oldx = e->x();
  _oldy = e->y();
  vx_ = e->x();  vy_ = e->y();
  camera_->mouse2view( vx_, vy_ );

  //camera_ = cam;
  if( camera_->isParallelMode() )
    _old_value = camera_->getParallelScale();
  else if( mode_ == Fly )
    {
      xspeed_ = 0;
      yspeed_ = 0;
      //ltime_  = time();
      world_->startTimer();
      connect( world_, SIGNAL(timerEvent()), SLOT(timerDone()) );
    }
  else
    {
      //TODO cout << "cc in mode " << mode_ << endl;
      _old_value = camera_->getFOV();
    }
}

void
MECameraControler::stop( )
{
  if( mode_ == Fly )
    {
      world_->stopTimer();
      world_->disconnect( this, SLOT(timerDone()) );
    }
  world_->updateCamera();
}

void
MECameraControler::timerDone()
{
  if( mode_ == Fly )
    {
      camera_->tuck( 0, 0, yspeed_ * 0.3 );
      camera_->yaw( deg2rad(xspeed_*0.3) );
    }
}

void
MECameraControler::tuckCamera( QMouseEvent *e )
{
  MeFloat scale = 1.0/50.0;
  if( camera_->isParallelMode() )
    {
      int xDiff = e->x() - _oldx;
      int yDiff = e->y() - _oldy;

      MeFloat xpos,ypos;
      camera_->getWindowCenter( xpos, ypos );
      xpos +=  xDiff * scale;
      ypos += -yDiff * scale;
      camera_->setWindowCenter( xpos, ypos );
    }
  else
    {
      MeFloat delx, dely = 0.0, delz = 0.0;

      delx = MeFloat( _oldx - e->x() ) * scale;
      if( e->state() & ShiftButton )
	delz = (MeFloat)(_oldy - e->y()) * scale;
      else
	dely = (MeFloat)(e->y() - _oldy) * scale;
      camera_->tuck( delx, dely, delz );
    }

  _oldx = e->x();
  _oldy = e->y();
}

void MECameraControler::increaseDynamicSpeed(float f)
{
	dynamicSpeedMultiplier_ = dynamicSpeedMultiplier_ * f;
}

void MECameraControler::decreaseDynamicSpeed(float f)
{
	dynamicSpeedMultiplier_ = dynamicSpeedMultiplier_ / f;
}
void MECameraControler::resetDynamicSpeed(float f)
{
	dynamicSpeedMultiplier_ =  f;
}

//================================EMD====================================
