/*
 * Project : "MindsEye"
 * Copyright (c)1996, 1997, 1998. All Rights Reserved.
 * Author(s) M. Kuball
 *
 * $Id: cam_controler.h,v 1.2 2002/06/16 20:50:39 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */
 
#ifndef __CAMERA_CONTROLER_H__
#define __CAMERA_CONTROLER_H__

#include "controler.h"
#include "3dlib/t_objects.h"

class QMouseEvent;
class G3DCamera;
class QTimer;
class MWorldObject;


/**
 */
class MECameraControler : public MEControler
{
  Q_OBJECT

public:
  enum { Fly=0, Orbit, Rotate, Zoom, Tuck, Roll };  //TODO Parallel
 //AllInOne, Pan, Orbit, Tuck, Rotate, Zoom, Roll, Dolly };

public:
  MECameraControler ();
  ~MECameraControler();

  int     getMode() { return mode_; }
  t_type  getType();
  void    setMode ( int );
  void    setWorld ( MWorldObject* w ) { world_ = w; }
  virtual void     evaluateInput( QMouseEvent*, int );
  virtual QCursor* getCursor ();
  virtual void     showFeedback ();
  virtual void     start( QMouseEvent* );
  virtual void     stop ();

  void increaseDynamicSpeed (float f = 1.2);
  void decreaseDynamicSpeed (float f = 1.2);
  void resetDynamicSpeed (float f = 1.0);

protected slots:
  void timerDone();

protected:
    ///Orbit/move/zoom all in one
  void cameraInteraction( QMouseEvent *, int );
    ///
  void flyCamera( QMouseEvent* );
    ///Change field-of-view, left mbtn zooms, right mbtn dolly
  void fovCamera( QMouseEvent*, int );
    ///Rotate eye around look at point
  void orbitCamera( QMouseEvent* );  
    /// change perspective, ie. field-of-view, without zoom
  void perspCamera( QMouseEvent* );
    /// roll camera sideways
  void rollCamera( QMouseEvent* );
    /// rotate look-at point around eye
  void rotateCamera(QMouseEvent *);
    ///Move the Camera parallel to the viewplane.
  void tuckCamera( QMouseEvent* );
	 /// simple pullback/forward of camera
  void dollyCamera( QMouseEvent* );

protected:
  MeFloat      _old_value;   //needed by fovCamera() [fm]
  MeFloat      xspeed_, yspeed_;
  MeFloat      vx_, vy_;
  int          _oldx, _oldy;
  int          mode_;
  QCursor      *panCursor_;
  MWorldObject *world_;
  MeFloat      dynamicSpeedMultiplier_;
};

#endif
