/*
 * Project : "MindsEye"
 * Copyright 1996 - 1997. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_viewl.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 * 
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 *
 */

#include "include/config.h"

#include <qkeycode.h>
#include <qlayout.h>

#include "dialogs/viewport/viewLayout.h"
#include "m_vmenu.h"
#include "m_viewl.h"
#include "viewgl.h"

typedef struct
{
  QBoxLayout::Direction gmBase;
  QBoxLayout::Direction gm2;  //if gm1 exists it has LeftoToRight
  char gmSub;                 //bit 1 for gm1, bit 2 for gm2
  char wnr;                   //# of views in gm2
} SLayout;

//A description of the 8 default layouts.
static SLayout Layouts[8] =
{ {QBoxLayout::TopToBottom, QBoxLayout::LeftToRight, 0, 1},
  {QBoxLayout::TopToBottom, QBoxLayout::LeftToRight, 3, 2},
  {QBoxLayout::TopToBottom, QBoxLayout::LeftToRight, 2, 3},
  {QBoxLayout::LeftToRight, QBoxLayout::TopToBottom, 2, 3},
  {QBoxLayout::LeftToRight, QBoxLayout::LeftToRight, 0, 2},
  {QBoxLayout::TopToBottom, QBoxLayout::LeftToRight, 0, 2},
  {QBoxLayout::TopToBottom, QBoxLayout::LeftToRight, 2, 2},
  {QBoxLayout::LeftToRight, QBoxLayout::TopToBottom, 2, 2}
};


class MESplitter : public QFrame
{
public:
  MESplitter( QWidget *p ) : QFrame(p)
    { setFrameStyle( WinPanel | Raised );
      setMinimumSize( 3, 3 );
    }
};


MViewPortLayout::MViewPortLayout( MModeler *model, QWidget *parent ) :
  QFrame(parent)
{
  setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
  makePopups();

  initViews( model );
  
  connect( parent, SIGNAL(worldChanged(QObject*)), SLOT(setWorld(QObject*)) );
  connect( parent, SIGNAL(cameraControlerChanged(MECameraControler*)),
	   SLOT(setCameraControler(MECameraControler*)) );
  connect( parent, SIGNAL(setWorkScale(MeFloat)),
	   SLOT(setWorkScale(MeFloat)) );
 //let's start with our LightWave "Modeler" view layout
  setLayout( LAYOUT_A );
}

MViewPortLayout::~MViewPortLayout (void)
{
  delete layout_popup;
}


void
MViewPortLayout::centerCamera()
{
  activeView_->centerCamera();
}

void
MViewPortLayout::editLayout ()
{
  MEViewlayoutDialog *tmp = new MEViewlayoutDialog( layout_ );
  connect( tmp, SIGNAL(setLayout(int)), SLOT(setLayout(int)) );
  tmp->show();
}

void
MViewPortLayout::initViews( MModeler *model )
{
  for( int i = 3; i >= 0; -- i )
    {
      views_[i] = new MGLPort( layout_popup, i, this );
      views_[i]->setModel( model );
      connect( views_[i], SIGNAL(setFocus(MGLPort*)),
	       SLOT(setActiveView(MGLPort*)) );

      spl_[i] = new MESplitter( this );
    }
  activeView_ = views_[0];

  gm = gm1 = gm2 = NULL;
}

void
MViewPortLayout::keyPressEvent( QKeyEvent *e )
{
  switch( e->key() )
    {  
    case Key_1 :
      setLayout( LAYOUT_A );  break;
    case Key_2 :
      setLayout( LAYOUT_B );  break;
    case Key_3 :
      setLayout( LAYOUT_C );  break;
    case Key_4 :
      setLayout( LAYOUT_D );  break;
    case Key_5 :
      setLayout( LAYOUT_E );  break;
    case Key_6 :
      setLayout( LAYOUT_F );  break;
    case Key_7 :
      setLayout( LAYOUT_G );  break;
    case Key_8 :
      setLayout( LAYOUT_H );  break;
    default:
      e->ignore();
    }
}

void
MViewPortLayout::makePopups()
{
  layout_popup = new MViewportPopup( this );
  connect( layout_popup,
	   SIGNAL(setShadingMode(int)),
	   SLOT(setShadingModeLocal(int)) );
  connect( layout_popup,
	   SIGNAL(changeCameraType(int)),
	   SLOT(setCameraType(int)) );
  connect( layout_popup,
	   SIGNAL(centerCamera()),
	   SLOT(centerCamera()) );
}

void
MViewPortLayout::setLayout( int new_layout )
{
  layout_ = new_layout;
  for( int i1 = 3; i1 >= 0; --i1 )
    {
      views_[i1]->hide();
      spl_[i1]->hide();
    }

  delete gm1;
  delete gm2;
  delete gm;
  gm1 = gm2 = NULL;

  SLayout &layout = Layouts[new_layout];
  int vn=0, sn=0;

  gm = new QBoxLayout( this, layout.gmBase );
  if( layout.gmSub & 0x1 )
    { //only for layout B
      gm1 = new QBoxLayout( QBoxLayout::LeftToRight );
      gm->addLayout( gm1, 100 );
      gm1->addWidget( views_[vn++], 100 );
      gm1->addWidget( spl_[sn], 0 );
      spl_[sn++]->show();
      gm1->addWidget( views_[vn++], 100 );
    }
  else
    gm->addWidget( views_[vn++], 100 );
  if( layout.gmSub & 0x2 )
    {
      gm->addWidget( spl_[sn], 0 );
      spl_[sn++]->show();
      gm2 = new QBoxLayout( layout.gm2 );
      gm->addLayout( gm2, 100 );

      gm2->addWidget( views_[vn++], 100 );
      gm2->addWidget( spl_[sn], 0 );
      spl_[sn++]->show();
      gm2->addWidget( views_[vn++], 100 );
      if( layout.wnr > 2 )
	{
	  gm2->addWidget( spl_[sn], 0 );
	  spl_[sn++]->show();
	  gm2->addWidget( views_[vn++], 100 );
	}
    }
  else if( layout.wnr > 1 )
    {
      gm->addWidget( spl_[sn], 0 );
      spl_[sn]->show();
      gm->addWidget( views_[vn++], 100 );
    }

  vn--;
  for( ; vn >= 0; --vn )
    views_[vn]->show();

  gm->activate();
  update();
}
/*---------------------------------------------------------------------------*/
void
MViewPortLayout::setActiveView( MGLPort *active )
{
  activeView_ = active;
}

void
MViewPortLayout::setCaching( int mode )
{
  //TODO this is a per scene thing
  activeView_->setDisplayListMode( mode );
}

void
MViewPortLayout::setCameraControler( MECameraControler *cc )
{
  for( int i = 3; i >= 0; --i )
    views_[i]->setCameraControler( cc );
  //TODO  if( activeView_ )
  // activeView_->setCameraControler( cc );
}

void
MViewPortLayout::setCameraType( int t )
{
  activeView_->setCameraType( t );
  
}

void
MViewPortLayout::setShadingModeAll( int new_dtype )
{
  for( int i = 3; i >= 0; --i )
    views_[i]->setShadingMode( new_dtype );
}

void
MViewPortLayout::setShadingModeLocal( int new_dtype )
{
  activeView_->setShadingMode( new_dtype );
}

void
MViewPortLayout::setMajorInteractionMode( int m )
{
  for( int i = 3; i >= 0; --i )
    views_[i]->setMajorInteractionMode( m );
}

void
MViewPortLayout::setWorkScale( MeFloat s )
{
  for( int i = 3; i >= 0; --i )
    views_[i]->setWorkScale( s );
}

void
MViewPortLayout::setWorld( QObject *world )
{
  activeView_->focusOutEvent( NULL );
  for( int i = 3; i >= 0; --i )
    views_[i]->setWorld( world );
  activeView_->focusInEvent(NULL);
}

void
MViewPortLayout::toggle_axes( bool val )
{
  for( int i = 3; i >= 0; --i )
    views_[i]->show_axes (val);
}

int
MViewPortLayout::toggle_caching ()
{
//TODO
  return( 0 );
}

void
MViewPortLayout::toggle_grid( bool val )
{
  for( int i = 3; i >= 0; --i )
    views_[i] ->show_grid (val);
}

//TODO do we need this?
void
MViewPortLayout::viewChanged()
{
  for( int i = 3; i >= 0; --i )
    views_[i]->updateGL();
}


//==================================END===================================
