/*
 * Project : "MindsEye"
 * Copyright 1996. All Rights Reserved.
 * Author(s) M. van Velsen, M. Kuball
 *
 * $Id: m_viewl.h,v 1.2 2002/06/16 20:50:39 ainvar Exp $
 *
 * 
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * OVERVIEW
 * ========
 *
 */
 
#ifndef __MVIEWPORT_L_H__
#define __MVIEWPORT_L_H__

#include "include/config.h"
#include <qframe.h>
#include "m_vdef.h"


#define LAYOUT_A      0
#define LAYOUT_B      1
#define LAYOUT_C      2 
#define LAYOUT_D      3
#define LAYOUT_E      4
#define LAYOUT_F      5
#define LAYOUT_G      6
#define LAYOUT_H      7

enum MEViewportMode { ViewMode, ObjectMode };
enum { OPERATION_OBJECT, OPERATION_CAMERA };
enum { DirectMode, DisplayListMode };


class QBoxLayout;
class QCursor;
class MViewportPopup;
class MModeler;
class MECameraControler;
class MESplitter;
class MGLPort;


/**A geometry manager for a quasi automatic layout of the
   3d view ports.
   The class knows about 8 predifined layouts which can be selected
   via a special dialog or keys.
 */
class MViewPortLayout : public QFrame
{
    Q_OBJECT

public:
    ///
  MViewPortLayout( MModeler*, QWidget* );
  ~MViewPortLayout( );

  void editLayout ();
    ///
  void setCaching( int mode );
    ///view or object
  void setMajorInteractionMode( int );
    ///Toggle the show axes flag on all 4 MGLPorts
  void toggle_axes    (bool);
    /**Toggle the caching of shapes in display lists.
       Returns the current mode.*/
  int  toggle_caching ();
    ///Toggle the show grid flag on all 4 MGLPorts
  void toggle_grid    (bool);

public slots:
    ///
  void centerCamera();
    ///Change the display mode of all viewports.
  void setShadingModeAll( int );
    ///Change the display mode of the active viewport.
  void setShadingModeLocal( int );
    ///
  void viewChanged();

protected:
    ///Initialize the 4 viewports.
  void initViews( MModeler* );
    ///Select a layout type
  void keyPressEvent( QKeyEvent* );
    ///Create some popup menus.
  void makePopups ();

protected slots:
  void setActiveView( MGLPort* );
  void setCameraControler( MECameraControler* );
  void setCameraType(int);
  void setLayout(int);
    ///
  void setWorkScale( MeFloat );
  void setWorld(QObject*);

private:
  int layout_;
      ///
  MViewportPopup *layout_popup;
 
  QBoxLayout *gm;  // master layout
  QBoxLayout *gm1; // row/column 1
  QBoxLayout *gm2; // row/column 2

  MGLPort    *views_[4];
  MGLPort    *activeView_;
  MESplitter *spl_[4];
};


#endif // __MVIEWPORT_L_H__
