/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997 by Martin Kuball. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: obj_controler.h,v 1.2 2002/06/16 20:50:39 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#ifndef _OBJ_CONTROLER_H_
#define _OBJ_CONTROLER_H_

#include "include/config.h"
#include <qobject.h>
#include <qevent.h>
#include "controler.h"
#include "3dlib/g3d_math.h"


class G3DObject;
class G3DCamera;
class Grid;


/**Abstract base class for Controler which are used to Transform
   3d objects. They translate mouse movements into suitable 3d
   coordinates.
   For special input devices you have to reimplement the transform
   function;
  */
class MEObjectControler : public MEControler
{
    Q_OBJECT

public:
  MEObjectControler () {
    selected_ = NULL;
    grid_     = NULL;
  };

  ///
  void    setGrid( Grid *grid ) { grid_ = grid; };
  ///
  void    setSelectedObject( G3DObject *sel ) { selected_ = sel; }

protected:
  G3DObject   *selected_;
  //G3DVector3d direction_;
  Grid        *grid_;

};


/**Objects of this class serve as a Translation controler.
   They perform a translation on the selected object based on
   the given mouse coordinates and the internal state
 */
class MoveControler : public MEObjectControler
{
public:
  MoveControler () {}

    ///Translate the selected object.
  virtual void evaluateInput ( QMouseEvent*, int );
    ///
  virtual void start ( QMouseEvent* );
    ///
  virtual void stop ();

protected:
    ///
  void doMove( int, int, int );

protected:
  G3DMatrix   map2local_;
  //G3DVector3d orig_;
  G3DPoint3d  position_, mousepos_, mstart_;
  int         oldy_, oldz_;
};


/**Objects of this class serve as a rotation controler.
   They perform a rotation on the selected object based on
   the given mouse coordinates and the internal state.
 */
class RotateControler : public MEObjectControler
{
public:
  RotateControler () {};

    ///Translate the selected object.
  virtual void evaluateInput ( QMouseEvent*, int buttonstate );
    ///
  virtual void start ( QMouseEvent* );
    ///
  virtual void stop ();

protected:
    ///Find the angle for the given mouse position.
  MeFloat calcAngle( G3DVector3d &mouse );
    ///
  void    doRotate( int, int );
    ///Initialize the translation from mouse pos to angle.
  void    initAngle( G3DVector3d&, G3DVector3d &e1, G3DVector3d &e2 );
  MoveControler *mover_;

protected:
  G3DQuaternion rot_;
  G3DVector3d   axis_, gaxis_, lcenter_, center_, radius_;
  G3DPoint3d    gcenter_, mouse_, origin_;

  bool        colinear_;
  MeFloat     angle0_, length_;
};


/**Objects of this class serve as a controler for scaling objects.
   A scaling controler must be asigned a widget object.
 */
class ScaleControler : public MEObjectControler
{
    Q_OBJECT

public:
  ScaleControler () {}

  virtual void evaluate ( G3DVector3d& );
    ///Translate the selected object.
  virtual void evaluateInput ( QMouseEvent*, int );
    ///
  virtual void start ( QMouseEvent* );
    ///
  virtual void stop ();

protected:
    ///
  void doScale( int, int, int );

protected:
  G3DMatrix   mlocal_, map2local_;
  G3DVector3d svector_, scale_, transl_;
  G3DPoint3d  mousepos_, gfixed_, lfixed_;
  int        _oldx, _oldy, _oldz;
  bool       uniform_;
};

#endif
