/*
 * Project : "MindsEye"
 * Copyright (c) 1996 - 1998. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: rcontroler.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdio.h>
//TODO include <stdlib.h>
#include <qkeycode.h>
#include <qwidget.h>

#include "3dlib/w_rotate.h"
#include "3dlib/camera.h"
#include "obj_controler.h"
#include "grid.h"


MeFloat
RotateControler::calcAngle( G3DVector3d &mouse )
{
  G3DVector3d v1(mouse - gcenter_);
  MeFloat      angle;

  if( colinear_ )
    {
      MeFloat r = v1.dot(radius_) / length_;
      if( r >= 1.0 )
	angle = 0.0;
      else if( r <= -1.0 )
	angle = M_PI;
      else
	angle = acos( r );
      angle = angle - angle0_;
    }
  else
    {
      G3DVector3d v2( mouse_ - gcenter_ );

      MeFloat ca = v1.dot( v2 ) / (v1.length() * v2.length());
      angle = acos( ca );
      if( (v2.cross( v1 )).dot(gaxis_) < 0 )
	angle = 2*M_PI - angle;
    }

  return( angle );
}

void
RotateControler::doRotate( int x, int y )
{
  G3DPoint3d mouse( gcenter_ );
  MeFloat    angle;

  camera_->unproject( x, y, mouse );
  angle = calcAngle( mouse );

  //TODO grid for angle or angles;
  G3DQuaternion rot( axis_, angle );
  rot *= rot_;
  selected_->setOrientation( rot, lcenter_, origin_ );

  char buffer[20];
  sprintf( buffer, "a = %.3f", rad2deg(angle) );
  emit setInfoText( buffer );
}

void
RotateControler::evaluateInput( QMouseEvent *e, int bt )
{
  if( mover_ == NULL )
    doRotate( e->x(), e->y() );
  else
    mover_->evaluateInput( e, bt );
}

void
RotateControler::initAngle( G3DVector3d &mouse, G3DVector3d &e1,
			    G3DVector3d &e2 )
{
  const G3DVector3d &up( camera_->getUp() );
  const G3DVector3d &r( camera_->getRight() );
  length_ = e1.length();
  e1 = up * e1.dot(up) + r * e1.dot(r);
  e2 = up * e2.dot(up) + r * e2.dot(r);
  if( (e1.cross( e2 )).length() < FG_EPSILON )
    {
      colinear_ = true;
      if( e1.length() > FG_EPSILON )
	{
	  radius_ = e1;
	}
      else
	radius_ = e2;
      radius_.normalize();

      //G3DVector3d v(mouse - gcenter_);
      angle0_ = (mouse - gcenter_).dot(radius_) / length_;
      if( angle0_ >= 1.0 )
	angle0_ = 0;
      else if( angle0_ <= -1.0 )
	angle0_ = M_PI;
      else
	angle0_ = acos( angle0_ );
    }
  else
    colinear_ = false;
}

void
RotateControler::start( QMouseEvent *e )
{
  if( selected_ && selected_->getType() == TW_ROTATE )
    {
      G3DRotateWidget *wid = (G3DRotateWidget*)selected_;
      if( e->state() == AltButton )
	{
	  mover_ = new MoveControler;
	  mover_->setSelectedObject( wid );
	  mover_->setCamera( camera_ );
	  mover_->start( e );
	}
      else
	mover_ = NULL;

      G3DPoint3d  center( wid->getCenter() );
      G3DVector3d axis;
      G3DMatrix   lmat, gmat;

      wid->getAxis( axis );  //based on the user selection
      wid->getLocalTrafo( lmat );
      wid->getWorldTrafo( gmat );
      center_  = center;
      gcenter_ = gmat * center;  //transform as a point here
      lcenter_ = lmat * center_; //and as a vector here
      axis_    = lmat * axis;
      gaxis_   = gmat * axis;

      mouse_   = gcenter_;
      camera_->unproject( e->x(), e->y(), mouse_ );

      G3DVector3d e1, e2;
      wid->getCircle( e1, e2 );
      e1 = gmat * e1;
      e2 = gmat * e2;
      initAngle( mouse_, e1, e2 );
      origin_  = wid->getParent()->getOrigin();
  
      rot_ = wid->getOrientation();

      emit setInfoText( "a = 0" );
    }
}

void
RotateControler::stop ()
{
  delete mover_;
  mover_ = NULL;
}


//================================END===================================
