/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997 by Martin Kuball. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: scontroler.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdio.h>
//include <stdlib.h>
#include <qkeycode.h>
#include <qwidget.h>

#include "3dlib/object.h"
#include "3dlib/camera.h"
#include "3dlib/w_scale.h"
#include "obj_controler.h"
#include "grid.h"


void
ScaleControler::evaluate( G3DVector3d &delta )
{
  if( selected_ )
    {
      G3DVector3d scale( selected_->getScale() );
      G3DVector3d transl(selected_->getOrigin() );
      G3DPoint3d  pos;

      selected_->getLocalTrafo( mlocal_ );
      pos = selected_->getOrigin();

      G3DMatrix scaler;
      scaler.scale( (scale + delta).cwDivide(scale) );
      scaler = mlocal_ * scaler;
      scale += delta;

      G3DVector3d bound( FG_EPSILON ); //don't make the object smaller
      scale.max( bound );

      selected_->setScale( scale );
      selected_->setOrigin( (mlocal_*pos) - (scaler*pos) + transl );

      char buffer[40];
      sprintf( buffer, "x: %.4f y: %.4f z: %.4f",
	       scale.x(),scale.y(),scale.z() );
      emit setInfoText( buffer );
    }
}

void
ScaleControler::evaluateInput( QMouseEvent *e, int button )
{
  if( button & LeftButton )
    doScale( e->x(), e->y(), _oldz );
  else
    doScale( e->x(), _oldy, e->y() );
//TODO if alt button, selected_->addWidget( new ConstraintW )
}

void
ScaleControler::start( QMouseEvent *e )
{
  _oldx = e->x();
  _oldy = e->y();
  _oldz = _oldy;

  //TODO if( dynamic_cast(G3DScaleWidget*) selected_ )
  //cout << "dyn cast" << endl;
  if( selected_ && (selected_->getType() == TW_SCALE) )
    {
      if( e->state() == AltButton )
	uniform_ = true;
      else
	uniform_ = false;

      G3DMatrix map2global;
      selected_->getWorldTrafo( map2global );
      lfixed_ = ((G3DScaleWidget*)selected_)->getFixpoint();
      ((G3DScaleWidget*)selected_)->getHandlePosition( mousepos_ );
      svector_   = mousepos_ - lfixed_;
      gfixed_    = map2global * lfixed_;
      mousepos_  = map2global * mousepos_;

      map2local_ = map2global;
      map2local_.invert3();

      selected_->getLocalTrafo( mlocal_ );
      scale_  = selected_->getScale();
      transl_ = selected_->getOrigin();

      char buffer[40];
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f",
	       scale_.x(),scale_.y(),scale_.z() );
      emit setInfoText( buffer );
    }
}

void
ScaleControler::stop ()
{
}

void
ScaleControler::doScale( int x, int y, int z )
{
  if( selected_ )
    {
      camera_->unproject( x, y, mousepos_ );
  
      G3DVector3d bound( FG_EPSILON ); //don't make the object smaller
      G3DVector3d svector( mousepos_ - gfixed_ );

      svector = map2local_ * svector;
      svector.cwDivide( svector_ );
      svector.max( bound );  //TODO before or after divide??
      if( uniform_ )
	{
	  MeFloat gt = svector.getMaxValue();
	  svector.setTo( gt, gt, gt );
	}

      G3DMatrix scaler;
      scaler.scale( svector );
      scaler = mlocal_ * scaler;
      svector.cwMultiply( scale_ );  //TODO grid

      selected_->setScale( svector );
      selected_->setOrigin( (mlocal_*lfixed_) - (scaler*lfixed_)
				 + transl_ );

      char buffer[40];
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f",
	       svector.x(),svector.y(),svector.z() );
      emit setInfoText( buffer );
    }
}


//================================END===================================
