/*
 * Project : "MindsEye"
 * Copyright (c) 1996, 1997 by Martin Kuball. All Rights Reserved.
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: tcontroler.cpp,v 1.2 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 *
 */

#include "include/config.h"
#include <stdio.h>
#include <qkeycode.h>
#include <qwidget.h>

#include "3dlib/object.h"
#include "3dlib/camera.h"
#include "obj_controler.h"
#include "grid.h"

//TODO a grid may be not general enough. Use a Constraint instead?
// external contraint vs. internal constraint between objects in the scene


void
MoveControler::doMove( int x, int y, int z )
{
  //TODO do some sanity checking?
  G3DPoint3d mousep(mousepos_);
  MeFloat delz  = (MeFloat)(z - oldz_) / 40.0;
  if( delz != 0 )
    mousep += camera_->getDirection() * delz;
  camera_->unproject( x, y, mousep );

  G3DVector3d delta( mousep - mousepos_ );
  delta = map2local_ * delta;
  G3DPoint3d  npos( position_ + delta );
  if( grid_ ) //w.r.t. parent coordinate system
    {
      grid_->projectToGrid( npos );
      delta = npos - position_;
    }
  if( !delta.isZero() )
    {
      selected_->setOrigin( position_ + delta );

      char buffer[40];
      //TODO show local or global pos? especially if moving objects or vertices
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f", npos.x(),
	       npos.y(), npos.z() );
      emit setInfoText( buffer );
    }
}

void
MoveControler::evaluateInput( QMouseEvent *e, int button )
{
  if( selected_ )
    {
      if( button & LeftButton )
	doMove( e->x(), e->y(), oldz_ );
      else
	doMove( e->x(), oldy_, e->y() );
    }
}

void
MoveControler::start( QMouseEvent *e )
{
  oldy_ = oldz_ = e->y();

  if( selected_ )
    {
      //TODO get the point on the surface
      //manipulation is done in the world coordinate system
      G3DMatrix map2global;
      //Get a matrix that maps from parent to global coordinates
      selected_->getWorldTrafo( map2global, false );
      position_ = selected_->getOrigin(); //ml * position_;
      mousepos_ = map2global * position_;
      camera_->unproject( e->x(), e->y(), mousepos_ );
      map2local_ = map2global;
      map2local_.invert3();
      //orig_ = selected_->getOrigin();

      //TODO ? selected_->addWidget();
      char buffer[40];
      sprintf( buffer, "x: %.3f y: %.3f z: %.3f", position_.x(),
	       position_.y(), position_.z() );
      emit setInfoText( buffer );
    }
}

void
MoveControler::stop ()
{
}


//================================END===================================
