/*
 * Project : MindsEye
 * Copyright (c) 1996, 1997, 1998
 * Author(s): M. Kuball, M. van Velsen, F. Meier
 * 
 * $Id: viewgl.cpp,v 1.7 2002/06/17 18:18:14 ainvar Exp $
 *
 *
 * COPYRIGHT
 * =========
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * OVERVIEW
 * ========
 * a widget which uses OpenGL to display a scene
 *
 */

#ifdef HAVE_CONFIG_H
#include "include/config.h"
#endif	 // HAVE_CONFIG_H
#include <stdio.h>

//include <stdlib.h>
#include <qkeycode.h>
#include <qpopmenu.h>
#include <qtimer.h>

#include "3dlib/object.h"
#include "3dlib/camera.h"
#include "3dlib/s_scene.h"
#include "3dlib/actions.h"
#include "3dlib/Attributes/color.h"
#include "kernel/world_obj.h"
#include "kernel/world_list.h"
#include "modeler/modeler.h"
#include "m_vmenu.h"
#include "m_vdef.h"
#include "m_viewl.h"
#include "viewgl.h"
#include "obj_controler.h"
#include "cam_controler.h"
//TODO #include "grid.h"

//TODO ? GLuint MGLPort::selectBuffer[SELECTSIZE];

int cnum = 0;

/*---------------------------------------------------------------------------*/

MGLPort::MGLPort( QPopupMenu *dpopup, int id, QWidget *parent ) :
  QGLWidget(parent)
{
  setFocusPolicy( QWidget::ClickFocus ); // we accept keyboard focus

  m_id      = id;
  world_    = NULL;
  scene_    = NULL; //TODO new G3DScene;
  camera_   = &defaultCamera_;
  defaultCamera_.setName( "default Camera" );
  defaultCamera_.setViewMode( (G3DCamera::TViewMode)id );

  _selectColor   = new G3DColor( 1, 0, 0 );  //TODO not used, yet
  selected_      = NULL;
  active_        = NULL;
  _defaultPopup  = dpopup;
  shadingMode_   = FLAT;
  see_grid       = TRUE;
  see_axes       = TRUE;

  focusFlag_    = false;
  mouseButton_  = NoButton;
  opMode_       = activeOpMode_ = ViewMode;
  dispListMode_ = DirectMode;
  camControler_    = NULL; //new MECameraControler;
  activeControler_ = NULL;
  damaged_      = false;
  timer_        = new QTimer (this);
  connect (timer_, SIGNAL(timeout()), this, SLOT(repairGL()));
}

MGLPort::~MGLPort()
{
  delete _selectColor;
  if( world_ && focusFlag_ )
    world_->activateDefaultCamera();
}

/*---------------------------------------------------------------------------*/
//TODO remove this -> camera::doAction()
void
MGLPort::centerCamera ()
{
  ASSERT(camera_);
  ASSERT(world_);
  
  camera_->setUVN( camera_->getPosition(), G3DPoint3d(0,0,0),
		   G3DVector3d(0,1,0) );
  world_->updateCamera();
}

void
MGLPort::drawAxes  ()
{
  //TODO compile the stuff
  //axisId = glGenLists(1) ;
  //glNewList(axisId,GL_COMPILE);
  //glEndList();
  if( see_axes == TRUE )
    {
      //TODO cout << "axes" << endl;
      glLineWidth( 1.6 );
      glDisable( GL_LIGHTING );
      glBegin( GL_LINES );

      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( -100, 0, 0 );
      glVertex3f( 0, 0, 0 );
      glColor3f( 1.0, 0.0, 0.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 100, 0, 0 );

      glColor3f( 0, 1.0, 0.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 0, 100, 0 );
      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( 0, -100, 0 );
      glVertex3f( 0, 0, 0 );

      glColor3f( 0, 0.0, 1.0 );
      glVertex3f( 0, 0, 100 );
      glVertex3f( 0, 0, 0 );
      glColor3f( 1.0, 1.0, 1.0 );
      glVertex3f( 0, 0, 0 );
      glVertex3f( 0, 0, -100 );

      glEnd();
    }
}

//TODO make this more configurable
void
MGLPort::drawGrid ()
{
  if( see_grid == TRUE )
    {
      MeFloat     width = 30;
      G3DVector3d step1, step2;
      G3DPoint3d  x1, x2, y1, y2;

      ASSERT(camera_);
      
      switch( camera_->getViewMode() )
	{
	case G3DCamera::Perspective:
	case G3DCamera::Top:
	case G3DCamera::Bottom:
	  step1.setTo(1, 0, 0);  //TODO step?
	  step2.setTo(0, 0, 1);
	  x1.setTo( -width, 0,  width );
	  x2.setTo( -width, 0, -width );
	  y1.setTo( -width, 0, -width );
	  y2.setTo(  width, 0, -width );
	  break;
	case G3DCamera::Left:
	case G3DCamera::Right:
	  step1.setTo( 0, 1, 0 );
	  step2.setTo( 0, 0, 1 );
	  x1.setTo( 0, -width, width);
	  x2.setTo( 0, -width, -width);
	  y1.setTo( 0, -width, -width);
	  y2.setTo( 0,  width, -width);
	  break;
	case G3DCamera::Front:
	case G3DCamera::Back:
	  step1.setTo( 1, 0, 0 );
	  step2.setTo( 0, 1, 0 );
	  x1.setTo( -width, -width, 0 );
	  x2.setTo( -width,  width, 0 );
	  y1.setTo( -width, -width, 0 );
	  y2.setTo(  width, -width, 0 );
	  break;
	}
      glDisable( GL_LIGHTING );
      glColor3f ( 0.8, 0.8, 0.8 );
      glLineWidth( 0.8 );
      glBegin( GL_LINES );
      for( int i = 60; i >= 0; i-- )
	{
	  if( i != 30 )
            {
	      glVertex3xv( (const MeFloat*)x1 );
	      glVertex3xv( (const MeFloat*)x2 );
	      glVertex3xv( (const MeFloat*)y1 );
	      glVertex3xv( (const MeFloat*)y2 );
            }
	  x1 += step1; x2 += step1;
	  y1 += step2; y2 += step2;
	}
      glEnd();
    }
}

/*---------------------------------------------------------------------------*/

G3DObject*
MGLPort::examineHitBuffer( int hits, GLuint *ptr, G3DObject *obj )
{
  // printf("-->MGLPort::examineHitBuffer(...)\n");
  
  CHECK_PTR(ptr);
  CHECK_PTR(obj);

  GLuint num_names, depth = (GLuint)-1, *picked = 0;

  for( int i = 0; i < hits; ++i )
    {
      num_names = *ptr++;
      if( num_names && (*ptr < depth) )
	{
	  depth  = *ptr ;
	  picked = ptr-1;
	}
      ptr += num_names + 2;
    }

  if (picked) {
    SelectData sdata( picked );
    obj->doAction (sdata);
    return (sdata.getSelection());
  }
  else
    return 0;
}

void
MGLPort::initPicking(QMouseEvent *e, GLsizei bs, GLuint buf[] )
{
  // printf("-->MGLPort::initPicking(...)\n");

  GLint         viewport[4];
  G3DMatrix     tM;

  initShader();
  glGetIntegerv(GL_VIEWPORT, viewport);

  glSelectBuffer( bs, buf );
  glRenderMode( GL_SELECT );
  glInitNames();

  glMatrixMode( GL_PROJECTION );
  glPushMatrix();

  glLoadIdentity();
  gluPickMatrix( (GLdouble)e->x()-2, 
		 (GLdouble)(viewport[3] - e->y() + 2), 
		 4,4,            // pick rectangle in pixels
		 viewport);

  ASSERT(camera_);

  camera_->getProjectionMatrix( tM );
  glMultMatrixx( (const MeFloat *)tM );
  
  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  camera_->getViewMatrix( tM ); 
  glLoadMatrixx( (const MeFloat *)tM );

  // printf("...leaving initPicking\n");
}

G3DObject*
MGLPort::pickObject(QMouseEvent *e, G3DGroup *elements )
{
  // printf("-->MGLPort::pickObject(...)\n");

  const GLsizei buf_size = 200;
  GLuint        buf[buf_size];

  initPicking( e, buf_size, buf );

  ASSERT(elements);
  elements->select();
  glPopMatrix();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glFlush();

  int hits = (int)glRenderMode(GL_RENDER);

  // printf("...leaving pickObject\n");
  return examineHitBuffer( hits, buf, elements );
}

G3DObject*
MGLPort::pickSubObject( QMouseEvent *e, G3DObject *obj )
{
  // printf("-->MGLPort::pickSubObject(...)\n");
  CHECK_PTR(obj);

  G3DRay ray;

  ASSERT(camera_);
  camera_->buildRay( e->x(), e->y(), ray );
  if( obj->select( *camera_, ray, allparts) < 4.0 )
    return (obj->getSelectionHandle());
  else
    return 0;
}

/*---------------------------------------------------------------------------*/
/*
G3DCamera*
MGLPort::getCamera ()
{
  return camera_;
}
*/
void
MGLPort::initShader ()
{
/* TODO set the quality of the output:
	HINT( enum target, enum hint )
  target = PERSPECTIVE_CORRECTION_HINT; FONG_HINT, POLYGON_SMOOTH_HINT
  hint = DONT_CARE, NICEST, FASTEST
  enable / disable POLYGON_SMOOTH (aliasing)
  // TODO must be set in MGLPort?? glEnable( GL_DITHER );
  //  glDisable( GL_DITHER );
*/

  if( shadingMode_ == WIRE )
    {
      glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
      glShadeModel( GL_FLAT );
      glDisable( GL_DEPTH_TEST );
      glDisable( GL_CULL_FACE );
    }
  else  //FLAT and SMOOTH  TODO what about TEXTURE
    {
      glClearDepth( 1.0 );  //TODO use near value? copy from a buffer
      glClear( (GLbitfield)GL_DEPTH_BUFFER_BIT );
      glDepthFunc( GL_LESS );
      glEnable( GL_DEPTH_TEST );
      glEnable( GL_CULL_FACE );
      glCullFace( GL_BACK );

      glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
      if( shadingMode_ == FLAT )
	glShadeModel( GL_FLAT );
      else
	glShadeModel (GL_SMOOTH);
    }

  glEnable( GL_NORMALIZE );  //TODO realy? maybe it slows things down?
  //clockwise orientation of polygons
  glFrontFace( GL_CW ); //TODO is this the best choice?

  /*
  //TODO should be GL_DIFFUSE only ???
  glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
#ifdef SUNOPENGL 
  glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, black ) ;
#endif
  glEnable( GL_COLOR_MATERIAL );
  */
}

/*---------------------------------------------------------------------------*/

void
MGLPort::focusInEvent ( QFocusEvent* )
{
  // printf("-->MGLPort::focusInEvent(...)\n");

  if (!focusFlag_ && world_) {
    focusFlag_ = true;  //TODO
    // cout << "fie2 " << m_id << " " << camera_->getName() << endl;
    connect( world_, SIGNAL(cameraActivated(G3DCamera*)),
	     SLOT(setCamera(G3DCamera*)) );
    connect( world_, SIGNAL(cameraChanged()),
	     SLOT(damageGL()) );
    world_->setActiveCamera( &defaultCamera_ );
    emit setFocus (this);
  }
}

void
MGLPort::focusOutEvent (QFocusEvent*)
{
  // printf("-->MGLPort::focusOutEvent(...)\n");

  if (camera_)
    defaultCamera_ = *camera_;
  focusFlag_ = false;
  // cout << "foe " << camera_->getName() << endl;
  if (world_) {
    // printf("...OK : world_ is non-null.\n");
    world_->disconnect( this, SLOT(setCamera(G3DCamera*)) );
  }
  // printf("...leaving.\n");
}

void
MGLPort::keyPressEvent( QKeyEvent* e )
{
  // printf("-->MGLPort::keyPressEvent(...)\n");

  int sign = -1;

  switch( e->key() )
    {
    case Key_Delete:
      world_->deleteObject( NULL );   //delete selected object
      break;
      /*
    case Key_1:
      camControler_->setMode( MECameraControler::AllInOne );
    case Key_2:
      camControler_->setMode( MECameraControler::Pan );
    case Key_3:
      camControler_->setMode( MECameraControler::Orbit );
    case Key_4:
      camControler_->setMode( MECameraControler::Tuck );
    case Key_5:
       camControler_->setMode( MECameraControler::Rotate );
    case Key_6:
       camControler_->setMode( MECameraControler::Zoom );
    case Key_7:
       camControler_->setMode( MECameraControler::Roll );
       break;
      */
    /* TODO use keys for exact movement
    case Key_Left:
      camera_->translate( 1,0,0 );
      break;
    case Key_Up:
      camera_->translate( 0,-1,0 );
      break;
    case Key_Right:
      camera_->translate( -1,0,0 );
      break;
    case Key_Down:
      camera_->translate( 0,1,0 );
      break;
    */

    case Key_Plus :
      sign = 1;
    case Key_Minus :
      if( selected_ )
	{
	  ScaleControler scnt;  //TODO not connected
	  scnt.setSelectedObject( selected_ );
	  G3DVector3d delta( sign*0.2 );
	  scnt.evaluate( delta );
          ASSERT(world_);
	  world_->update(); //TODO
	}
      else
	{
          ASSERT(camera_);
          ASSERT(world_);
	  camera_->setFOV( camera_->getFOV() + (sign*2) );
	  world_->updateCamera();
	}
      break;
    case Key_C :
      if( mouseButton_ == NoButton )
        activeOpMode_ = ViewMode;
      return;
      break;
    default:
      //activeControler::keyEvent( e );
      e->ignore();
      return;
    }
}

void
MGLPort::keyReleaseEvent( QKeyEvent* e )
{
  //don't change mode during an operation
  if (mouseButton_ == NoButton)
    activeOpMode_ = opMode_;
}

void
MGLPort::mouseMoveEvent( QMouseEvent * e )
{
  // printf("-->MGLPort::mouseMoveEvent(...)\n");

  if (activeControler_)
    activeControler_->evaluateInput (e, mouseButton_);
  /*
  if ( activeOpMode_ == ViewMode )
    {
      camControler_->move( e, mouseButton_ );
      world_->updateCamera();
    }
  else if( activeOpMode_ == ObjectMode )
    {
      if( active_ && tControler_ ) //TODO test for controler is enough?
	tControler_->transform( e, mouseButton_ );
    }
  */
  // updateGL();  //TODO
  damageGL ();
}

void
MGLPort::mousePressEvent( QMouseEvent * e )
{
  // printf("-->MGLPort::mousePressEvent(...)\n");

  // cout << "mpe" << endl;
  mouseButton_ = e->button();  // save mouse click state

  if (mouseButton_ != RightButton) {
      if (activeOpMode_ == ViewMode) {
        ASSERT(camControler_);
        camControler_->start( e );
        activeControler_ = camControler_;
      }
      else {   //( activeOpMode_ == ObjectMode )
        //TODO if( _collectPointMode )
        G3DObject *sub = 0;
        ASSERT(model_);
        G3DObject *tmp = model_->getNewObject();
        if (tmp == 0) { //try to select an object in the scene
          // cout << "state: " << e->state() << endl;
          if (e->state() & ShiftButton) {
            sub = pickObject( e, scene_ ); //does not select widgets,...
            if (sub) {
              ASSERT(world_);
              active_ = selected_ = world_->addToSelection (sub);
              setControler( e );
              damageGL();
            }
            // printf("...1 : leaving mousePressEvent(...)\n");
            return; //TODO
          }
          if (e->state() & ControlButton) {
            selected_ = pickObject( e, scene_ ); //does not select widget
            if (selected_) {
              selected_ = selected_->copy ();
              ASSERT(world_);
              world_->addObject (selected_);
            }
          }
          else {
            if (selected_ && (selected_->getType() != T_SEL_GROUP))
              sub = pickSubObject( e, selected_ );
            if (sub == 0) {
              // cout << "select" << endl;
              selected_ = pickObject (e, scene_);
            }
          }
        }
        else {  //we have a new object
          selected_ = tmp;
          ASSERT(camera_);
          G3DPoint3d pnt( camera_->getLookAt() );
          // cout << "fpt " << pnt << endl;
          camera_->unproject( e->x(), e->y(), pnt );
          selected_->setOrigin( pnt );
        }
        
        if (sub) {
          //TODO if( active_ != selected_ ) delete active_;
          active_ = sub;
        }
        else {
          active_ = selected_;
          ASSERT(world_);
          world_->selectObject( selected_ );
        }
        if (active_) {
          //selected_->setSelectColor( _selectColor );
          setControler (e);
        }
        else
          updateDisplayList();
        damageGL ();
      }
  }
  else {
    QPopupMenu *popup;
    QPoint     new_coors = mapToGlobal(e->pos());
    if ((activeOpMode_ == ViewMode) || (selected_ == 0))
      popup = _defaultPopup;
    else {
      ASSERT(model_);
      popup = model_->getPopupMenuFor( selected_ );	//TODO active?
    }
    if (popup != 0) {
      popup->move( new_coors.x(), new_coors.y() );
      popup->show();
    }
  }
  repairGL ();

  // printf("...leaving mousePressEvent(...)\n");
}

void
MGLPort::mouseReleaseEvent( QMouseEvent * e )
{
  // printf("-->MGLPort::mouseReleaseEvent(...)\n");

  if (activeControler_) {
    activeControler_->stop();
    activeControler_ = 0;
  }
  
  if (active_) {
    if (selected_ && (IS_CLASS(selected_->getType(), T_SCENE_ELEMENT))) {
      selected_->adjust();  //TODO for Helper's too?
    }
    
    //if( IS_CLASS(active_->getType(), T_HANDLE) )
    //active_->dispose(); if handle?
    active_ = 0;
    
    //save command
    if (activeOpMode_ != ViewMode) {
      ASSERT(world_);
      world_->update();  //TODO
    }
  }
  activeOpMode_ = opMode_;
  mouseButton_  = NoButton;
  // printf("..leaving mouseReleaseEvent(...)\n");
}

/*---------------------------------------------------------------------------*/

void
MGLPort::objectSelected ( G3DObject *obj )
{
  // cout << "--> MGLPort::objectSelected (" << (const char *)(obj?"ok":"null")
  //     << ")" << endl;;
  
  if (active_ == 0) {
    selected_ = obj;
    damageGL ();
  }
}

void
MGLPort::paintGL()
{
  // cout << "--> MGLPort::paintGL ()" << endl;
  
  initShader ();
  ASSERT(camera_);
  camera_->render (); // setup projection and modelview matrices
  
  if (scene_) {
    resetLights ();
    int start = 1;
    scene_->display (start); //1 = First available OGL light number
  }
  glPushMatrix ();
  glMultMatrixx ((const MeFloat*)workScale_);
  //Must be after rendering of the scene, cause the scene sets the background
  drawGrid ();
  drawAxes ();
  glPopMatrix ();
  
  //TODO controler_[0]->display();
  ASSERT(model_);
  if (model_->showSRegion ())
    camera_->showSecureRegion ();
  if (activeControler_)
    activeControler_->showFeedback ();  //must be last in the rendering queue
  
  //TODO write errors to console?
  int error;
  while (1) {
    error = glGetError();
    if (error == GL_NO_ERROR)
      break;
    fprintf (stderr, "GL error: %s\n", gluErrorString((GLenum)error));
  }
  //TODO if show save frame: camera->showFrame();
  
  // printf("...leaving paintGL\n");
}

void
MGLPort::damageGL ()
{
  // cout << "--> MGLPort::damageGL ()" << endl;

  damaged_ = true;
  timer_->start(0, true);
}

void
MGLPort::repairGL ()
{
  // cout << "MGLPort::repairGL ()" << endl;

  if (damaged_) {
    updateGL ();
    damaged_ = false;
    timer_->stop();
  }
}

void
MGLPort::resetLights ()
{
  // cout << "--> MGLPort::resetLights ()" << endl;
  
  // reset light sources:
  for (int i = 0; i < 8; i++)
    glDisable( (GLenum)(GL_LIGHT0 + i) );

  //light 0 is only ambient
  GLfloat tmpcol[4] = {0.0, 0.0, 0.0, 1.0};
  glLightfv( GL_LIGHT0, GL_DIFFUSE,  tmpcol );
  glLightfv( GL_LIGHT0, GL_SPECULAR, tmpcol );
 
  //TODO make this configurable?
  //glLightModelfi(GL_LIGHT_MODEL_LOCAL_VIEWER, 1 );
  //glLightModelfi(GL_LIGHT_MODEL_TWO_SIDE, 1 );
  
  glEnable(GL_LIGHTING);
  // printf("...leaving()\n");
}

void
MGLPort::resizeGL( int width, int height )
{
  // printf("-->MGLPort::resizeGL(...) w:%d, h:%d\n", width, height);
  
  if (camera_) {
    // printf("...OK : we got a camera.\n");
    camera_->setWindowSize (width, height);
  }
  //TODO adjust move buffers
}

void
MGLPort::setCamera( G3DCamera *cam )
{
  CHECK_PTR(cam);

  if (camera_ != cam) {  //TODO
    camera_ = cam;
    camera_->setWindowSize (width(), height());
    // updateGL();
    damageGL ();
  }
}

void
MGLPort::setCameraControler( MECameraControler *cc )
{
  camControler_ = cc;
}

void
MGLPort::setCameraType ( int t )
{
  ASSERT(camera_);
  ASSERT(world_);
  
  camera_->setViewMode( (G3DCamera::TViewMode)t );
  world_->updateCamera();
}

void
MGLPort::setControler( QMouseEvent *e )
{
  // printf("-->MGLPort::setControler (...)\n");
  
  ASSERT(model_);
  MEObjectControler *tmp = model_->getControlerFor( active_ );
  if (tmp) {
    tmp->setSelectedObject (active_);
    tmp->setCamera (camera_);
    tmp->start (e);
    activeControler_ = tmp;
  }
}

void
MGLPort::setDisplayListMode( int mode )
{
  // printf("-->MGLPort::setDisplayListMode (...)\n");

  dispListMode_ = mode;
  if (mode == DirectMode) {
    DeleteListAction action;
    ASSERT(scene_);
    scene_->doAction( action );
  }
  else if (mode == DisplayListMode)
    updateDisplayList();
}

//TODO
//camControler
//editControler
//setCameraControler( MEControler* )
void
MGLPort::setMajorInteractionMode( int mode )
{
  opMode_ = activeOpMode_ = mode;

  //activeControler_->releaseControl();
  //activeControler_ = controler_[mode];
  /*
  switch( opMode_ )
    {
    case ViewMode:
      setCursor( *hand_cursor );  //TODO move this into camControler
      break;
    case ObjectMode:
      setCursor( arrowCursor );
      break;
    }
  */
  //activeControler_->takeControl();
}

void
MGLPort::setShadingMode (int new_dtype)
{
  shadingMode_ = new_dtype;
  // updateGL();
  damageGL ();
}

void
MGLPort::setWorkScale( MeFloat s )
{
  // printf("-->MGLPort::setWorkScale (...)\n");

  workScale_.setToIdentity();
  workScale_.scale( G3DVector3d(s) );
  ASSERT(camera_);
  G3DPoint3d eye = camera_->getLookAt();
  eye += camera_->getDirection() * s * 5.0;
  camera_->setPosition( eye );
  camera_->setZFar( 300.0 * s );  //The dynamic range is constant
  camera_->setZNear( 0.05 * s );
  // updateGL();
  damageGL ();
}

void
MGLPort::setWorld ( QObject *world )
{
  // cout << "--> MGLPort::setWorld (" << (const char *)(world?"ok":"null")
  //      << ")" << endl;
  if (world) {
    if (world_)
      world_->disconnect (this);
    world_  = (MWorldObject*)world;
    scene_  = world_->getScene();
    
    connect( world_, SIGNAL(objectSelected(G3DObject*)),
             SLOT(objectSelected(G3DObject*)) );
    connect( world_, SIGNAL(sceneChanged(G3DGroup*)),
	     SLOT(damageGL()) );
    connect( world_, SIGNAL(objectChanged()),
	     SLOT(damageGL()) );
    
    updateDisplayList ();
    setCamera ((G3DCamera*)((world_->getCameras()).front()));
  }
}

void
MGLPort::show_axes (bool yes_or_no)
{
  see_axes = yes_or_no;
  update ();
}

void
MGLPort::show_grid (bool yes_or_no)
{
  see_grid = yes_or_no;
  update ();
}

void
MGLPort::updateDisplayList ()
{
  if (dispListMode_ == DisplayListMode) {
    // cout << "compiling display list" << endl;
    initShader();
    scene_->compileList( 0 );
  }
}


//================================EMD====================================
